/**
 * This file is part of FatturazioneElettronica - Core Extensions - Comune di Modena
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core Extensions - Comune di Modena.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.invoice;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.par.comunemodena.AreaCommercialeCC;
import it.fe.core.par.comunemodena.AreaCommercialeInvoiceLine;
import it.fe.core.par.comunemodena.CapitoloArticoloCC;
import it.fe.core.par.comunemodena.CapitoloArticoloInvoiceLine;
import it.fe.core.par.comunemodena.CentroDiCostoCC;
import it.fe.core.par.comunemodena.CentroDiCostoInvoiceLine;
import it.fe.core.par.comunemodena.ContoContrattuale;
import it.fe.core.par.comunemodena.InvoiceAdditionalData;
import it.fe.core.par.comunemodena.InvoiceLineAdditionalData;
import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.utility.Meter;
import it.fe.core.ws.comunemodena.sib.SibService;
import it.fe.core.ws.comunemodena.sib.SibServiceService;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.NoResultException;
import javax.persistence.Query;

@Stateless
@Remote(InvoiceManagerComuneModena.class)
public class InvoiceManagerComuneModenaBean extends InvoiceManagerBean implements InvoiceManagerComuneModena {
	
	public InvoiceManagerComuneModenaBean() {
		SibServiceService SIBproxy=null;
		try {
			//SIBproxy = new SibServiceService(new URL("http://localhost:8080/FECoMoBackend/SibService?wsdl"));
			SIBproxy = new SibServiceService(new URL(FEPropertiesUtil.getPA_Backend_WSDL_URL()));
			SIBservice = SIBproxy.getSibServicePort();
		} catch (MalformedURLException e) {
			e.printStackTrace();
			//Log.error("Errore durante inizializzazione Web Service SIB",e);
		}
		codiciIva = new HashMap<String, String>();
		codiciIva.put("S3", "  ");
		codiciIva.put("S2", "  ");
		codiciIva.put("S0", "  ");
		codiciIva.put("QA", "  ");
		codiciIva.put("Q1", "  ");

		codiciIva.put("T8", "60");
		codiciIva.put("T7", "  ");
		codiciIva.put("T6", "60");
		codiciIva.put("T5", "44");
		codiciIva.put("T4", "55");
		codiciIva.put("T2", "60");
		codiciIva.put("T1", "60");
		
		codiciIva.put("ZT", "65");
		codiciIva.put("ZX", "65");
		codiciIva.put("ZV", "65");
		codiciIva.put("WA", "  ");
		codiciIva.put("W4", "04");
		codiciIva.put("W2", "20");
		codiciIva.put("W0", "10");
		codiciIva.put("VA", "  ");
		codiciIva.put("V4", "04");
		codiciIva.put("V2", "20");
		codiciIva.put("V0", "10");
		codiciIva.put("U8", "60");
		codiciIva.put("U7", "  ");
		codiciIva.put("U6", "60");
		codiciIva.put("U5", "44");
		codiciIva.put("U4", "55");
		codiciIva.put("U2", "60");
		codiciIva.put("U1", "60");
	}
	
	private Map<String,String> codiciIva;
	
	protected SibService SIBservice;
	
	@Override
	public List<AreaCommercialeInvoiceLine> fetchInvoiceLineAdditionalDataAreaCommerciale(InvoiceLineAdditionalData invoiceLineAdditionalData) {
		invoiceLineAdditionalData = super.entityManager.merge(invoiceLineAdditionalData);
		invoiceLineAdditionalData.getAreeCommerciali().size();
		List<AreaCommercialeInvoiceLine> attachedList = invoiceLineAdditionalData.getAreeCommerciali();
		List<AreaCommercialeInvoiceLine> detachedList = new ArrayList<AreaCommercialeInvoiceLine>();
		for (AreaCommercialeInvoiceLine ad : attachedList) {
			detachedList.add(ad);
		}
		return detachedList;
	}

	@Override
	public long insertInvoice(Invoice invoice) {
		long id = super.insertInvoice(invoice);
		Invoice managedInvoice = super.entityManager.find(Invoice.class, id);

		for(InvoiceLine line: managedInvoice.getInvoiceLines()) {
			String contoContrattuale = line.getContoContrattuale();
			if(contoContrattuale != null) {
				Query q=super.entityManager.createQuery("from ContoContrattuale where contoContrattuale = :cc");
				q.setParameter("cc", contoContrattuale);
				ContoContrattuale cc=null;
				try {
					cc = (ContoContrattuale)q.getSingleResult();
				} catch (NoResultException e) {}
				if(cc == null) {
					cc = new ContoContrattuale();
					cc.setContoContrattuale(contoContrattuale);
					cc.setCodiceCliente("");
				}
				
				Meter managedMeter = super.entityManager.merge(line.getMeter());
				
				List<AdditionalData> meterAdditionalDataList=managedMeter.getAdditionalData();
				
				if(meterAdditionalDataList == null) {
					meterAdditionalDataList = new LinkedList<AdditionalData>();
					line.getMeter().setAdditionalData(meterAdditionalDataList);
				}
				
				ContoContrattuale contoContrattualeMeter=null;
				for(AdditionalData additionalData: meterAdditionalDataList) {
					if(additionalData instanceof ContoContrattuale)
						contoContrattualeMeter = (ContoContrattuale)additionalData;
				}
				if(contoContrattualeMeter == null) {
					logger.info("Contatore "+line.getMeter().getMeterNumber()+" senza conto contrattuale. Lo aggiungo");
					meterAdditionalDataList.add(cc);
				} else {
					if(!cc.getContoContrattuale().equals(contoContrattualeMeter.getContoContrattuale())) {
						logger.info("Contatore "+line.getMeter().getMeterNumber()+" con conto contrattuale non combaciante con anagrafica. Lo sostituisco");
						meterAdditionalDataList.remove(contoContrattualeMeter);
						meterAdditionalDataList.add(cc);
					}
				}
				
				List<AdditionalData> additionalDataList = line.getAdditionalData();
				if(additionalDataList == null) {
					additionalDataList = new LinkedList<AdditionalData>();
					line.setAdditionalData(additionalDataList);
				}
				additionalDataList.add(cc);
				//super.entityManager.persist(cc);
				//super.entityManager.flush();
			}
		}
		this.mergeInvoiceAdditionalDataFromMeter(managedInvoice);
		return id;
	}
	
	@Override
	public Invoice mergeInvoiceAdditionalDataFromMeter(Invoice invoice) {
		Invoice managedInvoice;
		//if(super.entityManager.contains(invoice))
		//	managedInvoice = invoice;
		//else
			managedInvoice = super.entityManager.merge(invoice);
		List<AdditionalData> invoiceAdditionalDataList = managedInvoice.getAdditionalData();
		
		if(invoiceAdditionalDataList == null) {
			invoiceAdditionalDataList = new LinkedList<AdditionalData>();
			managedInvoice.setAdditionalData(invoiceAdditionalDataList);
		}
		
		InvoiceAdditionalData invoiceAdditionalData = null;
		for(AdditionalData additionalData: invoiceAdditionalDataList) {
			if(additionalData instanceof InvoiceAdditionalData) {
				invoiceAdditionalData = (InvoiceAdditionalData)additionalData;
			}
		}
		if(invoiceAdditionalData==null) {
			invoiceAdditionalData = new InvoiceAdditionalData();
			invoiceAdditionalDataList.add(invoiceAdditionalData);
		}
		
		
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis(System.currentTimeMillis());
		invoiceAdditionalData.setAnnoEsercizio(c.get(Calendar.YEAR));
		
		//TODO: Non ricordo se dovevo prendere il mese precedente
		c.setTime(invoice.getIssueDate());
		c.set(Calendar.DAY_OF_MONTH, 1);
		invoiceAdditionalData.setInizioCompetenzaEconomica(c.getTime());
		c.add(Calendar.MONTH, 1);
		c.add(Calendar.DAY_OF_MONTH, -1);
		invoiceAdditionalData.setFineCompetenzaEconomica(c.getTime());
		
		if(invoice.getAllInclusiveTotal() >= 0)
			invoiceAdditionalData.setTipoFattura("FS");
		else
			invoiceAdditionalData.setTipoFattura("CS");
		
		for(InvoiceLine line: invoice.getInvoiceLines()) {
			//Meter meter=line.getMeter();
			List<AdditionalData> invoiceLineAdditionalDataList=line.getAdditionalData();
			
			if(invoiceLineAdditionalDataList == null) {
				invoiceLineAdditionalDataList = new LinkedList<AdditionalData>();
				line.setAdditionalData(invoiceLineAdditionalDataList);
			}
			
			ContoContrattuale contoContrattuale = null;
			for(AdditionalData additionalData: invoiceLineAdditionalDataList) {
				if(additionalData instanceof ContoContrattuale) {
					contoContrattuale = (ContoContrattuale)additionalData;
				}
			}
			if(contoContrattuale != null) {
				if(invoiceAdditionalData.getCodiceDivisione() == null)
					invoiceAdditionalData.setCodiceDivisione(contoContrattuale.getCodiceDivisione());
				if(invoiceAdditionalData.getCodiceSettore() == null)
					invoiceAdditionalData.setCodiceSettore(contoContrattuale.getCodiceSettore());
				if(invoiceAdditionalData.getCodiceUfficio() == null)
					invoiceAdditionalData.setCodiceUfficio(contoContrattuale.getCodiceUfficio());

				InvoiceLineAdditionalData invoiceLineAdditionalData = null;
				for(AdditionalData additionalData: invoiceLineAdditionalDataList) {
					if(additionalData instanceof InvoiceLineAdditionalData) {
						invoiceLineAdditionalData = (InvoiceLineAdditionalData)additionalData;
					}
				}
				
				if(invoiceLineAdditionalData == null) {
					invoiceLineAdditionalData = new InvoiceLineAdditionalData();
					invoiceLineAdditionalDataList.add(invoiceLineAdditionalData);
				}

				List<AreaCommercialeCC> areeCommerciali = contoContrattuale.getAreecommercialiccs();
				if(areeCommerciali != null) {
					List<AreaCommercialeInvoiceLine> lineAreeCommerciali=invoiceLineAdditionalData.getAreeCommerciali();
					if(lineAreeCommerciali == null) {
						lineAreeCommerciali = new LinkedList<AreaCommercialeInvoiceLine>();
						invoiceLineAdditionalData.setAreeCommerciali(lineAreeCommerciali);
					}
					int size=lineAreeCommerciali.size();
					for(int i=0;i<size;i++) {
						try {
							super.entityManager.remove(lineAreeCommerciali.remove(0));
						}
						catch(IllegalArgumentException e){
							//C'è qualcosa che non va
							e.printStackTrace();
						}
					}
					for(AreaCommercialeCC areaCommerciale: areeCommerciali) {
						AreaCommercialeInvoiceLine lineAreaCommerciale = new AreaCommercialeInvoiceLine();
						lineAreaCommerciale.setAreaCommercialeID(areaCommerciale.getId().getAreaCommerciale());
						lineAreaCommerciale.setPercentuale(areaCommerciale.getPercentuale());
						lineAreeCommerciali.add(lineAreaCommerciale);
					}
				}
				
				if(invoiceLineAdditionalData.getAreeCommerciali()!=null && invoiceLineAdditionalData.getAreeCommerciali().size()>0)
					invoiceLineAdditionalData.setFlagTipo("C");
				else
					invoiceLineAdditionalData.setFlagTipo("I");
				
				List<CapitoloArticoloCC> capitoliArticoli = contoContrattuale.getCapitoloarticoloccs();
				if(capitoliArticoli != null) {
					List<CapitoloArticoloInvoiceLine> lineCapitoliArticoli=invoiceLineAdditionalData.getCapitoliArticoli();
					if(lineCapitoliArticoli == null) {
						lineCapitoliArticoli = new LinkedList<CapitoloArticoloInvoiceLine>();
						invoiceLineAdditionalData.setCapitoliArticoli(lineCapitoliArticoli);
					}
					int size=lineCapitoliArticoli.size();
					for(int i=0;i<size;i++)
						try {
							super.entityManager.remove(lineCapitoliArticoli.remove(0));
						} catch(IllegalArgumentException e) {
							//C'è qualcosa che non va
							e.printStackTrace();
						}
					for(CapitoloArticoloCC capitoloArticolo: capitoliArticoli) {
						CapitoloArticoloInvoiceLine lineCapitoloArticolo = new CapitoloArticoloInvoiceLine();
						lineCapitoloArticolo.setArticolo(capitoloArticolo.getId().getArticolo());
						lineCapitoloArticolo.setCapitolo(capitoloArticolo.getId().getCapitolo());
						lineCapitoloArticolo.setPercentuale(capitoloArticolo.getPercentuale());
						lineCapitoliArticoli.add(lineCapitoloArticolo);
					}
				}
				
				List<CentroDiCostoCC> centriDiCosto = contoContrattuale.getCentridicostoccs();
				if(centriDiCosto != null) {
					List<CentroDiCostoInvoiceLine> lineCentriDiCosto=invoiceLineAdditionalData.getCentriDiCosto();
					if(lineCentriDiCosto == null) {
						lineCentriDiCosto = new LinkedList<CentroDiCostoInvoiceLine>();
						invoiceLineAdditionalData.setCentriDiCosto(lineCentriDiCosto);
					}
					int size=lineCentriDiCosto.size();
					for(int i=0;i<size;i++)
						try {
							super.entityManager.remove(lineCentriDiCosto.remove(0));
						} catch(IllegalArgumentException e){
							//C'è qualcosa che non va
							e.printStackTrace();
						}
					for(CentroDiCostoCC centroDiCosto: centriDiCosto) {
						CentroDiCostoInvoiceLine lineCentroDiCosto = new CentroDiCostoInvoiceLine();
						lineCentroDiCosto.setSettore(centroDiCosto.getSettore());
						lineCentroDiCosto.setServizio(centroDiCosto.getServizio());
						lineCentroDiCosto.setStruttura(centroDiCosto.getStruttura());
						lineCentroDiCosto.setPercentuale(centroDiCosto.getPercentuale());
						//super.entityManager.persist(lineCentroDiCosto);
						lineCentriDiCosto.add(lineCentroDiCosto);
					}
				}
				//Flush persisted additional data to assign id
				//super.entityManager.flush();
				String codiceIVA=this.codiciIva.get(line.getTaxDetail().getDescription());
				if(codiceIVA!=null) {
					invoiceLineAdditionalData.setCausaleRiga("IVA");
					invoiceLineAdditionalData.setSubCausaleRiga(codiceIVA);
				}
			}
		}
		
		return managedInvoice;
	}
	
	@Override
	public List<AdditionalData> fetchInvoiceLineAdditionalData(InvoiceLine invoiceLine) {
		invoiceLine = entityManager.merge(invoiceLine);
		invoiceLine.getAdditionalData().size();
		List<AdditionalData> additionalDataAttached = invoiceLine.getAdditionalData();
		List<AdditionalData> additionalDataDetached = new ArrayList<AdditionalData>();
		for (AdditionalData ad : additionalDataAttached) {
			additionalDataDetached.add(ad);
			if(ad.getClass().getName().equals(InvoiceLineAdditionalData.class.getName())) {
				InvoiceLineAdditionalData additionalData = (InvoiceLineAdditionalData)ad;
				if(additionalData.getAreeCommerciali() != null)
					additionalData.getAreeCommerciali().size();
				if(additionalData.getCapitoliArticoli() != null)
					additionalData.getCapitoliArticoli().size();
				if(additionalData.getCentriDiCosto() != null)
					additionalData.getCentriDiCosto().size();
			} else {
				int i;
				i=0;
				i++;
			}
		}
		return additionalDataDetached;
	}
}
