/**
 * This file is part of FatturazioneElettronica - Core Extensions - Comune di Modena
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core Extensions - Comune di Modena.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.process.comunemodena;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.invoice.InvoiceManagerComuneModena;
import it.fe.core.ejb.process.ProcessManager;
import it.fe.core.ejb.process.ProcessServiceFacade;
import it.fe.core.par.comunemodena.AreaCommercialeInvoiceLine;
import it.fe.core.par.comunemodena.InvoiceAdditionalData;
import it.fe.core.par.comunemodena.InvoiceLineAdditionalData;
import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.process.ProcessService;
import it.fe.core.process.exception.JbpmProcessException;
import it.fe.core.ws.comunemodena.sib.AreaCommercialeBean;
import it.fe.core.ws.comunemodena.sib.FatturaBean;
import it.fe.core.ws.comunemodena.sib.RigaBean;
import it.fe.core.ws.comunemodena.sib.SibService;
import it.fe.core.ws.comunemodena.sib.SibServiceService;

import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;

@Stateless
@Remote(ProcessManager.class)
public class ComuneModenaProcessManagerBean implements ProcessManager {

	private static Logger logger = Logger.getLogger(ComuneModenaProcessManagerBean.class);
	
	public ComuneModenaProcessManagerBean() {
		SibServiceService proxy=null;
		try {
			proxy = new SibServiceService(new URL(
					FEPropertiesUtil.getPA_Backend_WSDL_URL()));
		} catch (MalformedURLException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		sibService = proxy.getSibServicePort();
	}
	
	public String approveTechnically(Invoice invoice, String username) {
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_APPROVE, ProcessManager.STATE_TECHNICAL_VALIDATION, username);
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
	}

	public String discardAdditionalData(Invoice selectedInvoice, String username) {
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = selectedInvoice.getSupplier().getSupplierName() + "-" + selectedInvoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			try {
				result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_BACK, ProcessManager.STATE_ADDITIONAL_DATA_EDITING, username);
			} catch (JbpmProcessException e) {
				throw e;
			}
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		
		return result;
	}

	public String editAdditionalData(Invoice selectedInvoice, String username) {

		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = selectedInvoice.getSupplier().getSupplierName() + "-" + selectedInvoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			try {
				result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_EDIT, ProcessManager.STATE_TECHNICAL_VALIDATION, username);
			} catch (JbpmProcessException e) {
				throw e;
			}
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
		
	}

	SibService sibService;
	
	public Object[] insertIntoAccountableSystem(long invoiceId, String username) {
		Object[] results = new Object[2];
		
		// 1. estraggo la fattura dal db usando invoicemanager mediante InvoiceManagerBean
		Invoice invoice = JNDIUtils.getInvoiceManager().getInvoiceById(invoiceId);
		
		FatturaBean fatturaBean = new FatturaBean();

		List<AdditionalData> invoiceAdditionaDataList = invoice.getAdditionalData();
		
		if ( !(invoiceAdditionaDataList instanceof ArrayList)) {
			invoiceAdditionaDataList = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(invoice);
		}
		
		InvoiceAdditionalData invoiceAdditionalData = null;
		for(AdditionalData data: invoiceAdditionaDataList) {
			if(data instanceof InvoiceAdditionalData)
				invoiceAdditionalData = (InvoiceAdditionalData) data;
		}
		
		if(invoiceAdditionalData == null) {
			results[0] = ProcessManager.FAILURE;
			results[1] = "Mancano i dati aggiuntivi per l'inserimento nel sistema contabile";
			return results;
		}
			
		
		fatturaBean.setAnnoEsercizio(invoiceAdditionalData.getAnnoEsercizio());

		if(invoiceAdditionalData.getCodiceSettore() != null)
			fatturaBean.setCodiceSettore(invoiceAdditionalData.getCodiceSettore());
		if(invoiceAdditionalData.getCodiceDivisione() != null)
			fatturaBean.setCodiceDivisione(invoiceAdditionalData.getCodiceDivisione());
		if(invoiceAdditionalData.getCodiceUfficio() != null)
			fatturaBean.setCodiceUfficio(invoiceAdditionalData.getCodiceUfficio());

		fatturaBean.setNumeroEmissione(invoice.getInvoiceNumber());
		GregorianCalendar dataEmissione = new GregorianCalendar();
		 
		dataEmissione.setTime(invoice.getIssueDate());
		try {
			fatturaBean.setDataEmissione(DatatypeFactory.newInstance()
				    .newXMLGregorianCalendar(dataEmissione));
		} catch (DatatypeConfigurationException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		fatturaBean.setVatId(invoice.getSupplier().getVatId());

		Invoice documentoRef = invoiceAdditionalData.getDocumentoRef();
		
		if(documentoRef!=null) {
			GregorianCalendar dataEmissioneRef = new GregorianCalendar();
			dataEmissioneRef.setTime(documentoRef.getIssueDate());
			fatturaBean.setNumeroEmissioneRef(documentoRef.getInvoiceNumber());
			try {
				fatturaBean.setDataEmissioneFatturaRef(DatatypeFactory.newInstance()
					    .newXMLGregorianCalendar(dataEmissioneRef));
			} catch (DatatypeConfigurationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			fatturaBean.setVatIdRef(documentoRef.getSupplier().getVatId());
		} else {
			fatturaBean.setNumeroEmissioneRef("");
			fatturaBean.setDataEmissioneFatturaRef(null);
			fatturaBean.setVatIdRef("");
		}
		GregorianCalendar inizioCompetenzaEconomica = new GregorianCalendar();
		inizioCompetenzaEconomica.setTime(invoiceAdditionalData.getInizioCompetenzaEconomica());
		try {
			fatturaBean.setInizioCompetenzaEconomica(DatatypeFactory.newInstance()
				    .newXMLGregorianCalendar(inizioCompetenzaEconomica));
		} catch (DatatypeConfigurationException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		GregorianCalendar fineCompetenzaEconomica = new GregorianCalendar();
		fineCompetenzaEconomica.setTime(invoiceAdditionalData.getFineCompetenzaEconomica());
		try {
			fatturaBean.setFineCompetenzaEconomica(DatatypeFactory.newInstance()
				    .newXMLGregorianCalendar(fineCompetenzaEconomica));
		} catch (DatatypeConfigurationException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		List<RigaBean> righe = fatturaBean.getRighe();
		
		
		double totaleAliquota = 0;
		double totaleImponibile = 0;
		List<InvoiceLine> invoiceLines = invoice.getInvoiceLines();
		if ( !(invoiceLines instanceof ArrayList) ) {
			invoiceLines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(invoice);
		}
		
		for(InvoiceLine line: invoiceLines) {
			RigaBean riga = new RigaBean();
			List<AdditionalData> lineAdditionalDataList = line.getAdditionalData();
			if ( !(lineAdditionalDataList instanceof ArrayList)) {
				lineAdditionalDataList = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(line);
			}
			
			InvoiceLineAdditionalData lineAdditionalData = null;
			for(AdditionalData data: lineAdditionalDataList) {
				if(data.getClass().getName().equals(InvoiceLineAdditionalData.class.getName())) {
					lineAdditionalData = (InvoiceLineAdditionalData) data;
				} else {
					lineAdditionalData = null;
				}
			}
			
			if(lineAdditionalData == null) {
				results[0] = ProcessManager.FAILURE;
				results[1] = "Mancano i dati aggiuntivi righe per l'inserimento nel sistema contabile";
				return results;
			}
			
			if(lineAdditionalData.getAreaCommerciale() != null)
				riga.setAreaCommerciale(lineAdditionalData.getAreaCommerciale());
			
			List<AreaCommercialeBean> areeCommerciali = null;
			if(lineAdditionalData.getAreeCommerciali()!=null) {
				areeCommerciali = riga.getAreeCommerciali();
				List<AreaCommercialeInvoiceLine> areeCommercialiList = lineAdditionalData.getAreeCommerciali();
				if ( !(areeCommercialiList instanceof ArrayList)) {
					areeCommercialiList = ((InvoiceManagerComuneModena)JNDIUtils.getInvoiceManager()).fetchInvoiceLineAdditionalDataAreaCommerciale(lineAdditionalData);
				}
				for(AreaCommercialeInvoiceLine area: lineAdditionalData.getAreeCommerciali()) {
					AreaCommercialeBean areaCommercialeBean = new AreaCommercialeBean();
					areaCommercialeBean.setAreaCommercialeID(area.getAreaCommercialeID());
					areaCommercialeBean.setPercentuale(area.getPercentuale());
					areeCommerciali.add(areaCommercialeBean);
				}
			}
			
			//riga.setAreeCommerciali(areeCommerciali.toArray(new AreaCommercialeBean[areeCommerciali.size()]));
			riga.setCausaleRiga(lineAdditionalData.getCausaleRiga());
			riga.setSubCausaleRiga(lineAdditionalData.getSubCausaleRiga());
			riga.setFlagTipo(lineAdditionalData.getFlagTipo());
			riga.setImportoAliquota(line.getTaxDetail().getTaxAmount());
			riga.setImportoImponibile(line.getTaxDetail().getTaxableAmount());
			righe.add(riga);
			
			totaleAliquota+=riga.getImportoAliquota();
			totaleImponibile+=riga.getImportoImponibile();
		}
		//fatturaBean.setRighe(righe.toArray(new RigaBean[righe.size()]));
		fatturaBean.setTipoFattura(invoiceAdditionalData.getTipoFattura());
		
		fatturaBean.setTotaleAliquota(totaleAliquota);
		fatturaBean.setTotaleImponibile(totaleImponibile);
		fatturaBean.setUsername(username);

		try {
			results[0] = null;
			results[1] = sibService.saveFattura(fatturaBean);
		} catch (Exception e) {
			logger.error("Errore durante il salvataggio in SIB",e);
			results[0] = ProcessManager.FAILURE;
			results[1] = "Si è verificato un errore durante il salvataggio nel sistema contabile: "+e.getMessage();
		}
		if(results[1]!=null && results[1].equals("OK")) {
			results[0] = ProcessManager.SUCCESS;
			results[1] = "Salvataggio avvenuto con successo";
		} else {
			results[0] = ProcessManager.FAILURE;
		}
			
		return results;
	}

	public boolean isEditable(Invoice invoice) {
		
		boolean result = false;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {

			// la logica di WF mi assicura che se mi trovo in TECHNICAL_VALIDATION non posso in nessun modo trovarmi nello stato
			// ADDITIONAL_DATA_EDITING e di conseguenza la fattura non è lockata da nessun altro utente.
			if ( ProcessService.findActiveExecutionIn(processDefinitionKey, businessKey, ProcessManager.STATE_TECHNICAL_VALIDATION).equals(ProcessService.FAILURE) ) {
				// non sono nello stato di VT: ergo non posso modificare i dati aggiuntivi.
				result = false;
			}
			else if ( ProcessService.findActiveExecutionIn(processDefinitionKey, businessKey, ProcessManager.STATE_TECHNICAL_VALIDATION).equals(ProcessService.SUCCESS) ) {
				// mi trovo nello stato di VT ergo posso modificare i dati aggiuntivi
				result = true;
			}
		}
		else  {
			throw new JbpmProcessException("Could not obtain ProcessService");
		}
		
		return result;
	}

	public String rejectTechnically(Invoice invoice, String username) {

		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_REJECT, ProcessManager.STATE_TECHNICAL_VALIDATION, username);
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
	}

	public String saveAdditionalData(Invoice selectedInvoice, String username) {
		
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = selectedInvoice.getSupplier().getSupplierName() + "-" + selectedInvoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			try {
				result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_BACK, ProcessManager.STATE_ADDITIONAL_DATA_EDITING, username);
			} catch (JbpmProcessException e) {
				throw e;
			}
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		
		return result;
	}

	@Override
	public String approveInsertionIntoAccountableSystem(Invoice invoice,
			String username) {
		// TODO Auto-generated method stub
		return null;
	}

}
