/**
 * This file is part of FatturazioneElettronica - Core Extensions - UniMoRe
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core Extensions - UniMoRe is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core Extensions - UniMoRe is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core Extensions - UniMoRe.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
*
 *
 */
package it.fe.core.ejb.process.unimore;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.InvoiceUtils;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.invoice.InvoiceManager;
import it.fe.core.ejb.process.ProcessManager;
import it.fe.core.ejb.process.ProcessServiceFacade;
import it.fe.core.ejb.unimore.cia.UnimoreCIAGenericDAO;
import it.fe.core.ejb.unimore.sia.UnimoreSiaDao;
import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.unimore.cia.UnitaOrganizzativa;
import it.fe.core.process.ProcessService;
import it.fe.core.process.exception.JbpmProcessException;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;


/**
 * @author fulvio di marco
 * 
 *
 */
@Stateless
@Remote(ProcessManager.class)
public class UnimoreProcessManagerBean implements ProcessManager {
	
	private static Logger logger = Logger.getLogger(UnimoreProcessManagerBean.class);
	
	public static final String STATE_TECHNICAL_VALIDATION = "TechnicalValidation";
	public static final String STATE_CIA_PREINVOICE_INSERTION = "CIAPreInvoiceInsertion";
	
	
	public String approveInsertionIntoAccountableSystem(Invoice invoice, String username) {
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_INSERT, UnimoreProcessManagerBean.STATE_CIA_PREINVOICE_INSERTION, username);
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#approveTechnically(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String approveTechnically(Invoice invoice, String username) {
		
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_APPROVE, UnimoreProcessManagerBean.STATE_TECHNICAL_VALIDATION, username);
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
	}


	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#rejectTechnically(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String rejectTechnically(Invoice invoice, String username) {

		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_REJECT, UnimoreProcessManagerBean.STATE_TECHNICAL_VALIDATION, username);
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
	}
	
	
	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#insertIntoAccountableSystem(java.lang.String, java.lang.String)
	 */
	public Object[] insertIntoAccountableSystem(long invoiceId, String username) {
		
		Object[] results = new Object[2];
		
		InvoiceManager invoiceManager = JNDIUtils.getInvoiceManager();
		
		// 1. estraggo la fattura dal db usando invoicemanager mediante InvoiceManagerBean
		Invoice invoice = invoiceManager.getInvoiceById(invoiceId);
		
		// lazy fetching (Invoice.AdditionalData, InvoiceLines, InvoiceLines.AdditionalData)
		List<AdditionalData> lazilyFetchedAdditionalData = invoiceManager.fetchInvoiceAdditionalData(invoice);
		invoice.setAdditionalData(lazilyFetchedAdditionalData);
		
		List<InvoiceLine> lazilyFetchedInvoiceLines = invoiceManager.fetchInvoiceLines(invoice);
		
		for (InvoiceLine il : lazilyFetchedInvoiceLines) {
			List<AdditionalData> lazilyFetchedInvoiceLineAdditionalData = invoiceManager.fetchInvoiceLineAdditionalData(il);
			il.setAdditionalData(lazilyFetchedInvoiceLineAdditionalData);
		}
		invoice.setInvoiceLines(lazilyFetchedInvoiceLines);
		
		// pre-2: invoco il metodo di UnimoreSiaDao per ottenere il prossimo valore della sequence
		long id_imp_fattura = UnimoreSiaDao.getSequenceNextValue();
		
		// 2. eseguo il mapping Invoice -> ImpFattura e invoco il metodo del DAO SIA per l'inserimento
		UnimoreSiaDao.insert(UnimoreSiaDao.invoiceToImpFattura(invoice, id_imp_fattura));
	
		// 3. per ogni InvoiceLine eseguo il mapping InvoiceLine -> ImpFatturaDett e invoco il metodo del DAO sia per l'inserimento
		List<InvoiceLine> invoiceLines = invoice.getInvoiceLines();
		
		Map<String, Object> params = new HashMap<String, Object>();
		
		// esercizio, bilancio della testata
		if (InvoiceUtils.invoiceHasSavedAdditionalData(invoice, UnitaOrganizzativa.class)) {
			List<UnitaOrganizzativa> invoiceUos = InvoiceUtils.getInvoiceStoredAdditionalData(invoice, UnitaOrganizzativa.class);
			if ( (invoiceUos != null) && (!invoiceUos.isEmpty()) ) {
				params.put("esercizio", invoiceUos.get(0).getEsercizio());
				params.put("bilancio", invoiceUos.get(0).getIdentificativo_bilancio());
			}
		}
		
		UnimoreSiaDao.insertImpFatturaDetts(invoiceLines, id_imp_fattura, params);
		
		
//		long lineCount = 1;
//		long startTime = System.currentTimeMillis();
//		for (InvoiceLine invoiceLine : invoiceLines) {
//			UnimoreSiaDao.insert(this.invoiceLineToImpFatturaDett(invoiceLine, id_imp_fattura, lineCount, params));
//			lineCount++;
//		}
//		long endTime = System.currentTimeMillis();
//		logger.info("SERIAL INSERT TIME: "+(endTime-startTime)+" ms");
	
		// 4. invoco la stored procedure mediante il DAO CIA
		String[] storedProcedureResults= UnimoreCIAGenericDAO.invokeStoredProcedure(id_imp_fattura, null);
		
		// 5. gestisco i risultati di ritorno 
		// [0] CODICE_CIA: numero progressivo della prefattura assegnato da CIA
		// [1] ESISTE_ERR: 0 procedura eseguita correttamente; n numero degli errori riscontrati		
		// [2] ERRORE: 0 procedura eseguita correttamente; descrizioni autoesplicative degli errori
		String CDPREFATTURA = storedProcedureResults[0];
		String ESITO = storedProcedureResults[1];
		String ERRORE = storedProcedureResults[2];
		
		// eg: ERRORI = 0 AVVERTENZE = 343
		if (getErrorsNumber(ESITO) == 0) {
			logger.info("CIA stored procedure invoked successfully for CDPREFATTURA "+CDPREFATTURA);
			results[0] = ProcessManager.SUCCESS;
			// TODO: Estrarre avvertenze (da dove?)
			if (getWarningsNumber(ESITO) > 0) {
				results[1] = ERRORE;
			}
		}
		else if (getErrorsNumber(ESITO) > 0){
			logger.info("CIA stored procedure invocation failed. Errors found: "+ESITO+". Errors description: "+ERRORE);
			results[0] = ProcessManager.FAILURE;
			results[1] = ERRORE;
		} 
		
		return results;
	}


	/**
	 * Helper method to extract procedure invocation errors number
	 * 
	 * @param ESITO
	 * @return
	 */
	private int getErrorsNumber(String in) {
		return Integer.parseInt(in.substring(in.indexOf('=')+1, in.indexOf("AVVERTENZE")).trim());
	}
	
	/**
	 * Helper method to extract procedure invocation warnings number
	 * 
	 * @param in
	 * @return
	 */
	public static int getWarningsNumber(String in) {
		return Integer.parseInt(in.substring(in.lastIndexOf('=')+1).trim());
	}


	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#editAdditionalData(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String editAdditionalData(Invoice selectedInvoice, String username) {

		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = selectedInvoice.getSupplier().getSupplierName() + "-" + selectedInvoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			try {
				result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_EDIT, UnimoreProcessManagerBean.STATE_CIA_PREINVOICE_INSERTION, username);
			} catch (JbpmProcessException e) {
				throw e;
			}
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
		
	}


	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#isEditable(it.fe.core.par.invoice.Invoice)
	 */
	public boolean isEditable(Invoice invoice) {
		
		boolean result = false;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {

			// la logica di WF mi assicura che se mi trovo in STATE_CIA_PREINVOICE_INSERTION non posso in nessun modo trovarmi nello stato
			// ADDITIONAL_DATA_EDITING e di conseguenza la fattura non è lockata da nessun altro utente.
			if ( ProcessService.findActiveExecutionIn(processDefinitionKey, businessKey, UnimoreProcessManagerBean.STATE_CIA_PREINVOICE_INSERTION).equals(ProcessService.FAILURE) ) {
				// non sono nello stato CIAPreInvoiceInsertion : ergo non posso modificare i dati aggiuntivi.
				result = false;
			}
			else if ( ProcessService.findActiveExecutionIn(processDefinitionKey, businessKey, UnimoreProcessManagerBean.STATE_CIA_PREINVOICE_INSERTION).equals(ProcessService.SUCCESS) ) {
				// mi trovo nello stato CIAPreInvoiceInsertion ergo posso modificare i dati aggiuntivi
				result = true;
			}
		}
		else  {
			throw new JbpmProcessException("Could not obtain ProcessService");
		}
		
		return result;
	}


	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#saveAdditionalData(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String saveAdditionalData(Invoice selectedInvoice, String username) {
		
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = selectedInvoice.getSupplier().getSupplierName() + "-" + selectedInvoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			try {
				result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_BACK, ProcessManager.STATE_ADDITIONAL_DATA_EDITING, username);
			} catch (JbpmProcessException e) {
				throw e;
			}
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		
		return result;
	}


	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#discardAdditionalData(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String discardAdditionalData(Invoice selectedInvoice, String username) {
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = selectedInvoice.getSupplier().getSupplierName() + "-" + selectedInvoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			try {
				result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_BACK, ProcessManager.STATE_ADDITIONAL_DATA_EDITING, username);
			} catch (JbpmProcessException e) {
				throw e;
			}
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		
		return result;
	}

}
