/**
 * This file is part of FatturazioneElettronica - Core Extensions - UniMoRe
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core Extensions - UniMoRe is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core Extensions - UniMoRe is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core Extensions - UniMoRe.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
*
 *
 */
package it.fe.core.ejb.unimore.cia;

import it.fe.core.common.utils.db.oracle.OracleConnectionFactory;

import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.log4j.Logger;

/**
 * @author fulvio di marco
 *
 *	Generic ReadUpdate
 */
public class UnimoreCIAGenericDAO {
	
	private static Logger logger = Logger.getLogger(UnimoreCIAGenericDAO.class);
	
	private final static String INSERISCI_FATTURE_STATEMENT = "{ call PKG_IMPCIA_MAIN.INSERISCI_FATTURE( ?, ?, ?, ? ) }"; 
	private final static String GET_ERRORE_PROCEDURA = "select stringa_errore from errori_procedure where codice_errore=";
	
	/**
	 * Method used to retrieve by leveraging reflection DTOs from database given a map of parameters
	 * 
	 * @param <T>
	 * @param clazz
	 * @param parameters
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static  <T> List<T> findByParameters(Class<T> clazz, Map<String, Object> parameters) {
		
		List<T> results = new ArrayList<T>();
		Connection conn = OracleConnectionFactory.createConnection();
		
		try {
			
			PropertyDescriptor[] propertyDescriptors = PropertyUtils.getPropertyDescriptors(clazz);
			String query = "";
			query +="SELECT * FROM ";
			
			// estraggo il nome della tabella corrispondente
//			Entity entityAnnotation = clazz.getAnnotation(Entity.class);
//			String tableName = entityAnnotation.name();
//			query += tableName;
			
			String tableName = (String) clazz.getField("ciaViewName").get(null);
			query += tableName;
			
			//aggiungo condizioni in base al numero di parametri passati in input alla funzione
			Set<Entry<String, Object>> rawParameters = parameters.entrySet();
			
			if (rawParameters.size() > 0) {
				query += " WHERE ";
				Iterator<Entry<String, Object>> i = rawParameters.iterator();
				while (i.hasNext()) {
					Entry<String, Object> entry = i.next();
					String columnName = entry.getKey().toUpperCase();
					Object value = entry.getValue();
					if (value == null ) {
						// TODO: la presenza di parametri nulli non viene ancora gestita. La sintassi della query cambia.
						return results;
					}
					// se incontro una lista devo generare una IN ('','','')
					if (List.class.isAssignableFrom(value.getClass())) {
						List<String> values = (List<String>) value;
						query += columnName + " IN (";
						Iterator<String> j = values.iterator();
						while(j.hasNext()) {
							query += "'"+j.next()+"'";
							if(j.hasNext()) {
								query += ",";
							}
							else {
								query += ") ";
							}
						}
					}
					// se uno dei parametri è "null" devo generare una "is null"
					else if (String.class.isAssignableFrom(value.getClass()) && "null".equals((String)value)) {
						query += columnName + " is null";
					}
					// se uno dei parametri contiene '%' devono generare una like
					else if (String.class.isAssignableFrom(value.getClass()) && ((String)value).indexOf('%') > 0) {
						query += columnName + " like '"+value+"'";
					}
					// altrimenti aggiungo una condizione colonna='valore'
					else {
						query += columnName + "='"+value+"'";
					}
					
					if (i.hasNext()) {
						query += " AND ";
					}
				}
			}				
			
			PreparedStatement stmt = conn.prepareStatement(query);
			stmt.clearParameters();
			
			
			ResultSet resultSet = stmt.executeQuery();
			
			while(resultSet.next()) {
				Object obj = clazz.newInstance();
				for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
					if (propertyDescriptor.getName().equals("class"))
						continue;
					if (propertyDescriptor.getName().equals("null"))
						continue;
					if (propertyDescriptor.getName().equals("id"))
						continue;
					if (propertyDescriptor.getName().equals("ciaViewName"))
						continue;
					if (propertyDescriptor.getName().equals("additionalDataType"))
						continue;
					
					
					Object value = resultSet.getObject(propertyDescriptor.getName().toUpperCase());
					
					if (value != null) {
						if (value.getClass().equals(Class.forName("java.sql.Timestamp"))) {
							java.sql.Timestamp timestamp = (java.sql.Timestamp) value;
							value = new java.sql.Date(timestamp.getTime());
						}
						PropertyUtils.setSimpleProperty(obj, propertyDescriptor.getName(), value);
					}
						
				}
				results.add((T) obj);
			}
			resultSet.close();
			stmt.close();
		} catch (Exception e) {
			logger.error(e.getMessage());
			return null;
		}
		finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return results;
	}
	
	/**
	 * Method used to issue a specific sql native query against the database
	 * 
	 * @param <T>
	 * @param clazz
	 * @param query
	 * @return
	 */
	public static <T> List<T> findByNativeQuery(Class<T> clazz, String query) {		
		Connection conn = OracleConnectionFactory.createConnection();
		List<T> results = new ArrayList<T>();
		PropertyDescriptor[] propertyDescriptors = PropertyUtils.getPropertyDescriptors(clazz);
		try {
			PreparedStatement stmt = conn.prepareStatement(query);
			stmt.clearParameters();
			ResultSet resultSet = stmt.executeQuery();
			while(resultSet.next()) {
				Object obj = clazz.newInstance();
				for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
					if (propertyDescriptor.getName().equals("class"))
						continue;
					if (propertyDescriptor.getName().equals("null"))
						continue;
					if (propertyDescriptor.getName().equals("id"))
						continue;
					if (propertyDescriptor.getName().equals("ciaViewName"))
						continue;
					if (propertyDescriptor.getName().equals("additionalDataType"))
						continue;
					
					
					Object value = resultSet.getObject(propertyDescriptor.getName().toUpperCase());
					
					if (value != null) {
						if (value.getClass().equals(Class.forName("java.sql.Timestamp"))) {
							java.sql.Timestamp timestamp = (java.sql.Timestamp) value;
							value = new java.sql.Date(timestamp.getTime());
						}
						PropertyUtils.setSimpleProperty(obj, propertyDescriptor.getName(), value);
					}
						
				}
				results.add((T) obj);
			}
			resultSet.close();
			stmt.close();
		} catch (SQLException e) {
			logger.error(e.getMessage());
			return null;
		} catch (IllegalAccessException e) {
			logger.error(e.getMessage());
			return null;
		} catch (InvocationTargetException e) {
			logger.error(e.getMessage());
			return null;
		} catch (NoSuchMethodException e) {
			logger.error(e.getMessage());
			return null;
		} catch (ClassNotFoundException e) {
			logger.error(e.getMessage());
			return null;
		} catch (InstantiationException e) {
			logger.error(e.getMessage());
			return null;
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return results;
	}
	
	/**
	 * Invokes database's stored procedure to insert invoice within accountable systema (CIA) 
	 * 
	 * @param code
	 * @param CIACode
	 * @return
	 */
	public static String[] invokeStoredProcedure(Long code, String CIACode) {
		
		String[] out = new String[3];
		
		Connection connection = OracleConnectionFactory.createConnection();
		
		try {
			
			CallableStatement callableStatement = null;
			// prepare the callable statement
			// parameters (nome/tipo/in-out/significato):
			// 1. CODICE		NUMBER			IN			identificativo del record da importare in CIA, fornito dalla sequenza S_IMP_FATT_SEQ.
			// 2. CODICE_CIA	VARCHAR(10)		IN/OUT		se inserimento: in output numero progressivo della prefattura assegnato da CIA
			//												se modifica: in input numero della prefattura in stato provvisorio da modificare
			// 3. ESISTE_ERR	VARCHAR2(2)		OUT
			// 4. ERRORE		VARCHAR2(50)	OUT
			callableStatement = (CallableStatement) connection.prepareCall(INSERISCI_FATTURE_STATEMENT);
			
			// first parameter: IDPREFATTURA (IN)
			callableStatement.setLong(1, code);
			
			// second parameter: CDPREFATTURA (IN/OUT)
			callableStatement.setString(2, "");
			callableStatement.registerOutParameter(2, java.sql.Types.VARCHAR);
			
			// third parameter: ESITO (IN/OUT)
			callableStatement.setString(3, "");
			callableStatement.registerOutParameter(3, java.sql.Types.VARCHAR);
			
			
			// fourth parameter: ERRORE (IN/OUT)
			callableStatement.setString(4, "");
			callableStatement.registerOutParameter(4, java.sql.Types.VARCHAR);
			
			// execute the stored procedure
			callableStatement.execute();
			
			// extract the out parameters
			// CODICE_CIA
			out[0] = callableStatement.getString(2);
			// ESISTE_ERR
			out[1] = callableStatement.getString(3);
			// ERRORE
			out[2] = callableStatement.getString(4);
			 
			
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		} finally {
			OracleConnectionFactory.closeConnection(connection);
		}
		
		return out;
	}
	
	public static String getProcedureErrorDescription(String errorCode) {
		String result = "";
		Connection conn = OracleConnectionFactory.createConnection();
		try {
			PreparedStatement stmt = conn.prepareStatement(GET_ERRORE_PROCEDURA+"'"+errorCode+"'");
			stmt.clearParameters();
			ResultSet resultSet = stmt.executeQuery();
			while(resultSet.next())
				result = resultSet.getString(1);
			stmt.close();
		} catch (Exception e) {
			logger.error(e.getMessage());
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return result;
	}

}
