/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.common.utils;

import it.fe.core.ejb.process.ProcessManager;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * @author fulvio di marco
 * 
*
 * 
 * Generic interface to get EJB from JNDI Context, both local and remote, with caching 
 * functionalities to improve performances.
 *
 */
public class ContextUtils {
	
	private static final Map<String, Object> EJB_CACHE = new HashMap<String, Object>();

	private static String hostPort = FEPropertiesUtil.getJndiHost()+":"+FEPropertiesUtil.getJndiPort();
	
	private static final String COMO_PROCESS_MANAGER_KEY = "CoMoProcessManagerBean";
	
	private static final String COMO_PROCESS_MANAGER_JNDI_NAME = "CoMoProcessManagerBean/remote";
	
	private static final String COMO_PROCESS_MANAGER_KEY_LOCAL = "CoMoProcessManagerBeanLocal";
	
	private static final String COMO_PROCESS_MANAGER_JNDI_NAME_LOCAL = "CoMoProcessManagerBean/local";
	
	public static ProcessManager getCoMoProcessManager() {
		return retrieveEJB(COMO_PROCESS_MANAGER_KEY, COMO_PROCESS_MANAGER_JNDI_NAME);
	}
	
	public static ProcessManager getCoMoProcessManagerLocal() {
		return retrieveEJB(COMO_PROCESS_MANAGER_KEY_LOCAL, COMO_PROCESS_MANAGER_JNDI_NAME_LOCAL);
	}
	
	@SuppressWarnings("unchecked")
	private static InitialContext getInitialContext(String hostPort)
			throws Exception {
		Hashtable props = getInitialContextProperties(hostPort);
		return new InitialContext(props);
	}

	@SuppressWarnings("unchecked")
	private static Hashtable getInitialContextProperties(String hostPort) {
		Hashtable props = new Hashtable();
		props.put("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");
		props.put("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");
		props.put("java.naming.provider.url", hostPort);

		return props;
	}
	
	@SuppressWarnings("unchecked")
	private static <T> T retrieveEJB(String key, String name) {
		T ejbBean = null;
		
		if (EJB_CACHE.containsKey(key)) {
			ejbBean = (T) EJB_CACHE.get(key);
		} else {
			try {
				InitialContext context = getInitialContext( hostPort );
				ejbBean = (T) context.lookup(name);
				EJB_CACHE.put(key, ejbBean);
			} catch (NamingException e) {
				e.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		return ejbBean;
	}

}
