/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.common.utils;

import it.fe.core.par.common.FEFile;
import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.AllowanceCharge;
import it.fe.core.par.invoice.Client;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceLineNote;
import it.fe.core.par.invoice.InvoiceNote;
import it.fe.core.par.invoice.InvoicePeriod;
import it.fe.core.par.invoice.PaymentDetail;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.Supplier;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.utility.Consumption;
import it.fe.core.par.utility.Contact;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterProperty;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;
import it.fe.core.par.utility.Sector;
import it.fe.core.par.utility.SupplyInfo;
import it.fe.core.par.utility.UtilityConsumptionPoint;
import it.fe.core.par.utility.UtilitySupplier;

import java.io.IOException;
import java.io.PrintWriter;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class InvoiceParser extends XmlParser {
	
	private static List<Integer> parentDocumentLineReferenceList = new ArrayList<Integer>();
	
	private static List<InvoiceNote> noteList = new ArrayList<InvoiceNote>();
	
	private static List<MeterProperty> meterPropertyList = new ArrayList<MeterProperty>();
	
	private static List<AllowanceCharge> allowanceChargeList = new ArrayList<AllowanceCharge>();
	
	private static List<TaxDetail> taxDetailList = new ArrayList<TaxDetail>();
	
	private static List<Meter> meterList = new ArrayList<Meter>();
	
	//private static SimpleDateFormat formatterDate = new SimpleDateFormat("yyyyMMdd");

//	public void parse(Invoice invoice, Element rootNode, XPath xpath, Document doc, PrintWriter out) throws XPathExpressionException {
	
	@Override
	public void parse(Invoice invoice, Element rootNode, XPath xpath, Document doc) throws XPathExpressionException {
		
		taxDetailList.clear();
		allowanceChargeList.clear();
		parentDocumentLineReferenceList.clear();
		noteList.clear();
		meterPropertyList.clear();
		meterList.clear();
		
		setupXmlDocument(xpath, doc);
		
		//Numero fattura - cbc:ID
        String invoiceNumber = getNodeValue(rootNode, "cbc", "ID");
        invoiceNumber = SpecialCharacter.xmlEntityToString(invoiceNumber);
		invoice.setInvoiceNumber(invoiceNumber);
		logger.info("Invoice Number: " + invoiceNumber);
		
		
//		out.println("<cbc:ID>" + invoiceNumber + "</cbc:ID>");
		
        
        //Data emissione - cbc:IssueDate
    	try {
			String stringIssueDate = getNodeValue(rootNode, "cbc", "IssueDate");
			if(stringIssueDate != null){
				Date issueDate = dateFormat.parse(stringIssueDate);
				invoice.setIssueDate(issueDate);
				logger.info("Issue date: " + issueDate);
//				out.println("<cbc:IssueDate>" + issueDate + "</cbc:IssueDate>");
			}
		}
    	catch (Exception e) { 
			logger.error(e.getMessage());
		}		
		
    	//Note fattura
    	NodeList noteNodes = getNodes(rootNode, "cbc", "Note");
    			
//    	out.print("<cbc:Note>");
    			
    	for (int i = 0; i < noteNodes.getLength(); i++) {
    		Element note = (Element)noteNodes.item(i);
    		InvoiceNote noteTxt = new InvoiceNote();
    		noteTxt.setNote(getNodeValue(note, null, null));
//    		out.print(noteTxt.getNote());
    		noteList.add(noteTxt);
    	}
    	invoice.setNoteList(noteList);
//    	out.println("</cbc:Note>");
    	
    	//Periodo di consumo: cac:InvoicePeriod
    	Node invoicePeriodNode = getNode(rootNode, "cac", "InvoicePeriod");
    	InvoicePeriod invoicePeriod = new InvoicePeriod();
    		    
//    	out.println("<cac:InvoicePeriod>");
    		    
    	//StartDate
    	try {
    		String stringPeriodStartDate = getNodeValue((Element)invoicePeriodNode, "cbc", "StartDate");
    		if(stringPeriodStartDate != null) {
    		Date periodStartDate = dateFormat.parse(stringPeriodStartDate);
    		invoicePeriod.setStartDate(periodStartDate);
//    		out.println("\t<cbc:StartDate>" + stringPeriodStartDate + "</cbc:StartDate>");
    		}	
    	} catch (Exception e) { 
    		logger.error(e.getMessage());
    	}
    			
    	//EndDate
    	try {
    		String stringPeriodEndDate = getNodeValue((Element)invoicePeriodNode, "cbc", "EndDate");
    		if(stringPeriodEndDate != null){
    			Date periodEndDate = dateFormat.parse(stringPeriodEndDate);
    			invoicePeriod.setEndDate(periodEndDate);
//    			out.println("\t<cbc:EndDate>" + stringPeriodEndDate + "</cbc:EndDate>");
    		}
    	} catch (Exception e) { 
    		logger.error(e.getMessage());
    	}
    	
    	//Invoice Period Description
    	String periodDescription = getNodeValue((Element)invoicePeriodNode, "cbc", "Description");
		periodDescription = SpecialCharacter.xmlEntityToString(periodDescription);	
		invoicePeriod.setPeriodDescription(periodDescription);
		
//		out.println("\t<cbc:Description>" + periodDescription + "</cbc:Description>");
//		out.println("</cac:InvoicePeriod>\n");
    	
		//Conto Contrattuale
		//TODO: come lo mappo? Come Additional Data o come campo?
		String contoContrattuale=null;
		try {
			contoContrattuale = getNodeValue(rootNode, "cbc", "ContoContrattuale");
			if(contoContrattuale != null){
				logger.info("Conto Contrattuale: " + contoContrattuale);
//				out.println("<cbc:ContoContrattuale>" + contoContrattuale + "</cbc:ContoContrattuale>");
			}
		} 
		catch (Exception e) { 
			logger.error(e.getMessage());
		}
		
    	//Fornitore:
    	//- cac:PartyName = ragione sociale
    	//- cac:PartyIdentification = partita iva
    	//- cac:PostalAddress = indirizzo + cap + citta
		Node supplierInfoNode = getNode(rootNode, "cac", "AccountingSupplierParty");
		Node supplierPartyNode = getNode(supplierInfoNode, "cac", "Party");
		Node supplierPartyNameNode = getNode(supplierPartyNode, "cac", "PartyName");
		Node supplierPartyLegalEntityNode = getNode(supplierPartyNode, "cac", "PartyLegalEntity");
		
//		out.println("<cac:AccountingSupplierParty>");
//		out.println("\t<cac:Party>");
		
		//EndPoint
		String supplierEndPointID = getNodeValue((Element)supplierPartyNode, "cbc", "EndpointID");
		supplierEndPointID = SpecialCharacter.xmlEntityToString(supplierEndPointID);
		
//		out.println("\t<cbc:EndpointID>" + supplierEndPointID + "</cbc:EndpointID>");
//    	out.println("\t<cac:PartyName>");
		
	    //SupplierName
		String supplierName = getNodeValue((Element)supplierPartyNameNode, "cbc", "Name");
		supplierName = SpecialCharacter.xmlEntityToString(supplierName);
		logger.info("Supplier Short Name: " + supplierName);
		
//    	out.println("\t\t<cbc:Name>" + supplierName + "</cbc:Name>");
//    	out.println("\t</cac:PartyName>");
//    	out.println("\t<cac:PostalAddress>");
    	
		//Address
		Node supplierAddressNode = getNode(supplierPartyNode, "cac", "PostalAddress");		
		PostalAddress supplierPostalAddress = readAddress((Element)supplierAddressNode);
		
//		out.println("\t</cac:PostalAddress>");
//		out.println("\t<cac:PartyLegalEntity>");
//		out.println("\t\t<cac:RegistrationAddress>");
		
	    //RegistrationAddress
		Node supplierRegistrationAddressNode = getNode(supplierPartyLegalEntityNode, "cac", "RegistrationAddress");
		PostalAddress supplierRegistrationAddress = readAddress((Element)supplierRegistrationAddressNode);

//		out.println("\t\t\t</cac:RegistrationAddress>\n");
		
	    //RegistrationName
	    String supplierRegistrationName = getNodeValue((Element)supplierPartyLegalEntityNode, "cbc", "RegistrationName");
	    supplierRegistrationName = SpecialCharacter.xmlEntityToString(supplierRegistrationName);
	    if ((supplierName == null) || (supplierName.equals(""))){
	    	supplierName = supplierRegistrationName;
	    }
	    
//	    out.println("\t\t\t<cbc:RegistrationName>" + supplierRegistrationName + "</cbc:RegistrationName>");
    	
	    
	    //VatID
	    //In UBL la p.iva può essere indicata in vari punti di AccountingSupplierParty:
	    //1) in PartyLegalEntityNode/CompanyID
	    String supplierVatId = getNodeValue((Element)supplierPartyLegalEntityNode, "cbc", "CompanyID");
	    supplierVatId = SpecialCharacter.xmlEntityToString(supplierVatId);
	    
	    
	    //2) in PartyTaxScheme/CompanyID  (in questo caso deve essere specificato lo schemeIDType come VAT)
	    if ((supplierVatId == null) || (supplierVatId.equals(""))){
	    	NodeList supplierPartyTaxSchemeNodes = getNodes(supplierPartyNode, "cac", "PartyTaxScheme");
	    	
//	    	out.println("\t\t\t<cac:PartyTaxScheme>");
	    	
			if (supplierPartyTaxSchemeNodes != null){
				System.out.println("Numero nodi: " + supplierPartyTaxSchemeNodes.getLength());
				for (int i = 0;  i < supplierPartyTaxSchemeNodes.getLength(); i++) {
					//schemeIDType specificato come attriduto (CompanyID/@schemeID)
					String schemeIDType = getNodeValue((Element)supplierPartyTaxSchemeNodes.item(i), "cbc", "CompanyID/@schemeID");
					
					System.out.println("ID: " + schemeIDType);
					
					if ( (schemeIDType != null) && (schemeIDType.endsWith("VAT") ) ){
						supplierVatId = getNodeValue((Element)supplierPartyTaxSchemeNodes.item(i), "cbc", "CompanyID");
						supplierVatId = SpecialCharacter.xmlEntityToString(supplierVatId);
//						out.println("\t\t\t\t<cbc:CompanyID schemeID=\"" + schemeIDType + "\">" + supplierVatId + "</cbc:CompanyID>");
					}
					//schemeIDType specificato come nodo (CompanyID/TaxScheme/ID)
					else {
						
						System.out.println("Sono dentro else...");
						Node taxSchemeNode = getNode(supplierPartyTaxSchemeNodes.item(i), "cac", "TaxScheme");
						
//						out.println("\t\t\t\t<cac:TaxScheme>");
						
						if (taxSchemeNode != null){
							
							schemeIDType = getNodeValue((Element)taxSchemeNode, "cbc", "ID");
							
//							out.println("\t\t\t\t\t<cbc:ID>" + schemeIDType + "</cbc:ID>");
							
							if ( (schemeIDType != null) && (schemeIDType.endsWith("VAT") ) ){
								supplierVatId = getNodeValue((Element)supplierPartyTaxSchemeNodes.item(i), "cbc", "CompanyID");
								supplierVatId = SpecialCharacter.xmlEntityToString(supplierVatId);
//								out.println("\t\t\t\t<cbc:CompanyID schemeID=\"" + schemeIDType + "\">" + supplierVatId + "</cbc:CompanyID>");
								
							}
						}
						
//						out.println("\t\t\t\t</cac:TaxScheme>");
					}
				}
				
//				out.println("\t\t\t</cac:PartyTaxScheme>\n");
			}
	    }
	    else {
//	    	out.println("\t\t\t<cbc:CompanyID>" + supplierVatId + "</cbc:CompanyID>");
	    }
	    supplierVatId = SpecialCharacter.xmlEntityToString(supplierVatId);
	    
	    //Codice fiscale
	    String supplierFiscalCode = getNodeValue((Element)supplierPartyLegalEntityNode, "cbc", "CompanyID");
	    supplierFiscalCode = SpecialCharacter.xmlEntityToString(supplierFiscalCode);
	    //out.println("\t\t\t<cbc:CompanyID>" + supplierFiscalCode + "</cbc:CompanyID>");
	    
	    if ((supplierFiscalCode == null) || (supplierFiscalCode.equals(""))){
	    	NodeList supplierPartyTaxSchemeNodes = getNodes(supplierPartyNode, "cac", "PartyTaxScheme");
	    	
	    	//out.println("\t\t<cac:PartyTaxScheme>");
	    	
			if (supplierPartyTaxSchemeNodes  != null){
				for (int i = 0;  i < supplierPartyTaxSchemeNodes.getLength();i++  ) {
					String schemeIDType = getNodeValue((Element)supplierPartyTaxSchemeNodes.item(i), "cbc", "CompanyID/@schemeID");
					if ( (schemeIDType != null) && (schemeIDType.endsWith("CF") ) ){
						supplierFiscalCode = getNodeValue((Element)supplierPartyTaxSchemeNodes.item(i), "cbc", "CompanyID");
						supplierFiscalCode = SpecialCharacter.xmlEntityToString(supplierFiscalCode);
						//out.println("\t\t\t<cbc:CompanyID schemeID=\"" + schemeIDType + "\">" + supplierFiscalCode + "</cbc:CompanyID>");
					}
					else {
						//out.println("\t\t\t<cbc:CompanyID schemeID=\"" + schemeIDType + "\">" + supplierFiscalCode + "</cbc:CompanyID>");
					}
				}
			}
			//out.println("\t\t</cac:PartyTaxScheme>");
	    }
	   
	    Supplier supplier = new Supplier();
	    supplier.setVatId(supplierVatId);
	    supplier.setItalianFiscalCode(supplierFiscalCode);
	    supplier.setSupplierName(supplierName);
	    supplier.setPostalAddress(supplierPostalAddress);
	    supplier.setRegistrationAddress(supplierRegistrationAddress);
	    supplier.setEndpointId(supplierEndPointID);
	    invoice.setSupplier(supplier);
	    
	    	
//	    out.println("\t\t</cac:PartyLegalEntity>");
//	    out.println("\t</cac:Party>");
//	    out.println("</cac:AccountingSupplierParty>\n");
	    
    	//Cliente:
    	//- cac:PartyName = ragione sociale
    	//- cac:PartyIdentification = partita iva
    	//- cac:PostalAddress = indirizzo + cap + citta
		Node clientInfoNode = getNode(rootNode, "cac", "AccountingCustomerParty");
		Node clientPartyNode = getNode(clientInfoNode, "cac", "Party");
		Node clientPartyNameNode = getNode(clientPartyNode, "cac", "PartyName");
		Node clientPartyLegalEntityNode = getNode(clientPartyNode, "cac", "PartyLegalEntity");
		
	    //SupplierName
		String clientName = getNodeValue((Element)clientPartyNameNode, "cbc", "Name");
		clientName = SpecialCharacter.xmlEntityToString(clientName);
		logger.info("Client Short Name: " + clientName);
		
//		out.println("<cac:AccountingCustomerParty>");
	    //out.println("<SupplierAssignedAccountID: >");
//		out.println("\t<cac:Party>");
//		out.println("\t\t<cac:PartyName>");
//		out.println("\t\t\t<cbc:Name>" + clientName + "</cbc:Name>");
//		out.println("\t\t</cac:PartyName>");
//		out.println("\t\t<cac:PostalAddress>");
		
		//Address
		Node clientAddressNode = getNode(clientPartyNode, "cac", "PostalAddress");
		PostalAddress clientPostalAddress = readAddress((Element)clientAddressNode);
			    
//		out.println("\t\t</cac:PostalAddress>\n");
//		out.println("\t\t<cac:PartyLegalEntity>");
//		out.println("\t\t\t<cac:RegistrationAddress>");
		
	    //RegistrationAddress
		Node clientRegistrationAddressNode = getNode(clientPartyLegalEntityNode, "cac", "RegistrationAddress");
		PostalAddress clientRegistrationAddress = readAddress((Element)clientRegistrationAddressNode);

//		out.println("\t\t\t</cac:RegistrationAddress>\n");
		
	    //RegistrationName
	    String clientRegistrationName = getNodeValue((Element)clientPartyLegalEntityNode, "cbc", "RegistrationName");
	    clientRegistrationName = SpecialCharacter.xmlEntityToString(clientRegistrationName);
	    if ((clientName == null) || (clientName.equals(""))){
	    	clientName = clientRegistrationName;
	    }
	    
//	    out.println("\t\t\t<cbc:RegistrationName>" + clientRegistrationName + "</cbc:RegistrationName>");
		
	    
	    //VatID
	    String clientVatId = getNodeValue((Element)clientPartyLegalEntityNode, "cbc", "CompanyID");
	    if ((clientVatId == null) || (clientVatId.equals(""))){
	    	NodeList clientPartyTaxSchemeNodes = getNodes(clientPartyNode, "cac", "PartyTaxScheme");
			if (clientPartyTaxSchemeNodes  != null){
				for (int i = 0;  i < clientPartyTaxSchemeNodes.getLength();i++  ) {
					String schemeIDType = getNodeValue((Element)clientPartyTaxSchemeNodes.item(i), "cbc", "CompanyID/@schemeID");
					if ( (schemeIDType != null) && (schemeIDType.endsWith("VAT") ) ){
						clientVatId = getNodeValue((Element)clientPartyTaxSchemeNodes.item(i), "cbc", "CompanyID");
					}
					
					else {
						Node taxSchemeNode = getNode(clientPartyTaxSchemeNodes.item(i), "cac", "TaxScheme");
						if (taxSchemeNode != null){
							schemeIDType = getNodeValue((Element)taxSchemeNode, "cbc", "ID");
							if ( (schemeIDType != null) && (schemeIDType.endsWith("VAT") ) ){
								clientVatId = getNodeValue((Element)clientPartyTaxSchemeNodes.item(i), "cbc", "CompanyID");
							}
						}
					}
	
				}
			}
	    }
	    clientVatId = SpecialCharacter.xmlEntityToString(clientVatId);

	    //Codice fiscale
	    String clientFiscalCode = getNodeValue((Element)clientPartyLegalEntityNode, "cbc", "CompanyID");
	    if ((clientFiscalCode == null) || (clientFiscalCode.equals(""))){
	    	NodeList clientPartyTaxSchemeNodes = getNodes(clientPartyNode, "cac", "PartyTaxScheme");
			if (clientPartyTaxSchemeNodes  != null){
				for (int i = 0;  i < clientPartyTaxSchemeNodes.getLength();i++  ) {
					String schemeIDType = getNodeValue((Element)clientPartyTaxSchemeNodes.item(i), "cbc", "CompanyID/@schemeID");
					if ( (schemeIDType != null) && (schemeIDType.endsWith("CF") ) ){
						clientFiscalCode = getNodeValue((Element)clientPartyTaxSchemeNodes.item(i), "cbc", "CompanyID");
					}
	
				}
			}
	    }
	    clientFiscalCode = SpecialCharacter.xmlEntityToString(clientFiscalCode);
	    
	   
	    Client client = new Client();
	    client.setVatId(clientVatId);
	    client.setItalianFiscalCode(clientFiscalCode);
	    client.setClientName(clientName);
	    client.setPostalAddress(clientPostalAddress);
	    client.setRegistrationAddress(clientRegistrationAddress);
	    invoice.setClient(client);
		
//	    out.println("\t\t\t<cbc:CompanyID>" + clientFiscalCode + "</cbc:CompanyID>");
//	    out.println("\t\t</cac:PartyLegalEntity>");
//		out.println("\t</cac:Party>");
//		out.println("</cac:AccountingCustomerParty>\n");
	    
		
		//DescriptionCode
		String periodCode = getNodeValue((Element)invoicePeriodNode, "cbc", "DescriptionCode");
		periodCode = SpecialCharacter.xmlEntityToString(periodCode);	
		invoicePeriod.setPeriodCode(periodCode);
//		out.println("\t<cbc:DescriptionCode>" + periodCode + "</cbc:DescriptionCode>");
		
		/*//Description
		String periodDescription = getNodeValue((Element)invoicePeriodNode, "cbc", "Description");
		periodDescription = SpecialCharacter.xmlEntityToString(periodDescription);	
		invoicePeriod.setPeriodDescription(periodDescription);*/
				
		invoice.setInvoicePeriod(invoicePeriod);
		
		//Data di scadenza: cbc:PaymentDueDate
		Node paymentMeansNode = getNode(rootNode, "cac", "PaymentMeans");
		PaymentDetail paymentDetail = new PaymentDetail();
		
//		out.println("<cac:PaymentMeans>");
		
		String paymentMeansCode = getNodeValue((Element)paymentMeansNode, "cbc", "PaymentMeansCode");
		String paymentCodeId = getNodeValue((Element)paymentMeansNode, "cbc", "PaymentMeansCode/@listAgencyID");
//		out.println("\t<cbc:PaymentMeansCode listAgencyID=\"" + paymentCodeId + "\">" + paymentMeansCode + "</cbc:PaymentMeansCode>");
		paymentDetail.setPaymentMeansCode(paymentMeansCode);
		paymentDetail.setPaymentCodeId(paymentCodeId);
		
		//DueDate
		try {
			String stringPaymentDueDate = getNodeValue((Element)paymentMeansNode, "cbc", "PaymentDueDate");
			if(stringPaymentDueDate != null){
				Date paymentDueDate = dateFormat.parse(stringPaymentDueDate);
				paymentDetail.setPaymentDueDate(paymentDueDate);
//				out.println("\t<cbc:PaymentDueDate>" + stringPaymentDueDate + "</cbc:PaymentDueDate>");
			}
			
		} catch (Exception e) { 
			logger.error(e.getMessage());
		}
		
//		out.println("</cac:PaymentMeans>\n");
				
		//Termini di pagamento: cac:PaymentTerms
		Node paymentTermsNode = getNode(rootNode, "cac", "PaymentTerms");
		
//		out.println("<cac:PaymentTerms>");
		
		String paymentTerms = getNodeValue((Element)paymentTermsNode, "cbc", "Note");
		paymentTerms = SpecialCharacter.xmlEntityToString(paymentTerms);
		
//		out.println("\t<cbc:Note>" + paymentTerms + "</cbc:Note>");
		
		paymentDetail.setPaymentTerms(paymentTerms);
		
		invoice.setPaymentDetail(paymentDetail);

//		out.println("</cac:PaymentTerms>\n");
		
    	//Documenti pregressi
		List<PreviousDocument> previousDocuments = new ArrayList<PreviousDocument>();
		
		// - Ordine: cac:OrderReference
		Node orderReferenceNode =  getNode(rootNode, "cac", "OrderReference");	
		
//		out.println("<cac:OrderReference>");
		
		String orderCode = getNodeValue((Element)orderReferenceNode, "cbc", "ID");
		orderCode = SpecialCharacter.xmlEntityToString(orderCode);
		
//		out.println("\t<cbc:ID>" + orderCode + "</cbc:ID>");
		
		if((orderCode != null) && (!orderCode.equals(""))){
			PreviousDocument order;
			order = new PreviousDocument("ORDINE", orderCode); 
			order.setSupplier(supplier);
			previousDocuments.add(order);
		}
		
//		out.println("</cac:OrderReference>\n");
		
    	// - Contratto: cac:ContractDocumentReference
		NodeList contractDocumentNodes = getNodes(rootNode, "cac", "ContractDocumentReference");
		
		Node contractDocumentNode = null;
		for (int i = 0; i < contractDocumentNodes.getLength(); i++) {
			
//			out.println("<cac:ContractDocumentReference>");
			
			Element contractDocument = (Element)contractDocumentNodes.item(i);
			contractDocumentNode = contractDocumentNodes.item(i);
			
			String previousDocumentCode = getNodeValue(contractDocument, "cbc", "ID");
			previousDocumentCode = SpecialCharacter.xmlEntityToString(previousDocumentCode);
			String previousDocumentInfo = getNodeValue(contractDocument, "cbc", "DocumentType");
			previousDocumentInfo = SpecialCharacter.xmlEntityToString(previousDocumentInfo);
			
			PreviousDocument previousDocument = new PreviousDocument();
			previousDocument = new PreviousDocument("CONTRATTO", previousDocumentCode);
			previousDocument.setDocumentInfo(previousDocumentInfo);
			previousDocument.setSupplier(supplier);

			previousDocuments.add(previousDocument);
			
//			out.println("\t<cbc:ID>" + previousDocumentCode + "</cbc:ID>");
//			out.println("\t<cbc:DocumentType>" + previousDocumentInfo + "</cbc:DocumentType>");
//			out.println("</cac:ContractDocumentReference>\n");
		}
		
    	// - Documento generico: cac:AdditionalDocumentReference
		NodeList additionalDocumentNodes = getNodes(rootNode, "cac", "AdditionalDocumentReference");
		
//		out.println("<cac:AdditionalDocumentReference>");
		
		for (int i = 0; i < additionalDocumentNodes.getLength(); i++) {
			Element additionalDocument = (Element)additionalDocumentNodes.item(i);
			
			String previousDocumentCode = getNodeValue(additionalDocument, "cbc", "ID");
			previousDocumentCode = SpecialCharacter.xmlEntityToString(previousDocumentCode);
			String previousDocumentInfo = getNodeValue(additionalDocument, "cbc", "DocumentType");
			previousDocumentInfo = SpecialCharacter.xmlEntityToString(previousDocumentInfo);
			
			Node attachmentNode = getNode(additionalDocument, "cac", "Attachment");
			
			Node externalReferenceNode = getNode(attachmentNode, "cac", "ExternalReference");
			String previousDocumentURI = getNodeValue((Element)externalReferenceNode, "cbc", "URI");
			previousDocumentURI = SpecialCharacter.xmlEntityToString(previousDocumentURI);
			
			String embeddedDocumentBinaryObject = getNodeValue((Element)attachmentNode, "cbc", "EmbeddedDocumentBinaryObject");
			embeddedDocumentBinaryObject = SpecialCharacter.xmlEntityToString(embeddedDocumentBinaryObject);
			String embeddedDocumentMimeCode = getNodeValue((Element)attachmentNode, "cbc", "EmbeddedDocumentBinaryObject/@mimeCode");
			
//			out.println("\t<cbc:ID>" + previousDocumentCode + "</cbc:ID>");
//			out.println("\t<cbc:DocumentType>" + previousDocumentInfo + "</cbc:DocumentType>");
//			out.println("\t<cac:Attachment>");
//			out.println("\t\t<cac:ExternalReference>");
//			out.println("\t\t\t<cbc:URI>" + previousDocumentURI + "</cbc:URI>");
//			out.println("\t\t</cac:ExternalReference>");
//			out.println("\t\t<cbc:EmbeddedDocumentBinaryObject>" + embeddedDocumentBinaryObject + "</cbc:EmbeddedDocumentBinaryObject>");
//			out.println("\t\t<cbc:EmbeddedDocumentBinaryObject/@mimeCode>" + embeddedDocumentMimeCode + "</cbc:EmbeddedDocumentBinaryObject/@mimeCode>");
//			out.println("\t</cac:Attachment>\n");
			
			PreviousDocument previousDocument = new PreviousDocument();
			previousDocument = new PreviousDocument("DOCUMENTO AGGIUNTIVO", previousDocumentCode);
			previousDocument.setDocumentInfo(previousDocumentInfo);
			previousDocument.setDocumentURI(previousDocumentURI);
			previousDocument.setSupplier(supplier);
			
			if(embeddedDocumentBinaryObject != null){
					try {
						byte[] documentDecoded = new sun.misc.BASE64Decoder().decodeBuffer(embeddedDocumentBinaryObject);
						FEFile embeddedDocument = new FEFile(documentDecoded, "");
						embeddedDocument.setFileMimeType(embeddedDocumentMimeCode);
						previousDocument.setEmbeddedDocument(embeddedDocument);						
					} catch (IOException e) {
						logger.error(e.getMessage());
					}
			}
			previousDocuments.add(previousDocument);
		}
		
		invoice.setPreviousDocuments(previousDocuments);

//		out.println("</cac:AdditionalDocumentReference>\n");
		
		//Contatore utenze
		Node ublExtensionsNode = getNode(rootNode, "ext", "UBLExtensions");
		
		if(ublExtensionsNode != null) {
		
			Node ublExtensionNode = getNode(ublExtensionsNode, "ext", "UBLExtension");
			Node extensionContentNode = getNode(ublExtensionNode, "ext", "ExtensionContent");
			//Il prefisso non e indicato per cui, per compatibilita con xpath, va messo uno spazio vuoto
			Node utilityStatementNode = getNode(extensionContentNode, " ", "UtilityStatement");

//			out.println("\n\n");
//			out.println("<ext:UBLExtensions>");
//			out.println("\t<ext:UBLExtension>");
//			out.println("\t\t<ext:ExtensionContent>");
//			out.println("\t\t\t<UtilityStatement>\n");

			meterList = readUtilityStatementMeter((Element)utilityStatementNode, invoice);

//			out.println("\t\t\t</UtilityStatement>");
//			out.println("\t\t</ext:ExtensionContent>");
//			out.println("\t</ext:UBLExtension>");
//			out.println("</ext:UBLExtensions>");
//			out.println("\n\n");
		}
		
		/*//Contatore utenze: cac:Delivery
		NodeList deliveryNodes = getNodes(rootNode, "cac", "Delivery");
		
		out.println("<cac:Delivery>");
		
		List<Meter> meterList = new ArrayList<Meter>();		
		for (int i = 0; i < deliveryNodes.getLength(); i++) {
			Element delivery = (Element)deliveryNodes.item(i);
			String meterNumber = getNodeValue(delivery, "cbc", "ID");
			meterNumber = SpecialCharacter.xmlEntityToString(meterNumber);
			
			String connectionNumber = getNodeValue(delivery, "cbc", "PointOfDeliveryID");
			connectionNumber = SpecialCharacter.xmlEntityToString(connectionNumber);
			
			String meterContractCode = getNodeValue(delivery, "cbc", "TrackingID");
			meterContractCode = SpecialCharacter.xmlEntityToString(meterContractCode);
			
			PreviousDocument meterContract = new PreviousDocument("CONTRATTO", meterContractCode);
			
			SupplyInfo supplyInfo = new SupplyInfo();
			supplyInfo.setPreviousDocument(meterContract);

			List<SupplyInfo> supplyInfoList = new ArrayList<SupplyInfo>();
			supplyInfoList.add(supplyInfo);
			
			out.println("\t<cbc:ID>" + meterNumber + "</cbc:ID>");
			out.println("\t<cbc:PointOfDeliveryID>" + connectionNumber + "</cbc:PointOfDeliveryID>");
			out.println("\t<cbc:TrackingID>" + meterContractCode + "</cbc:TrackingID>");
			out.println("\t<cac:DeliveryAddress>");
			
			Node deliveryAddressNode = getNode(delivery, "cac", "DeliveryAddress");
			PostalAddress meterAddress = readAddress((Element)deliveryAddressNode, out);
			
		    
			//Settore merceologico
		    Node documentTypeNode = getNode(contractDocumentNode, "cbc", "DocumentType");
		    Node contractDocumentIdNode = getNode(contractDocumentNode, "cbc", "ID");
		    Sector sector = new Sector();
		    String sectorName = getNodeValueByExpr(doc, contractDocumentNode, documentTypeNode, contractDocumentIdNode, meterContractCode);
		    sectorName = SpecialCharacter.xmlEntityToString(sectorName);
			if(sectorName != null){
				sector.setSectorName(sectorName);
			}	
			
			out.println("\t</cac:DeliveryAddress>");
			
			
		    Meter meter = new Meter();
			meter = new Meter(meterNumber, connectionNumber, meterAddress, sector, new Date(), null, null);
			meter.setSupplyInfo(supplyInfoList);
			meterList.add(meter);
		}

		out.println("</cac:Delivery>"); */
		
		//Importo totale: cbc:PayableAmount
		Node legalMonetaryTotalNode = getNode(rootNode, "cac", "LegalMonetaryTotal");
		Double allInclusiveTotal = Double.parseDouble(getNumericNodeValue((Element)legalMonetaryTotalNode, "cbc", "PayableAmount"));
		String allInclusiveTotalCurrency = getNodeValue((Element)legalMonetaryTotalNode, "cbc", "PayableAmount/@currencyID");
		invoice.setAllInclusiveTotal(allInclusiveTotal);
		invoice.setAllInclusiveTotalCurrency(allInclusiveTotalCurrency);

//		out.println("<cac:LegalMonetaryTotal>");
//		out.println("\t<cbc:PayableAmount currencyID=\"" + allInclusiveTotalCurrency + "\">" + allInclusiveTotal + "</cbc:PayableAmount>");
//		out.println("</cac:LegalMonetaryTotal>\n");
		
		//AllowanceCharge (invoice level)
        allowanceChargeList = readAllowanceCharge(rootNode, false);
		if(allowanceChargeList.size() >= 0){
			invoice.setAllowanceCharges(allowanceChargeList);
		}
		
		//Data tasse: cbc:TaxPointDate
		try {
			String stringTaxPointDate = getNodeValue(rootNode, "cbc", "TaxPointDate");
			if(stringTaxPointDate != null){
				Date taxPointDate = dateFormat.parse(stringTaxPointDate);
				invoice.setTaxPointDate(taxPointDate);
				
//				out.println("<cbc:TaxPointDate>" + stringTaxPointDate + "</cbc:TaxPointDate>");
				
			}
			
		} catch (Exception e) { 
			logger.error(e.getMessage());
		}
		
		//Dettaglio IVA: cac:TaxTotal
		Node taxTotalNode = getNode(rootNode, "cac", "TaxTotal");
		Double roundingAmount = Double.parseDouble(getNumericNodeValue((Element)taxTotalNode, "cbc", "RoundingAmount"));
		NodeList taxSubTotalNodes = getNodes(taxTotalNode, "cac", "TaxSubtotal");
		
//		out.println("<cac:TaxTotal>");
//		out.println("\t<cbc:RoundingAmount>" + roundingAmount + "</cbc:RoundingAmount>");
		
		if (taxSubTotalNodes != null){
			for (int i = 0; i < taxSubTotalNodes.getLength(); i++) {
//				out.println("\t<cac:TaxSubtotal>");
				Element taxSubTotal = (Element)taxSubTotalNodes.item(i);
				TaxDetail taxDetail = new TaxDetail();
				taxDetail.setTaxableAmount(Double.parseDouble(getNumericNodeValue(taxSubTotal, "cbc", "TaxableAmount")));
				taxDetail.setTaxAmount(Double.parseDouble(getNumericNodeValue(taxSubTotal, "cbc", "TaxAmount")));
				taxDetail.setTaxAmountCurrency(getNodeValue(taxSubTotal, "cbc", "TaxAmount/@currencyID"));
				
				Node taxCategoryNode = getNode(taxSubTotal, "cac", "TaxCategory");
				taxDetail.setTaxPercent(Double.parseDouble(getNumericNodeValue((Element)taxCategoryNode, "cbc", "Percent")));
				taxDetail.setDescription(getNodeValue((Element)taxCategoryNode, "cbc", "ID"));	
				taxDetailList.add(taxDetail);
				
//				out.println("\t\t<cbc:TaxableAmount>" + taxDetail.getTaxableAmount() + "</cbc:TaxableAmount>");
//				out.println("\t\t<cbc:TaxAmount currencyID=\"" + taxDetail.getTaxAmountCurrency() + "\">" + taxDetail.getTaxAmount() + "</cbc:TaxAmount>");
//				out.println("\t\t<cac:TaxCategory>");
//				out.println("\t\t\t<cbc:Percent>" + taxDetail.getTaxPercent() + "</cbc:Percent>");
//				out.println("\t\t\t<cbc:ID>" + taxDetail.getDescription() + "</cbc:ID>");
//				out.println("\t\t</cac:TaxCategory>");
//				out.println("\t</cac:TaxSubtotal>\n");
			}
		}
		invoice.setTaxDetails(taxDetailList);
		
//		out.println("</cac:TaxTotal>");
		
		//Accounting cost: cbc:AccountingCost
		String accountingCost = getNodeValue(rootNode, "cbc", "AccountingCost");
		invoice.setAccountingCost(accountingCost);
		
//		out.println("<cbc:AccountingCost>" + accountingCost + "</cbc:AccountingCost>");
	    
		//Letture
		//List<Reading> readingList = new ArrayList<Reading>();
		
		//Riassunto contatore
		//List<MeterSummary> meterSummaryList = new ArrayList<MeterSummary>();
		
		//Righe
		NodeList invoiceLines = getNodes(rootNode, "cac", "InvoiceLine");
		
//		out.println("<cac:InvoiceLine>");
		
        for (int i = 0; i < invoiceLines.getLength(); i++) {
            Element line = (Element)invoiceLines.item(i);
            InvoiceLine invoiceLine = new InvoiceLine();
            
          //controllo che la linea non sia legata ai consumi, gia inseriti come dettaglio righe
            Boolean skipInvoiceLine = false;
            Integer invoiceLineNumber = Integer.parseInt(getNumericNodeValue(line, "cbc", "ID"));
            for (Integer num : parentDocumentLineReferenceList) {
            	 if(invoiceLineNumber == num){
            		 skipInvoiceLine = true;
            		 break;
            	 }
            }
            if(skipInvoiceLine){
            	continue;
            }
            
            invoiceLine.setLineNumber(Integer.parseInt(getNumericNodeValue(line, "cbc", "ID")));
            invoiceLine.setUnitCode(getNodeValue(line, "cbc", "InvoicedQuantity/@unitCode"));
            invoiceLine.setInvoicedQuantity(Double.parseDouble(getNumericNodeValue(line, "cbc", "InvoicedQuantity")));
            invoiceLine.setLineExtensionAmount(Double.parseDouble(getNumericNodeValue(line, "cbc", "LineExtensionAmount")));
            invoiceLine.setCurrencyID(getNodeValue(line, "cbc", "LineExtensionAmount/@currencyID"));
            
            //Setto il contoContrattuale per ogni riga
          	invoiceLine.setContoContrattuale(contoContrattuale);
            
//          	out.println("\t<cbc:ID>" + invoiceLine.getLineNumber() + "</cbc:ID>");
//          	out.println("\t<cbc:InvoicedQuantity unitCode=\"" + invoiceLine.getUnitCode() + "\">" + invoiceLine.getInvoicedQuantity() + "</cbc:InvoicedQuantity>");
//          	out.println("\t<cbc:LineExtensionAmount currencyID=\"" + invoiceLine.getCurrencyID() + "\">" + invoiceLine.getLineExtensionAmount() + "</cbc:LineExtensionAmount>");
          	
    		//AllowanceCharge (line level)
            List<AllowanceCharge> lineAllowanceChargeList = readAllowanceCharge((Element)line, false);
            if(lineAllowanceChargeList.size() >= 0){
    			invoiceLine.setAllowanceCharges(lineAllowanceChargeList);
    		}
    		
    	    Node priceNode = getNode(line, "cac", "Price");
    	    invoiceLine.setPriceAmount(Double.parseDouble(getNumericNodeValue((Element)priceNode, "cbc", "PriceAmount")));
    	    
//    	    out.println("\t<cac:Price>");
//    	    out.println("\t\t<cbc:PriceAmount>" + invoiceLine.getPriceAmount() + "</cbc:PriceAmount>");
    	    
    	    //AllowanceCharge (price)
    		List<AllowanceCharge> linePriceAllowanceChargeList = readAllowanceCharge((Element)priceNode, true);
    		if(linePriceAllowanceChargeList.size() >= 0){
    			lineAllowanceChargeList.addAll(linePriceAllowanceChargeList);
    		}
    	    
//    		out.println("\t</cac:Price>\n");
    		
    		//Totale tasse: cac:TaxTotal
    		Node invoiceLineTaxTotalNode = getNode(line, "cac", "TaxTotal");
//    		
//    		out.println("\t<cac:TaxTotal>");
    		
    		//Suppongo di avere solo un subTotal per prendere il taxCategory
    		Node taxSubtotaleNode = getNode(invoiceLineTaxTotalNode, "cac", "TaxSubtotal");
    		
//    		out.println("\t\t<cac:TaxSubtotal>");
    		
    		Node taxCategoryNode = getNode(taxSubtotaleNode, "cac", "TaxCategory");
    		
//    		out.println("\t\t\t<cac:TaxCategory>");
    		
    		TaxDetail invocieLineTaxDetail = new TaxDetail();
       		invocieLineTaxDetail.setTaxAmount(Double.parseDouble(getNumericNodeValue((Element)invoiceLineTaxTotalNode, "cbc", "TaxAmount")));
       		invocieLineTaxDetail.setTaxableAmount(Double.parseDouble(getNumericNodeValue(line, "cbc", "LineExtensionAmount")));
       		invocieLineTaxDetail.setDescription(getNodeValue((Element)taxCategoryNode, "cbc", "ID"));
       		invoiceLine.setTaxDetail(invocieLineTaxDetail);
       		
//       		out.println("\t\t<cbc:TaxAmount>" + invocieLineTaxDetail.getTaxAmount() + "</cbc:TaxAmount>");
//       		out.println("\t\t<cbc:LineExtensionAmount>" + invocieLineTaxDetail.getTaxableAmount() + "</cbc:LineExtensionAmount>");
//       		out.println("\t\t<cbc:ID>" + invocieLineTaxDetail.getDescription() + "</cbc:ID>");
//       		out.println("\t\t\t</cac:TaxCategory>");
//       		out.println("\t\t</cac:TaxSubtotal>");
//       		out.println("\t</cac:TaxTotal>\n");
       		
//       		out.print("\t<cbc:Note>");
       		
    		//Note
       		NodeList notes = getNodes(line, "cbc", "Note");
    		InvoiceLineNote invoiceLineNote = new InvoiceLineNote();
    		List<InvoiceLineNote> invoiceLineNoteList = new ArrayList<InvoiceLineNote>();
    		for (int z = 0; z < notes.getLength(); z++) {
    			Element note = (Element)notes.item(z);
    			String lineNote = getNodeValue(note, null, null);
    			if(lineNote != null){
    				invoiceLineNote.setNote(lineNote);
    				invoiceLineNoteList.add(invoiceLineNote);
    				
//    				out.print(invoiceLineNote.getNote());
    			}
    		}
    		if(invoiceLineNoteList.size() >= 0){
    			invoiceLine.setNoteList(invoiceLineNoteList);
    		}
//    		out.println("</cbc:Note>");
    		
    		//AccountingCost
    		String lineAccountingCost = getNodeValue((Element)line, "cbc", "AccountingCost");
    		lineAccountingCost = SpecialCharacter.xmlEntityToString(lineAccountingCost);
    		invoiceLine.setLineAccountingCost(lineAccountingCost);
    		
//    		out.println("\t<cbc:AccountingCost>" + lineAccountingCost + "</cbc:AccountingCost>");
    		
    		//OrderLineReference
    		Node orderLineReferenceNode = getNode(line, "cac", "OrderLineReference");
    		String orderLineReference = getNodeValue((Element)orderLineReferenceNode, "cbc", "LineID");
    		orderLineReference = SpecialCharacter.xmlEntityToString(orderLineReference);
    		invoiceLine.setOrderLineReference(orderLineReference);
    		
//    		out.println("\t<cac:OrderLineReference>");
//    		out.println("\t\t<cbc:LineID>" + orderLineReference + "</cbc:LineID>");
//    		out.println("\t</cac:OrderLineReference>\n");
    		
    		//Documento di riferimento
    		NodeList documentRefNodes = getNodes(line, "cac", "DocumentReference");
    		String documentRef;
    		String documentType;
    		
//    		out.println("\t<cac:DocumentReference>");
    		
    		for (int z = 0; z < documentRefNodes.getLength(); z++) {
    			Element docRef = (Element)documentRefNodes.item(z);
    			documentRef = getNodeValue(docRef, "cbc", "ID");
        		documentType = getNodeValue(docRef, "cbc", "DocumentType");
        		
//        		out.println("\t\t<cbc:ID>" + documentRef + "</cbc:ID>");
//        		out.println("\t\t<cbc:DocumentType>" + documentType + "</cbc:DocumentType>");
        		
        		//se il tipo documento è un contatore associa alla riga il contatore dichiarato in precedenza
        		/*if("CONTATORE".equalsIgnoreCase(documentType)){
        			for(int c = 0; c < meterList.size(); c++) {
    					if((meterList.get(c).getMeterNumber() != null) &&
    						(meterList.get(c).getMeterNumber().equalsIgnoreCase(documentRef))){
    						invoiceLine.setMeter(meterList.get(c));
    						break;
    					}
    				}
        		}
        		//se non è specificato il contatore cerca un contatore con lo stesso numero di contratto: es. caso ambiente
        		else if((invoiceLine.getMeter() == null) && ("CONTRATTO".equalsIgnoreCase(documentType))){
        			for(int c = 0; c < meterList.size(); c++) {
    					if(meterList.get(c).getSupplyInfo().size() > 0){
    	        			for(int w = 0; w < meterList.get(c).getSupplyInfo().size(); w++) {
    	        				if((meterList.get(c).getSupplyInfo().get(w).getPreviousDocument() != null) &&
    	    						(meterList.get(c).getSupplyInfo().get(w).getPreviousDocument().getDocumentCode().equalsIgnoreCase(documentRef))){
    	    						invoiceLine.setMeter(meterList.get(c));
    	    						break;
    	    					}
    	        			}
    					}
    				}
        		}*/
    		}
    		 
//    		out.println("\t</cac:DocumentReference>\n");
    		
    		//Item
    		Node itemNode = getNode(line, "cac", "Item");
       		invoiceLine.setItemDescription(getNodeValue((Element)itemNode, "cbc", "Description"));
       		
//       		out.println("\t<cac:Item>");
//       		out.println("\t\t<cbc:Description>" + invoiceLine.getItemDescription() + "</cbc:Description>");
       		
       		//Dettaglio tasse
       		Node classifiedTaxCategoryNode = getNode(itemNode, "cac", "ClassifiedTaxCategory");
    		invocieLineTaxDetail.setTaxPercent(Double.parseDouble(getNumericNodeValue((Element)classifiedTaxCategoryNode, "cbc", "Percent")));
    		invoiceLine.setTaxDetail(invocieLineTaxDetail);       		
       		
//    		out.println("\t\t<cac:ClassifiedTaxCategory>");
//    		out.println("\t\t\t<cbc:Percent>" + invocieLineTaxDetail.getTaxPercent() + "</cbc:Percent>");
//    		out.println("\t\t</cac:ClassifiedTaxCategory>\n");
    		
    		String itemName = getNodeValue((Element)itemNode, "cbc", "Name");
    		itemName = SpecialCharacter.xmlEntityToString(itemName);
    		invoiceLine.setItemName(itemName);
    		
//    		out.println("\t\t<cbc:Name>" + itemName + "</cbc:Name>");
    		
    		Node sellersItemIdentificationNode = getNode(itemNode, "cac", "SellersItemIdentification");
    		String sellersItemIdentification = getNodeValue((Element)sellersItemIdentificationNode, "cbc", "ID");
    		sellersItemIdentification = SpecialCharacter.xmlEntityToString(sellersItemIdentification);
    		invoiceLine.setSellersItemIdentification(sellersItemIdentification);
    		
//    		out.println("\t\t<cac:SellersItemIdentification>");
//    		out.println("\t\t\t<cbc:ID>" + sellersItemIdentification + "</cbc:ID>");
//    		out.println("\t\t</cac:SellersItemIdentification>\n");
    		
    		/* String itemKeyword = getNodeValue((Element)itemNode, "cbc", "Keyword");
    		
    		out.println("\t\t<cbc:Keyword>" + itemKeyword + "</cbc:Keyword>");
    		
    		if((itemKeyword == null) || (itemKeyword.equals(""))) {
				itemKeyword = "RIGA";
			}
    		
    		out.println("\t</cac:Item>");
    		
			switch (XmlFactory.itemKeyword.valueOf(itemKeyword)) {
    			case RIGA:
        			//logger.info("Invoice line num. " + invoiceLine + ", Invoiced quantity " + invoicedQuantity + ", Extension amount " + lineExtensionAmount + ", Item description " + itemDescription);				        			
        			invoice.addInvoiceLine(invoiceLine);
        			invoice.localAddInvoiceLine(invoiceLine);	    				
    				break;
    			case CONSUMO:
    				Date startConsumptionDate = null;
    				Date endConsumptionDate = null;
    				String consumptionBracket = null;
    				
    				if(invoiceLine.getNoteList() != null){
	    				for(int z=0; z<invoiceLine.getNoteList().size(); z++){
	    					if(invoiceLine.getNoteList().get(z).getNote() != null){
		    					String[] s = invoiceLine.getNoteList().get(z).getNote().split("-");
		    					if(s.length > 0){
		    						SimpleDateFormat formatterDate;
		    						try {
		    						     formatterDate = new SimpleDateFormat("yyyyMMdd");
		    						     startConsumptionDate = (Date)formatterDate.parse(s[0]);
		    						     endConsumptionDate = (Date)formatterDate.parse(s[1]);
			    					}
			    					catch (ParseException e){
			    						logger.error("Exception: " + e);
			    					}
		    					}
	    					}
	    				}
    				}

		        	Consumption consumption = new Consumption(startConsumptionDate, endConsumptionDate, consumptionBracket);
		        	invoiceLine.setConsumption(consumption);
		        	
		        	//logger.info("Consumo");
        			//logger.info("Invoice line num. " + invoiceLine + ", Invoiced quantity " + invoicedQuantity + ", Extension amount " + lineExtensionAmount + ", Item description " + itemDescription);				        			
        			invoice.addInvoiceLine(invoiceLine);
        			invoice.localAddInvoiceLine(invoiceLine);
        			
		            break;
		        case LETTURA:
		        	Date readingDate = null;
		        	if(invoiceLine.getNoteList() != null){
    					SimpleDateFormat formatterDate;
    					
    					for(int z=0; z<invoiceLine.getNoteList().size(); z++){
    						
	    					if(invoiceLine.getNoteList().get(z) != null){
	    						try {
	    						     formatterDate = new SimpleDateFormat("yyyyMMdd");
	    						     readingDate = (Date)formatterDate.parse(invoiceLine.getNoteList().get(z).getNote());
		    					}
		    					catch (ParseException e){
		    						logger.error("Exception: " + e);
		    					}
	    					}
	    				}
    				}
		        			        	
		        	Reading reading = new Reading(itemName, readingDate, invoiceLine.getItemDescription(), invoiceLine.getInvoicedQuantity(), invoiceLine.getUnitCode(), invoiceLine.getMeter());
		        	readingList.add(reading);
		        	invoice.setReadingList(readingList);
		        	
		        	break;
		        case RIASSUNTO_CONTATORE:
		        	MeterSummary meterSummary = new MeterSummary();
		        	meterSummary.setMeter(invoiceLine.getMeter());
		        	
		        	if(invoice.getMeterSummary() != null){
		        		if(invoice.getMeterSummary().size() > 0) {
			        		Meter meterPre = invoice.getMeterSummary().get(invoice.getMeterSummary().size()-1).getMeter();
			        		if((meterPre != null) && (meterPre.equals(invoiceLine.getMeter()))){
			        			meterSummary = invoice.getMeterSummary().get(invoice.getMeterSummary().size()-1);
			        		}
		        		}
		        	}
		        	
		        	if("ACCONTO".equalsIgnoreCase(itemName)){
		        		meterSummary.setMeterOnAccountQuantity(invoiceLine.getInvoicedQuantity());
		        	}
		        	else if("CONGUAGLIO".equalsIgnoreCase(itemName)){
		        		meterSummary.setMeterBalanceQuantity(invoiceLine.getInvoicedQuantity());
		        	}
		        	else{
		        		meterSummary.setMeterInvoicedQuantity(invoiceLine.getInvoicedQuantity());
		        		meterSummary.setMeterSummaryUnitCode(invoiceLine.getUnitCode());
		        	}
		        	
		        	if(invoice.getMeterSummary() != null){
		        		if(invoice.getMeterSummary().size() > 0) {
			        		if(invoice.getMeterSummary().get(invoice.getMeterSummary().size()-1).getMeter().equals(invoiceLine.getMeter())){
			        			meterSummary = invoice.getMeterSummary().set(invoice.getMeterSummary().size()-1, meterSummary);
			        		}
			        		else {
			        			meterSummaryList.add(meterSummary);
			        			invoice.setMeterSummary(meterSummaryList);
			        		}
		        		}
		        	}
		        	else {
	        			meterSummaryList.add(meterSummary);
	        			invoice.setMeterSummary(meterSummaryList);
		        	}

		        	break;
		        default:
		        	
		        	break;
			} */
        }
        
//        out.println("</cac:InvoiceLine> \n");
//        out.close();
	}
	
//	public static List<Meter> readUtilityStatementMeter(Element utilityStatementElement, Invoice invoice, PrintWriter out) throws XPathExpressionException {
	public static List<Meter> readUtilityStatementMeter(Element utilityStatementElement, Invoice invoice) throws XPathExpressionException {
		
		List<Meter> meterList = new ArrayList<Meter>();
		List<SupplyInfo> supplyInfoList = new ArrayList<SupplyInfo>();
		List<Contact> contactList = new ArrayList<Contact>();
		List<PreviousDocument> previousDocumentList = new ArrayList<PreviousDocument>();
		List<MeterSummary> meterSummaryList = new ArrayList<MeterSummary>();
		SimpleDateFormat formatterDate = new SimpleDateFormat("yyyyMMdd");
		boolean ambiente = false;
		
//		//Settore merceologico
//		Sector sector = new Sector();
//		String sectorName = getNodeValue((Element)utilityStatementElement, "cbc", "UtilityStatementTypeCode");
//	    sectorName = SpecialCharacter.xmlEntityToString(sectorName);
//		if(sectorName != null){
//			sector.setSectorName(sectorName);
//		}
		

		//SubscriberConsumption (0:n)
		NodeList subscriberConsumptionNodes = getNodes(utilityStatementElement, "cac", "SubscriberConsumption");
		
//		out.println("\t<!-- Fornitura -->");
//		out.println("\t<cac:SubscriberConsumption>");
		
		for (int i = 0; i < subscriberConsumptionNodes.getLength(); i++) {

			supplyInfoList.clear();
			previousDocumentList.clear();
			ambiente = false;
			
			//Settore merceologico
			Sector sector = new Sector();
			Node subscriberConsumptionNode = subscriberConsumptionNodes.item(i);
			
			String sectorName = getNodeValue((Element)subscriberConsumptionNode, "cbc", "SpecificationTypeCode");
			
//			out.println("\t\t<cbc:SpecificationTypeCode>" + sectorName + "</cbc:SpecificationTypeCode>");
			
		    sectorName = SpecialCharacter.xmlEntityToString(sectorName);
			if(sectorName != null){
				sector.setSectorName(sectorName);
			}
			
			String subscriberConsumptionNote = getNodeValue((Element)subscriberConsumptionNode, "cbc", "Note");
			
			InvoiceNote invoiceNote = new InvoiceNote();
			invoiceNote.setNote(subscriberConsumptionNote);
			noteList.add(invoiceNote);
			
//			out.println("\t\t<cbc:Note>" + subscriberConsumptionNote + "</cbc:Note>\n");
			
			/* Punto di consegna: cac:UtilityConsumptionPoint (0:1) */			
			Node utilityConsumptionPointNode = getNode(subscriberConsumptionNode, "cac", "UtilityConsumptionPoint");
			
//			out.println("\t<!-- Punto di consegna -->");
//			out.println("\t<cac:UtilityConsumptionPoint>");
			
			//Identificativo univoco (POD, PdR...)
			String connectionNumber = getNodeValue((Element)utilityConsumptionPointNode, "cbc", "ID");
			connectionNumber = SpecialCharacter.xmlEntityToString(connectionNumber);
			UtilityConsumptionPoint utilityConsumptionPoint = new UtilityConsumptionPoint(connectionNumber);
			
//			out.println("\t\t<cbc:ID>" + connectionNumber + "</cbc:ID>");
			
			//Ubicazione
			
//			out.println("\t\t<cac:Address>");
			
			Node utilityConsumptionPointAddressNode = getNode(utilityConsumptionPointNode, "cac", "Address");
			PostalAddress utilityConsumptionPointAddress = readAddress((Element)utilityConsumptionPointAddressNode);
			
//			out.println("\t\t</cac:Address>\n");
			
			/* Contatore: cac:UtilityMeter (0:n) */
			NodeList utilityMeterNodes = getNodes(utilityConsumptionPointNode, "cac", "UtilityMeter");

			//Nel caso di bollette Hera di tipo ambiente non ci sono contatori
			if(utilityMeterNodes == null || utilityMeterNodes.getLength() == 0)
				ambiente = true;
			
			for (int z = 0; z < utilityMeterNodes.getLength(); z++) {
				
//				out.println("\t\t<!-- Contatore -->");
//				out.println("\t\t<cac:UtilityMeter>");
				
				Meter meter = new Meter();
				MeterSummary meterSummary = new MeterSummary();
				Node utilityMeterNode = utilityMeterNodes.item(z);
				
				meterSummary.setMeter(meter);
				
				meter.setUtilityConsumptionPoint(utilityConsumptionPoint);
				meter.setMeterAddress(utilityConsumptionPointAddress);
				meter.setSector(sector);
				
				//Matricola contatore
//				out.println("\t\t\t<!-- Matricola -->");
				String meterNumber = getNodeValue((Element)utilityMeterNode, "cbc", "MeterNumber");
				meterNumber = SpecialCharacter.xmlEntityToString(meterNumber);
				meter.setMeterNumber(meterNumber);				
				
//				out.println("\t\t\t<cbc:MeterNumber>" + meterNumber + "</cbc:MeterNumber>");
				
				/* Letture */
//				out.println("\t\t\t<!-- Letture -->");
				Node meterReadingNode = getNode(utilityMeterNode, "cac", "MeterReading");
				
//				out.println("\t\t\t<cac:MeterReading>");
				
				List<Reading> readingList = new ArrayList<Reading>();
				//TODO Numero
				
				//Data
				Date previousMeterReadingDate = new Date();
				try {	
					String previousDate = getNodeValue((Element)meterReadingNode, "cbc", "PreviousMeterReadingDate");
			    	if (previousDate != null) {
			    		previousMeterReadingDate = (Date)formatterDate.parse(previousDate);
//			    		out.println("\t\t\t\t<cbc:PreviousMeterReadingDate>" + previousMeterReadingDate + "</cbc:PreviousMeterReadingDate>");
			    	}
				}
				catch (ParseException e){
					logger.error("Date exception: " + e);
				}
				
				Date latestMeterReadingDate = new Date();
				try {		
					String latestDate = getNodeValue((Element)meterReadingNode, "cbc", "LatestMeterReadingDate");
					if(latestDate != null) {
						latestMeterReadingDate = (Date)formatterDate.parse(latestDate);
//						out.println("\t\t\t\t<cbc:LatestMeterReadingDate>" + latestMeterReadingDate + "</cbc:LatestMeterReadingDate>");
					}
				}
				catch (ParseException e){
					logger.error("Date exception: " + e);
				}
				

				//Tipo lettura
				String meterReadingType = getNodeValue((Element)meterReadingNode, "cbc", "MeterReadingType");
				meterReadingType = SpecialCharacter.xmlEntityToString(meterReadingType);
				
//				out.println("\t\t\t\t<cbc:MeterReadingType>" + meterReadingType + "</cbc:MeterReadingType>");
				
				String meterReadingTypeCode = getNodeValue((Element)meterReadingNode, "cbc", "MeterReadingTypeCode");
				meterReadingTypeCode = SpecialCharacter.xmlEntityToString(meterReadingTypeCode);
				
//				out.println("\t\t\t\t<cbc:MeterReadingTypeCode>" + meterReadingType + "</cbc:MeterReadingTypeCode>");
				
				//Metodo lettura
				String meterReadingMethod = getNodeValue((Element)meterReadingNode, "cbc", "MeterReadingMethod");
				meterReadingMethod = SpecialCharacter.xmlEntityToString(meterReadingMethod);
				
//				out.println("\t\t\t\t<cbc:MeterReadingMethod>" + meterReadingMethod + "</cbc:MeterReadingMethod>");
				
				String meterReadingMethodCode = getNodeValue((Element)meterReadingNode, "cbc", "MeterReadingMethodCode");
				meterReadingMethodCode = SpecialCharacter.xmlEntityToString(meterReadingMethodCode);
				
//				out.println("\t\t\t\t<cbc:MeterReadingMethodCode>" + meterReadingMethodCode + "</cbc:MeterReadingMethodCode>");
				
				//Descrizione
				String meterReadingDescription = "";
				NodeList meterReadingDescriptionNodes =  getNodes((Element)meterReadingNode, "cbc", "MeterReadingComments");
				
				for (int y = 0; y < meterReadingDescriptionNodes.getLength(); y++) {	
					String readingDescription = getNodeValue((Element)meterReadingDescriptionNodes.item(y), "cbc", "MeterReadingComments");
					readingDescription = SpecialCharacter.xmlEntityToString(meterReadingDescription);
					
					meterReadingDescription = meterReadingDescription + ". " + readingDescription;
				}
				
//				out.println("\t\t\t\t<cbc:MeterReadingComments>" + meterReadingDescription + "</cbc:MeterReadingComments>");
				
				//Valore
				Double previousMeterQuantity = Double.parseDouble(getNumericNodeValue((Element)meterReadingNode, "cbc", "PreviousMeterQuantity"));
				Double latestMeterQuantity = Double.parseDouble(getNumericNodeValue((Element)meterReadingNode, "cbc", "LatestMeterQuantity"));
				
				//Unita di misura
				String previousMeterQuantityUnitCode = getNodeValue((Element)meterReadingNode, "cbc", "PreviousMeterQuantity/@unitCode");
				String latestMeterQuantityUnitCode = getNodeValue((Element)meterReadingNode, "cbc", "LatestMeterQuantity/@unitCode");
				
//				out.println("\t\t\t\t<cbc:PreviousMeterQuantity unitCode=\"" + previousMeterQuantityUnitCode + "\">" + previousMeterQuantity + "</cbc:PreviousMeterQuantity>");
//				out.println("\t\t\t\t<cbc:LatestMeterQuantity unitCode=\"" + latestMeterQuantityUnitCode + "\">" + latestMeterQuantity + "</cbc:LatestMeterQuantity>");
				
				//Consumo
				Double meterDeliveredQuantity = Double.parseDouble(getNumericNodeValue((Element)meterReadingNode, "cbc", "DeliveredQuantity"));
				meterSummary.setMeterInvoicedQuantity(meterDeliveredQuantity);
				
//				out.println("\t\t\t\t<cbc:DeliveredQuantity>" + meterDeliveredQuantity + "</cbc:DeliveredQuantity>");
				
				Reading previousMeterReading = new Reading("", previousMeterReadingDate, meterReadingType, meterReadingMethod, previousMeterQuantity, previousMeterQuantityUnitCode, meter);
				previousMeterReading.setReadingDescription(meterReadingDescription);
				readingList.add(previousMeterReading);
				
				Reading latestMeterReading = new Reading("", latestMeterReadingDate, meterReadingType, meterReadingMethod, latestMeterQuantity, latestMeterQuantityUnitCode, meter);
				latestMeterReading.setReadingDescription(meterReadingDescription);
				readingList.add(latestMeterReading);
				
				
				//SupplyInfo
				for(PreviousDocument pd: previousDocumentList) {
					SupplyInfo supplyInfo = new SupplyInfo();
					supplyInfo.setPreviousDocument(pd);
					
					supplyInfoList.add(supplyInfo);
				}
				meter.setSupplyInfo(supplyInfoList);
				
				meterList.add(meter);
				
//				out.println("\t\t\t</cac:MeterReading>\n");
				
				//MeterProperty
				NodeList meterPropertyNodeList = getNodes(utilityMeterNode, "cac", "MeterProperty");
				
				for (int k = 0; k < meterPropertyNodeList.getLength(); k++) {
					
					Node meterPropertyNode = meterPropertyNodeList.item(k); 
					String nameCode = getNodeValue((Element)meterPropertyNode, "cbc", "NameCode");
					String value = getNodeValue((Element)meterPropertyNode, "cbc", "Value");
					Double valueQuantity = Double.parseDouble(getNumericNodeValue((Element)meterPropertyNode, "cbc", "ValueQuantity"));
					String valueQuantityUnitCode = getNodeValue((Element)meterPropertyNode, "cbc", "ValueQuantity/@unitCode");
					MeterProperty meterProperty = new MeterProperty(nameCode, value, valueQuantity, valueQuantityUnitCode);
					
					meterPropertyList.add(meterProperty);
					
//					out.println("\t\t\t<cac:MeterProperty>");
//					out.println("\t\t\t\t<cbc:Name>" + nameCode + "</cbc:Name>");
//					out.println("\t\t\t\t<cbc:Value>" + value + "</cbc:Value>");
//					out.println("\t\t\t\t<cbc:ValueQuantity unitCode=\"" + valueQuantityUnitCode + "\">" + valueQuantity + "</cbc:ValueQuantity>");
//					out.println("\t\t\t</cac:MeterProperty>");
				}
				meter.setMeterPropertyList(meterPropertyList);
//				out.println("\t\t</cac:UtilityMeter>");
			}
	
//			out.println("\t</cac:UtilityConsumptionPoint>\n");
			
			//Supplier Consumption - Dettaglio consumi
			Node supplierConsumptionNode = getNode(subscriberConsumptionNode, "cac", "SupplierConsumption");			
			
//			out.println("\t<!-- Dettaglio consumi -->");
//			out.println("\t<cac:SupplierConsumption>");
			
			//UtilitySupplier
//			out.println("\t\t<cac:UtilitySupplierParty>");
			Node utilitySupplierPartyNode = getNode(supplierConsumptionNode, "cac", "UtilitySupplierParty");
			
//			out.println("\t\t\t<cac:PartyName>");
			Node partyNameNode = getNode(utilitySupplierPartyNode, "cac", "PartyName");
			String utilitySupplierName = getNodeValue((Element)partyNameNode, "cbc", "Name");
//			out.println("\t\t\t\t<cbc:Name>" + utilitySupplierName + "</cbc:Name>");
//			out.println("\t\t\t</cac:PartyName>\n");
			
//			out.println("\t\t\t<cac:PostalAddress>");
			Node postalAddressNode = getNode(utilitySupplierPartyNode, "cac", "PostalAddress");
			PostalAddress utilitySupplierPostalAddress = readAddress((Element)postalAddressNode);
//			out.println("\t\t\t</cac:PostalAddress>\n");
			
//			out.println("\t\t\t<cac:PartyTaxScheme>");
			Node supplierPartyTaxSchemeNode = getNode(utilitySupplierPartyNode, "cac", "PartyTaxScheme");
			String utilitySupplierCompanyId = getNodeValue((Element)supplierPartyTaxSchemeNode, "cbc", "CompanyID");
//			out.println("\t\t\t\t<cbc:CompanyID>" + utilitySupplierCompanyId + "</cbc:CompanyID>");
//			out.println("\t\t\t</cac:PartyTaxScheme>\n");
			
			UtilitySupplier utilitySupplier = new UtilitySupplier(utilitySupplierName, utilitySupplierPostalAddress, utilitySupplierCompanyId);
			utilitySupplier.setInvoice(invoice);
			invoice.setUtilitySupplier(utilitySupplier);
			
//			out.println("\t\t\t<cac:Contact>");
			NodeList contactNodeList = getNodes(utilitySupplierPartyNode, "cac", "Contact");
			
			for (int j = 0; j < contactNodeList.getLength(); j++) {
				
				Node contactNode = contactNodeList.item(j);
				String telephone = getNodeValue((Element)contactNode, "cbc", "Telephone");
//				out.println("\t\t\t\t<cbc:Telephone>" + telephone + "</cbc:Telephone>");
				String contactNotes = getNodeValue((Element)contactNode, "cbc", "Note");
//				out.println("\t\t\t\t<cbc:Note>" + contactNotes + "</cbc:Note>");
//				out.println("\t\t\t</cac:Contact>\n");
				Contact contact = new Contact(telephone, contactNotes);
				contactList.add(contact);
			}
			utilitySupplier.setContactList(contactList);
			
//			out.println("\t\t</cac:UtilitySupplierParty>\n");
			
			//Consumi - A livello di dominio non viene creato l'oggetto Consumption, 
			//ma i vari campi vengono salvati a livello di Invoice
			Node consumptionNode = getNode(supplierConsumptionNode, "cac", "Consumption");
//			out.println("\t\t\t<cac:Consumption>");
			
			//AllowanceCharge a livello di consumi (salvato a livello di invoice)
	        List<AllowanceCharge> usAllowanceChargeList = readUSAllowanceCharge((Element)consumptionNode, false);
	        if(allowanceChargeList.size() >= 0){
	        	if(invoice.getAllowanceCharges() != null)
	        		invoice.setAllowanceCharges(usAllowanceChargeList);
	        	else
	        		allowanceChargeList.addAll(usAllowanceChargeList);
			}
			
			//Dettaglio IVA: cac:TaxTotal - Salvato a livello di invoice
			Node taxTotalNode = getNode(consumptionNode, "cac", "TaxTotal");
			Double taxAmount = Double.parseDouble(getNumericNodeValue((Element)taxTotalNode, "cbc", "TaxAmount"));
			NodeList taxSubTotalNodesUS = getNodes(taxTotalNode, "cac", "TaxSubtotal");
			
//			out.println("\t\t\t<cac:TaxTotal>");
//			out.println("\t\t\t\t<cbc:TaxAmount>" + taxAmount + "</cbc:TaxAmount>");
			
			if (taxSubTotalNodesUS != null){
				for (int b = 0; b < taxSubTotalNodesUS.getLength(); b++) {
//					out.println("\t\t\t\t<cac:TaxSubtotal>");
					Element taxSubTotal = (Element)taxSubTotalNodesUS.item(b);
					TaxDetail taxDetail = new TaxDetail();
					taxDetail.setTaxAmount(Double.parseDouble(getNumericNodeValue(taxSubTotal, "cbc", "TaxAmount")));
					taxDetail.setTaxAmountCurrency(getNodeValue(taxSubTotal, "cbc", "TaxAmount/@unitCode"));
					
					Node taxCategoryNode = getNode(taxSubTotal, "cac", "TaxCategory");
					taxDetail.setName(getNodeValue((Element)taxCategoryNode, "cbc", "Name"));
					taxDetail.setPerUnitAmount(Double.parseDouble(getNumericNodeValue((Element)taxCategoryNode, "cbc", "PerUnitAmount")));	
					taxDetail.setTierRange(getNodeValue((Element)taxCategoryNode, "cbc", "TierRange"));	
					taxDetailList.add(taxDetail);
					
//					out.println("\t\t\t\t\t<cbc:TaxAmount unitCode=\"" + taxDetail.getTaxAmountCurrency() + "\">" + taxDetail.getTaxAmount() + "</cbc:TaxAmount>");
//					out.println("\t\t\t\t\t<cac:TaxCategory>");
//					out.println("\t\t\t\t\t\t<cbc:Name>" + taxDetail.getName() + "</cbc:Name>");
//					out.println("\t\t\t\t\t\t<cbc:PerUnitAmount>" + taxDetail.getPerUnitAmount() + "</cbc:PerUnitAmount>");
//					out.println("\t\t\t\t\t\t<cbc:TierRange>" + taxDetail.getTierRange() + "</cbc:TierRange>");
//					out.println("\t\t\t\t\t</cac:TaxCategory>");
//					out.println("\t\t\t\t</cac:TaxSubtotal>\n");
				}
			}
			
//			out.println("\t\t\t\t</cac:TaxTotal>");
			
			//ConsumptionReport (mappato in MeterSummary) - Dettaglio consumi
			//Se non sono presenti i nodi ConsumptionHistory salvo il valore totale (TotalConsumedQuantity) in un unico oggetto MeterSummary
			//altrimenti creo tanti MeterSummary quanti sono i nodi ConsumptionHistory e per ognuno di essi salvo la quantita' rilevata
			Node consumptionReportNode = getNode(consumptionNode, "cac", "ConsumptionReport");
			String consumptionReportId = getNodeValue((Element)consumptionReportNode, "cbc", "ID");
			String reportConsumptionType = getNodeValue((Element)consumptionReportNode, "cbc", "ConsumptionType");
			String consumptionReportDescription = getNodeValue((Element)consumptionReportNode, "cbc", "Description");
			Double consumedQuantity;
			String meterNumber = "";
			String consumedQuantityUnitCode = getNodeValue((Element)consumptionReportNode, "cbc", "TotalConsumedQuantity/@unitCode");
			String consumersEnergyLevel = getNodeValue((Element)consumptionReportNode, "cbc", "ConsumersEnergyLevel");
			

//			out.println("\t\t\t\t<cac:ConsumptionReport>");
//			out.println("\t\t\t\t\t<cbc:ID>" + consumptionReportId + "</cbc:ID>");
//			out.println("\t\t\t\t\t<cbc:ConsumptionType>" + reportConsumptionType + "</cbc:ConsumptionType>");
//			out.println("\t\t\t\t\t<cbc:Description>" + consumptionReportDescription + "</cbc:Description>");
//			out.println("\t\t\t\t\t<cbc:ConsumersEnergyLevel>" + consumersEnergyLevel + "</cbc:ConsumersEnergyLevel>");

			NodeList consumptionHistoryNodes = getNodes(consumptionReportNode, "cac", "ConsumptionHistory");
			//Se nella fattura sono presenti uno o piu' nodi ConsumptionHistory
			if(consumptionHistoryNodes != null) {
				for(int y = 0; y < consumptionHistoryNodes.getLength(); y++) {
					
					Node consumptionHistoryNode = consumptionHistoryNodes.item(y);
					
					MeterSummary meterSummary = new MeterSummary();
					meterSummary.setId(consumptionReportId);
					meterSummary.setConsumptionType(reportConsumptionType);
					meterSummary.setMeterSummaryDescription(consumptionReportDescription);
					meterSummary.setMeterSummaryUnitCode(consumedQuantityUnitCode);
					meterSummary.setConsumersEnergyLevel(consumersEnergyLevel);
					
					meterNumber = getNodeValue((Element)consumptionHistoryNode, "cbc", "MeterNumber");
					Double consumptionHistoryQuantity = Double.parseDouble(getNumericNodeValue((Element)consumptionHistoryNode, "cbc", "Quantity"));
					Double consumptionHistoryAmount = Double.parseDouble(getNumericNodeValue((Element)consumptionHistoryNode, "cbc", "Amount"));

					meterSummary.setConsumedQuantity(consumptionHistoryQuantity);
					meterSummary.setAmount(consumptionHistoryAmount);

//					out.println("\t\t\t\t\t<cac:ConsumptionHistory>");
//					out.println("\t\t\t\t\t\t<cbc:MeterNumber>" + meterNumber + "</cbc:MeterNumber>");
//					out.println("\t\t\t\t\t\t<cbc:Quantity>" + consumptionHistoryQuantity + "</cbc:Quantity>");
//					out.println("\t\t\t\t\t\t<cbc:Amount>" + consumptionHistoryAmount + "</cbc:Amount>");

					Node consumptionHistoryPeriodNode = getNode(consumptionReportNode, "cac", "Period");
//					out.println("\t\t\t\t\t\t<cac:Period>");
					
					//Data inizio consumo
					try {
						String startDateC = getNodeValue((Element)consumptionHistoryPeriodNode, "cbc", "StartDate");
						if(startDateC != null){
//							out.println("\t\t\t\t\t\t\t<cbc:StartDate>" + startDateC + "</cbc:StartDate>");
							Date startConsumptionDate = formatterDate.parse(startDateC);
							meterSummary.setStartDate(startConsumptionDate);
						}

					} catch (Exception e) { 
						logger.error("Date exception " + e.getMessage());
					}

					//Data fine consumo
					try {
						String endDateC = getNodeValue((Element)consumptionHistoryPeriodNode, "cbc", "EndDate");
						if(endDateC != null){
//							out.println("\t\t\t\t\t\t\t<cbc:EndDate>" + endDateC + "</cbc:EndDate>");
							Date endConsumptionDate = formatterDate.parse(endDateC);
							meterSummary.setEndDate(endConsumptionDate);
						}

					} catch (Exception e) { 
						logger.error("Date exception " + e.getMessage());
					}
//					out.println("\t\t\t\t\t\t</cac:Period>");
//					out.println("\t\t\t\t\t</cac:ConsumptionHistory>");
					
					//Setto il riferimento a Meter per MeterSummary - se il meter number di ConsumptionHistory e' vuoto 
					//non viene assegnato nessun Meter, anche se nella fattura sono presenti piu' contatori
					if(meterNumber.equals(""))
					{
						if(meterList.size() > 0)
							logger.error("Meter Number is null: no Meter is assigned to MeterSummary " +
								"even if meterList is not empty.");
						else
							logger.error("Meter Number is null: no Meter is assigned to MeterSummary.");
					}
					else {
						if(meterList.size() > 0) {
							for(int q = 0; q < meterList.size(); q++) {
								Meter currentMeter = meterList.get(q);
								if(meterNumber.equals(currentMeter.getMeterNumber()))
									meterSummary.setMeter(currentMeter);
							}
						}
						else 
							logger.error("MeterList is empty: no Meter is assigned to MeterSummary.");
					}
				}
			} 
			//Se nella fattura non sono presenti nodi ConsumptionHistory
			else {
				
				MeterSummary meterSummary = new MeterSummary();
				meterSummary.setId(consumptionReportId);
				meterSummary.setConsumptionType(reportConsumptionType);
				meterSummary.setMeterSummaryDescription(consumptionReportDescription);
				meterSummary.setMeterSummaryUnitCode(consumedQuantityUnitCode);
				meterSummary.setConsumersEnergyLevel(consumersEnergyLevel);
				
				consumedQuantity = Double.parseDouble(getNumericNodeValue((Element)consumptionReportNode, "cbc", "TotalConsumedQuantity"));
				meterSummary.setConsumedQuantity(consumedQuantity);
//				out.println("\t\t\t\t\t<cbc:TotalConsumedQuantity unitCode=" + consumedQuantityUnitCode + ">" + consumedQuantity + "</cbc:TotalConsumedQuantity>");
				Node consumptionReportPeriodNode = getNode(consumptionReportNode, "cac", "Period");
//				out.println("\t\t\t\t\t<cac:Period>");
				//Data inizio consumo
				try {
					String startDateC = getNodeValue((Element)consumptionReportPeriodNode, "cbc", "StartDate");
					if(startDateC != null){
//						out.println("\t\t\t\t\t\t<cbc:StartDate>" + startDateC + "</cbc:StartDate>");
						Date startConsumptionDate = formatterDate.parse(startDateC);
						meterSummary.setStartDate(startConsumptionDate);
					}
					
				} catch (Exception e) { 
					logger.error("Date exception " + e.getMessage());
				}
				
				//Data fine consumo
				try {
					String endDateC = getNodeValue((Element)consumptionReportPeriodNode, "cbc", "EndDate");
					if(endDateC != null){
//						out.println("\t\t\t\t\t\t<cbc:EndDate>" + endDateC + "</cbc:EndDate>");
						Date endConsumptionDate = formatterDate.parse(endDateC);
						meterSummary.setEndDate(endConsumptionDate);
					}
					
				} catch (Exception e) { 
					logger.error("Date exception " + e.getMessage());
				}
//				out.println("\t\t\t\t\t</cac:Period>");
			}
			
//			out.println("\t\t\t\t</cac:ConsumptionReport>");
//			out.println("\t\t\t</cac:Consumption>");
//			
//			//Contratto - Documento di riferimento (0,n)
//			out.println("\t<!-- Contratto. E possibile specificare piu contratti, il riferimento viene riportato in UtilityItem -->");
//			out.println("\t\t<cac:Contract>");
			
			Node contractNode = getNode(supplierConsumptionNode, "cac", "Contract");
			
			String documentId = getNodeValue((Element)contractNode, "cbc", "ID");
			String documentType = getNodeValue((Element)contractNode, "cbc", "ContractType");
			String documentDescription = getNodeValue((Element)contractNode, "cbc", "Description"); 
			
//			out.println("\t\t\t<cbc:ID>" + documentId + "</cbc:ID>");
//			out.println("\t\t\t<cbc:ContractType>" + documentType + "</cbc:ContractType>");
//			out.println("\t\t\t<cbc:Description>" + documentDescription + "</cbc:Description>");
			
			PreviousDocument pd = new PreviousDocument(documentType, documentId);
			pd.setSupplier(invoice.getSupplier());
			pd.setDocumentInfo(documentDescription);
			previousDocumentList.add(pd);
			
//			out.println("\t\t</cac:Contract>\n");
			
			//Consumption Lines - mappate in InvoiceLines, ad ognuno di esse viene associato un riferimento ad un oggetto Consumption
			NodeList consumptionLineNodes = getNodes(supplierConsumptionNode, "cac", "ConsumptionLine");
			
			for (int z = 0; z < consumptionLineNodes.getLength(); z++) {					
				InvoiceLine invoiceLineConsumption = new InvoiceLine();
				Consumption consumption = new Consumption();
				
				Node consumptionLine = consumptionLineNodes.item(z);				
				
//				out.println("\t\t<cac:ConsumptionLine>");
				
				Node periodNode = getNode(consumptionLine, "cac", "Period");
				
//				out.println("\t\t<cac:Period>");
				
				//Data inizio consumo
				try {
					String startDateC = getNodeValue((Element)periodNode, "cbc", "StartDate");
					if(startDateC != null){
//						out.println("\t\t\t<cbc:StartDate>" + startDateC + "</cbc:StartDate>");
						Date startConsumptionDate = formatterDate.parse(startDateC);
						consumption.setStartConsumptionDate(startConsumptionDate);
					}
					
				} catch (Exception e) { 
					logger.error("Date exception " + e.getMessage());
				}
				
				//Data fine consumo
				try {
					String endDateC = getNodeValue((Element)periodNode, "cbc", "EndDate");
					if(endDateC != null){
//						out.println("\t\t\t<cbc:EndDate>" + endDateC + "</cbc:EndDate>");
						Date endConsumptionDate = formatterDate.parse(endDateC);
						consumption.setEndConsumptionDate(endConsumptionDate);
					}
					
				} catch (Exception e) { 
					logger.error("Date exception " + e.getMessage());
				}
				
				String consumptionDescriptionCode = getNodeValue((Element)periodNode, "cbc", "DescriptionCode");
				consumption.setDescriptionCode(consumptionDescriptionCode);
//				out.println("\t\t\t<cbc:DescriptionCode>" + consumptionDescriptionCode + "</cbc:DescriptionCode>");
//				out.println("\t\t</cac:Period>\n");
				
				String consumptionBracket = null;
				consumption.setConsumptionBracket(consumptionBracket);
				
			
				invoiceLineConsumption.setLineNumber(Integer.parseInt(getNumericNodeValue((Element) consumptionLine, "cbc", "ID")));
				invoiceLineConsumption.setUnitCode(getNodeValue((Element) consumptionLine, "cbc", "InvoicedQuantity/@unitCode"));
				invoiceLineConsumption.setInvoicedQuantity(Double.parseDouble(getNumericNodeValue((Element) consumptionLine, "cbc", "InvoicedQuantity")));
				invoiceLineConsumption.setLineExtensionAmount(Double.parseDouble(getNumericNodeValue((Element) consumptionLine, "cbc", "LineExtensionAmount")));
				invoiceLineConsumption.setCurrencyID(getNodeValue((Element) consumptionLine, "cbc", "LineExtensionAmount/@currencyID"));				

//				out.println("\t\t<cbc:ID>" + invoiceLineConsumption.getLineNumber() + "</cbc:ID>");
//				out.println("\t\t<cbc:InvoicedQuantity unitCode=\"" + invoiceLineConsumption.getUnitCode() + "\">" + invoiceLineConsumption.getInvoicedQuantity() + "</cbc:InvoicedQuantity>");
//				out.println("\t\t<cbc:LineExtensionAmount currencyID=\"" + invoiceLineConsumption.getCurrencyID() + "\">" + invoiceLineConsumption.getLineExtensionAmount() + "</cbc:LineExtensionAmount>");
				
	    		//AllowanceCharge (line level)
	            List<AllowanceCharge> lineAllowanceChargeList = readAllowanceCharge((Element) consumptionLine, false);
	    		if(lineAllowanceChargeList.size() >= 0){
	    			invoiceLineConsumption.setAllowanceCharges(lineAllowanceChargeList);
	    		}
	    	        	       		
	    	    Node priceNode = getNode(consumptionLine, "cac", "Price");
	    	    invoiceLineConsumption.setPriceAmount(Double.parseDouble(getNumericNodeValue((Element)priceNode, "cbc", "PriceAmount")));
	    	    invoiceLineConsumption.setPriceAmountCurrencyId(getNodeValue((Element)priceNode, "cbc", "PriceAmount/@currencyID"));
	    	    invoiceLineConsumption.setBaseQuantity(Double.parseDouble(getNumericNodeValue((Element)priceNode, "cbc", "BaseQuantity")));
	    	    invoiceLineConsumption.setBaseQuantityUnitCode(getNodeValue((Element)priceNode, "cbc", "BaseQuantity/@unitCode"));
	    	    
//	    	    out.println("\t\t<!-- Costo calcolato su base annua -->");
//	    	    out.println("\t\t<cac:Price>");
//	    	    out.println("\t\t\t<cbc:PriceAmount currencyID=\"" + invoiceLineConsumption.getPriceAmountCurrencyId() + "\">" + invoiceLineConsumption.getPriceAmount() + "</cbc:PriceAmount>");
//	    	    out.println("\t\t\t<cbc:BaseQuantity unitCode=\"" + invoiceLineConsumption.getBaseQuantityUnitCode() + "\">" + invoiceLineConsumption.getBaseQuantity() + "</cbc:BaseQuantity>");
//	    	    out.println("\t\t</cac:Price>\n");
	    	    
	    	    //AllowanceCharge (price)
	    		List<AllowanceCharge> linePriceAllowanceChargeList = readAllowanceCharge((Element)priceNode, true);
	    		if(linePriceAllowanceChargeList.size() >= 0){
	    			lineAllowanceChargeList.addAll(linePriceAllowanceChargeList);
	    		}
	    	    
	    		//Totale tasse: cac:TaxTotal
	    		Node invoiceLineTaxTotalNode = getNode(consumptionLine, "cac", "TaxTotal");
	    		//Suppongo di avere solo un subTotal per prendere il taxCategory
	    		Node taxSubtotaleNode = getNode(invoiceLineTaxTotalNode, "cac", "TaxSubtotal");
	    		Node taxCategoryNode = getNode(taxSubtotaleNode, "cac", "TaxCategory");
	    		TaxDetail invocieLineTaxDetail = new TaxDetail();
	       		invocieLineTaxDetail.setTaxAmount(Double.parseDouble(getNumericNodeValue((Element)invoiceLineTaxTotalNode, "cbc", "TaxAmount")));
	       		invocieLineTaxDetail.setTaxAmountCurrency(getNodeValue((Element)invoiceLineTaxTotalNode, "cbc", "TaxAmount/@currencyID"));
	       		invocieLineTaxDetail.setTaxableAmount(Double.parseDouble(getNumericNodeValue((Element) consumptionLine, "cbc", "LineExtensionAmount")));
	       		invocieLineTaxDetail.setDescription(getNodeValue((Element)taxCategoryNode, "cbc", "ID"));
	       		invoiceLineConsumption.setTaxDetail(invocieLineTaxDetail);
	       		
//	       		out.println("\t\t<cbc:LineExtensionAmount>" + invocieLineTaxDetail.getTaxableAmount() + "</cbc:LineExtensionAmount>");
//	       		out.println("\t\t<cac:TaxTotal>");
//	       		out.println("\t\t\t<cbc:TaxAmount currencyID=\"" + invocieLineTaxDetail.getTaxAmountCurrency() + "\">" + invocieLineTaxDetail.getTaxAmount() + "</cbc:TaxAmount>");
//	       		out.println("\t\t\t<cac:TaxSubtotal>");
//	       		out.println("\t\t\t\t<cac:TaxCategory>");
//	       		out.println("\t\t\t\t\t<cbc:ID>" + invocieLineTaxDetail.getDescription() + "</cbc:ID>");
//	       		out.println("\t\t\t\t<cac:TaxCategory>");
//	       		out.println("\t\t\t<c/ac:TaxSubtotal>");
//	       		out.println("\t\t</cac:TaxTotal>\n");
	       		
	    		//Note
	       		NodeList notes = getNodes(consumptionLine, "cbc", "Note");
	    		InvoiceLineNote invoiceLineNote = new InvoiceLineNote();
	    		List<InvoiceLineNote> invoiceLineNoteList = new ArrayList<InvoiceLineNote>();
	    		
//	    		out.print("\t\t<cbc:Note>");
	    		
	    		for (int w = 0; w < notes.getLength(); w++) {
	    			Element note = (Element)notes.item(w);
	    			String lineNote = getNodeValue(note, null, null);
	    			if(lineNote != null){
	    				
//	    				out.print(lineNote);
	    				
	    				invoiceLineNote.setNote(lineNote);
	    				invoiceLineNoteList.add(invoiceLineNote);
	    			}
	    		}
	    		if(invoiceLineNoteList.size() >= 0){
	    			invoiceLineConsumption.setNoteList(invoiceLineNoteList);
	    		}
//	    		out.println("\t\t</cbc:Note>");
	    		//ParentDocumentLine (riferimento alla riga della fattura)
	    		Integer parentDocumentLineReference = Integer.parseInt(getNumericNodeValue((Element)consumptionLine, "cbc", "ParentDocumentLineReferenceID"));
	    		parentDocumentLineReferenceList.add(parentDocumentLineReference);
	    		
//	    		out.println("\t\t<cbc:ParentDocumentLineReferenceID>" + parentDocumentLineReference + "</cbc:ParentDocumentLineReferenceID>");
	    		    		
	    		//Item
	    		Node itemNode = getNode(consumptionLine, "cac", "UtilityItem");
	    		invoiceLineConsumption.setItemDescription(getNodeValue((Element)itemNode, "cbc", "Description"));
	       		//Dettaglio tasse
	       		Node classifiedTaxCategoryNode = getNode(itemNode, "cac", "ClassifiedTaxCategory");
	    		invocieLineTaxDetail.setTaxPercent(Double.parseDouble(getNumericNodeValue((Element)classifiedTaxCategoryNode, "cbc", "Percent")));
	    		invoiceLineConsumption.setTaxDetail(invocieLineTaxDetail);       		
	       		
	    		//Tipologia di consumo - itemName
	    		String consumptionType = getNodeValue((Element)itemNode, "cbc", "ConsumptionType");
	    		consumptionType = SpecialCharacter.xmlEntityToString(consumptionType);
	    		consumption.setConsumptionType(consumptionType);
	    		invoiceLineConsumption.setItemName(consumptionType);
	    		  
				invoiceLineConsumption.setConsumption(consumption);
				
				//Nel caso di bollette Hera di tipo ambiente creo un contatore fittizio
				if(ambiente == true) {
					Meter meter = new Meter();
					meter.setUtilityConsumptionPoint(utilityConsumptionPoint);
					meter.setMeterAddress(utilityConsumptionPointAddress);
					meter.setSector(sector);
					
					for(PreviousDocument previousDocument : previousDocumentList) {
						SupplyInfo supplyInfo = new SupplyInfo();
						supplyInfo.setPreviousDocument(previousDocument);
						
						supplyInfoList.add(supplyInfo);
					}
					meter.setSupplyInfo(supplyInfoList);
					
					meterList.add(meter);
					
					invoiceLineConsumption.setMeter(meter);
				}
				//TODO Attenzione: si presume che l'UtilityStatement abbia un solo contatore...
				else if(meterList.size() > 0)
					invoiceLineConsumption.setMeter(meterList.get(0));
				
				//logger.info("Consumo");
				//logger.info("Invoice line num. " + invoiceLine + ", Invoiced quantity " + invoicedQuantity + ", Extension amount " + lineExtensionAmount + ", Item description " + itemDescription);				        			
				invoice.addInvoiceLine(invoiceLineConsumption);
				invoice.localAddInvoiceLine(invoiceLineConsumption);
				
//				out.println("\t\t\t<cac:UtilityItem>");
//				out.println("\t\t\t\t<cbc:Description>" + invoiceLineConsumption.getItemDescription() + "</cbc:Description>");
//				out.println("\t\t\t\t<cac:ClassifiedTaxCategory>");
//				out.println("\t\t\t\t\t<cbc:Percent>" + invocieLineTaxDetail.getTaxPercent() + "</cbc:Percent>");
//				out.println("\t\t\t\t</cac:ClassifiedTaxCategory>");
//				out.println("\t\t\t\t<cbc:ConsumptionType>" + consumptionType + "</cbc:ConsumptionType>");
//				out.println("\t\t\t</cac:UtilityItem>");
//				out.println("\t\t</cac:ConsumptionLine>\n");
			}
			
//			out.println("\t</cac:SupplierConsumption>");
		}
		
//		out.println("\t<cac:SubscriberConsumption>\n");
		
		return meterList;
	}

	//Helper method
//	public static List<AllowanceCharge> readUSAllowanceCharge(Element element, boolean forInformationOnly, PrintWriter out) throws XPathExpressionException{
	public static List<AllowanceCharge> readUSAllowanceCharge(Element element, boolean forInformationOnly) throws XPathExpressionException{
		NodeList allowanceChargeNodes = getNodes(element, "cac", "AllowanceCharge");
		List<AllowanceCharge> allowanceChargeList = new ArrayList<AllowanceCharge>();
		
//		out.println("\t<cac:AllowanceCharge>");

		for (int z = 0; z < allowanceChargeNodes.getLength(); z++) {
			Element lineAllowanceChargeNode = (Element)allowanceChargeNodes.item(z);
			AllowanceCharge allowanceCharge = new AllowanceCharge();

			String chargeIndicator = getNumericNodeValue(lineAllowanceChargeNode, "cbc", "ChargeIndicator");
			double lineeAllowanceChargeAmount = Double.parseDouble(getNumericNodeValue(lineAllowanceChargeNode, "cbc", "Amount"));

			if("false".equalsIgnoreCase(chargeIndicator)){
				lineeAllowanceChargeAmount = lineeAllowanceChargeAmount * (-1);
			}
			allowanceCharge.setAmount(lineeAllowanceChargeAmount);
			allowanceCharge.setAllowanceChargeReason(getNodeValue(lineAllowanceChargeNode, "cbc", "AllowanceChargeReason"));
			allowanceCharge.setMultiplierFactorNumeric(Double.parseDouble(getNumericNodeValue(lineAllowanceChargeNode, "cbc", "MultiplierFactorNumeric")));
			allowanceCharge.setBaseAmount(Double.parseDouble(getNumericNodeValue(lineAllowanceChargeNode, "cbc", "BaseAmount")));
			allowanceCharge.setForInformationOnly(forInformationOnly);

//			out.println("\t\t<cbc:ChargeIndicator>" + chargeIndicator + "</cbc:ChargeIndicator>");
//			out.println("\t\t<cbc:AllowanceChargeReason>" + allowanceCharge.getAllowanceChargeReason() + "</cbc:AllowanceChargeReason>");
//			out.println("\t\t<cbc:Amount>" + lineeAllowanceChargeAmount + "</cbc:Amount>");
//			out.println("\t\t<cbc:MultiplierFactorNumeric>" + allowanceCharge.getMultiplierFactorNumeric() + "</cbc:MultiplierFactorNumeric>");
//			out.println("\t\t<cbc:BaseAmount>" + allowanceCharge.getBaseAmount() + "</cbc:BaseAmount>");

			//TO DO: sistemare la questione Tax Category
			
			allowanceChargeList.add(allowanceCharge);
		}

//		out.println("\t</cac:AllowanceCharge>");

		return allowanceChargeList;

	}
	
}
