/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.common.utils;

import it.fe.core.par.common.FEFile;
import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.AllowanceCharge;
import it.fe.core.par.invoice.Client;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.Invoice.DocumentType;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceLineNote;
import it.fe.core.par.invoice.InvoiceNote;
import it.fe.core.par.invoice.InvoicePeriod;
import it.fe.core.par.invoice.PaymentDetail;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.Supplier;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.utility.Consumption;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;
import it.fe.core.par.utility.Sector;
import it.fe.core.par.utility.SupplyInfo;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.log4j.Logger;
import org.apache.xml.utils.PrefixResolver;
import org.apache.xml.utils.PrefixResolverDefault;
import org.apache.xpath.NodeSet;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Classe per la conversione della fattura UBL nell'oggetto Invoice
 * 
 * @see it.fe.core.par.invoice.Invoice Invoice class
 * 
 * 
 */
public class InvoiceXmlToJava {

	protected static Logger logger = Logger.getLogger(InvoiceXmlToJava.class
			.getName());

	protected static XPathFactory factory = XPathFactory.newInstance();
	protected static XPath xpath = factory.newXPath();

	// protected static String root = ":Invoice";
	protected static Element rootNode;

	/**
	 * <p>
	 * Converte la fattura XML nell'oggetto Invoice
	 * </p>
	 * 
	 * @param (String) File XML contenente la fattura in standard UBL<br />
	 * 
	 * @return (Invoice)
	 * @throws XPathExpressionException
	 * @throws IOException
	 * @throws SAXException
	 * @throws ParserConfigurationException
	 * 
	 */
	public static Invoice xmlToJava(String xml) throws XPathExpressionException {

		// Documento XML
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		DocumentBuilder builder;
		Document doc = null;

		domFactory.setNamespaceAware(true);

		try {
			builder = domFactory.newDocumentBuilder();
			doc = builder.parse(new java.io.ByteArrayInputStream(xml
					.getBytes("UTF-8")));
			return xmlToJava(doc);
			// doc = builder.parse(xml);
		} catch (SAXParseException spe) {
			logger.error("Parse error: " + spe.getMessage());
		} catch (SAXException se) {
			logger.error("SAXException: " + se.getMessage());
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		return null;
	}

	public static Invoice xmlToJava(File file) throws XPathExpressionException {

		// Documento XML
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		DocumentBuilder builder;
		Document doc = null;

		domFactory.setNamespaceAware(true);

		try {
			builder = domFactory.newDocumentBuilder();
			doc = builder.parse(file);
			return xmlToJava(doc);
			// doc = builder.parse(xml);
		} catch (SAXParseException spe) {
			logger.error("Parse error: " + spe.getMessage());
		} catch (SAXException se) {
			logger.error("SAXException: " + se.getMessage());
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		return null;
	}

	public static Invoice xmlToJava(Document doc)
			throws XPathExpressionException {
		Invoice invoice = new Invoice();

		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
		final PrefixResolver resolver = new PrefixResolverDefault(
				doc.getDocumentElement());
		NamespaceContext ctx = new NamespaceContext() {
			public String getNamespaceURI(String prefix) {
				return resolver.getNamespaceForPrefix(prefix);
			}

			public String getPrefix(String arg0) {
				return null;
			}

			public Iterator getPrefixes(String arg0) {
				return null;
			}
		};

		String invoiceLineTag = "";
		String invoicedQuantityTag = "";
		xpath.setNamespaceContext(ctx);
		rootNode = doc.getDocumentElement();
		if (rootNode.getNodeName().contains(Invoice.DocumentType.Invoice)){
			invoice.setDocumentType(Invoice.DocumentType.Invoice);
			invoiceLineTag = "InvoiceLine";
			invoicedQuantityTag ="InvoicedQuantity";
		}
		else if (rootNode.getNodeName().contains(
				Invoice.DocumentType.CreditNote)){
			invoice.setDocumentType(Invoice.DocumentType.CreditNote);
			invoiceLineTag = "CreditNoteLine";
			invoicedQuantityTag ="CreditedQuantity";
		}
		else if (rootNode.getNodeName()
				.contains(Invoice.DocumentType.DebitNote)){
			invoice.setDocumentType(Invoice.DocumentType.DebitNote);
			invoiceLineTag = "DebitNoteLine";
			invoicedQuantityTag ="DebitedQuantity";
		}
		else
			throw new XPathExpressionException(
					"Unrecognized document: should be one of Invoice, CreditNote, DebitNote");

	
		
		/** INVOICE **/

		// Numero fattura - cbc:ID
		String invoiceNumber = getNodeValue(rootNode, "cbc", "ID");
		invoiceNumber = SpecialCharacter.xmlEntityToString(invoiceNumber);
		invoice.setInvoiceNumber(invoiceNumber);
		logger.info("Invoice Number: " + invoiceNumber);

		// Data emissione - cbc:IssueDate
		try {
			String stringIssueDate = getNodeValue(rootNode, "cbc", "IssueDate");
			if (stringIssueDate != null) {
				Date issueDate = dateFormat.parse(stringIssueDate);
				invoice.setIssueDate(issueDate);
				logger.info("Issue date: " + issueDate);
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		// Conto Contrattuale
		// TODO: come lo mappo? Come Additional Data o come campo?
		String contoContrattuale = null;
		try {
			contoContrattuale = getNodeValue(rootNode, "cbc",
					"ContoContrattuale");
			if (contoContrattuale != null) {
				logger.info("Conto Contrattuale: " + contoContrattuale);
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		// Fornitore:
		// - cac:PartyName = ragione sociale
		// - cac:PartyIdentification = partita iva
		// - cac:PostalAddress = indirizzo + cap + citta
		Node supplierInfoNode = getNode(rootNode, "cac",
				"AccountingSupplierParty");
		Node supplierPartyNode = getNode(supplierInfoNode, "cac", "Party");
		Node supplierPartyNameNode = getNode(supplierPartyNode, "cac",
				"PartyName");
		Node supplierPartyLegalEntityNode = getNode(supplierPartyNode, "cac",
				"PartyLegalEntity");

		// EndPoint
		String supplierEndPointID = getNodeValue(
				(Element) supplierPartyNameNode, "cbc", "EndpointID");
		supplierEndPointID = SpecialCharacter
				.xmlEntityToString(supplierEndPointID);

		// SupplierName
		String supplierName = getNodeValue((Element) supplierPartyNameNode,
				"cbc", "Name");
		supplierName = SpecialCharacter.xmlEntityToString(supplierName);
		logger.info("Supplier Short Name: " + supplierName);

		// Address
		Node supplierAddressNode = getNode(supplierPartyNode, "cac",
				"PostalAddress");
		PostalAddress supplierPostalAddress = readAddress((Element) supplierAddressNode);

		// RegistrationAddress
		Node supplierRegistrationAddressNode = getNode(
				supplierPartyLegalEntityNode, "cac", "RegistrationAddress");
		PostalAddress supplierRegistrationAddress = readAddress((Element) supplierRegistrationAddressNode);

		// RegistrationName
		String supplierRegistrationName = getNodeValue(
				(Element) supplierPartyLegalEntityNode, "cbc",
				"RegistrationName");
		supplierRegistrationName = SpecialCharacter
				.xmlEntityToString(supplierRegistrationName);
		if ((supplierName == null) || (supplierName.equals(""))) {
			supplierName = supplierRegistrationName;
		}

		// VatID
		// In UBL la p.iva può essere indicata in vari punti di
		// AccountingSupplierParty:
		// 1) in PartyLegalEntityNode/CompanyID
		String supplierVatId = getNodeValue(
				(Element) supplierPartyLegalEntityNode, "cbc", "CompanyID");

		// 2) in PartyTaxScheme/CompanyID (in questo caso deve essere
		// specificato lo schemeIDType come VAT)
		if ((supplierVatId == null) || (supplierVatId.equals(""))) {
			NodeList supplierPartyTaxSchemeNodes = getNodes(supplierPartyNode,
					"cac", "PartyTaxScheme");
			if (supplierPartyTaxSchemeNodes != null) {
				for (int i = 0; i < supplierPartyTaxSchemeNodes.getLength(); i++) {
					// schemeIDType specificato come attriduto
					// (CompanyID/@schemeID)
					String schemeIDType = getNodeValue(
							(Element) supplierPartyTaxSchemeNodes.item(i),
							"cbc", "CompanyID/@schemeID");
					if ((schemeIDType != null)
							&& (schemeIDType.endsWith("VAT"))) {
						supplierVatId = getNodeValue(
								(Element) supplierPartyTaxSchemeNodes.item(i),
								"cbc", "CompanyID");
					}
					// schemeIDType specificato come nodo
					// (CompanyID/TaxScheme/ID)
					else {
						Node taxSchemeNode = getNode(
								supplierPartyTaxSchemeNodes.item(i), "cac",
								"TaxScheme");
						if (taxSchemeNode != null) {
							schemeIDType = getNodeValue(
									(Element) taxSchemeNode, "cbc", "ID");
							if ((schemeIDType != null)
									&& (schemeIDType.endsWith("VAT"))) {
								supplierVatId = getNodeValue(
										(Element) supplierPartyTaxSchemeNodes
												.item(i),
										"cbc", "CompanyID");
							}
						}
					}

				}
			}
		}
		supplierVatId = SpecialCharacter.xmlEntityToString(supplierVatId);

		// Codice fiscale
		String supplierFiscalCode = getNodeValue(
				(Element) supplierPartyLegalEntityNode, "cbc", "CompanyID");
		if ((supplierFiscalCode == null) || (supplierFiscalCode.equals(""))) {
			NodeList supplierPartyTaxSchemeNodes = getNodes(supplierPartyNode,
					"cac", "PartyTaxScheme");
			if (supplierPartyTaxSchemeNodes != null) {
				for (int i = 0; i < supplierPartyTaxSchemeNodes.getLength(); i++) {
					String schemeIDType = getNodeValue(
							(Element) supplierPartyTaxSchemeNodes.item(i),
							"cbc", "CompanyID/@schemeID");
					if ((schemeIDType != null) && (schemeIDType.endsWith("CF"))) {
						supplierFiscalCode = getNodeValue(
								(Element) supplierPartyTaxSchemeNodes.item(i),
								"cbc", "CompanyID");
					}

				}
			}
		}
		supplierFiscalCode = SpecialCharacter
				.xmlEntityToString(supplierFiscalCode);

		Supplier supplier = new Supplier();
		supplier.setVatId(supplierVatId);
		supplier.setItalianFiscalCode(supplierFiscalCode);
		supplier.setSupplierName(supplierName);
		supplier.setPostalAddress(supplierPostalAddress);
		supplier.setRegistrationAddress(supplierRegistrationAddress);
		supplier.setEndpointId(supplierEndPointID);
		invoice.setSupplier(supplier);

		// Cliente:
		// - cac:PartyName = ragione sociale
		// - cac:PartyIdentification = partita iva
		// - cac:PostalAddress = indirizzo + cap + citta
		Node clientInfoNode = getNode(rootNode, "cac",
				"AccountingCustomerParty");
		Node clientPartyNode = getNode(clientInfoNode, "cac", "Party");
		Node clientPartyNameNode = getNode(clientPartyNode, "cac", "PartyName");
		Node clientPartyLegalEntityNode = getNode(clientPartyNode, "cac",
				"PartyLegalEntity");

		// SupplierName
		String clientName = getNodeValue((Element) clientPartyNameNode, "cbc",
				"Name");
		clientName = SpecialCharacter.xmlEntityToString(clientName);
		logger.info("Client Short Name: " + clientName);

		// Address
		Node clientAddressNode = getNode(clientPartyNode, "cac",
				"PostalAddress");
		PostalAddress clientPostalAddress = readAddress((Element) clientAddressNode);

		// RegistrationAddress
		Node clientRegistrationAddressNode = getNode(
				clientPartyLegalEntityNode, "cac", "RegistrationAddress");
		PostalAddress clientRegistrationAddress = readAddress((Element) clientRegistrationAddressNode);

		// RegistrationName
		String clientRegistrationName = getNodeValue(
				(Element) clientPartyLegalEntityNode, "cbc", "RegistrationName");
		clientRegistrationName = SpecialCharacter
				.xmlEntityToString(clientRegistrationName);
		if ((clientName == null) || (clientName.equals(""))) {
			clientName = clientRegistrationName;
		}

		// VatID
		String clientVatId = getNodeValue((Element) clientPartyLegalEntityNode,
				"cbc", "CompanyID");
		if ((clientVatId == null) || (clientVatId.equals(""))) {
			NodeList clientPartyTaxSchemeNodes = getNodes(clientPartyNode,
					"cac", "PartyTaxScheme");
			if (clientPartyTaxSchemeNodes != null) {
				for (int i = 0; i < clientPartyTaxSchemeNodes.getLength(); i++) {
					String schemeIDType = getNodeValue(
							(Element) clientPartyTaxSchemeNodes.item(i), "cbc",
							"CompanyID/@schemeID");
					if ((schemeIDType != null)
							&& (schemeIDType.endsWith("VAT"))) {
						clientVatId = getNodeValue(
								(Element) clientPartyTaxSchemeNodes.item(i),
								"cbc", "CompanyID");
					}

					else {
						Node taxSchemeNode = getNode(
								clientPartyTaxSchemeNodes.item(i), "cac",
								"TaxScheme");
						if (taxSchemeNode != null) {
							schemeIDType = getNodeValue(
									(Element) taxSchemeNode, "cbc", "ID");
							if ((schemeIDType != null)
									&& (schemeIDType.endsWith("VAT"))) {
								clientVatId = getNodeValue(
										(Element) clientPartyTaxSchemeNodes
												.item(i),
										"cbc", "CompanyID");
							}
						}
					}

				}
			}
		}
		clientVatId = SpecialCharacter.xmlEntityToString(clientVatId);

		// Codice fiscale
		String clientFiscalCode = getNodeValue(
				(Element) clientPartyLegalEntityNode, "cbc", "CompanyID");
		if ((clientFiscalCode == null) || (clientFiscalCode.equals(""))) {
			NodeList clientPartyTaxSchemeNodes = getNodes(clientPartyNode,
					"cac", "PartyTaxScheme");
			if (clientPartyTaxSchemeNodes != null) {
				for (int i = 0; i < clientPartyTaxSchemeNodes.getLength(); i++) {
					String schemeIDType = getNodeValue(
							(Element) clientPartyTaxSchemeNodes.item(i), "cbc",
							"CompanyID/@schemeID");
					if ((schemeIDType != null) && (schemeIDType.endsWith("CF"))) {
						clientFiscalCode = getNodeValue(
								(Element) clientPartyTaxSchemeNodes.item(i),
								"cbc", "CompanyID");
					}

				}
			}
		}
		clientFiscalCode = SpecialCharacter.xmlEntityToString(clientFiscalCode);

		Client client = new Client();
		client.setVatId(clientVatId);
		client.setItalianFiscalCode(clientFiscalCode);
		client.setClientName(clientName);
		client.setPostalAddress(clientPostalAddress);
		client.setRegistrationAddress(clientRegistrationAddress);
		invoice.setClient(client);

		// Periodo di consumo: cac:InvoicePeriod
		Node invoicePeriodNode = getNode(rootNode, "cac", "InvoicePeriod");
		InvoicePeriod invoicePeriod = new InvoicePeriod();

		// StartDate
		try {
			String stringPeriodStartDate = getNodeValue(
					(Element) invoicePeriodNode, "cbc", "StartDate");
			if (stringPeriodStartDate != null) {
				Date periodStartDate = dateFormat.parse(stringPeriodStartDate);
				invoicePeriod.setStartDate(periodStartDate);
			}

		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		// EndDate
		try {
			String stringPeriodEndDate = getNodeValue(
					(Element) invoicePeriodNode, "cbc", "EndDate");
			if (stringPeriodEndDate != null) {
				Date periodEndDate = dateFormat.parse(stringPeriodEndDate);
				invoicePeriod.setEndDate(periodEndDate);
			}

		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		// DescriptionCode
		String periodCode = getNodeValue((Element) invoicePeriodNode, "cbc",
				"DescriptionCode");
		periodCode = SpecialCharacter.xmlEntityToString(periodCode);
		invoicePeriod.setPeriodCode(periodCode);

		// Description
		String periodDescription = getNodeValue((Element) invoicePeriodNode,
				"cbc", "Description");
		periodDescription = SpecialCharacter
				.xmlEntityToString(periodDescription);
		invoicePeriod.setPeriodDescription(periodDescription);

		invoice.setInvoicePeriod(invoicePeriod);

		// Data di scadenza: cbc:PaymentDueDate
		Node paymentMeansNode = getNode(rootNode, "cac", "PaymentMeans");
		PaymentDetail paymentDetail = new PaymentDetail();

		// DueDate
		try {
			String stringPaymentDueDate = getNodeValue(
					(Element) paymentMeansNode, "cbc", "PaymentDueDate");
			if (stringPaymentDueDate != null) {
				Date paymentDueDate = dateFormat.parse(stringPaymentDueDate);
				paymentDetail.setPaymentDueDate(paymentDueDate);
			}

		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		// Termini di pagamento: cac:PaymentTerms
		Node paymentTermsNode = getNode(rootNode, "cac", "PaymentTerms");
		String paymentTerms = getNodeValue((Element) paymentTermsNode, "cbc",
				"Note");
		paymentTerms = SpecialCharacter.xmlEntityToString(paymentTerms);
		paymentDetail.setPaymentTerms(paymentTerms);

		invoice.setPaymentDetail(paymentDetail);

		// Documenti pregressi
		List<PreviousDocument> previousDocuments = new ArrayList<PreviousDocument>();

		// - Ordine: cac:OrderReference
		Node orderReferenceNode = getNode(rootNode, "cac", "OrderReference");
		String orderCode = getNodeValue((Element) orderReferenceNode, "cbc",
				"ID");
		orderCode = SpecialCharacter.xmlEntityToString(orderCode);
		if ((orderCode != null) && (!orderCode.equals(""))) {
			PreviousDocument order;
			order = new PreviousDocument("ORDINE", orderCode);
			order.setSupplier(supplier);
			previousDocuments.add(order);
		}

		// - Contratto: cac:ContractDocumentReference
		NodeList contractDocumentNodes = getNodes(rootNode, "cac",
				"ContractDocumentReference");
		Node contractDocumentNode = null;
		for (int i = 0; i < contractDocumentNodes.getLength(); i++) {
			Element contractDocument = (Element) contractDocumentNodes.item(i);
			contractDocumentNode = contractDocumentNodes.item(i);

			String previousDocumentCode = getNodeValue(contractDocument, "cbc",
					"ID");
			previousDocumentCode = SpecialCharacter
					.xmlEntityToString(previousDocumentCode);
			String previousDocumentInfo = getNodeValue(contractDocument, "cbc",
					"DocumentType");
			previousDocumentInfo = SpecialCharacter
					.xmlEntityToString(previousDocumentInfo);

			PreviousDocument previousDocument = new PreviousDocument();
			previousDocument = new PreviousDocument("CONTRATTO",
					previousDocumentCode);
			previousDocument.setDocumentInfo(previousDocumentInfo);
			previousDocument.setSupplier(supplier);

			previousDocuments.add(previousDocument);
		}

		// - Documento generico: cac:AdditionalDocumentReference
		NodeList additionalDocumentNodes = getNodes(rootNode, "cac",
				"AdditionalDocumentReference");
		for (int i = 0; i < additionalDocumentNodes.getLength(); i++) {
			Element additionalDocument = (Element) additionalDocumentNodes
					.item(i);

			String previousDocumentCode = getNodeValue(additionalDocument,
					"cbc", "ID");
			previousDocumentCode = SpecialCharacter
					.xmlEntityToString(previousDocumentCode);
			String previousDocumentInfo = getNodeValue(additionalDocument,
					"cbc", "DocumentType");
			previousDocumentInfo = SpecialCharacter
					.xmlEntityToString(previousDocumentInfo);

			Node attachmentNode = getNode(additionalDocument, "cac",
					"Attachment");

			Node externalReferenceNode = getNode(attachmentNode, "cac",
					"ExternalReference");
			String previousDocumentURI = getNodeValue(
					(Element) externalReferenceNode, "cbc", "URI");
			previousDocumentURI = SpecialCharacter
					.xmlEntityToString(previousDocumentURI);

			String embeddedDocumentBinaryObject = getNodeValue(
					(Element) attachmentNode, "cbc",
					"EmbeddedDocumentBinaryObject");
			embeddedDocumentBinaryObject = SpecialCharacter
					.xmlEntityToString(embeddedDocumentBinaryObject);
			String embeddedDocumentMimeCode = getNodeValue(
					(Element) attachmentNode, "cbc",
					"EmbeddedDocumentBinaryObject/@mimeCode");

			PreviousDocument previousDocument = new PreviousDocument();
			previousDocument = new PreviousDocument("DOCUMENTO AGGIUNTIVO",
					previousDocumentCode);
			previousDocument.setDocumentInfo(previousDocumentInfo);
			previousDocument.setDocumentURI(previousDocumentURI);
			previousDocument.setSupplier(supplier);

			if (embeddedDocumentBinaryObject != null) {
				try {
					// String p = new
					// sun.misc.BASE64Encoder().encodeBuffer("pippo".getBytes());
					// byte[] documentDecoded = new
					// sun.misc.BASE64Decoder().decodeBuffer(p);

					byte[] documentDecoded = new sun.misc.BASE64Decoder()
							.decodeBuffer(embeddedDocumentBinaryObject);
					FEFile embeddedDocument = new FEFile(documentDecoded, "");
					embeddedDocument.setFileMimeType(embeddedDocumentMimeCode);
					previousDocument.setEmbeddedDocument(embeddedDocument);

					// FileOutputStream fos;
					// try {
					//
					// fos = new FileOutputStream("prova.txt");
					// fos.write(documentDecoded);
					//
					//
					// fos.close();
					//
					// } catch (FileNotFoundException e) {
					// e.printStackTrace();
					// }

				} catch (IOException e) {
					logger.error(e.getMessage());
				}

			}

			previousDocuments.add(previousDocument);
		}

		invoice.setPreviousDocuments(previousDocuments);

		// Contatore utenze: cac:Delivery
		NodeList deliveryNodes = getNodes(rootNode, "cac", "Delivery");
		List<Meter> meterList = new ArrayList<Meter>();
		for (int i = 0; i < deliveryNodes.getLength(); i++) {
			Element delivery = (Element) deliveryNodes.item(i);
			String meterNumber = getNodeValue(delivery, "cbc", "ID");
			meterNumber = SpecialCharacter.xmlEntityToString(meterNumber);

			String connectionNumber = getNodeValue(delivery, "cbc",
					"PointOfDeliveryID");
			connectionNumber = SpecialCharacter
					.xmlEntityToString(connectionNumber);

			String meterContractCode = getNodeValue(delivery, "cbc",
					"TrackingID");
			meterContractCode = SpecialCharacter
					.xmlEntityToString(meterContractCode);

			PreviousDocument meterContract = new PreviousDocument("CONTRATTO",
					meterContractCode);

			SupplyInfo supplyInfo = new SupplyInfo();
			supplyInfo.setPreviousDocument(meterContract);

			List<SupplyInfo> supplyInfoList = new ArrayList<SupplyInfo>();
			supplyInfoList.add(supplyInfo);

			Node deliveryAddressNode = getNode(delivery, "cac",
					"DeliveryAddress");
			PostalAddress meterAddress = readAddress((Element) deliveryAddressNode);

			// Settore merceologico
			Node documentTypeNode = getNode(contractDocumentNode, "cbc",
					"DocumentType");
			Node contractDocumentIdNode = getNode(contractDocumentNode, "cbc",
					"ID");
			Sector sector = new Sector();
			String sectorName = getNodeValueByExpr(doc, contractDocumentNode,
					documentTypeNode, contractDocumentIdNode, meterContractCode);
			sectorName = SpecialCharacter.xmlEntityToString(sectorName);
			if (sectorName != null) {
				sector.setSectorName(sectorName);
			}

			Meter meter = new Meter();
			meter = new Meter(meterNumber, meterAddress, sector, new Date(),
					null, null);
			meter.setSupplyInfo(supplyInfoList);
			meterList.add(meter);
		}

		// Importo totale: cbc:PayableAmount
		Node legalMonetaryTotalNode = getNode(rootNode, "cac",
				"LegalMonetaryTotal");
		Double allInclusiveTotal = Double.parseDouble(getNumericNodeValue(
				(Element) legalMonetaryTotalNode, "cbc", "PayableAmount"));
		String allInclusiveTotalCurrency = getNodeValue(
				(Element) legalMonetaryTotalNode, "cbc",
				"PayableAmount/@currencyID");
		invoice.setAllInclusiveTotal(allInclusiveTotal);
		invoice.setAllInclusiveTotalCurrency(allInclusiveTotalCurrency);

		// AllowanceCharge (invoice level)
		List<AllowanceCharge> allowanceChargeList = readAllowanceCharge(
				rootNode, false);
		if (allowanceChargeList.size() >= 0) {
			invoice.setAllowanceCharges(allowanceChargeList);
		}

		// Data tasse: cbc:TaxPointDate
		try {
			String stringTaxPointDate = getNodeValue(rootNode, "cbc",
					"TaxPointDate");
			if (stringTaxPointDate != null) {
				Date taxPointDate = dateFormat.parse(stringTaxPointDate);
				invoice.setTaxPointDate(taxPointDate);
			}

		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		// Dettaglio IVA: cac:TaxTotal
		Node taxTotalNode = getNode(rootNode, "cac", "TaxTotal");
		Double roundingAmount = Double.parseDouble(getNumericNodeValue(
				(Element) taxTotalNode, "cbc", "RoundingAmount"));
		NodeList taxSubTotalNodes = getNodes(taxTotalNode, "cac", "TaxSubtotal");
		List<TaxDetail> taxDetailList = new ArrayList<TaxDetail>();
		if (taxSubTotalNodes != null) {
			for (int i = 0; i < taxSubTotalNodes.getLength(); i++) {
				Element taxSubTotal = (Element) taxSubTotalNodes.item(i);
				TaxDetail taxDetail = new TaxDetail();
				taxDetail.setTaxableAmount(Double
						.parseDouble(getNumericNodeValue(taxSubTotal, "cbc",
								"TaxableAmount")));
				taxDetail.setTaxAmount(Double.parseDouble(getNumericNodeValue(
						taxSubTotal, "cbc", "TaxAmount")));
				taxDetail.setTaxAmountCurrency(getNodeValue(taxSubTotal, "cbc",
						"TaxAmount/@currencyID"));
				taxDetail.setRoundingAmount(Double
						.parseDouble(getNumericNodeValue(taxSubTotal, "cbc",
								"RoundingAmount")));// TODO verificare

				Node taxCategoryNode = getNode(taxSubTotal, "cac",
						"TaxCategory");
				taxDetail.setTaxPercent(Double.parseDouble(getNumericNodeValue(
						(Element) taxCategoryNode, "cbc", "Percent")));
				taxDetail.setDescription(getNodeValue(
						(Element) taxCategoryNode, "cbc", "ID"));
				taxDetailList.add(taxDetail);
			}
		}
		invoice.setTaxDetails(taxDetailList);

		// Accounting cost: cbc:AccountingCost
		String accountingCost = getNodeValue(rootNode, "cbc", "AccountingCost");
		invoice.setAccountingCost(accountingCost);

		// Note fattura
		NodeList noteNodes = getNodes(rootNode, "cbc", "Note");
		List<InvoiceNote> noteList = new ArrayList<InvoiceNote>();
		for (int i = 0; i < noteNodes.getLength(); i++) {
			Element note = (Element) noteNodes.item(i);
			InvoiceNote noteTxt = new InvoiceNote();
			noteTxt.setNote(getNodeValue(note, null, null));

			noteList.add(noteTxt);
		}
		invoice.setNoteList(noteList);

		// Letture
		List<Reading> readingList = new ArrayList<Reading>();

		// Riassunto contatore
		List<MeterSummary> meterSummaryList = new ArrayList<MeterSummary>();
	
		// Righe
		NodeList invoiceLines = getNodes(rootNode, "cac", invoiceLineTag);
		for (int i = 0; i < invoiceLines.getLength(); i++) {

			Element singleLine = (Element) invoiceLines.item(i);
			singleLine.getParentNode().removeChild(singleLine);

			InvoiceLine invoiceLine = new InvoiceLine();

			invoiceLine.setLineNumber(Integer.parseInt(getNumericNodeValue(
					singleLine, "cbc", "ID")));
			invoiceLine.setUnitCode(getNodeValue(singleLine, "cbc",
					invoicedQuantityTag+"/@unitCode"));
			invoiceLine.setInvoicedQuantity(Double
					.parseDouble(getNumericNodeValue(singleLine, "cbc",
							invoicedQuantityTag)));
			invoiceLine.setLineExtensionAmount(Double
					.parseDouble(getNumericNodeValue(singleLine, "cbc",
							"LineExtensionAmount")));
			invoiceLine.setCurrencyID(getNodeValue(singleLine, "cbc",
					"LineExtensionAmount/@currencyID"));

			// Setto il contoContrattuale per ogni riga
			invoiceLine.setContoContrattuale(contoContrattuale);

			// AllowanceCharge (line level)
			List<AllowanceCharge> lineAllowanceChargeList = readAllowanceCharge(
					(Element) singleLine, false);
			if (lineAllowanceChargeList.size() >= 0) {
				invoiceLine.setAllowanceCharges(lineAllowanceChargeList);
			}

			Node priceNode = getNode(singleLine, "cac", "Price");
			invoiceLine.setPriceAmount(Double.parseDouble(getNumericNodeValue(
					(Element) priceNode, "cbc", "PriceAmount")));

			// AllowanceCharge (price)
			List<AllowanceCharge> linePriceAllowanceChargeList = readAllowanceCharge(
					(Element) priceNode, true);
			if (linePriceAllowanceChargeList.size() >= 0) {
				lineAllowanceChargeList.addAll(linePriceAllowanceChargeList);
			}

			// Totale tasse: cac:TaxTotal
			Node invoiceLineTaxTotalNode = getNode(singleLine, "cac",
					"TaxTotal");
			// Suppongo di avere solo un subTotal per prendere il taxCategory
			Node taxSubtotaleNode = getNode(invoiceLineTaxTotalNode, "cac",
					"TaxSubtotal");
			Node taxCategoryNode = getNode(taxSubtotaleNode, "cac",
					"TaxCategory");
			TaxDetail invocieLineTaxDetail = new TaxDetail();
			invocieLineTaxDetail.setTaxAmount(Double
					.parseDouble(getNumericNodeValue(
							(Element) invoiceLineTaxTotalNode, "cbc",
							"TaxAmount")));
			invocieLineTaxDetail.setTaxableAmount(Double
					.parseDouble(getNumericNodeValue(singleLine, "cbc",
							"LineExtensionAmount")));
			invocieLineTaxDetail.setDescription(getNodeValue(
					(Element) taxCategoryNode, "cbc", "ID"));
			invocieLineTaxDetail.setRoundingAmount(Double
					.parseDouble(getNumericNodeValue(singleLine, "cbc",
							"RoundingAmount")));// TODO verificare

			invoiceLine.setTaxDetail(invocieLineTaxDetail);

			// Note
			NodeList notes = getNodes(singleLine, "cbc", "Note");
			InvoiceLineNote invoiceLineNote = new InvoiceLineNote();
			List<InvoiceLineNote> invoiceLineNoteList = new ArrayList<InvoiceLineNote>();
			for (int z = 0; z < notes.getLength(); z++) {
				Element note = (Element) notes.item(z);
				String lineNote = getNodeValue(note, null, null);
				if (lineNote != null) {
					invoiceLineNote.setNote(lineNote);
					invoiceLineNoteList.add(invoiceLineNote);
				}
			}
			if (invoiceLineNoteList.size() >= 0) {
				invoiceLine.setNoteList(invoiceLineNoteList);
			}

			// AccountingCost
			String lineAccountingCost = getNodeValue((Element) singleLine,
					"cbc", "AccountingCost");
			lineAccountingCost = SpecialCharacter
					.xmlEntityToString(lineAccountingCost);
			invoiceLine.setLineAccountingCost(lineAccountingCost);

			// OrderLineReference
			Node orderLineReferenceNode = getNode(singleLine, "cac",
					"OrderLineReference");
			String orderLineReference = getNodeValue(
					(Element) orderLineReferenceNode, "cbc", "LineID");
			orderLineReference = SpecialCharacter
					.xmlEntityToString(orderLineReference);
			invoiceLine.setOrderLineReference(orderLineReference);

			// Documento di riferimento
			NodeList documentRefNodes = getNodes(singleLine, "cac",
					"DocumentReference");
			String documentRef;
			String documentType;
			for (int z = 0; z < documentRefNodes.getLength(); z++) {
				Element docRef = (Element) documentRefNodes.item(z);
				documentRef = getNodeValue(docRef, "cbc", "ID");
				documentType = getNodeValue(docRef, "cbc", "DocumentType");

				// se il tipo documento è un contatore associa alla riga il
				// contatore dichiarato in precedenza
				if ("CONTATORE".equalsIgnoreCase(documentType)) {
					for (int c = 0; c < meterList.size(); c++) {
						if ((meterList.get(c).getMeterNumber() != null)
								&& (meterList.get(c).getMeterNumber()
										.equalsIgnoreCase(documentRef))) {
							invoiceLine.setMeter(meterList.get(c));
							break;
						}
					}
				}
				// se non è specificato il contatore cerca un contatore con lo
				// stesso numero di contratto: es. caso ambiente
				else if ((invoiceLine.getMeter() == null)
						&& ("CONTRATTO".equalsIgnoreCase(documentType))) {
					for (int c = 0; c < meterList.size(); c++) {
						if (meterList.get(c).getSupplyInfo().size() > 0) {
							for (int w = 0; w < meterList.get(c)
									.getSupplyInfo().size(); w++) {
								if ((meterList.get(c).getSupplyInfo().get(w)
										.getPreviousDocument() != null)
										&& (meterList.get(c).getSupplyInfo()
												.get(w).getPreviousDocument()
												.getDocumentCode()
												.equalsIgnoreCase(documentRef))) {
									invoiceLine.setMeter(meterList.get(c));
									break;
								}
							}
						}
					}
				}
			}

			// Item
			Node itemNode = getNode(singleLine, "cac", "Item");
			// Dettaglio tasse
			Node classifiedTaxCategoryNode = getNode(itemNode, "cac",
					"ClassifiedTaxCategory");
			invocieLineTaxDetail.setTaxPercent(Double
					.parseDouble(getNumericNodeValue(
							(Element) classifiedTaxCategoryNode, "cbc",
							"Percent")));
			invoiceLine.setTaxDetail(invocieLineTaxDetail);

			String itemName = getNodeValue((Element) itemNode, "cbc", "Name");
			itemName = SpecialCharacter.xmlEntityToString(itemName);
			invoiceLine.setItemName(itemName);

			Node sellersItemIdentificationNode = getNode(itemNode, "cac",
					"SellersItemIdentification");
			String sellersItemIdentification = getNodeValue(
					(Element) sellersItemIdentificationNode, "cbc", "ID");
			sellersItemIdentification = SpecialCharacter
					.xmlEntityToString(sellersItemIdentification);
			invoiceLine.setSellersItemIdentification(sellersItemIdentification);

			String itemKeyword = getNodeValue((Element) itemNode, "cbc",
					"Keyword");
			if ((itemKeyword == null) || (itemKeyword.equals(""))) {
				itemKeyword = "RIGA";
			}
			switch (XmlFactory.itemKeyword.valueOf(itemKeyword)) {
			case RIGA:
				// logger.info("Invoice line num. " + invoiceLine +
				// ", Invoiced quantity " + invoicedQuantity +
				// ", Extension amount " + lineExtensionAmount +
				// ", Item description " + itemDescription);
				invoice.addInvoiceLine(invoiceLine);
				invoice.localAddInvoiceLine(invoiceLine);
				break;
			case CONSUMO:
				Date startConsumptionDate = null;
				Date endConsumptionDate = null;
				String consumptionBracket = null;

				if (invoiceLine.getNoteList() != null) {
					for (int z = 0; z < invoiceLine.getNoteList().size(); z++) {
						if (invoiceLine.getNoteList().get(z).getNote() != null) {
							String[] s = invoiceLine.getNoteList().get(z)
									.getNote().split("-");
							if (s.length > 0) {
								SimpleDateFormat formatterDate;
								try {
									formatterDate = new SimpleDateFormat(
											"yyyyMMdd");
									startConsumptionDate = (Date) formatterDate
											.parse(s[0]);
									endConsumptionDate = (Date) formatterDate
											.parse(s[1]);
								} catch (ParseException e) {
									logger.error("Exception: " + e);
								}
							}
						}
					}
				}

				Consumption consumption = new Consumption(startConsumptionDate,
						endConsumptionDate, consumptionBracket);
				invoiceLine.setConsumption(consumption);

				// logger.info("Consumo");
				// logger.info("Invoice line num. " + invoiceLine +
				// ", Invoiced quantity " + invoicedQuantity +
				// ", Extension amount " + lineExtensionAmount +
				// ", Item description " + itemDescription);
				invoice.addInvoiceLine(invoiceLine);
				invoice.localAddInvoiceLine(invoiceLine);

				break;
			case LETTURA:
				Date readingDate = null;
				if (invoiceLine.getNoteList() != null) {
					SimpleDateFormat formatterDate;

					for (int z = 0; z < invoiceLine.getNoteList().size(); z++) {

						if (invoiceLine.getNoteList().get(z) != null) {
							try {
								formatterDate = new SimpleDateFormat("yyyyMMdd");
								readingDate = (Date) formatterDate
										.parse(invoiceLine.getNoteList().get(z)
												.getNote());
							} catch (ParseException e) {
								logger.error("Exception: " + e);
							}
						}
					}
				}

				Reading reading = new Reading(itemName, readingDate,
						invoiceLine.getItemDescription(),
						invoiceLine.getItemDescription(),
						invoiceLine.getInvoicedQuantity(),
						invoiceLine.getUnitCode(), invoiceLine.getMeter());
				readingList.add(reading);
				invoice.setReadingList(readingList);

				break;
			case RIASSUNTO_CONTATORE:
				MeterSummary meterSummary = new MeterSummary();
				meterSummary.setMeter(invoiceLine.getMeter());

				if (invoice.getMeterSummary() != null) {
					if (invoice.getMeterSummary().size() > 0) {
						Meter meterPre = invoice.getMeterSummary()
								.get(invoice.getMeterSummary().size() - 1)
								.getMeter();
						if ((meterPre != null)
								&& (meterPre.equals(invoiceLine.getMeter()))) {
							meterSummary = invoice.getMeterSummary().get(
									invoice.getMeterSummary().size() - 1);
						}
					}
				}

				if ("ACCONTO".equalsIgnoreCase(itemName)) {
					meterSummary.setMeterOnAccountQuantity(invoiceLine
							.getInvoicedQuantity());
				} else if ("CONGUAGLIO".equalsIgnoreCase(itemName)) {
					meterSummary.setMeterBalanceQuantity(invoiceLine
							.getInvoicedQuantity());
				} else {
					meterSummary.setMeterInvoicedQuantity(invoiceLine
							.getInvoicedQuantity());
					meterSummary.setMeterSummaryUnitCode(invoiceLine
							.getUnitCode());
				}

				if (invoice.getMeterSummary() != null) {
					if (invoice.getMeterSummary().size() > 0) {
						if (invoice.getMeterSummary()
								.get(invoice.getMeterSummary().size() - 1)
								.getMeter().equals(invoiceLine.getMeter())) {
							meterSummary = invoice.getMeterSummary().set(
									invoice.getMeterSummary().size() - 1,
									meterSummary);
						} else {
							meterSummaryList.add(meterSummary);
							invoice.setMeterSummary(meterSummaryList);
						}
					}
				} else {
					meterSummaryList.add(meterSummary);
					invoice.setMeterSummary(meterSummaryList);
				}

				break;
			default:

				break;
			}

		}

		return invoice;
	}

	// public static void main(String[] args) throws XPathExpressionException {
	//
	// //Invoice out =
	// xmlToJava("C:/Users/Chiara/Desktop/testUBL_txtHera1_0000108005091572.xml");
	// Invoice out =
	// xmlToJava("C:/Users/Chiara/Desktop/FE - Fatture di Test/FatturePeppolDiEsempio/BII04+XML+example+full+core+data+01.xml");
	// //Invoice out =
	// xmlToJava("C:/Users/Chiara/Desktop/testUBL_txtHera3_0000110003332460.xml");
	// //Invoice out =
	// xmlToJava("C:/Users/Chiara/Desktop/BII04+XML+example+full+core+data+01.xml");
	// System.out.println("Invoice number: " + out.getInvoiceNumber());
	// System.out.println("Issue date: " + out.getIssueDate());
	// System.out.println("Supplier name: " +
	// out.getSupplier().getSupplierName());
	// System.out.println("Supplier vat id: " + out.getSupplier().getVatId());
	// System.out.println("Supplier address: " +
	// out.getSupplier().getPostalAddress());
	//
	// System.out.println("Client name: " + out.getClient().getClientName());
	// System.out.println("Client vat id: " + out.getClient().getVatId());
	// System.out.println("Client address: " +
	// out.getClient().getPostalAddress());
	//
	// System.out.println("Invoice period: " +
	// out.getInvoicePeriod().getStartDate() + " - " +
	// out.getInvoicePeriod().getEndDate());
	// System.out.println("Invoice period code: " +
	// out.getInvoicePeriod().getPeriodCode());
	// System.out.println("Invoice period description: " +
	// out.getInvoicePeriod().getPeriodDescription());
	//
	// System.out.println("Invoice payment due date: " +
	// out.getPaymentDetail().getPaymentDueDate());
	//
	// for (int z = 0; z < out.getPreviousDocuments().size(); z++) {
	// System.out.println("Invoice document type: " +
	// out.getPreviousDocuments().get(z).getDocumentType());
	// System.out.println("Invoice document code: " +
	// out.getPreviousDocuments().get(z).getDocumentCode());
	// }
	//
	// System.out.println("Invoice allinclusive total: " +
	// out.getAllInclusiveTotal());
	//
	// for (int z = 0; z < out.getTaxDetails().size(); z++) {
	// System.out.println("Invoice taxable amount: " +
	// out.getTaxDetails().get(z).getTaxableAmount());
	// System.out.println("Invoice tax amount: " +
	// out.getTaxDetails().get(z).getTaxAmount());
	// System.out.println("Invoice tax percent: " +
	// out.getTaxDetails().get(z).getTaxPercent());
	// }
	//
	// for (int z = 0; z < out.getInvoiceLines().size(); z++) {
	// System.out.println("Line number: " +
	// out.getInvoiceLines().get(z).getLineNumber());
	// System.out.println("Item description: " +
	// out.getInvoiceLines().get(z).getItemDescription());
	// System.out.println("Invoiced quantity: " +
	// out.getInvoiceLines().get(z).getInvoicedQuantity());
	// System.out.println("Unit code: " +
	// out.getInvoiceLines().get(z).getUnitCode());
	// System.out.println("Price: " +
	// out.getInvoiceLines().get(z).getPriceAmount());
	// System.out.println("Line extension amount: " +
	// out.getInvoiceLines().get(z).getLineExtensionAmount());
	// System.out.println("Line tax total: " +
	// out.getInvoiceLines().get(z).getTaxDetail().getTaxAmount());
	// System.out.println("Line taxable total: " +
	// out.getInvoiceLines().get(z).getTaxDetail().getTaxableAmount());
	// System.out.println("Line tax percent: " +
	// out.getInvoiceLines().get(z).getTaxDetail().getTaxPercent());
	// if(out.getInvoiceLines().get(z).getMeter() != null){
	// System.out.println("Line meter: " +
	// out.getInvoiceLines().get(z).getMeter().getMeterNumber());
	// System.out.println("Meter sector: " +
	// out.getInvoiceLines().get(z).getMeter().getSector().getSectorName());
	// }
	// }
	//
	// for (int z = 0; z < out.getNoteList().size(); z++) {
	// System.out.println("Invoice note: " +
	// out.getNoteList().get(z).getNote());
	// }
	//
	// System.out.println("Tax Point Date: " + out.getTaxPointDate());
	// System.out.println("Accounting cost: " + out.getAccountingCost());
	//
	// out.setReceiptDate(out.getIssueDate());
	// JNDIUtils.getInvoiceManager().insertInvoice(out);
	// }

	public static NodeList getNodes(Node parentN, String prefix, String nodeName)
			throws XPathExpressionException {
		NodeList result = new NodeSet();

		if (parentN == null) {
			return result;
		}
		if (nodeName == null) {
			return result;
		}

		result = (NodeList) xpath.evaluate(nodeName, parentN,
				XPathConstants.NODESET);
		if (result.getLength() <= 0) {
			prefix = prefix + ":";

			result = (NodeList) xpath.evaluate(prefix + nodeName, parentN,
					XPathConstants.NODESET);
		}

		return result;
	}

	public static Node getNode(Node parentN, String prefix, String nodeName)
			throws XPathExpressionException {
		if (parentN == null) {
			return null;
		}
		if (nodeName == null) {
			return null;
		}

		Node result = (Node) xpath.evaluate(nodeName, parentN,
				XPathConstants.NODE);
		if (result == null) {
			prefix = prefix + ":";
			result = (Node) xpath.evaluate(prefix + nodeName, parentN,
					XPathConstants.NODE);
			logger.debug(prefix + nodeName);
		}

		return result;
	}

	public static String getNodeValue(Element e, String prefix, String nodeName)
			throws XPathExpressionException {
		if (e == null) {
			return null;
		}
		if (nodeName == null) {
			if (e.getFirstChild() != null) {
				return e.getFirstChild().getNodeValue();
			}
			return null;
		}

		// controlla la presenza del namespace
		if (!(xpath.evaluate(nodeName, e).equals(""))) {
			return xpath.evaluate(nodeName, e);
		} else if (!xpath.evaluate(prefix + ":" + nodeName, e).equals("")) {
			return xpath.evaluate(prefix + ":" + nodeName, e);
		}

		return null;
	}

	public static String getNumericNodeValue(Element e, String prefix,
			String nodeName) throws XPathExpressionException {
		String s = getNodeValue(e, prefix, nodeName);
		if (s == null) {
			return "0";
		}

		return s;
	}

	public static String getNodeValueByExpr(Document doc, Node parent,
			Node nodeToFind, Node nodeQuery, String value) {
		if (parent == null) {
			return null;
		}
		if (nodeToFind == null) {
			return null;
		}
		if (nodeQuery == null) {
			return null;
		}

		String expression = "[" + nodeQuery.getNodeName() + "='" + value + "']";
		XPathExpression ex;
		try {
			ex = xpath.compile("//" + parent.getNodeName() + expression + "/"
					+ nodeToFind.getNodeName() + "/text()");
			return (String) ex.evaluate(doc, XPathConstants.STRING);

		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}

		return null;
	}

	// Helper method
	public static List<AllowanceCharge> readAllowanceCharge(Element element,
			boolean forInformationOnly) throws XPathExpressionException {
		NodeList allowanceChargeNodes = getNodes(element, "cac",
				"AllowanceCharge");
		List<AllowanceCharge> allowanceChargeList = new ArrayList<AllowanceCharge>();
		for (int z = 0; z < allowanceChargeNodes.getLength(); z++) {
			Element lineAllowanceChargeNode = (Element) allowanceChargeNodes
					.item(z);
			AllowanceCharge allowanceCharge = new AllowanceCharge();

			double lineeAllowanceChargeAmount = Double
					.parseDouble(getNumericNodeValue(lineAllowanceChargeNode,
							"cbc", "Amount"));
			if ("false".equalsIgnoreCase(getNumericNodeValue(
					lineAllowanceChargeNode, "cbc", "ChargeIndicator"))) {
				lineeAllowanceChargeAmount = lineeAllowanceChargeAmount * (-1);
			}
			allowanceCharge.setAmount(lineeAllowanceChargeAmount);
			allowanceCharge.setAllowanceChargeReason(getNodeValue(
					lineAllowanceChargeNode, "cbc", "AllowanceChargeReason"));
			allowanceCharge.setMultiplierFactorNumeric(Double
					.parseDouble(getNumericNodeValue(lineAllowanceChargeNode,
							"cbc", "MultiplierFactorNumeric")));
			allowanceCharge.setBaseAmount(Double
					.parseDouble(getNumericNodeValue(lineAllowanceChargeNode,
							"cbc", "BaseAmount")));
			allowanceCharge.setForInformationOnly(forInformationOnly);

			allowanceChargeList.add(allowanceCharge);
		}

		return allowanceChargeList;

	}

	private static PostalAddress readAddress(Element addressElement)
			throws XPathExpressionException {
		PostalAddress address = new PostalAddress();
		address.setStreetName(SpecialCharacter.xmlEntityToString(getNodeValue(
				addressElement, "cbc", "StreetName")));
		address.setBuildingNumber(SpecialCharacter
				.xmlEntityToString(getNodeValue(addressElement, "cbc",
						"BuildingNumber")));
		address.setPostalZone(SpecialCharacter.xmlEntityToString(getNodeValue(
				addressElement, "cbc", "PostalZone")));
		address.setCityName(SpecialCharacter.xmlEntityToString(getNodeValue(
				addressElement, "cbc", "CityName")));

		Node addressCountryNode = getNode(addressElement, "cac", "Country");
		address.setCountry(SpecialCharacter.xmlEntityToString(getNodeValue(
				(Element) addressCountryNode, "cbc", "IdentificationCode")));

		address.setPostBox(SpecialCharacter.xmlEntityToString(getNodeValue(
				addressElement, "cbc", "Postbox")));
		address.setAdditionalStreetName(SpecialCharacter
				.xmlEntityToString(getNodeValue(addressElement, "cbc",
						"AdditionalStreetName")));
		address.setDepartment(SpecialCharacter.xmlEntityToString(getNodeValue(
				addressElement, "cbc", "Department")));
		address.setCountrySubentityCode(SpecialCharacter
				.xmlEntityToString(getNodeValue(addressElement, "cbc",
						"CountrySubentityCode")));

		return address;
	}

	public static String readSupplierName(String xml) {
		// Documento XML

		try {
			DocumentBuilderFactory domFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder;
			Document doc = null;
			domFactory.setNamespaceAware(true);

			builder = domFactory.newDocumentBuilder();
			doc = builder
					.parse(new java.io.ByteArrayInputStream(xml.getBytes()));

			final PrefixResolver resolver = new PrefixResolverDefault(
					doc.getDocumentElement());
			NamespaceContext ctx = new NamespaceContext() {
				public String getNamespaceURI(String prefix) {
					return resolver.getNamespaceForPrefix(prefix);
				}

				public String getPrefix(String arg0) {
					return null;
				}

				public Iterator getPrefixes(String arg0) {
					return null;
				}
			};

			xpath.setNamespaceContext(ctx);
			rootNode = doc.getDocumentElement();

			Node supplierInfoNode = getNode(rootNode, "cac",
					"AccountingSupplierParty");
			Node supplierPartyNode = getNode(supplierInfoNode, "cac", "Party");
			Node supplierPartyNameNode = getNode(supplierPartyNode, "cac",
					"PartyName");
			Node supplierPartyLegalEntityNode = getNode(supplierPartyNode,
					"cac", "PartyLegalEntity");

			// SupplierName
			String supplierName = getNodeValue((Element) supplierPartyNameNode,
					"cbc", "Name");
			supplierName = SpecialCharacter.xmlEntityToString(supplierName);

			return supplierName;
		} catch (SAXParseException spe) {
			logger.error("Parse error: " + spe.getMessage());
		} catch (SAXException se) {
			logger.error("SAXException: " + se.getMessage());
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		return "";
	}

}