/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.common.utils;

import java.lang.reflect.Field;

/**
 * This class provides static methods to deal with java reflection
 * 
 * @author fulvio di marco
 *
 */
public class ReflectionUtils {
	
	/**
	 * 
	 * Given an object and a property name, returns the property value
	 * 
	 * @param <T>
	 * @param obj
	 * @param propertyName
	 * @return
	 * 
	 * @author fulvio di marco
	 */
	public static <T> Object getPropertyValue(Object obj, String propertyName) {
		try {
			Field field = obj.getClass().getDeclaredField(propertyName);
			field.setAccessible(true);
			return field.get(obj);
		} catch (SecurityException e) {
			e.printStackTrace();
			return null;
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			return null;
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			return null;
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * 
	 * Given an object and a property name, returns the type of the specified property.
	 * 
	 * @param <T>
	 * @param obj
	 * @param propertyName
	 * @return
	 * 
	 * @author fulvio di marco
	 */
	public static <T> Class<?> getType(Object obj, String propertyName) {
		Field field;
		try {
			field = obj.getClass().getDeclaredField(propertyName);
			field.setAccessible(true);
			return field.getType();
		} catch (SecurityException e) {
			e.printStackTrace();
			return null;
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			return null;
		}
		
	}

}
