/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */

package it.fe.core.common.utils;

/**
 * 
 * Classe per la gestione dei caratteri speciali
 * 
 * 
 * @author chiara.pezzi@epocaricerca.it
 * 
 */
public class SpecialCharacter {

	/**
	 * Controlla la presenza di caratteri speciali e li converte in xml entity.
	 * <br /><br />
	 * 
	 * N.B. Prima di generare il file xml è necessario richiamare il metodo adjustXmlEntity
	 * che corregge la conversione automatica di & in &amp; rendendo possibile la visione 
	 * corretta dei caratteri speciali
	 * 
	 * @param txt - (String) stringa da controllare
	 * @return (String) stringa contente le xml entity dei caratteri speciali
	 * 
	 * @see adjustXmlEntity(String txt)
	 */
	public static String stringToXmlEntity(String txt) {
		String xmlEntity = "";
		
		if (txt != null) {
			xmlEntity = txt;
		
			// Carattere ° - Codice ASCII 176
			xmlEntity = xmlEntity.replace(Character.toString((char)176), "&#176;");
			
			// Carattere ^ - Codice ASCII 136
			xmlEntity = xmlEntity.replace(Character.toString((char)136), "&#136;");
			
			// Carattere / - Codice ASCII 47
			xmlEntity = xmlEntity.replace(Character.toString((char)47), "&#47;");
			
			// Carattere è - Codice ASCII 232
			xmlEntity = xmlEntity.replace(Character.toString((char)232), "&#232;");
			
			// Carattere é - Codice ASCII 233
			xmlEntity = xmlEntity.replace(Character.toString((char)233), "&#233;");
			
			// Carattere à - Codice ASCII 224
			xmlEntity = xmlEntity.replace(Character.toString((char)224), "&#224;");
			
			// Carattere ò - Codice ASCII 242
			xmlEntity = xmlEntity.replace(Character.toString((char)242), "&#242;");
			
			// Carattere ù - Codice ASCII 249
			xmlEntity = xmlEntity.replace(Character.toString((char)249), "&#249;");
			
			// Carattere ì - Codice ASCII 236
			xmlEntity = xmlEntity.replace(Character.toString((char)236), "&#236;");
		}
		
		return xmlEntity;
	}
	
	/**
	 * Corregge la conversione automatica di & in &amp; 
	 * ripristinando la corretta xml Entity
	 * <br /><br />
	 * 
	 * @param txt - (String) stringa da controllare
	 * @return (String) stringa contente le xml entity dei caratteri speciali
	 */
	public static String adjustXmlEntity(String txt) {
		String xmlEntity = "";
		
		if (txt != null) {
			xmlEntity = txt;

			// Carattere ° - Codice ASCII 176
			xmlEntity = xmlEntity.replace("&amp;#176;", "&#176;");
			
			// Carattere ^ - Codice ASCII 136
			xmlEntity = xmlEntity.replace("&amp;#136;", "&#136;");
			
			// Carattere / - Codice ASCII 47
			xmlEntity = xmlEntity.replace("&amp;#47;", "&#47;");
			
			// Carattere è - Codice ASCII 232
			xmlEntity = xmlEntity.replace("&amp;#232;", "&#232;");
			
			// Carattere é - Codice ASCII 233
			xmlEntity = xmlEntity.replace("&amp;#233;", "&#233;");
			
			// Carattere à - Codice ASCII 224
			xmlEntity = xmlEntity.replace("&amp;#224;", "&#224;");
			
			// Carattere ò - Codice ASCII 242
			xmlEntity = xmlEntity.replace("&amp;#242;", "&#242;");
			
			// Carattere ù - Codice ASCII 249
			xmlEntity = xmlEntity.replace("&amp;#249;", "&#249;");
			
			// Carattere ì - Codice ASCII 236
			xmlEntity = xmlEntity.replace("&amp;#236;", "&#236;");
		}
		
		return xmlEntity;
	}
	
	
	/**
	 * Converte le xml entity in caratteri
	 * 
	 * @param textXml - (String) stringa xml da convertire
	 * @return (String) - testo convertito
	 * 
	 */
	public static String xmlEntityToString(String textXml) {
		String text = "";
		
		if (textXml != null) {
			text = textXml;
		
			// Carattere ° - Codice ASCII 176
			text = text.replace("&#176;", Character.toString((char)176));
			
			// Carattere ^ - Codice ASCII 136
			text = text.replace("&#136;", Character.toString((char)136));
			
			// Carattere / - Codice ASCII 47
			text = text.replace("&#47;", Character.toString((char)47));
			
			// Carattere è - Codice ASCII 232
			text = text.replace("&#232;", Character.toString((char)232));
			
			// Carattere é - Codice ASCII 233
			text = text.replace("&#233;", Character.toString((char)233));
			
			// Carattere à - Codice ASCII 224
			text = text.replace("&#224;", Character.toString((char)224));
			
			// Carattere ò - Codice ASCII 242
			text = text.replace("&#242;", Character.toString((char)242));
			
			// Carattere ù - Codice ASCII 249
			text = text.replace("&#249;", Character.toString((char)249));
			
			// Carattere ì - Codice ASCII 236
			text = text.replace("&#236;", Character.toString((char)236));
		}
		
		return text;
	}
}
