/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
*
 *
 */
package it.fe.core.common.utils.liferay.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.events.Action;
import com.liferay.portal.kernel.events.ActionException;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.util.PortalUtil;


/**
 * 
 *	Logout event. Please refer to Liferay Documentation.
 *	ref. logout.events.pre and logout.events.post in portal-ext.properties
 *
 *	This action deletes liferay account on user logout. Remember that a new portal
 *	account is created on every login.
 *	
 */
public class UserDeleteLogoutAction extends Action {
	
	private Logger logger = Logger.getLogger(UserDeleteLogoutAction.class);

	@Override
	public void run(HttpServletRequest request, HttpServletResponse response)
			throws ActionException {
		
		try {
			
			logger.info("Executing pre logout actions");
			User user = PortalUtil.getUser(request);
			// Liferay user Test, used for admininistration/Testing
			if (user.getScreenName().equals("test"))
				return;
			else {
				try {
					UserLocalServiceUtil.deleteUser(user.getUserId());
				} catch (PortalException e) {
					logger.error(e.getMessage());
				} catch (SystemException e) {
					logger.error(e.getMessage());
				}
			}
			
		} catch (Exception e) {
			throw new ActionException(e);
		}
		
	}

}
