/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb;

import it.fe.core.par.Quicknote;

import java.util.Date;
import java.util.logging.Logger;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@Stateless
@Remote(QuicknoteManager.class)
public class QuicknoteManagerBean implements QuicknoteManager {
	
	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(QuicknoteManagerBean.class.getName());

	public Quicknote getQuicknoteByUser(String username) {
		Quicknote result = null;
		Query query = entityManager.createQuery("SELECT qn FROM Quicknote qn WHERE qn.username = :username");
		query.setParameter("username", username);
		try {
			result = (Quicknote) query.getSingleResult();
		} catch (NoResultException e) {
			return null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			return null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			return null;
		}
		
		return result;
	}

	public void saveQuicknoteByUser(String username, String text, Date currentDate) {
		Quicknote quicknote = this.getQuicknoteByUser(username);
		if (quicknote == null) {
			quicknote = new Quicknote();
			quicknote.setText(text);
			quicknote.setUsername(username);
			quicknote.setLastChange(currentDate);
			entityManager.persist(quicknote);
		}
		else {
			quicknote.setText(text);
			quicknote.setUsername(username);
			quicknote.setLastChange(currentDate);
			entityManager.merge(quicknote);
		}	
	}


}
