/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.invoice;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.TransmissionType;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.log4j.Logger;

@Stateless
@Remote(OriginalDocumentManager.class)
public class OriginalDocumentManagerBean implements OriginalDocumentManager {

	private static final long serialVersionUID = 593464069843086636L;

	@PersistenceContext(name = "FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;

	protected static Logger logger = Logger.getLogger(InvoiceManagerBean.class
			.getName());

	public OriginalDocument getOriginalDocumentById(long id) {
		OriginalDocument originalDocument = new OriginalDocument();

		try {
			originalDocument = entityManager.find(OriginalDocument.class, id);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			originalDocument = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			originalDocument = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			originalDocument = null;
		}

		return originalDocument;
	}

	public long insertOriginalDocument(OriginalDocument originalDocument) {
		long result = 0;

		try {
			entityManager.persist(originalDocument);
			result = originalDocument.getOriginalDocumentId();
			entityManager.flush();
		} catch (EntityExistsException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalStateException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (PersistenceException e) {
			logger.error(e.getMessage());
			result = -1;
		}

		return result;
	}

	public OriginalDocument updateReceiveStatus(long idOriginalDocument,
			String receiveStatus, String exceptionMessage) {
		OriginalDocument originalDocument = entityManager.find(
				OriginalDocument.class, idOriginalDocument);
		if (originalDocument != null) {
			originalDocument.setReceiveStatus(receiveStatus);
			originalDocument.setExceptionMessage(exceptionMessage);
		}
		return originalDocument;
	}

	@SuppressWarnings("unchecked")
	public List<OriginalDocument> getOriginalDocuments() {
		Query query = entityManager
				.createQuery("SELECT o FROM OriginalDocument o");
		return query.getResultList();
	}

	public OriginalDocument fetchOriginalDocument(long idOriginalDocument) {
		OriginalDocument originalDocument = entityManager.find(
				OriginalDocument.class, idOriginalDocument);
		List<FEFile> documents = new ArrayList<FEFile>();
		if (originalDocument.getReceiptDocuments() != null)
			for (FEFile receiptDocument : originalDocument
					.getReceiptDocuments()) {
				FEFile retrieved = entityManager.find(FEFile.class,
						receiptDocument.getFileId());
				retrieved.getFileBlob();
				documents.add(retrieved);
			}
		originalDocument.setReceiptDocuments(documents);
		return originalDocument;
	}

	@Override
	public List<OriginalDocument> findOriginalDocumentByTypeAndAddditionalData(
			String transmissionTypeDescription, String additionalInfo) {
		TransmissionType type = JNDIUtils.getTransmissionTypeManager()
				.getTransmissionTypeByDescription(transmissionTypeDescription);
		Query query = entityManager
				.createQuery(
						"SELECT o FROM OriginalDocument o WHERE o.transmissionType = :type AND o.additionalInfo=:additionalInfo")
				.setParameter("type", type).setParameter("additionalInfo", additionalInfo);
		return query.getResultList();
	}

}
