/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.invoice;

import it.fe.core.par.invoice.TransmissionType;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.log4j.Logger;

@Stateless
@Remote(TransmissionTypeManager.class)
public class TransmissionTypeManagerBean implements TransmissionTypeManager {

	private static final long serialVersionUID = -7033900420110602255L;

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(TransmissionTypeManagerBean.class.getName());
	
	public TransmissionType getTransmissionTypeById(long id) {
		TransmissionType transmissionType = new TransmissionType();
		
		try {
			transmissionType = entityManager.find(TransmissionType.class, id);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			transmissionType = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			transmissionType = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			transmissionType = null;
		}
		
		return transmissionType;
	}
	
	
	public TransmissionType getTransmissionTypeByDescription(String description) {
		TransmissionType transmissionType = new TransmissionType();
		Query query = entityManager.createQuery("SELECT t FROM TransmissionType t WHERE t.description = :description");
		query.setParameter("description", description);
		
		try {
			transmissionType = (TransmissionType) query.getSingleResult();
			logger.info("Find transmission type " + description);
			
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			transmissionType = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			transmissionType = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			transmissionType = null;
		}
		
		return transmissionType;
	}

}
