/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.process;

import it.fe.core.par.invoice.Invoice;

public interface ProcessManager {
	
	public final static String SUCCESS = "success";
	public final static String FAILURE = "failure";
	public final static String STATE_TECHNICAL_VALIDATION = "TechnicalValidationSign";
	public final static String STATE_ADDITIONAL_DATA_EDITING = "AdditionalDataEditing";
	public final static String ACTION_APPROVE = "approve";
	public final static String ACTION_REJECT = "reject";
	public final static String ACTION_EDIT = "edit";
	public final static String ACTION_BACK = "back";
	public final static String ACTION_INSERT = "insert";
	
	/**
	 * @param invoice
	 * @param username
	 * @return
	 */
	public String approveTechnically(Invoice invoice, String username);
	
	/**
	 * @param invoice
	 * @param username
	 * @return
	 */
	public String rejectTechnically(Invoice invoice, String username);
	
	/**
	 * @param invoice
	 * @param username
	 * @return
	 */
	public String approveInsertionIntoAccountableSystem(Invoice invoice, String username);
	
	/**
	 * Method invoked by custom action in the underlying workflow.
	 * 
	 * Must be implemented with accountable system insertion business logic.
	 * 
	 * @param invoiceId
	 * @param username
	 * @return
	 */
	public Object[] insertIntoAccountableSystem(long invoiceId, String username);

	/**
	 * @param selectedInvoice
	 * @param username
	 * @return
	 */
	public String editAdditionalData(Invoice selectedInvoice, String username);
	
	/**
	 * @param selectedInvoice
	 * @param username
	 * @return
	 */
	public String saveAdditionalData(Invoice selectedInvoice, String username);
	
	/**
	 * @param selectedInvoice
	 * @param username
	 * @return
	 */
	public String discardAdditionalData(Invoice selectedInvoice, String username);
	
	/**
	 * Method used to know if the workflow may be triggered from TechnicalValidation to AdditionalDataEditing. 
	 * Returns true if and only if the workflow is in TechnicalValidation state.
	 * 
	 * @param invoice
	 * @return
	 */
	public boolean isEditable(Invoice invoice);

}
