/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.process;

import java.util.List;
import java.util.Map;

import org.jbpm.api.ProcessInstance;
import org.jbpm.api.task.Task;

/**
 * @author fdimarco, stefano
 *
 */
public interface ProcessServiceFacade {
	
	/**
	 * Starts a new process instance
	 * 
	 * @param processDefinitionKey - is the key of the process definition for which the latest version will be taken.
	 * @param processInstanceKey - is a user provided reference for the new execution that must be unique over all process versions with the same name
	 * @return the process instance id
	 */
	public String start(String processDefinitionKey, String processInstanceKey);
	
	/**
	 * Starts a new process instance
	 * 
	 * @param processDefinitionKey - is the key of the process definition for which the latest version will be taken.
	 * @param variables - are the initial values of the process variables that will be set before the execution starts.
	 * @param processInstanceKey - is a user provided reference for the new execution that must be unique over all process versions with the same name
	 * @return the process instance id
	 */
	public String start(String processDefinitionKey, Map<String, ?> variables, String processInstanceKey);
	
	/**
	 * 
	 * @param processDefinitionKey
	 * @param processInstanceKey
	 * @return
	 */
	public ProcessInstance findProcessInstanceById(String processDefinitionKey, String processInstanceKey);
	
	
    /**
     * Provides an external trigger to an execution.
     * 
     * @param executionId
     * @param currentState
     * @param username
     */
    public String signal(String processDefinitionKey, String businessKey, String currentState, String username) ;
    
    
    /**
     * Provides an external trigger to an execution.
     * 
     * @param executionId
     * @param signalName
     * @param currentState
     * @param username
     */
    public String signal(String processDefinitionKey, String businessKey, String signalName, String currentState, String username);
	
    /**
     * List of tasks that are assigned to the given user. Returns an empty list in case no such tasks exist.
     * 
     * @param userId
     * @return
     */
    public List<Task> findPersonalTasks(String userId);
    
    /**
     * Deletes this task, marks the related history task as completed. 
     * If the task was created in the context of a process execution, this operation may result 
     * in a process instance being triggered.
     * 
     * @param taskId
     */
    public void completeTask(String taskId);
    
    /**
     * Deletes this task, marks the related history task as completed with the specified outcome. 
     * If the task was created in the context of a process execution, this operation may result in 
     * a process instance being triggered. The outcome in that case corresponds to an outgoing 
     * transition in the process.
     * 
     * @param taskId
     * @param outcome
     */
    public void completeTask(String taskId, String outcome);
  
    /**
     * Ends a process instance 
     * 
     * @param processDefinitionKey
     * @param processInstanceKey
     */
    public void end(String processDefinitionKey, String processInstanceKey);
	

}
