/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.user;

import it.fe.core.par.authorization.Permission;
import it.fe.core.par.authorization.RoleDetail;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.user.Role;

import java.math.BigInteger;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@Stateless
@Remote(RoleManager.class)
public class RoleManagerBean implements RoleManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(RoleManagerBean.class.getName());

	@SuppressWarnings("unchecked")
	public List<Role> getRoles() {
		Query q = entityManager.createQuery("SELECT r FROM Role r");
		List<Role> roles = q.getResultList();
		return roles;
	}
	
	public Role getRoleById(long roleId) {
		Query q = entityManager.createQuery("SELECT r FROM Role r WHERE roleId = :roleId");
		q.setParameter("roleId", roleId);
		return (Role) q.getSingleResult();
	}
	
	public Role getRoleByDescription(String description) {
		Query q = entityManager.createQuery("SELECT r FROM Role r WHERE description = :description");
		q.setParameter("description", description);
		return (Role) q.getSingleResult();
	}
	

	@SuppressWarnings("unchecked")
	public Role loadRoleDetail (Role role) {
//		Query q = entityManager.createQuery("SELECT r FROM Role r WHERE r.description = :description");
////		Query q = entityManager.createQuery("SELECT r FROM Role r join r.roleDetail WHERE r.description = :description");
//
//		q.setParameter("description", role.getDescription());
//		List<Role> roles  = q.getResultList();
//		(Role) q.getSingleResult();
		if (role == null )
			return null;
		
		Role detachedRole = new Role();
		detachedRole.setStartDate(role.getStartDate());
		detachedRole.setEndDate(role.getEndDate());
		detachedRole.setDescription(role.getDescription());
		
		
		Query q1 = entityManager.createNativeQuery("select * FROM RoleDetails r WHERE r.roleId = "+role.getRoleId()+";");
		Set<RoleDetail> detachedDetails = new HashSet<RoleDetail>();
		List<Object[]> retrievedRoleDetails = q1.getResultList();
		
		//OTTENGO OGGETTI DI TIPO [roleId,invoiceStateId,permissionId]
		
		if (retrievedRoleDetails != null)
			for (Object[] rd : retrievedRoleDetails ){
				RoleDetail newDetail = new RoleDetail();
				
				InvoiceState state = (InvoiceState) entityManager.find(InvoiceState.class, ((BigInteger)rd[1]).longValue()) ;   
				InvoiceState detachedState = new InvoiceState ();
				if (state != null){
					detachedState.setDescription(state.getDescription());
					detachedState.setEndDate(state.getEndDate());
					detachedState.setInvoiceStateId(state.getInvoiceStateId());
					detachedState.setStartDate(state.getStartDate());
				}
				
				Permission p = entityManager.find(Permission.class,((BigInteger)rd[2]).longValue());
				Permission detachedPermission = new Permission();
				if (p != null){
					detachedPermission = new Permission(p.getPermissionId(),p.getCode(),p.getDescription(),p.getStartDate(),p.getEndDate());
				}
					
				
				newDetail.setInvoiceState(detachedState);
				newDetail.setPermission(detachedPermission);
				newDetail.setRole(detachedRole);
				detachedDetails.add(newDetail);
			}
		
		
		detachedRole.setRoleDetail(detachedDetails);
		
		return detachedRole;
		
//		if ( (role != null)&& (roleAttached.getRoleDetail())!=null) {
//			roleAttached.getRoleDetail().size();
//			for (RoleDetail detail : roleAttached.getRoleDetail()) {
//				Query q1 = entityManager.createNativeQuery("select * FROM RoleDetails r WHERE r.roleId = "+role.getRoleId()+";");
//				
//				RoleDetail newDetail = new RoleDetail();
//				newDetail.setInvoiceState( detail.getInvoiceState());
//				newDetail.setPermission(detail.getPermission());
//				newDetail.setRole(detail.getRole());
//				attachedDetails.add(newDetail);
//			}
//			roleAttached.setRoleDetail(attachedDetails);
//		}
//		if ((roles != null) && (roles.size() > 0 ) ) 
//			return roles.get(0) ;
//		else 
//			return null;
	}

}
