/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.user;

import it.fe.core.par.user.User;

import java.util.List;

/**
 * 
 * Interfaccia gestione utenti.  
 * 
 * @author fdimarco
 *
 */
public interface UserManager {

	/**
	 * Get an instance of Utente by id.
	 * 
	 * @param id
	 * @return
	 */
	public User getUserById(long id);
	
	/**
	 * Get an instance of Utente by username.
	 * 
	 * @param username
	 * @return
	 */
	public User getUserByUsername(String username);
	
	/**
	 * Get an instance of Utente by email address.
	 * 
	 * @param email
	 * @return
	 */
	public User getUserByEmail(String email);
	
	/**
	 * Make a Utente instance managed and persistent.
	 * 
	 * @param user
	 * @return
	 */
	public int insertUser(User user);
	
	/**
	 * Merge the state of the given user into the current persistence context.
	 * 
	 * @param user
	 * @return
	 */
	public User updateUser(User user);
	
	/**
	 * Remove a Utente instance by id.
	 * 
	 * @param id
	 * @return
	 */
	public int removeUserById(long id);
	
	/**
	 * Remove a Utente instance by username.
	 * 
	 * @param username
	 * @return
	 */
	public int removeUserByUsername(String username);
	
	/**
	 * Get all Utente instances.
	 * 
	 * @return
	 */
	public List<User> getUsers();
	
	//public Set<DettaglioProfilo> getDettagliProfili();
	
	/**
	 * Remove all ProfileDetails related to user by userId
	 * 
	 * @param userId
	 * @return the number of rows deleted
	 */
	public int removeUserProfileDetails(long userId);
	
}
