/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.user;

import it.fe.core.par.user.ProfileDetail;
import it.fe.core.par.user.User;

import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;
import javax.persistence.TransactionRequiredException;

@Stateless
@Remote(UserManager.class)
public class UserManagerBean implements UserManager {
	
	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(UserManagerBean.class.getName());
	
	private static final String _REMOVE_PROFILE_DETAILS = "delete from ProfileDetails where userId = ";

	
	/* (non-Javadoc)
	 * @see it.epocaricerca.fatturazione.core.ejb.utenti.UtentiManager#getUtenteByEmail(java.lang.String)
	 */
	public User getUserByEmail(String email) {
		User user;
		Query query = entityManager.createQuery("SELECT u FROM User u WHERE u.email = :email");
		query.setParameter("email", email);
		try {
			user = (User) query.getSingleResult();
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			user = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			user = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			user = null;
		}
		return user;
	}

	/* (non-Javadoc)
	 * @see it.epocaricerca.fatturazione.core.ejb.utenti.UtentiManager#getUtenteById(long)
	 */
	public User getUserById(long id) {
		User user;
//		Query query = entityManager.createQuery("SELECT u FROM Utente u WHERE u.id = :id");
//		query.setParameter("id", id);
		try {
//			utente = (Utente) query.getSingleResult();
			user = entityManager.find(User.class, id);
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			user = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			user = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			user = null;
		}
		
		return user;
	}

	/* (non-Javadoc)
	 * @see it.epocaricerca.fatturazione.core.ejb.utenti.UtentiManager#getUtenteByUsername(java.lang.String)
	 */
	public User getUserByUsername(String username) {
		User user;
		Query query = entityManager.createQuery("SELECT u FROM User u WHERE u.username = :username");
		query.setParameter("username", username);
		try {
			user = (User) query.getSingleResult();
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			user = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			user = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			user = null;
		}
		return user;
	}

	/* (non-Javadoc)
	 * @see it.epocaricerca.fatturazione.core.ejb.utenti.UtentiManager#getUtenti()
	 */
	@SuppressWarnings("unchecked")
	public List<User> getUsers() {
		Query query = entityManager.createQuery("SELECT u FROM User u");
		return query.getResultList();
	}

	/* (non-Javadoc)
	 * @see it.epocaricerca.fatturazione.core.ejb.utenti.UtentiManager#insertUtente(it.epocaricerca.fatturazione.core.par.utente.Utente)
	 */
	public int insertUser(User user) {
		int result;
		try {
			 entityManager.persist(user);
			 result = user.getUserId().intValue();
			 entityManager.flush();
		} catch (EntityExistsException e) {
			logger.severe(e.getMessage());
			result = -1;
		} catch (IllegalStateException e) {
			logger.severe(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.severe(e.getMessage());
			result = -1;
		} catch (PersistenceException e) {
			logger.severe(e.getMessage());
			result = -1;
		}
		return result;
	}
	
	public User updateUser(User user) {
		if (user != null) {
			return entityManager.merge(user); 
		}
		else return user;
	}

	/* (non-Javadoc)
	 * @see it.epocaricerca.fatturazione.core.ejb.utenti.UtentiManager#removeUtenteById(long)
	 */
	public int removeUserById(long id) {
		int result;
		try {
			User author = entityManager.find(User.class, id);
			entityManager.remove(author);
			result = 1;
		} catch (IllegalStateException e) {
			logger.severe(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.severe(e.getMessage());
			result = -1;
		} catch (TransactionRequiredException e) {
			logger.severe(e.getMessage());
			result = -1;
		}	
		return result;
	}

	/* (non-Javadoc)
	 * @see it.epocaricerca.fatturazione.core.ejb.utenti.UtentiManager#removeUtenteByUsername(java.lang.String)
	 */
	public int removeUserByUsername(String username) {
		// TODO Auto-generated method stub
		return 0;
	}
	
	/*
	public Set<DettaglioProfilo> getDettagliProfiliById(long id) {
		Utente utente = getUtenteById(id);
		Set<DettaglioProfilo> dettagliProfili = new HashSet<DettaglioProfilo>();
		if (utente != null) {
			dettagliProfili = utente.getProfilo();
			// nd fdimarco: this does the trick and triggers the lazy loading and 
			// all the collections will be populated
			dettagliProfili.size();
			for(Iterator<DettaglioProfilo> i = dettagliProfili.iterator(); i.hasNext();) {
				DettaglioProfilo profilo = i.next();
				Contesto contesto = profilo.getContesto();
				Ruolo ruolo = profilo.getRuolo();
			}
		}
		return dettagliProfili;
	}
	*/
	
	// ---------------------------------------------------------------------------------
	// Utils
	// ---------------------------------------------------------------------------------
	
	/**
	 * This method is used to trigger the fetching of the whole set of DettaglioProfilo of the 
	 * entity Utente, otherwise lazy loaded.
	 * 
	 * @param user
	 * @return
	 */
	private User loadProfileDetails(User user) {
		Set<ProfileDetail> profileDetails = user.getProfile();
		// nd fdimarco: this does the trick and triggers the lazy loading and 
		// all the collections will be populated
		profileDetails.size();
		return user;
	}

	public int removeUserProfileDetails(long userId) {
		String query = _REMOVE_PROFILE_DETAILS+userId;
		return entityManager.createNativeQuery(query).executeUpdate();
	}

}
