/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.utility;

import it.fe.core.par.utility.Sector;

import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.log4j.Logger;

@Stateless
@Remote(SectorManager.class)
public class SectorManagerBean implements SectorManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(MeterManagerBean.class.getName());
	

	@SuppressWarnings("unchecked")
	public List<Sector> getMeters() {
		Query query = entityManager.createQuery("SELECT s FROM Sector s");
		return query.getResultList();
	}

	public Sector findSectorById(long id) {
		Sector sector = new Sector();
		
		try {
			sector = entityManager.find(Sector.class, id);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			sector = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			sector = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			sector = null;
		}
		
		return sector;
	}
	
	public Sector findSectorByName(String sectorName) {
		Sector sector = new Sector();
		
		Query query = entityManager.createQuery("SELECT s FROM Sector s WHERE s.sectorName = ?1)");
		query.setParameter(1, sectorName);
		
		try {
			logger.info("Find sector by name " + sectorName);
			sector = (Sector) query.getSingleResult();
		} catch (NoResultException e) {
			logger.info("Unable to find sector by name " + sectorName);
			sector = findGenericSector();
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			sector = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			sector = null;
		}
		
		return sector;
	}

	public Sector findGenericSector() {
		Sector sector;
		
		Query query = entityManager.createQuery("SELECT s FROM Sector s WHERE s.sectorName = ?1)");
		query.setParameter(1, "ALTRO");
		
		try {
			logger.info("Find generic sector by name " + "ALTRO");
			sector = (Sector) query.getSingleResult();
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			sector = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			sector = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			sector = null;
		}
		
		return sector;
	}

}
