/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.utility;

import it.fe.core.par.utility.SpecificSector;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.log4j.Logger;

@Stateless
@Remote(SpecificSectorManager.class)
public class SpecificSectorManagerBean implements SpecificSectorManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(MeterManagerBean.class.getName());
	
	public SpecificSector findSpecificSectorByName(String specificSectorName) {
		SpecificSector specificSector;
		
		Query query = entityManager.createQuery("SELECT s FROM SpecificSector s WHERE s.specificSectorName = :specificSectorName");
		query.setParameter("specificSectorName", specificSectorName);
		
		try {
			logger.info("Find specific sector by name " + specificSectorName);
			specificSector = (SpecificSector) query.getSingleResult();
		} catch (NoResultException e) {
			logger.info("Unable to find specifc sector by name " + specificSectorName);
			specificSector = findGenericSpecificSector();
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			specificSector = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			specificSector = null;
		}
		
		return specificSector;
	}

	public SpecificSector findGenericSpecificSector() {
		SpecificSector specificSector;
		
		Query query = entityManager.createQuery("SELECT s FROM SpecificSector s WHERE s.specificSectorName = :specificSectorName");
		query.setParameter("specificSectorName", "ALTRO");
		
		try {
			logger.info("Find generic specific sector by name " + "ALTRO");
			specificSector = (SpecificSector) query.getSingleResult();
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			specificSector = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			specificSector = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			specificSector = null;
		}
		
		return specificSector;
	}

}
