/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.common;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Embeddable;

/**
 * La classe <b><i>Address</i></b> contiene tutte le informazioni relative agli indirizzi.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Embeddable
public class PostalAddress implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Column(name="address_postBox")
	private String postBox;
	
	@Column(name="address_streetName")
	private String streetName;
	
	@Column(name="address_floor")
	private String floor;
	
	@Column(name="address_room")
	private String room;
	
	@Column(name="address_additionalStreetName")
	private String additionalStreetName;
	
	@Column(name="address_buildingNumber")
	private String buildingNumber;
	
	@Column(name="address_department")
	private String department;
	
	@Column(name="address_cityName")
	private String cityName;
	
	@Column(name="address_postalZone")
	private String postalZone;
	
	@Column(name="address_countrySubentityCode")
	private String countrySubentityCode;
	
	@Column(name="address_country")
	private String country;
	
	@Column(name="address_countryName")
	private String countryName;
	

	//--------------- Ctors --------------------------	

	public PostalAddress() {
		super();
	}
	
	
	
	//--------------- Getters/Setters --------------------------

	/**
	 * @param postBox the postBox to set
	 */
	public void setPostBox(String postBox) {
		this.postBox = postBox;
	}

	/**
	 * @return the postBox
	 */
	public String getPostBox() {
		return postBox;
	}



	/**
	 * @return the streetName
	 */
	public String getStreetName() {
		return streetName;
	}



	/**
	 * @param streetName the streetName to set
	 */
	public void setStreetName(String streetName) {
		this.streetName = streetName;
	}



	/**
	 * @return the additionalStreetName
	 */
	public String getAdditionalStreetName() {
		return additionalStreetName;
	}



	/**
	 * @param additionalStreetName the additionalStreetName to set
	 */
	public void setAdditionalStreetName(String additionalStreetName) {
		this.additionalStreetName = additionalStreetName;
	}



	/**
	 * @return the buildingNumber
	 */
	public String getBuildingNumber() {
		return buildingNumber;
	}



	/**
	 * @param buildingNumber the buildingNumber to set
	 */
	public void setBuildingNumber(String buildingNumber) {
		this.buildingNumber = buildingNumber;
	}



	/**
	 * @return the department
	 */
	public String getDepartment() {
		return department;
	}



	/**
	 * @param department the department to set
	 */
	public void setDepartment(String department) {
		this.department = department;
	}



	/**
	 * @return the cityName
	 */
	public String getCityName() {
		return cityName;
	}



	/**
	 * @param cityName the cityName to set
	 */
	public void setCityName(String cityName) {
		this.cityName = cityName;
	}



	/**
	 * @return the postalZone
	 */
	public String getPostalZone() {
		return postalZone;
	}



	/**
	 * @param postalZone the postalZone to set
	 */
	public void setPostalZone(String postalZone) {
		this.postalZone = postalZone;
	}



	/**
	 * @return the countrySubentityCode
	 */
	public String getCountrySubentityCode() {
		return countrySubentityCode;
	}



	/**
	 * @param countrySubentityCode the countrySubentityCode to set
	 */
	public void setCountrySubentityCode(String countrySubentityCode) {
		this.countrySubentityCode = countrySubentityCode;
	}



	/**
	 * @return the country
	 */
	public String getCountry() {
		return country;
	}



	/**
	 * @param country the country to set
	 */
	public void setCountry(String country) {
		this.country = country;
	}

	/**
	 * @return the floor
	 */
	public String getFloor() {
		return floor;
	}

	/**
	 * @param floor the floor to set
	 */
	public void setFloor(String floor) {
		this.floor = floor;
	}

	/**
	 * @return the room
	 */
	public String getRoom() {
		return room;
	}

	/**
	 * @param room the room to set
	 */
	public void setRoom(String room) {
		this.room = room;
	}

	/**
	 * @return the countryName
	 */
	public String getCountryName() {
		return countryName;
	}

	/**
	 * @param countryName the countryName to set
	 */
	public void setCountryName(String countryName) {
		this.countryName = countryName;
	}
	
	public boolean equals(PostalAddress postalAddress) {
		if(!this.streetName.equalsIgnoreCase(postalAddress.getStreetName()) || !this.buildingNumber.equalsIgnoreCase(postalAddress.getBuildingNumber())
				|| !this.cityName.equalsIgnoreCase(postalAddress.getCityName()) || !this.postalZone.equalsIgnoreCase(postalAddress.getPostalZone()))
//				|| !this.countryName.equalsIgnoreCase(postalAddress.getCountryName()) || !this.floor.equalsIgnoreCase(postalAddress.getFloor())
//				|| !this.room.equalsIgnoreCase(postalAddress.getRoom()))
			return false;
		else
			return true;
	}
}
