/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.compliance;

import it.fe.core.par.invoice.TransmissionType;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

/**
 * La classe <b><i>SenderRegister</i></b> contiene le informazioni necessarie a verificare l'attendibiltà del mittente.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="SendersRegister")
public class SenderRegister implements Serializable {
	
	private static final long serialVersionUID = 1L;

	/**
	 * Identificativo
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long senderRegisterId;
	
	/**
	 * Identificativo del mittente (es. dominio della e-mail, participantId Peppol...)
	 */
	private String identificationCode;
	
	/**
	 * Canale di ricezione della fattura
	 */
	@ManyToOne
	@JoinColumn(name="transmissionTypeId", referencedColumnName="transmissionTypeId", nullable=false)
	private TransmissionType transmissionType;
	
	/**
	 * Tipologia di traduzione da applicare al file
	 */
	@ManyToOne
	@JoinColumn(name="translationTypeId", referencedColumnName="translationTypeId", nullable=false)
	private TranslationType translationType;

	
	//--------------- Ctors --------------------------
	
	public SenderRegister() {
		super();
	}


	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the senderRegisterId
	 */
	public long getSenderRegisterId() {
		return senderRegisterId;
	}

	/**
	 * @param senderRegisterId the senderRegisterId to set
	 */
	public void setSenderRegisterId(long senderRegisterId) {
		this.senderRegisterId = senderRegisterId;
	}

	/**
	 * @return the identificationCode
	 */
	public String getIdentificationCode() {
		return identificationCode;
	}

	/**
	 * @param identificationCode the identificationCode to set
	 */
	public void setIdentificationCode(String identificationCode) {
		this.identificationCode = identificationCode;
	}

	/**
	 * @return the transmissionType
	 */
	public TransmissionType getTransmissionType() {
		return transmissionType;
	}

	/**
	 * @param transmissionType the transmissionType to set
	 */
	public void setTransmissionType(TransmissionType transmissionType) {
		this.transmissionType = transmissionType;
	}

	/**
	 * @return the translationType
	 */
	public TranslationType getTranslationType() {
		return translationType;
	}

	/**
	 * @param translationType the translationType to set
	 */
	public void setTranslationType(TranslationType translationType) {
		this.translationType = translationType;
	}
}
