/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.invoice;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

/**
 * La classe <b><i>TaxDetail</i></b> mantiene le informazioni relative alle tasse; 
 * nello specifico in Italia fa riferimento al dettaglio Iva; 
 * i dati, disaggregati per aliquota, vengono calcolati sull'importo della fattura o sull'importo di ogni singola riga.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="TaxDetails")
public class TaxDetail implements Serializable {
	
	private static final long serialVersionUID = 1L;

	/**
	 * Numero progressivo assegnato al dettaglio IVA.
	 * E' un Long poichè nel ManagerBean deve essere possibile richiamare intValue()
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long taxDetailId;
	
	/**
	 * Importo totale dell'IVA applicata.
	 */
	@Column
	private Double taxAmount;
	
	/**
	 * Valuta delle tasse
	 */
	@Column
	private String taxAmountCurrency;
	
	/**
	 * La normativa richiede un arrotondamento degli importi al centesimo di euro.
	 */
	private Double roundingAmount;

	/**
	 * Base imponibile: ammontare complessivo dei beni.
	 */
	@Column
	private Double taxableAmount;	
	
	/**
	 * Aliquota IVA applicata.
	 */
	@Column
	private Double taxPercent;
	
	/**
	 * Codice IVA così come arriva dal fornitore
	 */
	@Column
	private String description;
	
	/**
	 * Descrizione
	 */
	@Column
	private String name;
	
	/**
	 * Percentuale per unita'
	 */
	@Column
	private Double perUnitAmount;
	
	/**
	 * Range di applicabilita'
	 */
	@Column
	private String tierRange;

	public TaxDetail() {
		super();
	}


	/**
	 * @param taxAmount
	 * @param roundingAmount
	 * @param taxableAmount
	 * @param taxPercent
	 */
	public TaxDetail(Double taxAmount, Double roundingAmount,
			Double taxableAmount, Double taxPercent) {
		super();
		this.taxAmount = taxAmount;
		this.roundingAmount = roundingAmount;
		this.taxableAmount = taxableAmount;
		this.taxPercent = taxPercent;
	}
	
	

	/**
	 * @return the taxDetailId
	 */
	public Long getTaxDetailId() {
		return taxDetailId;
	}

	/**
	 * @param taxDetailId the taxDetailId to set
	 */
	public void setTaxDetailId(Long taxDetailId) {
		this.taxDetailId = taxDetailId;
	}

	/**
	 * @return the taxAmount
	 */
	public double getTaxAmount() {
		return taxAmount;
	}

	/**
	 * @param taxAmount the taxAmount to set
	 */
	public void setTaxAmount(double taxAmount) {
		this.taxAmount = taxAmount;
	}

	/**
	 * @return the roundingAmount
	 */
	public double getRoundingAmount() {
		return roundingAmount;
	}

	/**
	 * @param roundingAmount the roundingAmount to set
	 */
	public void setRoundingAmount(double roundingAmount) {
		this.roundingAmount = roundingAmount;
	}

	/**
	 * @return the taxableAmount
	 */
	public double getTaxableAmount() {
		return taxableAmount;
	}

	/**
	 * @param taxableAmount the taxableAmount to set
	 */
	public void setTaxableAmount(double taxableAmount) {
		this.taxableAmount = taxableAmount;
	}

	/**
	 * @param taxAmountCurrency the taxAmountCurrency to set
	 */
	public void setTaxAmountCurrency(String taxAmountCurrency) {
		this.taxAmountCurrency = taxAmountCurrency;
	}


	/**
	 * @return the taxAmountCurrency
	 */
	public String getTaxAmountCurrency() {
		return taxAmountCurrency;
	}


	/**
	 * @return the taxPercent
	 */
	public double getTaxPercent() {
		return taxPercent;
	}

	/**
	 * @param taxPercent the taxPercent to set
	 */
	public void setTaxPercent(double taxPercent) {
		this.taxPercent = taxPercent;
	}


	public void setDescription(String description) {
		this.description = description;
	}


	public String getDescription() {
		return description;
	}


	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}


	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}


	/**
	 * @return the perUnitAmount
	 */
	public Double getPerUnitAmount() {
		return perUnitAmount;
	}


	/**
	 * @param perUnitAmount the perUnitAmount to set
	 */
	public void setPerUnitAmount(Double perUnitAmount) {
		this.perUnitAmount = perUnitAmount;
	}


	/**
	 * @return the tierRange
	 */
	public String getTierRange() {
		return tierRange;
	}


	/**
	 * @param tierRange the tierRange to set
	 */
	public void setTierRange(String tierRange) {
		this.tierRange = tierRange;
	}
	
}
