/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.par.user;

import it.fe.core.par.authorization.RoleDetail;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.CollectionOfElements;

/**
 * @author fdimarco, cpezzi
 *
 */
@Entity
@Table(name="Roles")
public class Role implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	public static final String CONSULTATORE = "CONSULTATORE";
	public static final String FIRMATARIO_CONTABILE = "FIRMATARIO_CONTABILE";
	public static final String FIRMATARIO_TECNICO = "FIRMATARIO_TECNICO";
	public static final String FUNZIONARIO_CONTABILE = "FUNZIONARIO_CONTABILE";
	public static final String FUNZIONARIO_TECNICO = "FUNZIONARIO_TECNICO";
	public static final String FUNZIONARIO_AMMINISTRATIVO = "FUNZIONARIO_AMMINISTRATIVO";
	public static final String GESTORE = "GESTORE";
	public static final String AMMINISTRATORE = "AMMINISTRATORE";
	public static final String FORNITORE = "FORNITORE";	
	
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long roleId;
	
	@Column(nullable=false)
	private String description;
	
	@Temporal(TemporalType.DATE)
	private Date startDate;
	
	@Temporal(TemporalType.DATE)
	private Date endDate;
	
	@CollectionOfElements(targetElement = RoleDetail.class, fetch=FetchType.LAZY)
	@JoinTable(name = "RoleDetails", joinColumns = @JoinColumn(name = "roleId"))
	private Set<RoleDetail> roleDetail = new HashSet<RoleDetail>();
	
	
	public Role() {
		super();
	}

	/**
	 * @return the roleId
	 */
	public long getRoleId() {
		return roleId;
	}

	/**
	 * @param roleId the roleId to set
	 */
	public void setRoleId(long roleId) {
		this.roleId = roleId;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}



	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}



	/**
	 * @return the startDate
	 */
	public Date getStartDate() {
		return startDate;
	}



	/**
	 * @param startDate the startDate to set
	 */
	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}



	/**
	 * @return the endDate
	 */
	public Date getEndDate() {
		return endDate;
	}



	/**
	 * @param endDate the endDate to set
	 */
	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}



	/**
	 * @return the roleDetail
	 */
	public Set<RoleDetail> getRoleDetail() {
		return roleDetail;
	}



	/**
	 * @param roleDetail the roleDetail to set
	 */
	public void setRoleDetail(Set<RoleDetail> roleDetail) {
		this.roleDetail = roleDetail;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((endDate == null) ? 0 : endDate.hashCode());
		result = prime * result
				+ ((startDate == null) ? 0 : startDate.hashCode());
		result = prime * result
				+ ((description == null) ? 0 : description.hashCode());

		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Role other = (Role) obj;
		if (endDate == null) {
			if (other.endDate != null)
				return false;
		} else if (!endDate.equals(other.endDate))
			return false;
		if (startDate == null) {
			if (other.startDate != null)
				return false;
		} else if (!startDate.equals(other.startDate))
			return false;
		if (description == null) {
			if (other.description != null)
				return false;
		} else if (!description.equals(other.description))
			return false;
		if (roleDetail == null) {
			if (other.roleDetail != null)
				return false;
		} else if (!roleDetail.equals(other.roleDetail))
			return false;
		if (roleId != other.roleId)
			return false;
		return true;
	}
	
}
