/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.par.user;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.CollectionOfElements;

/**
 * @author Fulvio Di Marco
 * 
 * L'entità Utente raccoglie i dati generali relativi all'utente che accede al sistema FE.
 *
 */
@Entity
@Table(name="Users")
public class User implements Serializable {

	private static final long serialVersionUID = 1L;
	
	
	/**
	 * Identificativo
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long userId;
	
	/**
	 * Anagrafica: nome utente
	 */
	@Column(nullable=false)
	private String name;
	
	/**
	 * Anagrafica: cognome utente
	 */
	@Column(nullable=false)
	private String surname;
	
	/**
	 * Anagrafica: indirizzo email utente
	 */
	private String email;
	
	/**
	 * Posizione lavorativa
	 */
	@Column(nullable=true)
	private String registrationNumber;
	
	/**
	 * Posizione lavorativa - Indica il codice dell'Unità Organizzativa di appartenenza dell'utente.
	 */
	private String uoId;
	
	/**
	 * Posizione lavorativa - Indica la descrizione dell'Unità Organizzativa di appartenenza dell'utente.
	 */
	
	private String uoDenomination;
	
	/**
	 * Autenticazione: univoco nel proprio sistema di contesto.
	 */
	@Column(nullable=false)
	private String username;
	
	@OneToMany(	mappedBy="user", 
				fetch=FetchType.EAGER, 
				cascade=CascadeType.ALL)
	private List<UserPassword> passwords;
	
//	@OneToMany(fetch=FetchType.EAGER, cascade=CascadeType.ALL)
//	@JoinTable(
//			name="User_Passwords",
//			joinColumns={ @JoinColumn(name="userId", referencedColumnName="userId") },
//			inverseJoinColumns={ @JoinColumn(name="passwordId", referencedColumnName="passwordId") }
//	)
//	private List<UserPassword> passwords;
	
	@CollectionOfElements(targetElement = ProfileDetail.class, fetch=FetchType.EAGER)
	@JoinTable(name = "ProfileDetails", joinColumns = @JoinColumn(name = "userId"))
	private Set<ProfileDetail> profile = new HashSet<ProfileDetail>();
	
	public User() {}

	public List<UserPassword> getPasswords() {
		if(passwords == null) {
			passwords = new ArrayList<UserPassword>();
		}
		return passwords;
	}

	public void setPasswords(List<UserPassword> passwords) {
		this.passwords = passwords;
	}
	
	public void addUserPassword(UserPassword userPassword) {
		userPassword.setUser(this);
	}
	
	public void removeUserPassword(UserPassword userPassword) {
		userPassword.setUser(this);
	}
	
	public void localAddUserPassword(UserPassword userPassword) {
		if (passwords == null)
			passwords = new ArrayList<UserPassword>();
		passwords.add(userPassword);
	}
	
	public void localRemoveUserPassword(UserPassword userPassword) {
		if (passwords == null)
			passwords = new ArrayList<UserPassword>();
		//passwords.remove(userPassword);
		int index = passwords.indexOf(userPassword);
		userPassword.setEndDate(new Date());
		passwords.set(index, userPassword);
	}

	public void setProfile(Set<ProfileDetail> profile) {
		this.profile = profile;
	}

	public Set<ProfileDetail> getProfile() {
		return profile;
	}

	

	/**
	 * @return the userId
	 */
	public Long getUserId() {
		return userId;
	}

	/**
	 * @param userId the userId to set
	 */
	public void setUserId(Long userId) {
		this.userId = userId;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the surname
	 */
	public String getSurname() {
		return surname;
	}

	/**
	 * @param surname the surname to set
	 */
	public void setSurname(String surname) {
		this.surname = surname;
	}

	/**
	 * @return the email
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * @param email the email to set
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @return the registrationNumber
	 */
	public String getRegistrationNumber() {
		return registrationNumber;
	}

	/**
	 * @param registrationNumber the registrationNumber to set
	 */
	public void setRegistrationNumber(String registrationNumber) {
		this.registrationNumber = registrationNumber;
	}

	/**
	 * @return the uoId
	 */
	public String getUoId() {
		return uoId;
	}

	/**
	 * @param uoId the uoId to set
	 */
	public void setUoId(String uoId) {
		this.uoId = uoId;
	}

	/**
	 * @return the username
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * @param username the username to set
	 */
	public void setUsername(String username) {
		this.username = username;
	}

	/**
	 * @return the uoDenomination
	 */
	public String getUoDenomination() {
		return uoDenomination;
	}

	/**
	 * @param uoDenomination the uoDenomination to set
	 */
	public void setUoDenomination(String uoDenomination) {
		this.uoDenomination = uoDenomination;
	}

	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((surname == null) ? 0 : surname.hashCode());
		result = prime * result
				+ ((uoDenomination == null) ? 0 : uoDenomination.hashCode());
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		result = prime * result + ((uoId == null) ? 0 : uoId.hashCode());
		result = prime * result
				+ ((userId == null) ? 0 : userId.hashCode());
		result = prime * result
				+ ((registrationNumber == null) ? 0 : registrationNumber.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result
				+ ((passwords == null) ? 0 : passwords.hashCode());
		result = prime * result
				+ ((username == null) ? 0 : username.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof User))
			return false;
		User other = (User) obj;
		if (surname == null) {
			if (other.surname != null)
				return false;
		} else if (!surname.equals(other.surname))
			return false;
		if (uoDenomination == null) {
			if (other.uoDenomination != null)
				return false;
		} else if (!uoDenomination.equals(other.uoDenomination))
			return false;
		if (email == null) {
			if (other.email != null)
				return false;
		} else if (!email.equals(other.email))
			return false;
		if (uoId == null) {
			if (other.uoId != null)
				return false;
		} else if (!uoId.equals(other.uoId))
			return false;
		if (userId == null) {
			if (other.userId != null)
				return false;
		} else if (!userId.equals(other.userId))
			return false;
		if (registrationNumber == null) {
			if (other.registrationNumber != null)
				return false;
		} else if (!registrationNumber.equals(other.registrationNumber))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (passwords == null) {
			if (other.passwords != null)
				return false;
		} else if (!passwords.equals(other.passwords))
			return false;
		if (profile == null) {
			if (other.profile != null)
				return false;
		} else if (!profile.equals(other.profile))
			return false;
		if (username == null) {
			if (other.username != null)
				return false;
		} else if (!username.equals(other.username))
			return false;
		return true;
	}

	
}


