/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.par.utility;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@Table(name="Consumptions")
public class Consumption implements Serializable{
	
	private static final long serialVersionUID = 1L;

	/**
	 * Numero progressivo assegnato alla riga fattura.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long consumptionId;
	
	/**
	 * Data di inizio consumo addebitato.
	 */
	@Temporal(TemporalType.DATE)
	private Date startConsumptionDate;
	
	/**
	 * Data di fine consumo addebitato.
	 */
	@Temporal(TemporalType.DATE)
	private Date endConsumptionDate;
	
	/**
	 * Fascia di consumo.
	 */
	private String consumptionBracket;

	/**
	 * Tipologia di consumo.
	 */
	private String consumptionType;
	
	/**
	 * Indica il periodo su cui e' calcolato il consumo.
	 */
	private String descriptionCode;
	
	//--------------- Ctors --------------------------
	
	public Consumption() {
		super();
	}

	public Consumption(Date startConsumptionDate, Date endConsumptionDate,
			String consumptionBracket) {
		super();
		this.startConsumptionDate = startConsumptionDate;
		this.endConsumptionDate = endConsumptionDate;
		this.consumptionBracket = consumptionBracket;
	}

	
	//--------------- Getters/Setters --------------------------
	
	public long getConsumptionId() {
		return consumptionId;
	}

	public void setConsumptionId(long consumptionId) {
		this.consumptionId = consumptionId;
	}

	public Date getStartConsumptionDate() {
		return startConsumptionDate;
	}

	public void setStartConsumptionDate(Date startConsumptionDate) {
		this.startConsumptionDate = startConsumptionDate;
	}

	public Date getEndConsumptionDate() {
		return endConsumptionDate;
	}

	public void setEndConsumptionDate(Date endConsumptionDate) {
		this.endConsumptionDate = endConsumptionDate;
	}

	public String getConsumptionBracket() {
		return consumptionBracket;
	}

	public void setConsumptionBracket(String consumptionBracket) {
		this.consumptionBracket = consumptionBracket;
	}

	/**
	 * @return the consumptionType
	 */
	public String getConsumptionType() {
		return consumptionType;
	}

	/**
	 * @param consumptionType the consumptionType to set
	 */
	public void setConsumptionType(String consumptionType) {
		this.consumptionType = consumptionType;
	}

	/**
	 * @return the descriptionCode
	 */
	public String getDescriptionCode() {
		return descriptionCode;
	}

	/**
	 * @param descriptionCode the descriptionCode to set
	 */
	public void setDescriptionCode(String descriptionCode) {
		this.descriptionCode = descriptionCode;
	}
	
}
