/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author mattia.generali@epocaricerca.it
 * 
 * <br /><br />
* 
 *
 */

package it.fe.core.par.utility;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.JoinColumn;

import org.hibernate.annotations.CascadeType;

/**
 * La classe <b><i>MeterProperty</i></b> contiene le caratteristiche del contatore.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="MeterProperty")
public class MeterProperty implements Serializable {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo univoco associato alla proprieta
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long meterPropertyId;
	
	/**
	 * Nome della proprieta
	 */
	@Column
	private String nameCode;
	
	/**
	 * Valore della proprieta (in caso di dati di tipo testuale)
	 */
	@Column
	private String value;
	
	/**
	 * Valore della proprieta (in caso di dati di tipo numerico)
	 */
	@Column
	private Double valueQuantity;
	
	/**
	 * Unita di misura riferita al valore
	 */
	@Column
	private String valueQuantityUnitCode;

	public MeterProperty(String nameCode, String value, Double valueQuantity, String valueQuantityUnitCode) {
		this.nameCode = nameCode;
		this.value = value;
		this.valueQuantity = valueQuantity;
		this.valueQuantityUnitCode = valueQuantityUnitCode;
	}
	
	/**
	 * Riferimento all'oggetto Meter
	 */
	@ManyToOne(fetch=FetchType.EAGER)
	@JoinColumn(name="meterId", referencedColumnName="meterId")
	private Meter meter;
	
	/**
	 * @return the meterPropertyId
	 */
	public Long getMeterPropertyId() {
		return meterPropertyId;
	}

	/**
	 * @param meterPropertyId the meterPropertyId to set
	 */
	public void setMeterPropertyId(Long meterPropertyId) {
		this.meterPropertyId = meterPropertyId;
	}

	/**
	 * @return the nameCode
	 */
	public String getNameCode() {
		return nameCode;
	}

	/**
	 * @param nameCode the nameCode to set
	 */
	public void setNameCode(String nameCode) {
		this.nameCode = nameCode;
	}

	/**
	 * @return the value
	 */
	public String getValue() {
		return value;
	}

	/**
	 * @param value the value to set
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/**
	 * @return the valueQuantity
	 */
	public Double getValueQuantity() {
		return valueQuantity;
	}

	/**
	 * @param valueQuantity the valueQuantity to set
	 */
	public void setValueQuantity(Double valueQuantity) {
		this.valueQuantity = valueQuantity;
	}

	/**
	 * @return the valueQuantityUnitCode
	 */
	public String getValueQuantityUnitCode() {
		return valueQuantityUnitCode;
	}

	/**
	 * @param valueQuantityUnitCode the unitCode to set
	 */
	public void setValueQuantityUnitCode(String valueQuantityUnitCode) {
		this.valueQuantityUnitCode = valueQuantityUnitCode;
	}

	/**
	 * @return the meter
	 */
	public Meter getMeter() {
		return meter;
	}

	/**
	 * @param meter the meter to set
	 */
	public void setMeter(Meter meter) {
		this.meter = meter;
	}
}
