/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.utility;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

 
 /**
  * La classe <b><i>MeterSummary</i></b> contiene ub riepilogo dei costi e dei consumi specifici di un conatatore per un certo periodo fatturato.
  * 
  * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
  *
  */
 @Entity
 @Table(name="MeterSummaries")
 public class MeterSummary implements Serializable {
	
	private static final long serialVersionUID = 1L;
	

	/**
	 * Numero progressivo assegnato al riassunto
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long meterSummaryId;
	
	/**
	 * Indentificativo all'interno dell'Utility Statement
	 */
	private String id;
	
	/**
	 * Quantità consumata nel periodo di consumo complessivo della fattura.
	 */
	private Double meterInvoicedQuantity;
	
	/**
	 * Quantità in acconto alla data di emissione della fattura.
	 */
	private Double meterOnAccountQuantity;
	
	/**
	 * Quantità complessiva a conguaglio alla data di emissione della fattura.
	 */
	private Double meterBalanceQuantity;
	
	/**
	 * Unità di misura della quantità fatturata.
	 */
	private String meterSummaryUnitCode;
	
	/**
	 * Tipologia corrispettivo.
	 */
	private String consumptionType;
	
	/**
	 * Quantita' rilevata.
	 */
	private Double consumedQuantity;
	
	/**
	 * Descrizione.
	 */
	@Lob
	private String meterSummaryDescription;
	
	/**
	 * Fascia - scaglione.
	 */
	private String consumersEnergyLevel;
	
	/**
	 * Importo dovuto per la quantita' rilevata.
	 */
	private double amount;
	
	/**
	 * Data inizio consumo
	 */
	@Temporal(TemporalType.DATE)
	private Date startDate;
	
	/**
	 * Data fine consumo
	 */
	@Temporal(TemporalType.DATE)
	private Date endDate;
	
	/**
	 * Riferimento al contatore.
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="meterId", referencedColumnName="meterId", nullable=false)
	private Meter meter;
	

	/**
	 * @return the meterSummaryId
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	public Long getMeterSummaryId() {
		return meterSummaryId;
	}
	
	
	//--------------- Ctors --------------------------
	
	public MeterSummary() {
		super();
	}
	
	
	public MeterSummary(Double meterInvoicedQuantity,
			Double meterOnAccountQuantity, Double meterBalanceQuantity,
			String meterSummaryUnitCode, Meter meter) {
		super();
		this.meterInvoicedQuantity = meterInvoicedQuantity;
		this.meterOnAccountQuantity = meterOnAccountQuantity;
		this.meterBalanceQuantity = meterBalanceQuantity;
		this.meterSummaryUnitCode = meterSummaryUnitCode;
		this.meter = meter;
	}



	//--------------- Getters/Setters --------------------------
	
	/**
	 * @param meterSummaryId the meterSummaryId to set
	 */
	public void setMeterSummaryId(Long meterSummaryId) {
		this.meterSummaryId = meterSummaryId;
	}

	/**
	 * @return the meterInvoicedQuantity
	 */
	public Double getMeterInvoicedQuantity() {
		return meterInvoicedQuantity;
	}

	/**
	 * @param meterInvoicedQuantity the meterInvoicedQuantity to set
	 */
	public void setMeterInvoicedQuantity(Double meterInvoicedQuantity) {
		this.meterInvoicedQuantity = meterInvoicedQuantity;
	}

	/**
	 * @return the meterOnAccountQuantity
	 */
	public Double getMeterOnAccountQuantity() {
		return meterOnAccountQuantity;
	}

	/**
	 * @param meterOnAccountQuantity the meterOnAccountQuantity to set
	 */
	public void setMeterOnAccountQuantity(Double meterOnAccountQuantity) {
		this.meterOnAccountQuantity = meterOnAccountQuantity;
	}

	/**
	 * @return the meterBalanceQuantity
	 */
	public Double getMeterBalanceQuantity() {
		return meterBalanceQuantity;
	}

	/**
	 * @param meterBalanceQuantity the meterBalanceQuantity to set
	 */
	public void setMeterBalanceQuantity(Double meterBalanceQuantity) {
		this.meterBalanceQuantity = meterBalanceQuantity;
	}

	/**
	 * @return the meterSummaryUnitCode
	 */
	public String getMeterSummaryUnitCode() {
		return meterSummaryUnitCode;
	}

	/**
	 * @param meterSummaryUnitCode the meterSummaryUnitCode to set
	 */
	public void setMeterSummaryUnitCode(String meterSummaryUnitCode) {
		this.meterSummaryUnitCode = meterSummaryUnitCode;
	}

	/**
	 * @return the meter
	 */
	public Meter getMeter() {
		return meter;
	}

	/**
	 * @param meter the meter to set
	 */
	public void setMeter(Meter meter) {
		this.meter = meter;
	}


	/**
	 * @return the consumptionType
	 */
	public String getConsumptionType() {
		return consumptionType;
	}


	/**
	 * @param consumptionType the consumptionType to set
	 */
	public void setConsumptionType(String consumptionType) {
		this.consumptionType = consumptionType;
	}


	/**
	 * @return the meterSummaryDescription
	 */
	public String getMeterSummaryDescription() {
		return meterSummaryDescription;
	}


	/**
	 * @param meterSummaryDescription the meterSummaryDescription to set
	 */
	public void setMeterSummaryDescription(String meterSummaryDescription) {
		this.meterSummaryDescription = meterSummaryDescription;
	}


	/**
	 * @return the consumersEnergyLevel
	 */
	public String getConsumersEnergyLevel() {
		return consumersEnergyLevel;
	}


	/**
	 * @param consumersEnergyLevel the consumersEnergyLevel to set
	 */
	public void setConsumersEnergyLevel(String consumersEnergyLevel) {
		this.consumersEnergyLevel = consumersEnergyLevel;
	}


	/**
	 * @return the startDate
	 */
	public Date getStartDate() {
		return startDate;
	}


	/**
	 * @param startDate the startDate to set
	 */
	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}


	/**
	 * @return the endDate
	 */
	public Date getEndDate() {
		return endDate;
	}


	/**
	 * @param endDate the endDate to set
	 */
	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}


	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}


	/**
	 * @param id the id to set
	 */
	public void setId(String id) {
		this.id = id;
	}


	/**
	 * @return the totalConsumedQuantity
	 */
	public Double getConsumedQuantity() {
		return consumedQuantity;
	}


	/**
	 * @param totalConsumedQuantity the totalConsumedQuantity to set
	 */
	public void setConsumedQuantity(Double consumedQuantity) {
		this.consumedQuantity = consumedQuantity;
	}


	/**
	 * @return the amount
	 */
	public double getAmount() {
		return amount;
	}


	/**
	 * @param amount the amount to set
	 */
	public void setAmount(double amount) {
		this.amount = amount;
	}
}
