/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.utility;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@Table(name="Readings")
public class Reading implements Serializable{
	
	private static final long serialVersionUID = 1L;

	/**
	 * Numero progressivo assegnato alla riga fattura.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long readingId;
	
	/**
	 * Codice lettura dato dal fornitore.
	 */
	private String readingNumber;
	
	/**
	 * Data della lettura.
	 */
	@Temporal(TemporalType.DATE)
	private Date readingDate;
	
	/**
	 * Tipo lettura (es. lettura periodica...).
	 */
	private String readingType;	
	
	/**
	 * Metodo di lettura (es. operatore, autolettura, stimata...).
	 */
	private String readingMethod;
	
	/**
	 * Descrizione della lettura (es. lettura periodica, lettura stimata...).
	 */
	private String readingDescription;
	
	/**
	 * Valore della lettura.
	 */
	private double readingValue;
	
	/**
	 * Unità di misura della lettura.
	 */
	private String readingUnitCode;
	
	/**
	 * Riferimento al contatore.
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="meterId", referencedColumnName="meterId")
	private Meter meter;

	
	//--------------- Ctors --------------------------
	
	public Reading() {
		super();
	}

	public Reading(String readingNumber, Date readingDate,
			String readingType, String readingMethod, double readingValue, String readingUnitCode,
			Meter meter) {
		super();
		this.readingNumber = readingNumber;
		this.readingDate = readingDate;
		this.readingType = readingType;
		this.readingMethod = readingMethod;
		this.readingValue = readingValue;
		this.readingUnitCode = readingUnitCode;
		this.meter = meter;
	}

	
	//--------------- Getters/Setters --------------------------
	
	public long getReadingId() {
		return readingId;
	}

	public void setReadingId(long readingId) {
		this.readingId = readingId;
	}

	public String getReadingNumber() {
		return readingNumber;
	}

	public void setReadingNumber(String readingNumber) {
		this.readingNumber = readingNumber;
	}

	public Date getReadingDate() {
		return readingDate;
	}

	public void setReadingDate(Date readingDate) {
		this.readingDate = readingDate;
	}

	public String getReadingDescription() {
		return readingDescription;
	}

	public void setReadingDescription(String readingDescription) {
		this.readingDescription = readingDescription;
	}

	public double getReadingValue() {
		return readingValue;
	}

	public void setReadingValue(double readingValue) {
		this.readingValue = readingValue;
	}

	public String getReadingUnitCode() {
		return readingUnitCode;
	}

	public void setReadingUnitCode(String readingUnitCode) {
		this.readingUnitCode = readingUnitCode;
	}

	public Meter getMeter() {
		return meter;
	}

	public void setMeter(Meter meter) {
		this.meter = meter;
	}

	/**
	 * @return the readingType
	 */
	public String getReadingType() {
		return readingType;
	}

	/**
	 * @param readingType the readingType to set
	 */
	public void setReadingType(String readingType) {
		this.readingType = readingType;
	}

	/**
	 * @return the readingMethod
	 */
	public String getReadingMethod() {
		return readingMethod;
	}

	/**
	 * @param readingMethod the readingMethod to set
	 */
	public void setReadingMethod(String readingMethod) {
		this.readingMethod = readingMethod;
	}

}
