/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.utility;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * La classe <b><i>SpecificSector</i></b> mantiene un'anagrafica delle forniture.
 * er fornitura si intende una specializzazione del settore merceologico.
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="SpecificSectors")
public class SpecificSector implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Numero progressivo assegnato alla fornitura
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long specificSectorId;
	
	/**
	 * Descrizione della fornitura (es. idrante)
	 */
	private String specificSectorName;
	
	/**
	 * Riferimento al settore merceologico.
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="sectorId", referencedColumnName="sectorId", nullable=false)
	private Sector sector;
	
	/**
	 * Data attivazione fornitura
	 */
	@Temporal(TemporalType.DATE)
	private Date startDate;
	
	/**
	 * Data cessazione fornitura
	 */
	@Temporal(TemporalType.DATE)
	private Date endDate;

	
	
	//--------------- Ctors --------------------------
		
	public SpecificSector() {
		super();
	}

	public SpecificSector(String specificSectorName, Sector sector,
			Date startDate) {
		super();
		this.specificSectorName = specificSectorName;
		this.sector = sector;
		this.startDate = startDate;
	}

	
	//--------------- Getters/Setters --------------------------
	
	public Long getSpecificSectorId() {
		return specificSectorId;
	}

	public void setSpecificSectorId(Long specificSectorId) {
		this.specificSectorId = specificSectorId;
	}

	public String getSpecificSectorName() {
		return specificSectorName;
	}

	public void setSpecificSectorName(String specificSectorName) {
		this.specificSectorName = specificSectorName;
	}

	public Sector getSector() {
		return sector;
	}

	public void setSector(Sector sector) {
		this.sector = sector;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

}
