/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.utility;

import it.fe.core.par.invoice.PreviousDocument;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * La classe <b><i>SupplyInfo</i></b> mantiene le informazioni legate alla fornitura dipendenti anche dai contratti (e.g. potenza massima).
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="SupplyInfos")
public class SupplyInfo implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Numero progressivo assegnato alle informazioni fornitura
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long supplyInfoId;
	
	/**
	 * Riferimento alla fornitura
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="specificSectorId", referencedColumnName="specificSectorId")
	private SpecificSector specificSector;
	
	/**
	 * Riferimento al contratto
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="previousDocumentId", referencedColumnName="previousDocumentId")
	private PreviousDocument previousDocument;
	
	/**
	 * Valore del limite massimo di consumo (potenza massima per l'energia elettrica, il gas, l'acqua o valore dei mq per l'ambiente)
	 */
	private Double consumptionLimit;

	/**
	 * Unità di misura del valore riportato come limite massimo (es. KW per energia elettrica, Mcal per gas, M3/mese per acqua, mq per ambiente)
	 */
	private String limitUnitCode;
	
	/**
	 * Data inizio validità fornitura
	 */
	@Temporal(TemporalType.DATE)
	private Date startDate;
	
	/**
	 * Data fine validità fornitura
	 */
	@Temporal(TemporalType.DATE)
	private Date endDate;

	
	//--------------- Ctors --------------------------
	
	public SupplyInfo() {
		super();
	}

	public SupplyInfo(SpecificSector specificSector, PreviousDocument previousDocument,
			Double consumptionLimit, String limitUnitCode, Date startDate) {
		super();
		this.specificSector = specificSector;
		this.previousDocument = previousDocument;
		this.consumptionLimit = consumptionLimit;
		this.limitUnitCode = limitUnitCode;
		this.startDate = startDate;
	}
	
	
	//--------------- Getters/Setters --------------------------
	
	public Long getSupplyInfoId() {
		return supplyInfoId;
	}

	public void setSupplyInfoId(Long supplyInfoId) {
		this.supplyInfoId = supplyInfoId;
	}

	public SpecificSector getSpecificSector() {
		return specificSector;
	}

	public void setSpecificSector(SpecificSector specificSector) {
		this.specificSector = specificSector;
	}

	public PreviousDocument getPreviousDocument() {
		return previousDocument;
	}

	public void setPreviousDocument(PreviousDocument previousDocument) {
		this.previousDocument = previousDocument;
	}

	public Double getConsumptionLimit() {
		return consumptionLimit;
	}

	public void setConsumptionLimit(Double consumptionLimit) {
		this.consumptionLimit = consumptionLimit;
	}

	public String getLimitUnitCode() {
		return limitUnitCode;
	}

	public void setLimitUnitCode(String limitUnitCode) {
		this.limitUnitCode = limitUnitCode;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}
}
