/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 * <br /><br />
* 
 *
 */
package it.fe.core.par.utility;


import it.fe.core.par.common.PostalAddress;

import java.io.Serializable;
import java.util.List;

import javax.persistence.Entity;

import javax.persistence.CascadeType;
import javax.persistence.Embedded;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;



/**
 * La classe <b><i>UtilityConsumptionPoint</i></b> contiene le informazioni relative al punto di connessione di un contatore.
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="UtilityConsumptionPoints")
public class UtilityConsumptionPoint implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	public enum RegistrationUserType { AUTOMATICO }
	

	/**
	 * Numero progressivo assegnato al contatore
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long utilityConsumptionPointId;
	
	
	/**
	 * Numero di presa (punto di connessione): POD per l'elettricità, PDR per il gas
	 */
	private String connectionNumber;

	
	/**
	 * Descrizione del punto di connessione
	 */
	private String description;
	
	
	/**
	 * Indirizzo del contatore.
	 */
	@Embedded
	private PostalAddress utilityConsumptionPointAddress;
	
	
	/**
	 * Contatori associati
	 */
	@OneToMany( fetch=FetchType.LAZY)
	@JoinColumn(name="utilityConsumptionPointId", referencedColumnName="utilityConsumptionPointId")
	private List<Meter> meterList;

	
	
	//--------------- Ctors --------------------------

	public UtilityConsumptionPoint() {
		super();
		// TODO Auto-generated constructor stub
	}


	public UtilityConsumptionPoint(String connectionNumber) {
		super();
		this.connectionNumber = connectionNumber;
	}



	
	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the utilityConsumptionPointId
	 */
	public Long getUtilityConsumptionPointId() {
		return utilityConsumptionPointId;
	}


	/**
	 * @param utilityConsumptionPointId the utilityConsumptionPointId to set
	 */
	public void setUtilityConsumptionPointId(Long utilityConsumptionPointId) {
		this.utilityConsumptionPointId = utilityConsumptionPointId;
	}


	/**
	 * @return the connectionNumber
	 */
	public String getConnectionNumber() {
		return connectionNumber;
	}


	/**
	 * @param connectionNumber the connectionNumber to set
	 */
	public void setConnectionNumber(String connectionNumber) {
		this.connectionNumber = connectionNumber;
	}


	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}


	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}


	/**
	 * @return the utilityConsumptionPointAddress
	 */
	public PostalAddress getUtilityConsumptionPointAddress() {
		return utilityConsumptionPointAddress;
	}


	/**
	 * @param utilityConsumptionPointAddress the utilityConsumptionPointAddress to set
	 */
	public void setUtilityConsumptionPointAddress(
			PostalAddress utilityConsumptionPointAddress) {
		this.utilityConsumptionPointAddress = utilityConsumptionPointAddress;
	}


	public void setMeterList(List<Meter> meterList) {
		this.meterList = meterList;
	}


	public List<Meter> getMeterList() {
		return meterList;
	}

}
