/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author mattia.generali@epocaricerca.it
 * 
 * <br /><br />
* 
 *
 */

package it.fe.core.par.utility;

import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.Invoice;

import java.io.Serializable;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

/**
 * La classe <b><i>UtilitySupplier</i></b> mantiene le informazioni relative al gestore della rete di distribuzione.
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */

@Entity
@Table(name="UtilitySupplier")
public class UtilitySupplier implements Serializable{

private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo univoco associato al gestore
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long utilitySupplierId;
	
	/**
	 * Nome del gestore
	 */
	@Column
	private String name;
	
	/**
	 * Indirizzo del gestore
	 */
	@Column
	private PostalAddress address;
	
	/**
	 * Partita IVA del gestore
	 */
	@Column
	private String companyId;
	
	/**
	 * Relazione con l'oggetto Invoice
	 */
	@OneToOne( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="invoiceId", referencedColumnName="invoiceId")
	private Invoice invoice;
	
	/**
	 * Informazioni sui contatti del gestore
	 */
	@OneToMany( fetch=FetchType.EAGER, cascade=CascadeType.ALL )
	@JoinColumn(name="utilitySupplierId", referencedColumnName="utilitySupplierId")
	private List<Contact> contactList;
	
	public UtilitySupplier(String name, PostalAddress address, String companyId) {
		this.setName(name);
		this.setAddress(address);
		this.setCompanyId(companyId);
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the address
	 */
	public PostalAddress getAddress() {
		return address;
	}

	/**
	 * @param address the address to set
	 */
	public void setAddress(PostalAddress address) {
		this.address = address;
	}

	/**
	 * @return the companyId
	 */
	public String getCompanyId() {
		return companyId;
	}

	/**
	 * @param companyId the companyId to set
	 */
	public void setCompanyId(String companyId) {
		this.companyId = companyId;
	}

	/**
	 * @return the invoice
	 */
	public Invoice getInvoice() {
		return invoice;
	}

	/**
	 * @param invoice the invoice to set
	 */
	public void setInvoice(Invoice invoice) {
		this.invoice = invoice;
	}

	/**
	 * @return the contactList
	 */
	public List<Contact> getContactList() {
		return contactList;
	}

	/**
	 * @param contactList the contactList to set
	 */
	public void setContactList(List<Contact> contactList) {
		this.contactList = contactList;
	}
}
