/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.process.listener;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.process.ProcessVariables;

import java.util.Date;

import org.apache.log4j.Logger;
import org.jbpm.api.listener.EventListener;
import org.jbpm.api.listener.EventListenerExecution;

public class PerformanceLogOnEntry implements EventListener {

	private static Logger logger = Logger
			.getLogger(PerformanceLogOnEntry.class);

	// E' necessario configurare il processo in questa maniera
	//	<event-listener
	//   class="it.fe.core.process.listener.PerformanceLogOnEntry">
	// </event-listener>

	/**
	 * Scrive su DB la Tabella TaskTiming, con i dati necessari per l'apertura di un task
	 * 
	 * 
	 */
	public void notify(EventListenerExecution execution) throws Exception {

		Long invoiceId = (Long) execution
				.getVariable(ProcessVariables.INVOICE_ID);
		if (invoiceId == null) {
			logger.warn(this.getClass().getSimpleName()
					+ " - No valid invoiceId in execution context: "
					+ execution.getVariable(ProcessVariables.INVOICE_ID));
			throw new PerformanceLogException(this.getClass().getSimpleName()
					+ " - No valid invoiceId in execution context");
		}

		if (!JNDIUtils.getInvoiceManager().invoiceExists(invoiceId)) {
			logger.warn(this.getClass().getSimpleName()
					+ " - Wrong invoice id or no valid invoice: " + invoiceId);
			throw new PerformanceLogException(this.getClass().getSimpleName()
					+ " - Wrong invoice id or no valid invoice");
		}

		JNDIUtils.getProcessPerformanceManager().addTaskTiming(invoiceId,
				execution.getProcessDefinitionId(), execution.getId(),
				execution.getState(),
				new Date());
	}

}
