/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.process.listener;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.process.ProcessVariables;

import java.util.Date;

import org.apache.log4j.Logger;
import org.jbpm.api.listener.EventListener;
import org.jbpm.api.listener.EventListenerExecution;

public class PerformanceLogOnExit implements EventListener {

	private static Logger logger = Logger
			.getLogger(PerformanceLogOnExit.class);

	// E' necessario configurare il processo in questa maniera
	//	<event-listener
	//   class="it.fe.core.process.listener.PerformanceLogOnExit">
	// </event-listener>

	/**
	 * Aggiorna su DB la Tabella TaskTiming, con i dati necessari a "chiudere" la gestione di un task
	 * 
	 * ATTENZIONE: lo stato della fattura da salvare nella tabella TaskTiming viene preso dallo stato corrente della fattura 
	 * 	--> il meccanismo di apertura/chiusura  TaskTiming rimane coerente solo se i due Listener (PerformanceLogOnEntry e PerformanceLogOnExit) 
	 * vengono chiamati rispettivamente dopo la transizione da stato vecchio a nuovo e prima della transizione da corrente a nuovo.
	 * 
	 *  Esempio: supponendo di avere la seguente transizione e di avere agganciato i due listener a Task2
	 *  	
	 *  	Task1			--->		Task2				--->	Task3				 
	 *  Stato:RICEVUTA				Stato:VALIDATA TECH			Stato:CONTABILIZZATA
	 *  
	 *  allora 
	 *  	PerformanceLogOnEntry dovrà essere chiamata dopo aver fatto transitare la fattura da stato RICEVUTA a VALIDATA TECH
	 *  	PerformanceLogOnExit  dovrà essere chiamata PRIMA della transizione da da stato RICEVUTA a VALIDATA TECH
	 * 
	 */
	public void notify(EventListenerExecution execution) throws Exception {

		Long invoiceId = (Long) execution
				.getVariable(ProcessVariables.INVOICE_ID);
		if (invoiceId == null) {
			logger.warn(this.getClass().getSimpleName()
					+ " - No valid invoiceId in execution context: "
					+ execution.getVariable(ProcessVariables.INVOICE_ID));
			throw new PerformanceLogException(this.getClass().getSimpleName()
					+ " - No valid invoiceId in execution context");
		}

		if (!JNDIUtils.getInvoiceManager().invoiceExists(invoiceId)) {
			logger.warn(this.getClass().getSimpleName()
					+ " - Wrong invoice id or no valid invoice: " + invoiceId);
			throw new PerformanceLogException(this.getClass().getSimpleName()
					+ " - Wrong invoice id or no valid invoice");
		}
		
		String username = (String) execution.getVariable(ProcessVariables.USERNAME);
		if (  (username == null) || (username.equals("")) ){
			logger.warn(this.getClass().getSimpleName()
					+ " - No valid username");
		}

		JNDIUtils.getProcessPerformanceManager().updateTaskTimingEndDate
		(invoiceId,
				execution.getId(),
				execution.getState(),
				username ,new Date());
	}

}
