/**
 * This file is part of FatturazioneElettronica - ESB
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - ESB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - ESB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - ESB.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.esb.common;

/**
 * Verifiche di attendibilità dei file ricevuti.
 * <br /><br />
 * Ricezione: fase di accettazione
 *
 */
public class CheckedDocumentUtils {
	
	/**
	 * Fornitori gestiti
	 */
	public enum supplier { 
		HERASpa("HERA", "04245520376", "gruppohera.it", "txt"), 
		HERAComm("HERA", "02221101203", "gruppohera.it", "txt"), 
		Generic("UBL", "", "", "xml"),
		TestHera("HERA", "04245520376", "epocaricerca.it", "txt");
				
		private String invoiceType;
		private String vatId;
		private String emailDomain;
		private String fileExtension;
		
		supplier(String invoiceType, String vatId, String emailDomain, String fileExtension){
			this.invoiceType = invoiceType;
			this.vatId = vatId;
			this.emailDomain = emailDomain;
			this.fileExtension = fileExtension;
		}
		
		
		/**
		 * @return il tipo di fattura
		 */
		public String getInvoiceType() { 
		    return this.invoiceType; 
		}
		
		/**
		 * @return la partita iva del fornitore gestito
		 */
		public String getVatId() { 
		    return this.vatId; 
		}
		
		/**
		 * @return il dominio email del fornitore
		 */
		public String getEmailDomain() { 
		    return this.emailDomain; 
		}
		
		/**
		 * @return il file gestito per il fornitore
		 */
		public String getFileExtension() { 
		    return this.fileExtension; 
		}
		
		
	}
	
	/**
	 * Estensioni gestite
	 */
	//public enum fileExtension { TXT, XML }
	
	/**
	 * Domini e-mail gestiti, associati ai relativi fornitori
	 */
	public enum emailDomain { 
		HERASpa("epocaricerca.it"), HERAComm("epocaricerca.it");
		
		private String txt;
		
		emailDomain(String txt){
			this.txt = txt;
		}
		
		/**
		 * @return il dominio e-mail
		 */
		public String getTxt() { 
		    return this.txt; 
		}
	}
	
	
	/**
	 * Verifica che il fornitore associato al file ricevuto sia riconosciuto nel sistema, 
	 * attraverso il dominio dell'e-mail e l'estensione del file
	 * 
	 * @param emailDomain - (String) dominio dell'e-mail ricevuta
	 * @return (String) tipo di fattura
	 */
	public static String checkSupplierByEmailDomainAndFileExtension(String supplierEmailDomain, String fileExtension){
		for(supplier s : supplier.values()){
			if((supplierEmailDomain.equalsIgnoreCase(s.getEmailDomain())) &&
					(fileExtension.equalsIgnoreCase(s.getFileExtension()))){
				return s.getInvoiceType();
			}
		}

		return null;
	}
	
	/**
	 * Verifica che il fornitore associato al file ricevuto sia riconosciuto nel sistema, 
	 * attraverso la partita iva e l'estensione del file
	 * 
	 * @param fileExtension - (String) estensione del file ricevuto
	 * @param expectedFileExtension - (String) estensione del file aspettata
	 * @return (String) tipo di fattura
	 */
	public static String checkSupplierByVatIdAndFileExtension(String supplierVatId, String fileExtension){
		for(supplier s : supplier.values()){
			if((supplierVatId.equalsIgnoreCase(s.getVatId())) &&
					(fileExtension.equalsIgnoreCase(s.getFileExtension()))){
				return s.getInvoiceType();
			}
		}

		return null;
	}
}
