/**
 * This file is part of FatturazioneElettronica - ESB
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - ESB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - ESB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - ESB.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.esb.compliance;

import java.util.ArrayList;
import java.util.List;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.TransmissionType;
import it.fe.esb.common.CheckedDocument;
import it.fe.esb.common.CheckedDocumentUtils;

import org.apache.log4j.Logger;

/**
 * Classe che si occupa di verificare l'attendibilità dei file ricevti via e-mail
 * <br /><br />
 * Ricezione: fase di accettazione
 *
 */
public class EMailCompliance {

	protected static Logger logger = Logger.getLogger(EMailCompliance.class.getName());

	
	/**
	 * Se i file ricevuti rispettano i criteri di accettazione,
	 * viene settato il relativo valore di <i>invoiceType</i>;
	 * altrimenti <i>invoiceType</i> è lasciata vuota e viene segnalata l'anomalia nel log.
	 * 
	 * @param documentList - (List&lt;ReceiptDocument&gt;) lista dei documenti ricevuti
	 * @return (CheckedDocument) documento verificato
	 * 
	 * <br /><br />
	 * @see it.fe.esb.common.CheckedDocumentUtils CheckedDocumentUtils
	 */
	public CheckedDocument check(CheckedDocument documentRecived) {
		
		CheckedDocument out = new CheckedDocument();
		out.setOriginalDocument(documentRecived.getOriginalDocument());
		out.setInvoiceType("");	
		
		List<FEFile> documents = new ArrayList<FEFile>();
				
		if(documentRecived != null) {
			OriginalDocument originalDocument = documentRecived.getOriginalDocument();
			String emailDomain = originalDocument.getSender().substring(originalDocument.getSender().indexOf("@") + 1);
			emailDomain = emailDomain.replace(">", "");
			String fileExtension = "";
						
			if(! (TransmissionType.type.EMAIL.toString().equalsIgnoreCase(originalDocument.getTransmissionType().getDescription())) ){
				out.setInvoiceType("");				
				logger.error("ATTENZIONE: mezzo di trasmissione" + originalDocument.getTransmissionType().getDescription() + ", aspettato EMAIL!");
				
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						originalDocument.getOriginalDocumentId(), 
						OriginalDocument.receiveStatusType.ERRORE.toString(),
						"Verifica di accettazione fallita: mezzo di trasmissione non riconosciuto");
			}
			else {
				if(documentRecived.getDocumentList() != null){
					for (int i=0; i < documentRecived.getDocumentList().size(); i++) {
						fileExtension = documentRecived.getDocumentList().get(i).getFileName().substring(documentRecived.getDocumentList().get(i).getFileName().lastIndexOf(".") + 1);
						
						if(CheckedDocumentUtils.checkSupplierByEmailDomainAndFileExtension(emailDomain, fileExtension) != null){
							String invoiceType = CheckedDocumentUtils.checkSupplierByEmailDomainAndFileExtension(emailDomain, fileExtension);
							logger.info("Email ricevuta da: " + emailDomain);
							logger.info("Tipologia fattura riconosciuta: " + invoiceType);
							
							documents.add(documentRecived.getDocumentList().get(i));
							out.setInvoiceType(invoiceType);
						}
						else {
							logger.error("ATTENZIONE: impossibile riconoscere il file con estensione " + fileExtension + ", proveniente da: " + emailDomain);
							
							JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
																		originalDocument.getOriginalDocumentId(), 
																		OriginalDocument.receiveStatusType.ERRORE.toString(),
																		"Verifica di accettazione fallita: file non riconosciuto");						
						}
					}
				}
				else {
					logger.error("ATTENZIONE: e-mail senza allegati!");
					JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
							originalDocument.getOriginalDocumentId(), 
							OriginalDocument.receiveStatusType.ERRORE.toString(),
							"Verifica di accettazione fallita: e-mail senza allegati");	
				}				
			}
		}		
		else {
			logger.error("ERRORE RICEZIONE FILE");
		}
		
		out.setDocumentList(documents);
		return out;
	}

}
