/**
 * This file is part of FatturazioneElettronica - ESB
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - ESB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - ESB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - ESB.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.esb.enrollment;

import it.fe.core.common.exception.InvalidInvoiceException;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.common.utils.InvoiceXmlToJava;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.esb.common.HolderOriginalDocument;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.xpath.XPathExpressionException;

import org.apache.log4j.Logger;


public class InvoiceEnrollment {
	
	protected static Logger logger = Logger.getLogger(InvoiceEnrollment.class.getName());
	
	String jbpmProcessDefinitionKey;
	String exceptionJbpm = "";
	String exceptionInvoice = "";
	String messageInvoice = "";
	int countExceptionInvoice = 0;
	
	String processInstanceKey = "";
	
	public void readInvoices(HolderOriginalDocument holder) {
		
		Invoice invoice = new Invoice();
		byte[] invoiceXML;
		String invoiceXMLString;
		long invoiceId;
		
		List<byte[]> invoices = holder.getByteArrayList();
		
		
		
		for (int i=0; i < invoices.size(); i++) {
			
			invoiceXML = invoices.get(i);
			invoiceXMLString = new String(invoiceXML);
			//logger.info("XML: " + invoiceXML);
			
			
			try {
				invoice = InvoiceXmlToJava.xmlToJava(invoiceXMLString);
			} catch (XPathExpressionException x) {
				logger.error(x.getMessage());
			}
			
			//setta nella fattura la data di ricezione del documento originale
			invoice.setReceiptDate(holder.getOriginalDocument().getReceiptDateTime());
			
			//setta il documento originale
			//per ora gestisce solo 1 documento originale (caso Hera)
			List<OriginalDocument> originalDocuments = new ArrayList<OriginalDocument>();
			originalDocuments.add(holder.getOriginalDocument());
			invoice.setOriginalDocuments(originalDocuments);
			
			//setta il file xml
			//nome file: p.iva + invoiceNumber
			invoice.setXmlFile(new FEFile(invoiceXML, invoice.getSupplier().getVatId() + "-" + invoice.getInvoiceNumber() + ".xml"));
			
			try {
				int checkInvoice = JNDIUtils.getInvoiceManager().checkInvoice(invoice);
				if(checkInvoice < 0){
					JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
							holder.getOriginalDocument().getOriginalDocumentId(), 
							OriginalDocument.receiveStatusType.ERRORE.toString(),
							"Verifica di consistenza dei dati fallita");
				}
				else{
					
					//conrolla che la fattura non sia già  stata inserita in FE
					if(JNDIUtils.getInvoiceManager().findInvoiceByNumberAndVatId(invoice.getInvoiceNumber(), invoice.getSupplier().getVatId()) == null){
					
						invoiceId = JNDIUtils.getInvoiceManager().insertInvoice(invoice);
						logger.info("Salvata fattura num. " + invoiceId + " nel db");
						
						//Chiamata wf
						startWorkflow(JNDIUtils.getInvoiceManager().getInvoiceById(invoiceId));

					}
					else {
						//controlla che la fattura già inserita abbia un wf esistente associato
						invoice = JNDIUtils.getInvoiceManager().findInvoiceByNumberAndVatId(invoice.getInvoiceNumber(), invoice.getSupplier().getVatId());
						if((invoice.getProcessInstanceId() == null) || (invoice.getProcessInstanceId().equals(""))){
							startWorkflow(invoice);
						}
						else {
							countExceptionInvoice++;
							messageInvoice = "FATTURA NUM. " + invoice.getInvoiceNumber() + " di " + invoice.getSupplier().getSupplierName();
							logger.error(messageInvoice  + " GIA' INSERITA IN FE DB");
							
							exceptionInvoice = exceptionInvoice + "||" + messageInvoice;
						}
					}				
				
					if((countExceptionInvoice > 0) || (!exceptionJbpm.equals(""))) {
						String exceptionMessage = "";
						if(!exceptionInvoice.equals("")){
							exceptionMessage = "Trovate " + countExceptionInvoice + " fatture già  prensenti nel db FE:" + exceptionInvoice;
						}
						
						if(!exceptionJbpm.equals("")){
							exceptionMessage = exceptionMessage + exceptionJbpm;
						}
						
						JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
								holder.getOriginalDocument().getOriginalDocumentId(), 
								OriginalDocument.receiveStatusType.ERRORE.toString(),
								exceptionMessage);
					}
					else{
						JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
								holder.getOriginalDocument().getOriginalDocumentId(), 
								OriginalDocument.receiveStatusType.OK.toString(),
								null);
					}
				
				}
				
			} catch (InvalidInvoiceException e) {
				logger.error(e.getMessage());
			}
		}
	}
	
	
	public void startWorkflow(Invoice invoice) {
		processInstanceKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		if(JNDIUtils.getProcessServiceFacade().findProcessInstanceById(jbpmProcessDefinitionKey, processInstanceKey) == null){
			
			Map<String,Object> variables = new HashMap<String,Object>();
			variables.put("invoiceId", invoice.getInvoiceId());
			variables.put("invoiceNumber", invoice.getInvoiceNumber());
			variables.put("supplierName", invoice.getSupplier().getSupplierName());

			String processInstanceId = JNDIUtils.getProcessServiceFacade().start(jbpmProcessDefinitionKey, variables, processInstanceKey);
			if(processInstanceId != null){
				logger.info("Start workflow with Process Instance Key " + processInstanceId);
				
				//Aggiorna il Process Instance Id nella fattura
				invoice = JNDIUtils.getInvoiceManager().updateProcessInstanceId(invoice.getInvoiceId(), processInstanceId);
			}
			else {
				logger.error("Unable to start workflow");
			}
		}
		else{
			logger.error("Process Instance " + processInstanceKey + " alredy exist");
			exceptionJbpm = exceptionJbpm + "||" + "Process Instance " + processInstanceKey + " alredy exist";
		}
	}
	
	

	public String getJbpmProcessDefinitionKey() {
		return jbpmProcessDefinitionKey;
	}

	public void setJbpmProcessDefinitionKey(String jbpmProcessDefinitionKey) {
		this.jbpmProcessDefinitionKey = jbpmProcessDefinitionKey;
	}
}
