/**
 * This file is part of FatturazioneElettronica - ESB
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - ESB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - ESB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - ESB.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.esb.storage;

import java.io.IOException;
import java.util.Date;
import java.util.Properties;

import javax.mail.BodyPart;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;

import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.apache.log4j.Logger;

public class EMailForwarding {

	protected static Logger logger = Logger.getLogger(EMailForwarding.class.getName());

	String smtpFrom;
	String smtpTo;
	String smtpSubject;
	
	public Message forwarding(Message messageRecived) throws MessagingException, IOException {
	    // Create empty properties
	    Properties props = new Properties();
	    // Get session
	    Session session = Session.getDefaultInstance(props, null);
		
		Message forward = new MimeMessage(session);
				
		String from = InternetAddress.toString(messageRecived.getFrom());
       	logger.info("From: " + from);
 
        String replyTo = InternetAddress.toString(messageRecived.getReplyTo());
        logger.info("Reply-to: " + replyTo);

        String to = InternetAddress.toString(messageRecived.getRecipients(Message.RecipientType.TO));
        logger.info("To: " + to);

        String subject = messageRecived.getSubject();
        logger.info("Subject: " + subject);

        Date sent = messageRecived.getSentDate();
        logger.info("Sent: " + sent);

        
        // Header
        forward.setSubject(smtpSubject);
        forward.setFrom(new InternetAddress(smtpFrom));
        forward.setRecipient(Message.RecipientType.TO, new InternetAddress(smtpTo));

        // Message
        String bodyText = "---- Messaggio originale ----\n" +
        					"Oggetto: " + subject + "\n" +	
        					"Data: " + sent + "\n" +
        					"Mittente: " + from + "\n" +
        					"A: " + to + "\n\n"; 
        
        
        BodyPart messageBodyPart = new MimeBodyPart();
        messageBodyPart.setText(bodyText);

        Multipart multipart = new MimeMultipart();
        multipart.addBodyPart(messageBodyPart);

        // Forwarded content
        messageBodyPart = new MimeBodyPart();
        messageBodyPart.setDataHandler(messageRecived.getDataHandler());

        multipart.addBodyPart(messageBodyPart);

        forward.setContent(multipart);
		
        return forward;		
	}
	
	
	
    public String getSmtpFrom() {
		return smtpFrom;
	}

	public void setSmtpFrom(String smtpFrom) {
		this.smtpFrom = smtpFrom;
	}

	public String getSmtpTo() {
		return smtpTo;
	}

	public void setSmtpTo(String smtpTo) {
		this.smtpTo = smtpTo;
	}

	public String getSmtpSubject() {
		return smtpSubject;
	}

	public void setSmtpSubject(String smtpSubject) {
		this.smtpSubject = smtpSubject;
	}	
}
