/**
 * This file is part of FatturazioneElettronica - ESB
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - ESB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - ESB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - ESB.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.esb.storage;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.TransmissionType;
import it.fe.esb.common.CheckedDocument;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.activation.MimetypesFileTypeMap;

import org.apache.log4j.Logger;

public class ManualUploadStorage {
	
	protected static Logger logger = Logger.getLogger(ManualUploadStorage.class.getName());
	
	public CheckedDocument readData(File file) throws IOException {
		CheckedDocument out = new CheckedDocument();
		
		List<FEFile> attachements = new ArrayList<FEFile>();
		
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
		
		String fileName = file.getName();
		String[] fileNameSplit = fileName.split("_");
		Date receiptDate = dateFormat.parse(fileNameSplit[0], new ParsePosition(0));
		String userName = fileNameSplit[1];
		String supplierName = fileNameSplit[2];
		String originalFileName = fileNameSplit[3];
		
		String additionalInfo = "User: " + userName + " - Original file name: " + originalFileName;

		
		InputStream is = new FileInputStream(file);
		byte[] fileByteArray = inputStreamToBytes(is);
		is.close();
		file.delete();
		
		//TODO: gestire recipient
		OriginalDocument originalDocument = new OriginalDocument(null, receiptDate, supplierName, "",additionalInfo);
		
		FEFile document = new FEFile(fileByteArray, fileName);
		document.setFileMimeType(new MimetypesFileTypeMap().getContentType(fileName));
		
		attachements.add(document);
		originalDocument.setReceiptDocuments(attachements);
	
		originalDocument.setTransmissionType(
			JNDIUtils.getTransmissionTypeManager().getTransmissionTypeByDescription(TransmissionType.type.MANUAL.toString())
		);
		
		long originalDocumentId = saveIntoDb(originalDocument);	
		originalDocument.setOriginalDocumentId(originalDocumentId);

		out.setDocumentList(attachements);
		out.setOriginalDocument(originalDocument);
		
		return out;
	}
	
	
	public long saveIntoDb(OriginalDocument originalDocument) {
		long originalDocumentId = JNDIUtils.getOriginalDocumentManager().insertOriginalDocument(originalDocument);
		logger.info("Salvato file nel db");
		
		return originalDocumentId;
	}
	
	static byte[] inputStreamToBytes(InputStream in)  throws IOException {

		ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
		byte[] buffer = new byte[1024];
		int len;
		   
		while((len = in.read(buffer)) >= 0)
			out.write(buffer, 0, len);

		in.close();
		out.close();
		   
		return out.toByteArray();
	}
}
