/**
 * This file is part of FatturazioneElettronica - ESB
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - ESB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - ESB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - ESB.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */

package it.fe.esb.translation;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.common.utils.SpecialCharacter;
import it.fe.core.common.utils.XmlFactory;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.AllowanceCharge;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.esb.common.CheckedDocument;
import it.fe.esb.common.HolderOriginalDocument;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.jdom.Element;

/**
 * Classe che effettua la transcodifica del txt fornito da Hera in UBL
 *
 */
public class HeraTranslation {
	
	protected static Logger logger = Logger.getLogger(HeraTranslation.class.getName());
	
	static OriginalDocument originalDocument;
	
	public enum recordHera { FF0000, F01000, F02000, F03000, F04000, F05000, F06000, F07000, F08000, F08500, F09000, F10000, F11000, F12000, F13000, F99000, F }

	// Dati record 0 - Flusso
	static String codiceEnte;
    static String codiceFlusso;
    static String dataEmissione;
    
    //Dati record 1 - Busta
    static String categoriaPIva;
    static String pIva;
    static String nome;
    static String cognome;
    static String indirizzo;
    static String localita;
    static String cap;
    static String civico;
    static String civico2;
    static String codiceCliente;
    static String contoContrattuale;
    static String dataRegistrazione;
    static String dataScadenza;
    static String flag;
    static String periodoCompetenza;
    
    //Quando cambia la busta la fattura precedentemente viene salvata
    //il flag evita che la fattura venga salvata nuovamente al record 3
    static boolean giaSalvata = false;

    
    //Dati record 2 - AddebitoAccredito
    static String numeroBolletta;
    static String descrizione;
    static String importoAddebitoAccredito;
    static String documentoCA; //conto contrattuale
    static String dataFatturaAA;
    
    //Dati record 3 - Fattura
	static String numeroFattura;
    static String codiceSocietaEmittente;
    static String importoFattura;
    static String numeroDocumentoCA;
    
    //Dati record 4 - ContrattoFattura
    static String codiceSettoreMerceologico;
    static String settoreMerceologico;
    static String codiceContratto;
    static String indirizzoFornitura;
    static String civicoFornitura;
    static String civicoFornitura2;
    static String capFornitura;
    static String localitaFornitura;
    static ContractDocument contratto =  new HeraTranslation().new ContractDocument();
    
    static String testoContratto;
    static String categoriaTariffa;
    static String tipoFattura;
    static String portion;
    static String opzioneVenditaContratto;
    static String POD;
    
    static List<ContractDocument> contratti = new ArrayList<ContractDocument>();
    
    static List<String> contatoriPerContratto = new ArrayList<String>();
    
    //Dati record 5 - Lettura
    static String matricolaContatore;
    static String mcFatturati;
    static String mcAcconto;
    static String mcConguaglio;
    static String numeratore;
    static String unitaDiMisura;
    //TODO: non mappato nel file Excel di Hera
    static String incognita1;
    
    //Dati record 6 - DettaglioLettura
    static String dataLettura;
    static String tipologiaLettura;
    static String descrizioneLettura;
    static String valoreLettura;
    static String idLettura;
    
    //Dati record 7 - DettaglioFattura
    static int numRiga = 0;
    static String codiceContrattoVoce;
    static String testoVoce;
    static String inizioPeriodo;
    static String finePeriodo;
    static String unitaDiMisuraRigaFattura;
    static String quantita;
    static String prezzo;
    static String importoRiga;
    static String codiceIvaRigaFattura;
    static String importoAcconto;
    static String importoConguaglio;
    
    //Dati record 8 - AddebitoContratto
    static String tipoVoceUlterioreC;
    static String testoVoceUlterioreC;
    static String importoUlterioreC;
    static String codiceIvaUlterioreC;
    
    //Dati record 85 - DettaglioDiritti
    static String contrattoDirittiAttivazione;
    static String testoVoceDirittiAttivazione;
    static String inizioPeriodoDirittiAttivazione;
    static String finePeriodoDirittiAttivazione;
    static String unitaDiMisuraDirittiAttivazione;
    static String quantitaDirittiAttivazione;
    static String prezzoDirittiAttivazione;
    static String importoRigaDirittiAttivazione;
    static String codiceIvaDirittiAttivazione;
    static String importoAccontoDirittiAttivazione;
    static String importoConguaglioDirittiAttivazione;
    
    //Dati record 9 - DettaglioIva
    static String codiceIva;
    static String aliquotaIva;
    static String imponibile;
    static String importoIva;
    
    static List<CodiciIva> codiciIva = new ArrayList<CodiciIva>();
    
    //Dati record 10 - AltriServizi
    static String tipoVoceAltriServizi;
    static String testoVoceAltriServizi;
    static String importoAltriServizi;
    static String codiceIvaAltriServizi;
    
    //Dati record 11 - UlterioriAddebiti
    static String tipoVoceUlterioriAddebiti;
    static String testoVoceUlterioriAddebiti;
    static String importoUlterioriAddebiti;
    static String codiceIvaUlterioriAddebiti;
    
    //Dati record 12 - RiepilogoIva
    static String codiceIvaRiepilogo;
    static String aliquotaIvaRiepilogo;
    static String imponibileRiepilogo;
    static String importoIvaRiepilogo;
    static Double totaleTasse;
    
    static List<RiepilogoIva> righeRiepilogoIva = new ArrayList<RiepilogoIva>();
    
    //Dati record 13 - Update Busta
    static String importoTotale;

    //Dati record 99 - Totali Flusso
    static String totBusteFlusso;
    static String totFattureHeraSpa;
    static String totFattureHeraComm;
    static String importoTotaleFlusso;
    
    //Righe fattura + righe riepilogo
    static List<InvoiceLineHera> invoiceLines = new ArrayList<InvoiceLineHera>();
    static List<InvoiceLineHera> invoiceLinesExtra = new ArrayList<InvoiceLineHera>();
	
    /** CAMPI RELATIVI ALL'UTILITY STATEMENT **/
    
    //ID dell'Utility Statement
    static int utilityStatementIndex;
    
	/**
	 * Per ogni allegato richiama il metodo di conversione in UBL
	 * 
	 * @param documents (List&ltCheckedDocument&gt) lista dei documenti riconosciuti
	 * @return (HolderOriginalDocument) lista delle fatture xml generate
	 * 
	 */
    public static HolderOriginalDocument readFiles(CheckedDocument documents) throws IOException {
		
    	List<FEFile> files = documents.getDocumentList();
    	
		InputStream fileHera;
		List<byte[]> xmlList = new ArrayList<byte[]>();
		
		HolderOriginalDocument out = null;
		
		originalDocument = documents.getOriginalDocument();
		if((files != null) && (files.size() > 0)) {
			for (int i=0; i < files.size(); i++) {

				fileHera = new ByteArrayInputStream(files.get(i).getFileBlob());
				xmlList.addAll(txtToXml(fileHera));

			}
		
			out = new HolderOriginalDocument(originalDocument, xmlList);
		}
		
		return out;
	}
	
	/**
	 * Richiama una serie di metodi che permettono di recuperare dal file txt di Hera
	 * i dati necessari alla creazione del file XML UBL
	 * 
	 * @param file (InputStream) inputstream del file da leggere
	 * @return (List &ltbyte[]&gt) lista delle fatture xml generate
	 * 
	 */
	public static List<byte[]> txtToXml(InputStream file) throws IOException {
		List<byte[]> out = new ArrayList<byte[]>();
		
		// Apertura del file in lettura
		BufferedReader in = new BufferedReader(new InputStreamReader(file));
		
		// Lettura di una riga del file
		String line = in.readLine();
		
		String tipoRecord;
		int count = 0;
		
		
		while((line!=null)) {
			
			if((!line.equals(""))){
				
				if(line.length() >= 13){
					tipoRecord = "F" + line.substring(8, 13);
								
					for(recordHera r : recordHera.values()){
						if(tipoRecord.equalsIgnoreCase(r.toString())){
							
							switch (r) {
						        case FF0000:
						        	f0000(line);
						            break;
						        case F01000:						        	
						        	if(count > 0) {
						        		utilityStatementIndex = 1;
						        		out.add(buildXml());
						        		giaSalvata = true;
						        	}
						        	
						        	f01000(line);
						        	break;
						        case F02000:
						            f02000(line);
						        	break;
						        case F03000:
						        	if((count > 0) && (giaSalvata == false)) {
						        		utilityStatementIndex = 1;
						        		out.add(buildXml());
						        	}
						        	giaSalvata = false;
					        		
					        	    //Dati record 2 - AddebitoAccredito
					        	    numeroBolletta = "";
					        	    descrizione = "";
					        	    importoAddebitoAccredito = "";
					        	    documentoCA = "";
					        	    dataFatturaAA = "";
					        		
					        		//Dati record 3 - Fattura
					        		numeroFattura = "";
					        	    codiceSocietaEmittente = "";
					        	    importoFattura = "";
					        	    numeroDocumentoCA = "";
					        	    
					        	    contratti = new ArrayList<ContractDocument>();
					        	    
					        	    //Periodo di consumo (record 7)
					        	    inizioPeriodo = "";
					        	    finePeriodo = "";
					        	    
					        	    //Dati record 10 - AltriServizi
					        	    tipoVoceAltriServizi = "";
					        	    testoVoceAltriServizi = "";
					        	    importoAltriServizi = "";
					        	    codiceIvaAltriServizi = "";
					        	    
					        	    //Dati record 11 - UlterioriAddebiti
					        	    tipoVoceUlterioriAddebiti = "";
					        	    testoVoceUlterioriAddebiti = "";
					        	    importoUlterioriAddebiti = "";
					        	    codiceIvaUlterioriAddebiti = "";
					        	    
					        	    //Dati record 12 - RiepilogoIva
					        	    codiceIvaRiepilogo = "";
					        	    aliquotaIvaRiepilogo = "";
					        	    imponibileRiepilogo = "";
					        	    importoIvaRiepilogo = "";
					        		
					        		numRiga = 0;
					        		invoiceLines = new ArrayList<InvoiceLineHera>();
					        		invoiceLinesExtra = new ArrayList<InvoiceLineHera>();
								        	
						        	count++;		        			            
						        	f03000(line);
						        	break;
						        case F04000:
						            //Dati record 4 - ContrattoFattura
						            codiceSettoreMerceologico = "";
						            settoreMerceologico = "";
						            codiceContratto = "";
						            indirizzoFornitura = "";
						            civicoFornitura = "";
						            civicoFornitura2 = "";
						            capFornitura = "";
						            localitaFornitura = "";
						            contratto =  new HeraTranslation().new ContractDocument();
						            
						            testoContratto = "";
						            categoriaTariffa = "";
						            tipoFattura = "";
						            portion = "";
						            opzioneVenditaContratto = "";
						            POD = "";
						            
						            //Dati record 5 - Contatore
						            matricolaContatore = "";
						            mcFatturati = "";
						            mcAcconto = "";
						            mcConguaglio = "";
						            numeratore = "";
						            unitaDiMisura = "";
						            incognita1 = "";
						            
						            //Dati record 8 - AddebitoContratto
						            tipoVoceUlterioreC = "";
						            testoVoceUlterioreC = "";
						            importoUlterioreC = "";
						            codiceIvaUlterioreC = "";
						            
						            //Dati record 85 - DettaglioDiritti
						            contrattoDirittiAttivazione = "";
						            testoVoceDirittiAttivazione = "";
						            inizioPeriodoDirittiAttivazione = "";
						            finePeriodoDirittiAttivazione = "";
						            unitaDiMisuraDirittiAttivazione = "";
						            quantitaDirittiAttivazione = "";
						            prezzoDirittiAttivazione = "";
						            importoRigaDirittiAttivazione = "";
						            codiceIvaDirittiAttivazione = "";
						            importoAccontoDirittiAttivazione = "";
						            importoConguaglioDirittiAttivazione = "";
						            
						            //Dati record 9 - DettaglioIva
						            codiceIva = "";
						            aliquotaIva = "";
						            imponibile = "";
						            importoIva = "";
						            
						            righeRiepilogoIva = new ArrayList<RiepilogoIva>();
						            totaleTasse = 0.0;
						            contatoriPerContratto = new ArrayList<String>();
						            
						        	f04000(line);
						        	break;
						        case F05000:
						            f05000(line);
						        	break;
						        case F06000:
						        	
						            //Dati record 6 - DettaglioLettura
						            dataLettura = "";
						            tipologiaLettura = "";
						            descrizioneLettura = "";
						            valoreLettura = "";
						            idLettura = "";		            
						        	
						        	f06000(line);
						        	break;
						        case F07000:
						            f07000(line);
						        	break;
						        case F08000:
						            f08000(line);
						    		break;
						        case F08500:
						            f08500(line);
						    		break;
						        case F09000:
						        	f09000(line);
						    		break;
						        case F10000:
						            f10000(line);
						    		break;
						        case F11000:
						            f11000(line);
						    		break;
						        case F12000:
						            f12000(line);
						    		break;
						        case F13000:
						            f13000(line);
						    		break;	
						        case F99000:
						            //Fine flusso: salvataggio ultima fattura
						        	utilityStatementIndex = 1;
						        	out.add(buildXml());
						        	
						        	f99000(line);
						    		break;
						        case F:
						            //In fondo al file c'e' una riga vuota
						        	break;
						        default:
						        	logger.error("Messaggio inesistente! " + tipoRecord);
						        	break;
							}
						}
					}
				}
			}

			line = in.readLine();
		}
		
		if(count > 0){
			logger.info("Tot file generati: " + count);
			logger.info("Tot buste: " + totBusteFlusso);
			logger.info("Tot fatture: " + totFattureHeraSpa + " HeraSpa  +  " + totFattureHeraComm + " HeraComm");
		}
		else {
			logger.error("Impossibile generare fatture!");
			
			JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
					originalDocument.getOriginalDocumentId(), 
					OriginalDocument.receiveStatusType.ERRORE.toString(),
					"Verifica di consistenza fallita: impossibile generare fatture");	
		}
		
		return out;
	}
	
	
	/**
	 * Codice record txt F0000
	 * Dati generali flusso (codice flusso, codice ente, data emissione)
	 * 
	 */
	private static void f0000(String line) {
		//logger.debug(line);

		codiceFlusso = substringNN(line,13, 17); //fisso "Hera"
		codiceEnte = substringNN(line,17, 27);
	    dataEmissione = substringNN(line,27, 35); //aaaammgg
	}

	/**
	 * Codice record txt 01000
	 * Dati intestazione del cliente + date
	 * 
	 * N.B. le fatture dovrebbero avere un'unica data di scadenza
	 * 
	 */
	private static void f01000(String line) {
		//logger.debug(line);


		categoriaPIva = substringNN(line,23, 25);
		pIva = substringNN(line,27, 47);
		nome = substringNN(line,47, 82).trim();
		cognome = substringNN(line,82, 117).trim();
		indirizzo = substringNN(line,117, 177);
		localita = substringNN(line,177, 217);
		cap = substringNN(line,217, 227);
		civico = substringNN(line,227, 237);
		civico2 = substringNN(line,237, 247);
		codiceCliente = substringNN(line,247, 257);
		contoContrattuale = substringNN(line,257, 269);
		dataRegistrazione = substringNN(line,269, 277);
		dataScadenza = substringNN(line,277, 285);
		flag = substringNN(line,285, 285); //Economato, Patrimoniale
		periodoCompetenza = substringNN(line,287, 291);
	}
	
	/**
	 * Codice record txt 02000
	 * Uteriori addebiti/accrediti per bolletta
	 * 
	 */
	private static void f02000(String line) {
		//logger.debug(line);
		numRiga++;
		
	    descrizione = substringNN(line,13,63);
	    numeroBolletta = substringNN(line,63,79);
	    documentoCA = substringNN(line,79,91);
	    dataFatturaAA = substringNN(line,100,99);
	    
	    importoAddebitoAccredito = substringNN(line,99,116);
	    if((importoAddebitoAccredito == null) || (importoAddebitoAccredito.equals(""))){
	    	importoAddebitoAccredito = "0";
	    }
	    else {
	    	importoAddebitoAccredito = importoAddebitoAccredito.replace(",", ".");
	    }
	    if(substringNN(line,116,117).equals("-")){
	    	importoAddebitoAccredito = "-" + importoAddebitoAccredito;
	    }
	    	    
		List<String> note = new ArrayList<String>();	    
	    note.add("Rif. documento CA num. " + documentoCA);
	    note.add("Numero bolletta fiscale: " + numeroBolletta);
	    note.add("Data fattura: " + dataFatturaAA);
	    
	    InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
				numRiga, 												//numero riga
				1, 														//quantita'
				"",														//unita' di misura
				descrizione, 											//descrizione
				Double.parseDouble(importoAddebitoAccredito), 			//prezzo cad
				Double.parseDouble(importoAddebitoAccredito), 			//importo totale (quantita' x prezzo)
				"EUR");													//valuta
	    
	    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIGA.toString());
	    invoiceLine.setNotes(note);
	    
	    invoiceLines.add(invoiceLine);
	}

	
	/**
	 * Codice record txt 03000
	 * Dati bolletta fiscale: considerata fattura
	 * 
	 */
	private static void f03000(String line) {
		//logger.debug(line);
		
	    numeroFattura = substringNN(line,13,29).trim();
	    codiceSocietaEmittente = substringNN(line,29,33).trim();  //5010 Hera Spa / 1070 Hera Comm srl
	    
	    importoFattura = substringNN(line,33,50).trim();
	    if((importoFattura == null) || (importoFattura.equals(""))){
	    	importoFattura = "0";
	    }
	    else {
	    	importoFattura = importoFattura.replace(",", ".");
	    }
	    if(substringNN(line,50,51).equals("-")){
	    	importoFattura = "-" + importoFattura;
	    }
	    
	    numeroDocumentoCA = substringNN(line,51,63).trim();
	}
	
	
	/**
	 * Codice record txt 04000
	 * Dati contratto: ogni bolletta può avere più contratti
	 * 
	 * In UBL la fornitura è memorizzata in Delivery
	 * 
	 */
	private static void f04000(String line) {
		//logger.debug(line);
			
		/* 
		 * Settore merceologico:
		 * 10: Gas
		 * 11: Acqua
		 * 12: Igiene Ambientale
		 * 13: Energia elettrica
		 * 14: Teleriscaldamento
		*/
		codiceSettoreMerceologico = substringNN(line,13,15).trim();
		if((codiceSettoreMerceologico != null) && (!codiceSettoreMerceologico.equals(""))){
			switch (Integer.parseInt(codiceSettoreMerceologico)) {
		        case 10:
		        	settoreMerceologico = "Gas";
		            break;
		        case 11:
		        	settoreMerceologico = "Acqua";
		            break;
		        case 12:
		        	settoreMerceologico = "Ambiente";
		            break;
		        case 13:
		        	settoreMerceologico = "Energia elettrica";
		            break;
		        case 14:
		        	settoreMerceologico = "Teleriscaldamento";
		            break;
		        default:
		        	settoreMerceologico = "";		        	
		        	break;
			}
		}
		
		codiceContratto = substringNN(line,15,25);
		
		//Indirizzo fornitura (anagrafica utenza)
		indirizzoFornitura = substringNN(line,25,85).trim();
		localitaFornitura = substringNN(line,85,125).trim();		
		capFornitura = substringNN(line,125,135).trim();		
		civicoFornitura = substringNN(line,135,145).trim();
		civicoFornitura2 = substringNN(line,145,155).trim();
		if(!civicoFornitura2.equals(""))
			civicoFornitura = civicoFornitura + " " + civicoFornitura2;
		
		testoContratto = substringNN(line,155,190).trim(); //per ora vuoto
		categoriaTariffa = substringNN(line,190,200).trim();
	    tipoFattura = substringNN(line,200,203).trim(); //per ora vuoto
	    portion = substringNN(line,203,211).trim();
	    //opzioneVenditaContratto = substringNN(line,211,221).trim(); //per ora vuoto
	    POD = substringNN(line,211,261);
	    
	    
	    if(!codiceContratto.equals("")){
	    	contratto = new HeraTranslation().new ContractDocument(codiceContratto);
	    	contratto.setSettoreMerceologico(settoreMerceologico);
	    	contratto.setIndirizzoFornitura(indirizzoFornitura);
	    	contratto.setCivicoFornitura(civicoFornitura);
	    	contratto.setLocalitaFornitura(localitaFornitura);
	    	contratto.setCapFornitura(capFornitura);
	    	contratto.setPOD(POD);
	    	
	    	contratti.add(contratto);
	    }
	}
	
	
	/**
	 * Codice record txt 05000
	 * Dati contatore: un contratto, che fa riferimento ad un unico indirizzo, 
	 * potrebbe avere più contatori
	 * 
	 */
	private static void f05000(String line) {
		//logger.debug(line);
		
		InvoiceLineHera invoiceLine;
		
		matricolaContatore = substringNN(line,13,31).trim();
	    mcFatturati = substringNN(line,31,51).trim().replace(",", ".");
	    if((mcFatturati == null) || (mcFatturati.equals("")))
	    	mcFatturati = "0";
	    
	    mcAcconto = substringNN(line,51,71).trim();
	    if((mcAcconto == null) || (mcAcconto.equals("")))
	    	mcAcconto = "0";
	    mcAcconto = mcAcconto.replace(",", ".");
	    
	    mcConguaglio = substringNN(line,71,91).trim().replace(",", ".");
	    if((mcConguaglio == null) || (mcConguaglio.equals("")))
	    	mcConguaglio = "0";
	    mcConguaglio = mcConguaglio.replace(",", ".");
	    
	    numeratore = substringNN(line,91,94).trim();
	    unitaDiMisura = substringNN(line,94,100).trim();
	    incognita1 = substringNN(line,100,105).trim();
	    
	    
	    //Aggiunge il contatore alla lista contatori del contratto
	    if((contratto.getContatori() == null) || (!contratto.getContatori().contains(matricolaContatore))){
	    	contatoriPerContratto.add(matricolaContatore);
	    	contratto.setContatori(contatoriPerContratto);
	    }
	    
	    //RIGA MC FATTURATI
	    invoiceLine = new HeraTranslation().new InvoiceLineHera(
				0, 											//numero riga
				Double.parseDouble(mcFatturati), 			//quantita'
				unitaDiMisura,								//unita' di misura
				unitaDiMisura + " fatturati", 				//descrizione
				0.0, 										//prezzo cad
				0.0, 										//importo totale (quantita' x prezzo)
				"EUR");										//valuta
		
		invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		invoiceLine.setContatore(matricolaContatore);
		invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIASSUNTO_CONTATORE.toString());
		invoiceLine.setItemName("");
		
		invoiceLinesExtra.add(invoiceLine);
		
	    //RIGA MC ACCONTO
	    invoiceLine = new HeraTranslation().new InvoiceLineHera(
				0, 											//numero riga
				Double.parseDouble(mcAcconto), 				//quantita'
				unitaDiMisura,								//unita' di misura
				unitaDiMisura + " acconto", 				//descrizione
				0.0, 										//prezzo cad
				0.0, 										//importo totale (quantita' x prezzo)
				"EUR");										//valuta
		
		invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		invoiceLine.setContatore(matricolaContatore);
		invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIASSUNTO_CONTATORE.toString());
		invoiceLine.setItemName("ACCONTO");
		
		invoiceLinesExtra.add(invoiceLine);
		
	    //RIGA MC CONGUAGLIO
	    invoiceLine = new HeraTranslation().new InvoiceLineHera(
				0, 											//numero riga
				Double.parseDouble(mcConguaglio), 			//quantita'
				unitaDiMisura,								//unita' di misura
				unitaDiMisura + " conguaglio", 				//descrizione
				0.0, 										//prezzo cad
				0.0, 										//importo totale (quantita' x prezzo)
				"EUR");										//valuta
		
		invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		invoiceLine.setContatore(matricolaContatore);
		invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIASSUNTO_CONTATORE.toString());
		invoiceLine.setItemName("CONGUAGLIO");
		
		invoiceLinesExtra.add(invoiceLine);
	}
	
	/**
	 * Codice record txt 06000
	 * Dati lettura: ogni contatore ha più letture
	 * 
	 */
	private static void f06000(String line) {
		//logger.debug(line);
		List<String> note = new ArrayList<String>();
		
		dataLettura = substringNN(line,13,21).trim();
		tipologiaLettura = substringNN(line,21,23).trim();
	    descrizioneLettura = substringNN(line,23,63).trim();
	    valoreLettura = substringNN(line,63,83).trim();
	    if((valoreLettura == null) || (valoreLettura.equals("")))
	    	valoreLettura = "0";
	    valoreLettura = valoreLettura.replace(",", ".");
	    
	    idLettura = substringNN(line,83,103).trim();
	    
	    String descrizioneCompleta = descrizioneLettura + " - Tipologia " + tipologiaLettura + " - Codice " + idLettura;
	    
	    note.add(dataLettura);
	    
	    InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
				0, 											//numero riga
				Double.parseDouble(valoreLettura), 			//quantita'
				"",											//unita' di misura
				descrizioneCompleta, 						//descrizione
				0.0, 										//prezzo cad
				0.0, 										//importo totale (quantita' x prezzo)
				"EUR");										//valuta
		
		invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		invoiceLine.setContatore(matricolaContatore);
		invoiceLine.setItemKeyword(XmlFactory.itemKeyword.LETTURA.toString());
		invoiceLine.setItemName(idLettura);
		invoiceLine.setNotes(note);
		invoiceLine.setDescrizioneLettura(descrizioneLettura);
		invoiceLine.setTipologiaLettura("Tipologia " + tipologiaLettura);
		
		invoiceLinesExtra.add(invoiceLine);

	}
	
	/**
	 * Codice record txt 07000
	 * Righe bolletta relative al contratto corrente
	 * 
	 */
	private static void f07000(String line) throws NullPointerException {
		//logger.debug(line);
		numRiga++;
		
		List<String> note = new ArrayList<String>();
		String inizioPeriodo_old = inizioPeriodo;
		String finePeriodo_old = finePeriodo;
		
		codiceContrattoVoce = substringNN(line,13,23).trim();
		testoVoce = substringNN(line,23,109).trim();
	    inizioPeriodo = substringNN(line,109,117).trim();
	    finePeriodo = substringNN(line,117,125).trim();
	    
	    note.add(inizioPeriodo + "-" + finePeriodo);
	    
	    //Va mantenuta la data di inizio più piccola
	    if((inizioPeriodo == null) || (inizioPeriodo.equals("")) || (inizioPeriodo.equals("00000000"))){
	    	inizioPeriodo = inizioPeriodo_old;
	    }
	    else if((inizioPeriodo_old != null) && (!inizioPeriodo_old.equals("")) && (inizioPeriodo_old.compareTo(inizioPeriodo) < 0)){
	    	if(!inizioPeriodo_old.equals("00000000"))		
	    		inizioPeriodo = inizioPeriodo_old;
	    }


	    //Va mantenuta la data di fine più grande
	    if((finePeriodo_old != null) && (finePeriodo_old.compareTo(finePeriodo) > 0)) {
	    	finePeriodo = finePeriodo_old;
	    }

	    
	    unitaDiMisuraRigaFattura = substringNN(line,125,133).trim();
	    
	    //puo' essere negativa oppure una percentuale
	    quantita = substringNN(line,133,152).trim();
	    if((quantita == null) || (quantita.equals("")))
	    	quantita = "0";
	    if(substringNN(line,152,153).equals("-")) {
	    	quantita = "-" + quantita;
	    }
	    else if(substringNN(line,152,153).equals("%")) {
	    	quantita = "0";
	    	note.add(quantita + "%");
	    }
	    
	    //puo' essere negativo
	    prezzo = substringNN(line,153,172).trim();
	    if((prezzo == null) || (prezzo.equals("")))
	    	prezzo = "0";
	    if(substringNN(line,172,173).equals("-")) {
	    	prezzo = "-" + prezzo;
	    }
	    
	    //importo totale: prezzo * quantita'
	    //puo' essere negativo
	    importoRiga = substringNN(line,173,190).trim();
	    if((importoRiga == null) || (importoRiga.equals("")))
	    	importoRiga = "0";
	    if(substringNN(line,190,191).equals("-")) {
	    	importoRiga = "-" + importoRiga;
	    }
	    
	    codiceIvaRigaFattura = substringNN(line,191,193).trim();
	    importoAcconto = substringNN(line,193,210).trim();
	    if((importoAcconto == null) || (importoAcconto.equals("")))
	    	importoAcconto = "0";
	    if(substringNN(line,210,211).equals("-")) {
	    	importoAcconto = "-" + importoAcconto;
	    }

	    importoConguaglio = substringNN(line,211,228).trim();
	    if((importoConguaglio == null) || (importoConguaglio.equals("")))
	    	importoConguaglio = "0";
	    if(substringNN(line,228,229).equals("-")) {
	    	importoConguaglio = "-" + importoConguaglio;
	    }
	    
	    
	    InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
	    							numRiga, 									//numero riga
	    							Double.parseDouble(quantita), 				//quantita'
	    							unitaDiMisuraRigaFattura,					//unita' di misura
	    							testoVoce, 									//descrizione
	    							Double.parseDouble(prezzo), 				//prezzo cad
	    							Double.parseDouble(importoRiga), 			//importo totale (quantita' x prezzo)
	    							"EUR");										//valuta
	    
	    
	    if ((codiceIvaRigaFattura != null ) && (!codiceIvaRigaFattura.equals(""))) {
	    	invoiceLine.setCodIva(codiceIvaRigaFattura);
	    }
	    
	    invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
	    invoiceLine.setContatore(matricolaContatore);
	    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.CONSUMO.toString());
	    invoiceLine.setNotes(note);
	    invoiceLine.setInizioPeriodo(inizioPeriodo);
	    invoiceLine.setFinePeriodo(finePeriodo);
	    
	    invoiceLines.add(invoiceLine);
	    
	    
	    //Acconto
	    if((importoAcconto != null) && (!importoAcconto.equals("")) && (!importoAcconto.equals("0"))) {
	    	numRiga++;

		    invoiceLine = new HeraTranslation().new InvoiceLineHera(
					numRiga, 												//numero riga
					1, 														//quantita'
					unitaDiMisuraRigaFattura,								//unita' di misura
					"**ACCONTO** " + testoVoce, 							//descrizione
					Double.parseDouble(importoAcconto), 					//prezzo cad
					Double.parseDouble(importoAcconto), 					//importo totale (quantita' x prezzo)
					"EUR");													//valuta
	    	
		    if ((codiceIvaRigaFattura != null ) && (!codiceIvaRigaFattura.equals(""))) {
		    	invoiceLine.setCodIva(codiceIvaRigaFattura);
		    }
		    
		    invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		    invoiceLine.setContatore(matricolaContatore);
		    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.CONSUMO.toString());
		    invoiceLine.setNotes(note);
		    invoiceLine.setItemName("ACCONTO");
		    invoiceLine.setInizioPeriodo(inizioPeriodo);
		    invoiceLine.setFinePeriodo(finePeriodo);
		    
	    	invoiceLines.add(invoiceLine);
	    }
	    
	    //Conguaglio
	    if((importoConguaglio != null) && (!importoConguaglio.equals("")) && (!importoConguaglio.equals("0"))) {
	    	numRiga++;

		    invoiceLine = new HeraTranslation().new InvoiceLineHera(
					numRiga, 														//numero riga
					1, 																//quantita'
					unitaDiMisuraRigaFattura,										//unita' di misura
					"**CONGUAGLIO** " + testoVoce, 									//descrizione
					Double.parseDouble(importoConguaglio), 							//prezzo cad
					Double.parseDouble(importoConguaglio), 							//importo totale (quantita' x prezzo)
					"EUR");															//valuta
	    	
		    if ((codiceIvaRigaFattura != null ) && (!codiceIvaRigaFattura.equals(""))) {
		    	invoiceLine.setCodIva(codiceIvaRigaFattura);
		    }
		    
		    invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		    invoiceLine.setContatore(matricolaContatore);
		    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.CONSUMO.toString());
		    invoiceLine.setNotes(note);
		    invoiceLine.setItemName("CONGUAGLIO");
		    invoiceLine.setInizioPeriodo(inizioPeriodo);
		    invoiceLine.setFinePeriodo(finePeriodo);
		    
	    	invoiceLines.add(invoiceLine);
	    }
	    
	}
	
	/**
	 * Codice record txt 08000
	 * Voci per contratto (es. deposito cauzionale)
	 * L'importo può essere anche negativo.
	 * 
	 */
	private static void f08000(String line) {
		//logger.debug(line);
		
	    tipoVoceUlterioreC = substringNN(line,13,19).trim();
		testoVoceUlterioreC = substringNN(line,19,69).trim();
	    importoUlterioreC = substringNN(line,69,86).trim();
	    if((importoUlterioreC == null) || (importoUlterioreC.equals(""))){
	    	importoUlterioreC = "0";
	    }
	    if(substringNN(line,86,87).equals("-")) {
	    	importoUlterioreC = "-" + importoUlterioreC;
	    }
	    codiceIvaUlterioreC = substringNN(line,87,89).trim();
	    
	    if((importoUlterioreC != null) && (!importoUlterioreC.equals("")) && (!importoUlterioreC.equals("0"))) {
	    	numRiga++;

		    InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
					numRiga, 									//numero riga
					1, 											//quantita'
					"",											//unita' di misura
					testoVoceUlterioreC, 						//descrizione
					Double.parseDouble(importoUlterioreC), 		//prezzo cad
					Double.parseDouble(importoUlterioreC), 		//importo totale (quantita' x prezzo)
					"EUR");										//valuta
	    	
		    invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIGA.toString());
		    
	    	invoiceLines.add(invoiceLine);
	    }
	}
	
	/**
	 * Codice record txt 08500
	 * Voci per contratto relative ai diritti di attivazione
	 * 
	 */
	private static void f08500(String line) {
		//logger.debug(line);
		List<String> note = new ArrayList<String>();
		
		contrattoDirittiAttivazione = substringNN(line,13,23).trim();
		testoVoceDirittiAttivazione = substringNN(line,23,109).trim();
	    inizioPeriodoDirittiAttivazione = substringNN(line,109,117).trim();
	    finePeriodoDirittiAttivazione = substringNN(line,117,125).trim();
	    unitaDiMisuraDirittiAttivazione = substringNN(line,125,133).trim();

	    //puo' essere negativa o una percetuale
	    quantitaDirittiAttivazione = substringNN(line,133,152).trim();
	    if((quantitaDirittiAttivazione == null) || (quantitaDirittiAttivazione.equals("")))
	    	quantitaDirittiAttivazione = "0";
	    if(substringNN(line,152,153).equals("-")) {
	    	quantitaDirittiAttivazione = "-" + quantitaDirittiAttivazione;
	    }
	    else if(substringNN(line,152,153).equals("%")) {
	    	quantitaDirittiAttivazione = "0";
	    	note.add(quantitaDirittiAttivazione + "%");
	    }
	    
	    prezzoDirittiAttivazione = substringNN(line,153,172).trim();
	    if((prezzoDirittiAttivazione == null) || (prezzoDirittiAttivazione.equals("")))
	    	prezzoDirittiAttivazione = "0";
	    if(substringNN(line,172,173).equals("-")) {
	    	prezzoDirittiAttivazione = "-" + prezzoDirittiAttivazione;
	    }
	    
	    importoRigaDirittiAttivazione = substringNN(line,173,190).trim();
	    if((importoRigaDirittiAttivazione == null) || (importoRigaDirittiAttivazione.equals("")))
	    	importoRigaDirittiAttivazione = "0";
	    if(substringNN(line,190,191).equals("-")) {
	    	importoRigaDirittiAttivazione = "-" + importoRigaDirittiAttivazione;
	    }
	    
	    codiceIvaDirittiAttivazione = substringNN(line,191,193).trim();
	    
	    importoAccontoDirittiAttivazione = substringNN(line,193,210).trim();
	    if((importoAccontoDirittiAttivazione == null) || (importoAccontoDirittiAttivazione.equals("")))
	    	importoAccontoDirittiAttivazione = "0";
	    if(substringNN(line,210,211).equals("-")) {
	    	importoAccontoDirittiAttivazione = "-" + importoAccontoDirittiAttivazione;
	    }

	    importoConguaglioDirittiAttivazione = substringNN(line,211,228).trim();
	    if((importoConguaglioDirittiAttivazione == null) || (importoConguaglioDirittiAttivazione.equals("")))
	    	importoConguaglioDirittiAttivazione = "0";
	    if(substringNN(line,228,229).equals("-")) {
	    	importoConguaglioDirittiAttivazione = "-" + importoConguaglioDirittiAttivazione;
	    }

	    
	    if((importoRigaDirittiAttivazione != null) && (!importoRigaDirittiAttivazione.equals("")) && (!importoRigaDirittiAttivazione.equals("0"))) {
	    	numRiga++;
	    	
		    InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
					numRiga, 												//numero riga
					Double.parseDouble(quantitaDirittiAttivazione), 		//quantita'
					unitaDiMisuraDirittiAttivazione,						//unita' di misura
					testoVoceDirittiAttivazione, 							//descrizione
					Double.parseDouble(prezzoDirittiAttivazione), 			//prezzo cad
					Double.parseDouble(importoRigaDirittiAttivazione), 		//importo totale (quantita' x prezzo)
					"EUR");													//valuta
	    	
		    if ((codiceIvaDirittiAttivazione != null ) && (!codiceIvaDirittiAttivazione.equals(""))) {
		    	invoiceLine.setCodIva(codiceIvaDirittiAttivazione);
		    }
		    
		    invoiceLine.setPreviousDocument(new PreviousDocument("contratto", contrattoDirittiAttivazione));
		    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIGA.toString());
		    
		    
		    note.add("Periodo diritti di attivazione dal " + inizioPeriodoDirittiAttivazione + " al " + finePeriodoDirittiAttivazione);
		    invoiceLine.setNotes(note);
		    
	    	invoiceLines.add(invoiceLine);
	    }
	    
	    
	    //Acconto Diritti di Attivazione
	    if((importoAccontoDirittiAttivazione != null) && (!importoAccontoDirittiAttivazione.equals("")) && (!importoAccontoDirittiAttivazione.equals("0"))) {
	    	numRiga++;

	    	InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
					numRiga, 																	//numero riga
					1, 																			//quantita'
					"",																			//unita' di misura
					"**ACCONTO** " + testoVoceDirittiAttivazione, 								//descrizione
					Double.parseDouble(importoAccontoDirittiAttivazione), 						//prezzo cad
					Double.parseDouble(importoAccontoDirittiAttivazione), 						//importo totale (quantita' x prezzo)
					"EUR");																		//valuta
	    	
		    if ((codiceIvaRigaFattura != null ) && (!codiceIvaRigaFattura.equals(""))) {
		    	invoiceLine.setCodIva(codiceIvaRigaFattura);
		    }

		    invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIGA.toString());
		    
	    	invoiceLines.add(invoiceLine);
	    }
	    
	    //Conguaglio Diritti di Attivazione
	    if((importoConguaglioDirittiAttivazione != null) && (!importoConguaglioDirittiAttivazione.equals("")) && (!importoConguaglioDirittiAttivazione.equals("0"))) {
	    	numRiga++;
	    	InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
					numRiga, 																		//numero riga
					1, 																				//quantita'
					"",																				//unita' di misura
					"**CONGUAGLIO** " + testoVoceDirittiAttivazione, 								//descrizione
					Double.parseDouble(importoConguaglioDirittiAttivazione),						//prezzo cad
					Double.parseDouble(importoConguaglioDirittiAttivazione), 						//importo totale (quantita' x prezzo)
					"EUR");																			//valuta
	    	
		    if ((codiceIvaRigaFattura != null ) && (!codiceIvaRigaFattura.equals(""))) {
		    	invoiceLine.setCodIva(codiceIvaRigaFattura);
		    }
		    
		    invoiceLine.setPreviousDocument(new PreviousDocument("contratto", codiceContratto));
		    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIGA.toString());
		    
	    	invoiceLines.add(invoiceLine);
	    }
	}

	
	/**
	 * Codice record txt 09000
	 * riassunto imponibile e aliquota del contratto corrente
	 * 
	 */
	private static void f09000(String line) {
		//logger.debug(line);
		
		codiceIva = substringNN(line,13,15).trim();
		aliquotaIva = substringNN(line,15,23).trim();
		if((aliquotaIva == null) || (aliquotaIva.equals("")))
			aliquotaIva = "0";
		
		imponibile = substringNN(line,23,40).trim();
		if((imponibile == null) || (imponibile.equals("")))
			imponibile = "0";
		if(substringNN(line,40,41).equals("-")) 
			imponibile = "-" + imponibile;
		
		importoIva = substringNN(line,41,58).trim();
		if((importoIva == null) || (importoIva.equals("")))
			importoIva = "0";
		if(substringNN(line,58,59).equals("")) 
			importoIva = "-" + importoIva;
		
		totaleTasse = totaleTasse + Double.parseDouble(importoIva);
		
		CodiciIva iva = new HeraTranslation().new CodiciIva(codiceContratto, codiceIva, Double.parseDouble(aliquotaIva));
		
		codiciIva.add(iva);
	}
	
	
	/**
	 * Codice record txt 10000
	 * altri servizi riferiti alla bolletta
	 * 
	 */
	private static void f10000(String line) {
		//logger.debug(line);
		numRiga++;
		
		tipoVoceAltriServizi = substringNN(line,13,19).trim();
		testoVoceAltriServizi = substringNN(line,19,69).trim();
		importoAltriServizi = substringNN(line,69,86).trim();
		if((importoAltriServizi == null) || (importoAltriServizi.equals("")))
			importoAltriServizi = "0";
		if(substringNN(line,86,87).equals("-"))
			importoAltriServizi = "-" + importoAltriServizi;
		
		codiceIvaAltriServizi = substringNN(line,87,89).trim();
		
	    InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
				numRiga, 												//numero riga
				1, 														//quantita'
				"",														//unita' di misura
				testoVoceAltriServizi, 									//descrizione
				Double.parseDouble(importoAltriServizi),				//prezzo cad
				Double.parseDouble(importoAltriServizi), 				//importo totale (quantita' x prezzo)
				"EUR");													//valuta

	    
		if ((codiceIvaAltriServizi != null ) && (!codiceIvaAltriServizi.equals(""))) {
			invoiceLine.setCodIva(codiceIvaAltriServizi);
		}
	    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIGA.toString());
	    
		invoiceLines.add(invoiceLine);
	}
	
	
	/**
	 * Codice record txt 11000
	 * ulteriori addebiti riferiti alla bolletta
	 * 
	 */
	private static void f11000(String line) {
		//logger.debug(line);
		numRiga ++;
		
	    tipoVoceUlterioriAddebiti = substringNN(line,13,19).trim();
		testoVoceUlterioriAddebiti = substringNN(line,19,69).trim();
	    importoUlterioriAddebiti = substringNN(line,69,86).trim();
		if((importoUlterioriAddebiti == null) || (importoUlterioriAddebiti.equals("")))
			importoUlterioriAddebiti = "0";
		if(substringNN(line,86,87).equals("-"))
			importoUlterioriAddebiti = "-" + importoUlterioriAddebiti;
		
	    codiceIvaUlterioriAddebiti = substringNN(line,87,89).trim();
	    
	    InvoiceLineHera invoiceLine = new HeraTranslation().new InvoiceLineHera(
									numRiga, 												//numero riga
									1, 														//quantita'
									"",														//unita' di misura
									testoVoceUlterioriAddebiti.trim(), 						//descrizione
									Double.parseDouble(importoUlterioriAddebiti),			//prezzo cad
									Double.parseDouble(importoUlterioriAddebiti), 			//importo totale (quantita' x prezzo)
									"EUR");													//valuta
	    
	    if ((codiceIvaUlterioriAddebiti != null ) && (!codiceIvaUlterioriAddebiti.equals(""))) {
	    	invoiceLine.setCodIva(codiceIvaUlterioriAddebiti);
	    }
	    
	    invoiceLine.setItemKeyword(XmlFactory.itemKeyword.RIGA.toString());
	    
	    invoiceLines.add(invoiceLine);
	}
	
	/**
	 * Codice record txt 12000
	 * riassunto imponibile e aliquota della bolletta corrente 
	 * (quindi se la bolletta ha un solo contratto, il codice 12000 deve essere uguale al codice 09000)
	 * 
	 */
	private static void f12000(String line) {
		//logger.debug(line);
		RiepilogoIva riepilogoIva = new HeraTranslation().new RiepilogoIva();
		
		codiceIvaRiepilogo = substringNN(line,13,15).trim();
		
		aliquotaIvaRiepilogo = substringNN(line,15,23).trim();			
		if((aliquotaIvaRiepilogo == null) || (aliquotaIvaRiepilogo.equals("")))
			aliquotaIvaRiepilogo = "0";
		
		imponibileRiepilogo = substringNN(line,23,40).trim();
		if((imponibileRiepilogo == null) || (imponibileRiepilogo.equals("")))
			imponibileRiepilogo = "0";
		if(substringNN(line,40,41).equals("-"))
			imponibileRiepilogo = "-" + imponibileRiepilogo;
		
		importoIvaRiepilogo = substringNN(line,41,58).trim();
		if((importoIvaRiepilogo == null) || (importoIvaRiepilogo.equals("")))
			importoIvaRiepilogo = "0";
		if(substringNN(line,58,59).equals("-"))
			importoIvaRiepilogo = "-" + importoIvaRiepilogo;
		
		riepilogoIva.setCodiceIvaRiepilogo(codiceIvaRiepilogo);
		riepilogoIva.setAliquotaIvaRiepilogo(Double.parseDouble(aliquotaIvaRiepilogo));
		riepilogoIva.setImponibileRiepilogo(Double.parseDouble(imponibileRiepilogo));
		riepilogoIva.setImportoIvaRiepilogo(Double.parseDouble(importoIvaRiepilogo));
		
		righeRiepilogoIva.add(riepilogoIva);
	}
	
	/**
	 * Codice record txt 13000
	 * importo totale a pagare della busta(quindi se la busta contiene una sola bolletta, il codice 13000 deve essere uguale al codice 12000)
	 */
	private static void f13000(String line) {
		//logger.debug(line);
		
		importoTotale = substringNN(line,13,31);
	}
	
	private static void f99000(String line) {
		//logger.debug(line);
		
		totBusteFlusso = substringNN(line,13,19);
		totFattureHeraSpa = substringNN(line,19,25);
		totFattureHeraComm = substringNN(line,25,31);
		importoTotaleFlusso = substringNN(line,31,49);
	}
	
	
	/**
	 * Restitutisce la sottostringa delimitata da inizio e fine
	 * se il limite superiore non eccede la lunghezza della stringa originale
	 * se no restituice una stringa vuota
	 * 
	 * @param str (String) stringa da cui ricavare la sottostringa <br />
	 * @param inizio (int) offset iniziale
	 * @param fine (int) offset finale
	 * 
	 * @return (String) la sottostringa richiesta oppure stringa vuota
	 */
	private static String substringNN(String str, int inizio,int fine) {
		String substring = "";
		
		if(fine<inizio){
			substring = "";
		}
		else {
			if(str.length()>=fine){		
				substring = str.substring(inizio,fine);
			}
			else {
				substring = substringNN(str, inizio, fine-1);
			}
		}
		
		substring = SpecialCharacter.stringToXmlEntity(substring);
		return substring;
	}
	
	
	private static byte[] buildXml() {
	    Element root = XmlFactory.createDocument();
	    
	    //CAMPI RELATIVI ALLA CREZIONE DELL'UTILITY STATEMENT NELLA FATTURA

	    List<String> settoriMerceologici = new ArrayList<String>();
	    List<PostalAddress> indirizzi = new ArrayList<PostalAddress>();
	    String utenza = "Mono-utenza";
	    String sito = "Mono-sito";
	    
	    for(int i = 0; i < contratti.size(); i++){
	    	
	    	/** Salvataggio di informazioni necessarie per l'Utility Statement: UtilityStatementType Code  **/
	    	HeraTranslation.ContractDocument contrattoCorrente = contratti.get(i);
	    	settoriMerceologici.add(contrattoCorrente.getSettoreMerceologico());
	    	
	    	if(settoriMerceologici.size() > 1) {
		    	for(int j = 0; j < settoriMerceologici.size(); j++) {
		    		for(int k = 0; k < settoriMerceologici.size(); k++) {
		    			if(!settoriMerceologici.get(j).equalsIgnoreCase(settoriMerceologici.get(k))) {
		    				utenza = "Multi-utenza";
		    				break;
		    			}
		    		}
		    	}
		    }
	    	
	    	PostalAddress indirizzoCorrente = new PostalAddress();
	    	indirizzoCorrente.setStreetName(contrattoCorrente.getIndirizzoFornitura());
	    	indirizzoCorrente.setBuildingNumber(contrattoCorrente.getCivicoFornitura());
	    	indirizzoCorrente.setCityName(contrattoCorrente.getLocalitaFornitura());
	    	indirizzoCorrente.setPostalZone(contrattoCorrente.getCapFornitura());
	    	indirizzi.add(indirizzoCorrente);
	    	/**		**/
	    }

	    /** Salvataggio di informazioni necessarie per l'Utility Statement: UtilityStatementType Code  **/
	    if(indirizzi.size() > 1) {
	    	for(int j = 0; j < indirizzi.size(); j++) {
	    		for(int k = 0; k < indirizzi.size(); k++) {
	    			if(!indirizzi.get(j).equals(indirizzi.get(k))) {
	    				sito = "Multi-sito";
	    				break;
	    			}
	    		}
	    	}
	    }
	    /**		**/
	    
	    //UTILITY STATEMENT
	    String utilityStatementId = "HERA - " + numeroFattura + " - " + utilityStatementIndex;
	    utilityStatementIndex++;
	    
	    double totalLineExtensionAmount = 0.0;
	    double totImportoIva = 0.0;
	    //Totale spese
	    double totalCharge = 0.0;
	    //Totale sconti/abbuoni
	    double totalAllowance = 0.0;
	    
	    Element utilityStatement = XmlFactory.setUtilityStatement(root, utilityStatementId, dataRegistrazione, sito + " " + utenza);

	    XmlFactory.setDocumentCurrencyCode(utilityStatement);
	    
	    //PARENT DOCUMENT REFERENCE
	    XmlFactory.setUSParentDocumentReference(utilityStatement, numeroFattura);
	    
	    //SENDER PARTY
	    if(codiceSocietaEmittente != null) {
		    if(codiceSocietaEmittente.equals("1070")) {
		    	//Hera Comm srl
		    	XmlFactory.setUSSenderParty(utilityStatement, "HERA Comm", "Via Molino Rosso", "8", "Imola", "40026", "Bologna", "IT", "Italia");
		    }
		    else if(codiceSocietaEmittente.equals("5010")) {
		    	//Hera Spa
		    	XmlFactory.setUSSenderParty(utilityStatement, "HERA", "Viale Berti Pichat", "2/4", "Bologna", "40100", "Bologna", "IT", "Italia");
		    }
		    else {
		    	logger.error("Impossibile identificare la societa' Hera (comm o spa)");
		    }
	    }
	    else {
	    	logger.error("Impossibile identificare la societa' Hera (comm o spa)");
	    }
	    
	    //RECEIVER PARTY
	    XmlFactory.setUSReceiverParty(utilityStatement, nome.trim() + cognome.trim(), indirizzo, civico, localita, cap, "", "", "");
	    
	    //SUBSCRIBER PARTY
	    String usRagioneSociale = "";
	    if(!cognome.equals("")){
	    	usRagioneSociale = cognome.trim();
	    	if(!nome.equals(""))
	    		usRagioneSociale = usRagioneSociale + " " + nome.trim();
	    }
	    else if(!nome.equals("")){
	    	usRagioneSociale = nome.trim();
	    }
	    
	    if(usRagioneSociale.equals("")) {
	    	logger.error("Impossibile identificare il cliente: ragione sociale non indicata");
	    }
	    else if(pIva.equals("")) {
	    	logger.error("Impossibile identificare il cliente: partita Iva non indicata");
	    }
	    else if((indirizzo.equals("")) || (civico.equals("")) || (localita.equals("")))  {
	    	logger.error("Impossibile identificare il cliente: indirizzo non indicato o indicato parzialmente");
	    }
	    XmlFactory.setUSSubscriberParty(utilityStatement, usRagioneSociale, indirizzo, civico, localita, cap, "", "", "");
	    
	    for(int k = 0; k < contratti.size(); k++) {
	    	HeraTranslation.ContractDocument contrattoCorrente = contratti.get(k);
	    	
	    	//SUBSCRIBER CONSUMPTION - Ripetuto per ogni punto di fornitura
	    	Element cacSubscriberConsumption = XmlFactory.setUSSubscriberConsumption(utilityStatement, contrattoCorrente.getSettoreMerceologico(), "");

	    		//UTILITY CONSUMPTION POINT
	    	Element cacUtilityConsumptionPoint = XmlFactory.setUSUtilityConsumptionPoint(cacSubscriberConsumption, contrattoCorrente.getPOD(), "", "", 
	    			contrattoCorrente.getIndirizzoFornitura(), contrattoCorrente.getCivicoFornitura(), 
	    			contrattoCorrente.getLocalitaFornitura(), contrattoCorrente.getCapFornitura(), "", "", "");

	    	List<String> contatori = contrattoCorrente.getContatori();
	    	
	    			//UTILITY METER
	    	if(contatori != null) {
	    		for(int i = 0; i < contatori.size(); i++) {
	    			String meterNumber = contatori.get(i);
	    			Element cacUtilityMeter = XmlFactory.setUSUtilityMeter(cacUtilityConsumptionPoint, meterNumber);
	    			
    				boolean isLatestReading = false;
    				String meterReadingId = "";
    				String previousDate = "";
    				double previousQuantity = 0.0;
    				String latestDate = "";
    				double latestQuantity = 0.0;
    				String prevoiusMethod = "";
    				String latestMethod = "";
    				String unitCode = "";
	    			
	    				//METER READING: nelle fatture Hera per ogni contatore ci sono due lettura, quella precedente e quella corrente
	    			for(int z = 0; z < invoiceLinesExtra.size(); z++) {
	    				
	    				InvoiceLineHera meterReading = invoiceLinesExtra.get(z);
	    				if(meterReading.getItemKeyword().equals("RIASSUNTO_CONTATORE") && meterReading.getContatore().equals(meterNumber))
	    					unitCode = meterReading.getUnitCode();
	    				
	    				if(meterReading != null && meterReading.getItemKeyword().equals("LETTURA") && meterNumber.equals(meterReading.getContatore())) {
	    					if(!isLatestReading) {
	    						previousDate = meterReading.getNotes().get(0);
	    						previousQuantity = meterReading.getInvoicedQuantity();
	    						prevoiusMethod = meterReading.getDescrizioneLettura() + " " + meterReading.getTipologiaLettura();
	    						isLatestReading = true;
	    					} 
	    					else {
	    						meterReadingId = meterReading.getItemName();
	    						latestDate = meterReading.getNotes().get(0);
	    						latestQuantity = meterReading.getInvoicedQuantity();
	    						latestMethod = meterReading.getDescrizioneLettura() + " " + meterReading.getTipologiaLettura();
	    						isLatestReading = false;
	    					}
	    				}
	    			}
	    			
					XmlFactory.setUSMeterReading(cacUtilityMeter, meterReadingId, previousDate, previousQuantity,
							latestDate, latestQuantity, prevoiusMethod, latestMethod, latestQuantity - previousQuantity, unitCode);
	    				
						//TODO: METER PROPERTY: non presenti nella fattura Hera quindi non inserite
	    		}
	    	}
	    
	    	//TODO: SUPPLIER CONSUMPTION - mancano i campi PartyTaxScheme SchemeID, TaxScheme ID, TaxScheme SchemeID
	    	Element cacSupplierConsumption = XmlFactory.setUSSupplierConsumption(cacSubscriberConsumption, "HERA", "Viale Berti Pichat", "2/4", 
	    			"Bologna", "40100", "Bologna", "IT", "Italia", "04245520376");

	    	//TODO: CONSUMPTION - non riempito
	    	Element cacConsumption = XmlFactory.setUSConsumption(cacSupplierConsumption, inizioPeriodo, finePeriodo, "");
	    	
	    	TaxDetail taxDetailAllowCharge = new TaxDetail();
		    Double imponibileAllowCharge = 0.0;
		    Double importoIvaAllowCharge = 0.0;
		    String codiceIvaHeraAllowCharge;
	    	
	    	for (int j = 0; j < invoiceLines.size(); j++) {
	    		HeraTranslation.InvoiceLineHera currentLineAllowCharge = invoiceLines.get(j);
	    		String itemNameAllowCharge = currentLineAllowCharge.getItemName();
	    		
	    		taxDetailAllowCharge = new TaxDetail();
	    		imponibileAllowCharge = 0.0;
	    		importoIvaAllowCharge = 0.0;  
	    		codiceIvaHeraAllowCharge = "";
	    		
	    		List<AllowanceCharge> allowanceCharges = currentLineAllowCharge.getAllowanceCharges();
	    			
	    		if(allowanceCharges != null && allowanceCharges.size() > 0) {
	    			
	    			//iva
				    if(currentLineAllowCharge.getCodIva() != null){
				    	codiceIvaHeraAllowCharge = currentLineAllowCharge.getCodIva();
				    	taxDetailAllowCharge.setDescription(codiceIvaHeraAllowCharge);
			    		
				    	if((currentLineAllowCharge.getPreviousDocument() != null)) {
				    		for(int z = 0; z < codiciIva.size(); z++) {		    			
				    			if((currentLineAllowCharge.getPreviousDocument().getDocumentCode().equals(codiciIva.get(z).getCodiceContratto())) 
				    					&& (codiceIvaHeraAllowCharge.equals(codiciIva.get(z).getCodice()))) {
				    				
				    				double lineExtensionAmount = currentLineAllowCharge.getLineExtensionAmount();
				    				totalLineExtensionAmount += lineExtensionAmount;
				    				
				    				imponibileAllowCharge = lineExtensionAmount * (1 - codiciIva.get(z).getAliquota()/100);
				    				importoIvaAllowCharge = lineExtensionAmount - imponibileAllowCharge;
			    	    			//Arrotondamento a 2 decimali
			    	    			imponibileAllowCharge = BigDecimal.valueOf(imponibileAllowCharge).setScale(2, RoundingMode.HALF_UP).doubleValue();
			    	    			importoIvaAllowCharge = BigDecimal.valueOf(importoIvaAllowCharge).setScale(2, RoundingMode.HALF_UP).doubleValue();
			    	    			
			    	    			totImportoIva += importoIvaAllowCharge;
			    	    			
			    	    			taxDetailAllowCharge.setTaxPercent(codiciIva.get(z).getAliquota());
			    	    			taxDetailAllowCharge.setTaxableAmount(imponibileAllowCharge);
			    	    			taxDetailAllowCharge.setTaxAmount(importoIvaAllowCharge);
			    	    			
			    	    			break;
			    	    		}
				    		}
				    	}
				    	else {
				    		for(int z = 0; z < righeRiepilogoIva.size(); z++) {		    	    	
				    			if((codiceIvaHeraAllowCharge.equals(righeRiepilogoIva.get(z).getCodiceIvaRiepilogo()))) {
			    	    			
				    				double lineExtensionAmount = currentLineAllowCharge.getLineExtensionAmount();
				    				totalLineExtensionAmount += lineExtensionAmount;
				    				
				    				imponibileAllowCharge = lineExtensionAmount * (1 - righeRiepilogoIva.get(z).getAliquotaIvaRiepilogo()/100);
				    				importoIvaAllowCharge = lineExtensionAmount - imponibileAllowCharge;
			    	    			//Arrotondamento a 2 decimali
			    	    			imponibileAllowCharge = BigDecimal.valueOf(imponibileAllowCharge).setScale(2, RoundingMode.HALF_UP).doubleValue();
			    	    			importoIvaAllowCharge = BigDecimal.valueOf(importoIvaAllowCharge).setScale(2, RoundingMode.HALF_UP).doubleValue();
		
			    	    			totImportoIva += importoIvaAllowCharge;
			    	    			
			    	    			taxDetailAllowCharge.setTaxPercent(righeRiepilogoIva.get(z).getAliquotaIvaRiepilogo());
			    	    			taxDetailAllowCharge.setTaxableAmount(imponibileAllowCharge);
			    	    			taxDetailAllowCharge.setTaxAmount(importoIvaAllowCharge);
			    	    			
			    	    			break;
			    	    		}
				    		}
				    	}
			    	}
	    			
	    			for(int i = 0; i < allowanceCharges.size(); i++) {
	    				AllowanceCharge allowanceCharge = allowanceCharges.get(i);
	    				double amount = allowanceCharge.getAmount();
	    				String chargeIndicator = "";
	    				if(amount > 0){
	    					chargeIndicator = "true";
	    					totalCharge += amount;
	    				}
	    				else {
	    					chargeIndicator = "false";
	    					totalAllowance += amount;
	    				}
	    				XmlFactory.setUSAllowanceCharge(cacConsumption, chargeIndicator, allowanceCharge.getAllowanceChargeReason(), amount, currentLineAllowCharge.getCurrencyID(),
	    						currentLineAllowCharge.getCodIva(), taxDetailAllowCharge.getTaxPercent(), taxDetailAllowCharge.getTaxableAmount(), taxDetailAllowCharge.getTaxAmount(), true);
	    			}
	    		}
	    	}
	    	
	    	Element cacEnergyWaterSupply = XmlFactory.setUSEnergyWaterSupply(cacConsumption);
	    	
	    	//CONSUMPTION REPORT - contiene le Invoice Line con keyword RIASSUNTO_CONTATORE
//	    	for (int i = 0; i < invoiceLinesExtra.size(); i++) {
//	    		HeraTranslation.InvoiceLineHera currentLine = invoiceLinesExtra.get(i);
//
//	    		if(currentLine.getItemKeyword().equals("RIASSUNTO_CONTATORE")) {
//	    			Element cacConsumptionReport = XmlFactory.setUSConsumptionReport(cacConsumption, "" + (i+1), 
//	    					currentLine.getItemName(), currentLine.getItemDescription(),
//	    					currentLine.getInvoicedQuantity(), currentLine.getUnitCode(), "", currentLine.getInizioPeriodo(),
//	    					currentLine.getFinePeriodo());
//	    		}
//
//	    	}
	    	
	    	//LEGAL MONETARY TOTAL
	    	XmlFactory.setUSLegalMonetaryTotal(cacConsumption, Double.parseDouble(importoFattura));
	    	
	    	//CONTRACT
	    	for(int i = 0; i < contratti.size(); i++){
		    	XmlFactory.setUSContract(cacSupplierConsumption, contratti.get(i).getCodiceContratto(), contratti.get(i).getSettoreMerceologico(), "", "");
		    }

	    	//CONSUMPTION LINES
		    TaxDetail taxDetail = new TaxDetail();
		    Double imponibile = 0.0;
		    Double importoIva = 0.0;
		    String codiceIvaHera;
		    
		    for(int i = 0; i < invoiceLines.size(); i++){
		    	InvoiceLineHera currentLine = invoiceLines.get(i);
		    	taxDetail = new TaxDetail();
		    	imponibile = 0.0;
			    importoIva = 0.0;
			    
			    codiceIvaHera = "";
			    
			    //iva
			    if(invoiceLines.get(i).getCodIva() != null){
		    		codiceIvaHera = currentLine.getCodIva();
		    		taxDetail.setDescription(codiceIvaHera);
		    		
			    	if((currentLine.getPreviousDocument() != null)) {
			    		for(int z = 0; z < codiciIva.size(); z++) {		    			
			    			if((currentLine.getPreviousDocument().getDocumentCode().equals(codiciIva.get(z).getCodiceContratto())) 
			    					&& (codiceIvaHera.equals(codiciIva.get(z).getCodice()))) {
			    				
			    				double lineExtensionAmount = currentLine.getLineExtensionAmount();
			    				totalLineExtensionAmount += lineExtensionAmount;
			    				
		    	    			imponibile = lineExtensionAmount * (1 - codiciIva.get(z).getAliquota()/100);
		    	    			importoIva = lineExtensionAmount - imponibile;
		    	    			//Arrotondamento a 2 decimali
		    	    			imponibile = BigDecimal.valueOf(imponibile).setScale(2, RoundingMode.HALF_UP).doubleValue();
		    	    			importoIva = BigDecimal.valueOf(importoIva).setScale(2, RoundingMode.HALF_UP).doubleValue();
		    	    			
		    	    			totImportoIva += importoIva;
		    	    			
		    	    			taxDetail.setTaxPercent(codiciIva.get(z).getAliquota());
		    	    			taxDetail.setTaxableAmount(imponibile);
		    	    			taxDetail.setTaxAmount(importoIva);
		    	    			
		    	    			break;
		    	    		}
			    		}
			    	}
			    	else {
			    		for(int z = 0; z < righeRiepilogoIva.size(); z++) {		    	    	
			    			if((codiceIvaHera.equals(righeRiepilogoIva.get(z).getCodiceIvaRiepilogo()))) {
		    	    			
			    				double lineExtensionAmount = currentLine.getLineExtensionAmount();
			    				totalLineExtensionAmount += lineExtensionAmount;
			    				
		    	    			imponibile = lineExtensionAmount * (1 - righeRiepilogoIva.get(z).getAliquotaIvaRiepilogo()/100);
		    	    			importoIva = lineExtensionAmount - imponibile;
		    	    			//Arrotondamento a 2 decimali
		    	    			imponibile = BigDecimal.valueOf(imponibile).setScale(2, RoundingMode.HALF_UP).doubleValue();
		    	    			importoIva = BigDecimal.valueOf(importoIva).setScale(2, RoundingMode.HALF_UP).doubleValue();
	
		    	    			totImportoIva += importoIva;
		    	    			
		    	    			taxDetail.setTaxPercent(righeRiepilogoIva.get(z).getAliquotaIvaRiepilogo());
		    	    			taxDetail.setTaxableAmount(imponibile);
		    	    			taxDetail.setTaxAmount(importoIva);
		    	    			
		    	    			break;
		    	    		}
			    		}
			    	}
		    	}
			    XmlFactory.setUSConsumptionLine(cacSupplierConsumption, "" + (i+1), "1", currentLine.getInvoicedQuantity(), 
			    		currentLine.getUnitCode(), currentLine.getLineExtensionAmount(), currentLine.getCurrencyID(), 
			    		currentLine.getInizioPeriodo(), currentLine.getFinePeriodo(), "", taxDetail.getTaxAmount(), currentLine.getCurrencyID(),
			    		"" + (i+1), currentLine.getItemDescription(), currentLine.getCodIva(), taxDetail.getTaxPercent(), currentLine.getPriceAmount(), 
			    		currentLine.getCurrencyID(), currentLine.getInvoicedQuantity());
	    	}
	    }
	    
	    //UBL VERSION ID
	    XmlFactory.setUBLVersionID(root, "2.0");
	    
	    //CUSTOMIZATION ID E PROFILE ID
	    XmlFactory.setCustomizationID(root);
	    XmlFactory.setProfileID(root);
	    
	    //NUMERO FATTURA
	    if(numeroFattura != null) {
	    	XmlFactory.setNumeroFattura(root, numeroFattura);
	    }
	    else {
	    	logger.error("Impossibile identificare il numero della fattura");
	    }
	    
	    //DATA EMISSIONE
	    if(dataRegistrazione != null) {
	    	XmlFactory.setDataEmissione(root, dataRegistrazione);
	    }
	    else {
	    	logger.error("Impossibile identificare la data di emissione della fattura");
	    }
	    
	    //INVOICE TYPE CODE
	    XmlFactory.setInvoiceTypeCode(root, "380");
	    
	    //DOCUMENT CURRENCY CODE
	    XmlFactory.setDocumentCurrencyCode(root);
	    
//	    //CONTO CONTRATTUALE
//	    if(contoContrattuale != null) {
//	    	XmlFactory.setContoContrattuale(root, contoContrattuale);
//	    }
//	    else {
//	    	logger.error("Impossibile identificare il conto contrattuale della fattura");
//	    }
	    
	    //PERIODO DI CONSUMO
	    XmlFactory.setPeriodoFatturazione(root, inizioPeriodo, finePeriodo, "");
	    
	    //CONTRATTI DI RIFERIMENTO
	    for(int i = 0; i < contratti.size(); i++){
	    	XmlFactory.setContrattoDiRiferimento(root, contratti.get(i).getCodiceContratto(), contratti.get(i).getSettoreMerceologico());
	    }
	    
	    //FORNITORE: Hera Comm o Hera spa
	    if(codiceSocietaEmittente != null) {
		    if(codiceSocietaEmittente.equals("1070")) {
		    	//Hera Comm srl
		    	XmlFactory.setFornitore(root, "", "HERA Comm", "", "Via Molino Rosso", "8", "Imola (BO)", "40026", "Bologna", "IT", "Italia", 
		    								"HERA Comm S.R.L.", "Via Molino Rosso", "8", "Imola (BO)", "40026", "Bologna", "IT", "Italia", "02221101203");
		    }
		    else if(codiceSocietaEmittente.equals("5010")) {
		    	//Hera Spa
		    	XmlFactory.setFornitore(root, "", "HERA", "", "Viale Berti Pichat", "2/4", "Bologna", "40100", "Bologna", "IT", "Italia",  
		    								"HERA S.p.a.", "Viale Berti Pichat", "2/4", "Bologna", "40100", "Bologna", "IT", "Italia", "04245520376");
		    }
		    else {
		    	logger.error("Impossibile identificare la società Hera (comm o spa)");
		    }
	    }
	    else {
	    	logger.error("Impossibile identificare la società Hera (comm o spa)");
	    }
	    
	    //CLIENTE
	    String ragioneSociale = "";
	    if(!cognome.equals("")){
	    	ragioneSociale = cognome.trim();
	    	if(!nome.equals(""))
	    		ragioneSociale = ragioneSociale + " " + nome.trim();
	    }
	    else if(!nome.equals("")){
	    	ragioneSociale = nome.trim();
	    }
	    
	    if(ragioneSociale.equals("")) {
	    	logger.error("Impossibile identificare il cliente: ragione sociale non indicata");
	    }
	    else if(pIva.equals("")) {
	    	logger.error("Impossibile identificare il cliente: partita Iva non indicata");
	    }
	    else if((indirizzo.equals("")) || (civico.equals("")) || (localita.equals("")))  {
	    	logger.error("Impossibile identificare il cliente: indirizzo non indicato o indicato parzialmente");
	    }
	    XmlFactory.setCliente(root, codiceCliente, ragioneSociale, indirizzo, civico + " " + civico2, localita, cap, "", "IT", "Italia",
	    		ragioneSociale, indirizzo, civico + " " + civico2, localita, cap, "", "IT", "Italia", categoriaPIva + pIva);
	    
	    //FORNITURE
//	    for(int i = 0; i < contratti.size(); i++){
//	    	
//	    	if(contratti.get(i).getContatori() != null){
//		    	for(int y = 0; y < contratti.get(i).getContatori().size(); y++){
//		    		XmlFactory.setFornitura(root, contratti.get(i).getContatori().get(y), contratti.get(i).getPOD(), contratti.get(i).getCodiceContratto(), contratti.get(i).getIndirizzoFornitura(), contratti.get(i).getCivicoFornitura(), contratti.get(i).getCapFornitura(), contratti.get(i).getLocalitaFornitura());
//		    	}
//	    	}
//	    	else if(contratti.get(i).getIndirizzoFornitura() != null){
//	    		XmlFactory.setFornitura(root, "", contratti.get(i).getPOD(), contratti.get(i).getCodiceContratto(), contratti.get(i).getIndirizzoFornitura(), contratti.get(i).getCivicoFornitura(), contratti.get(i).getCapFornitura(), contratti.get(i).getLocalitaFornitura());	    		
//	    	}
//	    }
    
	    //DATI PAGAMENTO
	    XmlFactory.setDataScadenza(root, dataScadenza, null);
	    
	    //RIEPILOGO IVA
	    if((righeRiepilogoIva != null) && (righeRiepilogoIva.size() > 0)){
	    	totaleTasse = Math.abs(totaleTasse);
	    	Element taxTotal = XmlFactory.setRiepilogoIva(root, null, null, null, String.valueOf(totaleTasse), false);
	    	
	    	for(int i = 0; i < righeRiepilogoIva.size(); i++) {
	    		
	    		if(imponibileRiepilogo != null && !imponibileRiepilogo.isEmpty())
	    			XmlFactory.setRiepilogoIva(taxTotal, righeRiepilogoIva.get(i).getCodiceIvaRiepilogo(), 
		    									righeRiepilogoIva.get(i).getAliquotaIvaRiepilogo(), 
		    									String.valueOf(righeRiepilogoIva.get(i).getImponibileRiepilogo()), 
		    									String.valueOf(righeRiepilogoIva.get(i).getImportoIvaRiepilogo()),
		    									true);
		    }
	    }
	    
	    //TOTALE FATTURA
	    totalLineExtensionAmount = BigDecimal.valueOf(totalLineExtensionAmount).setScale(2, RoundingMode.HALF_UP).doubleValue();
	    totImportoIva = BigDecimal.valueOf(totImportoIva).setScale(2, RoundingMode.HALF_UP).doubleValue();
	    double totaleTasseEscluse = totalLineExtensionAmount + totalCharge - totalAllowance;
	    double totaleTasseIncluse = totalLineExtensionAmount + totImportoIva;
	    totaleTasseIncluse = BigDecimal.valueOf(totaleTasseIncluse).setScale(2, RoundingMode.HALF_UP).doubleValue();
	    
	    XmlFactory.setTotaleFattura(root, importoFattura,"EUR", totalLineExtensionAmount, totaleTasseEscluse, totaleTasseIncluse, totalAllowance, totalCharge);

//	    codiceIva = substringNN(line,13,15).trim();
//		aliquotaIva = substringNN(line,15,23).trim();
//		if((aliquotaIva == null) || (aliquotaIva.equals("")))
//			aliquotaIva = "0";
//		
//		imponibile = substringNN(line,23,40).trim();
//		if((imponibile == null) || (imponibile.equals("")))
//			imponibile = "0";
//		if(substringNN(line,40,41).equals("-")) 
//			imponibile = "-" + imponibile;
//		
//		importoIva = substringNN(line,41,58).trim();
//		if((importoIva == null) || (importoIva.equals("")))
//			importoIva = "0";
//		if(substringNN(line,58,59).equals("")) 
//			importoIva = "-" + importoIva;
//		
//		totaleTasse = totaleTasse + Double.parseDouble(importoIva);
//	    
//	    //DETTAGLIO FATTURA
	    TaxDetail taxDetail = new TaxDetail();
	    Double imponibile = 0.0;
	    Double importoIva = 0.0;
	    String codiceIvaHera;
	    
	    for(int i = 0; i < invoiceLines.size(); i++){
	    	taxDetail = new TaxDetail();
	    	imponibile = 0.0;
		    importoIva = 0.0;
		    
		    codiceIvaHera = "";
		    
		    //iva
		    if(invoiceLines.get(i).getCodIva() != null){
	    		codiceIvaHera = invoiceLines.get(i).getCodIva();
	    		taxDetail.setDescription(codiceIvaHera);
	    		
		    	if((invoiceLines.get(i).getPreviousDocument() != null)) {
		    		for(int z = 0; z < codiciIva.size(); z++) {		    			
		    			if((invoiceLines.get(i).getPreviousDocument().getDocumentCode().equals(codiciIva.get(z).getCodiceContratto())) 
		    					&& (codiceIvaHera.equals(codiciIva.get(z).getCodice()))) {
	    	    			
	    	    			imponibile = invoiceLines.get(i).getLineExtensionAmount() * (1 - codiciIva.get(z).getAliquota()/100);
	    	    			importoIva = invoiceLines.get(i).getLineExtensionAmount() - imponibile;
	    	    			//Arrotondamento a 2 decimali
	    	    			imponibile = BigDecimal.valueOf(imponibile).setScale(2, RoundingMode.HALF_UP).doubleValue();
	    	    			importoIva = BigDecimal.valueOf(importoIva).setScale(2, RoundingMode.HALF_UP).doubleValue();
	    	    			
	    	    			taxDetail.setTaxPercent(codiciIva.get(z).getAliquota());
	    	    			taxDetail.setTaxableAmount(imponibile);
	    	    			taxDetail.setTaxAmount(importoIva);
	    	    			
	    	    			break;
	    	    		}
		    		}
		    	}
		    	else {
		    		for(int z = 0; z < righeRiepilogoIva.size(); z++) {		    	    	
		    			if((codiceIvaHera.equals(righeRiepilogoIva.get(z).getCodiceIvaRiepilogo()))) {
	    	    			
	    	    			imponibile = invoiceLines.get(i).getLineExtensionAmount() * (1 - righeRiepilogoIva.get(z).getAliquotaIvaRiepilogo()/100);
	    	    			importoIva = invoiceLines.get(i).getLineExtensionAmount() - imponibile;
	    	    			//Arrotondamento a 2 decimali
	    	    			imponibile = BigDecimal.valueOf(imponibile).setScale(2, RoundingMode.HALF_UP).doubleValue();
	    	    			importoIva = BigDecimal.valueOf(importoIva).setScale(2, RoundingMode.HALF_UP).doubleValue();

	    	    			taxDetail.setTaxPercent(righeRiepilogoIva.get(z).getAliquotaIvaRiepilogo());
	    	    			taxDetail.setTaxableAmount(imponibile);
	    	    			taxDetail.setTaxAmount(importoIva);
	    	    			
	    	    			break;
	    	    		}
		    		}
		    	}
	    	}
	    	
	    	XmlFactory.setRigaFattura(root,
	    			invoiceLines.get(i).getItemKeyword(),
	    			invoiceLines.get(i).getItemName(),
	    			invoiceLines.get(i).getLineNumber(),
	    			null, null, null,
	    			invoiceLines.get(i).getInvoicedQuantity(),
	    			invoiceLines.get(i).getPriceAmount(),
	    			invoiceLines.get(i).getAllowanceCharges(),
	    			invoiceLines.get(i).getLineExtensionAmount(),
	    			taxDetail,
	    			codiceIvaHera,
	    			invoiceLines.get(i).getItemDescription(),
	    			invoiceLines.get(i).getUnitCode(),
	    			invoiceLines.get(i).getCurrencyID(),
	    			invoiceLines.get(i).getPreviousDocument(), 
	    			invoiceLines.get(i).getNotes(),
	    			invoiceLines.get(i).getContatore());
	    }
	    
	    //RIGHE DI RIEPILOGO
//	    if((invoiceLinesExtra != null) && (invoiceLinesExtra.size() > 0)) {
//	    	for(int i = 0; i < invoiceLinesExtra.size(); i++){
//	    		numRiga++;
//	    		
//		    	XmlFactory.setRigaFattura(root,
//		    			invoiceLinesExtra.get(i).getItemKeyword(),
//		    			invoiceLinesExtra.get(i).getItemName(),
//		    			numRiga,
//		    			null, null, null,
//		    			invoiceLinesExtra.get(i).getInvoicedQuantity(),
//		    			invoiceLinesExtra.get(i).getPriceAmount(),
//		    			invoiceLinesExtra.get(i).getAllowanceCharges(),
//		    			invoiceLinesExtra.get(i).getLineExtensionAmount(),
//		    			null,
//		    			null,
//		    			invoiceLinesExtra.get(i).getItemDescription(),
//		    			invoiceLinesExtra.get(i).getUnitCode(),
//		    			invoiceLinesExtra.get(i).getCurrencyID(),
//		    			invoiceLinesExtra.get(i).getPreviousDocument(), 
//		    			invoiceLinesExtra.get(i).getNotes(),
//		    			invoiceLinesExtra.get(i).getContatore());
//	    	}
//	    } else {
//	    	XmlFactory.setRigaFattura(root,
//	    			"",
//	    			"",
//	    			1,
//	    			null, null, null,
//	    			0.0,
//	    			0.0,
//	    			null,
//	    			Double.parseDouble(importoFattura),
//	    			null,
//	    			null,
//	    			"Totale della bolletta",
//	    			"",
//	    			"EUR",
//	    			null, 
//	    			null,
//	    			"");
//	    }
//	    XmlFactory.setRigaFattura(root,
//    			"",
//    			"Totale della bolletta",
//    			1,
//    			null, null, null,
//    			1,
//    			Double.parseDouble(importoFattura),
//    			null,
//    			Double.parseDouble(importoFattura),
//    			null,
//    			null,
//    			"",
//    			"",
//    			"EUR",
//    			null, 
//    			null,
//    			"");
	    return XmlFactory.generateXml(root); 
	}
	
	
	
	
	//---- UTILS ----//
	
	class ContractDocument {
		String codiceContratto;
		String settoreMerceologico;
		String indirizzoFornitura;
		String civicoFornitura;
		String capFornitura;
		String localitaFornitura;
		String POD;
		List<String> contatori;
		
		ContractDocument(String codiceContratto) {
			this.codiceContratto = codiceContratto;
		}

		public ContractDocument() {
		}

		public String getCodiceContratto() {
			return codiceContratto;
		}

		public void setCodiceContratto(String codiceContratto) {
			this.codiceContratto = codiceContratto;
		}

		public String getSettoreMerceologico() {
			return settoreMerceologico;
		}

		public void setSettoreMerceologico(String settoreMerceologico) {
			this.settoreMerceologico = settoreMerceologico;
		}

		public String getIndirizzoFornitura() {
			return indirizzoFornitura;
		}

		public void setIndirizzoFornitura(String indirizzoFornitura) {
			this.indirizzoFornitura = indirizzoFornitura;
		}

		public String getCivicoFornitura() {
			return civicoFornitura;
		}

		public void setCivicoFornitura(String civicoFornitura) {
			this.civicoFornitura = civicoFornitura;
		}

		public String getCapFornitura() {
			return capFornitura;
		}

		public void setCapFornitura(String capFornitura) {
			this.capFornitura = capFornitura;
		}

		public String getLocalitaFornitura() {
			return localitaFornitura;
		}

		public void setLocalitaFornitura(String localitaFornitura) {
			this.localitaFornitura = localitaFornitura;
		}

		public List<String> getContatori() {
			return contatori;
		}

		public void setContatori(List<String> contatori) {
			this.contatori = contatori;
		}

		public String getPOD() {
			return POD;
		}

		public void setPOD(String pOD) {
			POD = pOD;
		}
	
	}
	
	class CodiciIva {
		String codiceContratto;
		String codice;
		double aliquota;
		
		public CodiciIva(String codiceContratto, String codice, double aliquota) {
			super();
			this.codiceContratto = codiceContratto;
			this.codice = codice;
			this.aliquota = aliquota;
		}

		public String getCodiceContratto() {
			return codiceContratto;
		}

		public void setCodiceContratto(String codiceContratto) {
			this.codiceContratto = codiceContratto;
		}
		
		public String getCodice() {
			return codice;
		}
		public void setCodice(String codice) {
			this.codice = codice;
		}
		public double getAliquota() {
			return aliquota;
		}
		public void setAliquota(double aliquota) {
			this.aliquota = aliquota;
		}
	}
	
	class RiepilogoIva {
		String codiceIvaRiepilogo;
		Double aliquotaIvaRiepilogo;
		Double imponibileRiepilogo;
		Double importoIvaRiepilogo;
		public RiepilogoIva(String codiceIvaRiepilogo,
				Double aliquotaIvaRiepilogo, Double imponibileRiepilogo,
				Double importoIvaRiepilogo) {
			super();
			this.codiceIvaRiepilogo = codiceIvaRiepilogo;
			this.aliquotaIvaRiepilogo = aliquotaIvaRiepilogo;
			this.imponibileRiepilogo = imponibileRiepilogo;
			this.importoIvaRiepilogo = importoIvaRiepilogo;
		}
		
		public RiepilogoIva() {

		}

		public String getCodiceIvaRiepilogo() {
			return codiceIvaRiepilogo;
		}
		public void setCodiceIvaRiepilogo(String codiceIvaRiepilogo) {
			this.codiceIvaRiepilogo = codiceIvaRiepilogo;
		}
		public Double getAliquotaIvaRiepilogo() {
			return aliquotaIvaRiepilogo;
		}
		public void setAliquotaIvaRiepilogo(Double aliquotaIvaRiepilogo) {
			this.aliquotaIvaRiepilogo = aliquotaIvaRiepilogo;
		}
		public Double getImponibileRiepilogo() {
			return imponibileRiepilogo;
		}
		public void setImponibileRiepilogo(Double imponibileRiepilogo) {
			this.imponibileRiepilogo = imponibileRiepilogo;
		}
		public Double getImportoIvaRiepilogo() {
			return importoIvaRiepilogo;
		}
		public void setImportoIvaRiepilogo(Double importoIvaRiepilogo) {
			this.importoIvaRiepilogo = importoIvaRiepilogo;
		}
	}
	
	class InvoiceLineHera extends InvoiceLine {
		
		private static final long serialVersionUID = 5084027370115789749L;
		
		String codIva;
		List<String> notes;
		String contatore;
		String itemKeyword;
		String itemName;
		String descrizioneLettura;
		String tipologiaLettura;
		String inizioPeriodo;
		String finePeriodo;

		public InvoiceLineHera() {
			super();
		}


		public InvoiceLineHera(int lineNumber, double invoicedQuantity,
				String unitCode, String itemDescription, double priceAmount,
				double lineExtensionAmount, String currencyID) {
			super(lineNumber, invoicedQuantity, unitCode, itemDescription, priceAmount,
					lineExtensionAmount, currencyID);
		}

		public String getCodIva() {
			if (codIva == null) {
				codIva = "";
			}
			
			return codIva;
		}

		public void setCodIva(String codiceIva) {
			codIva = codiceIva;
		}

		public List<String> getNotes() {
			return notes;
		}

		public void setNotes(List<String> notes) {
			this.notes = notes;
		}

		public String getContatore() {
			return contatore;
		}

		public void setContatore(String contatore) {
			this.contatore = contatore;
		}

		public String getItemKeyword() {
			return itemKeyword;
		}

		public void setItemKeyword(String itemKeyword) {
			this.itemKeyword = itemKeyword;
		}

		public String getItemName() {
			return itemName;
		}

		public void setItemName(String itemName) {
			this.itemName = itemName;
		}
		
		public String getDescrizioneLettura() {
			return descrizioneLettura;
		}

		public void setDescrizioneLettura(String descrizioneLettura) {
			this.descrizioneLettura = descrizioneLettura;
		}
		
		public String getTipologiaLettura() {
			return tipologiaLettura;
		}

		public void setTipologiaLettura(String tipologiaLettura) {
			this.tipologiaLettura = tipologiaLettura;
		}
		
		public String getInizioPeriodo() {
			return inizioPeriodo;
		}

		public void setInizioPeriodo(String inizioPeriodo) {
			this.inizioPeriodo = inizioPeriodo;
		}
		
		public String getFinePeriodo() {
			return finePeriodo;
		}

		public void setFinePeriodo(String finePeriodo) {
			this.finePeriodo = finePeriodo;
		}
	}
}
