/**
 * This file is part of FatturazioneElettronica - Signature and timestamp facilities
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Signature and timestamp facilities is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Signature and timestamp facilities is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Signature and timestamp facilities.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.signature;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;



public class ByteArrayUtils {
	
	public static List<byte[]> split(byte[] large, byte[] subset){
		if ( (large == null) || (large.length <= 0 ))
			return  null;
		if ( (subset == null) || (subset.length <= 0 ))
			return null;
		if ( subset.length > large.length)
			return null;
		List<byte[]> out = new ArrayList<byte[]>();
		int index = firstOccurrence(large,subset);
		while (index >= 0) {
			
			out.add(Arrays.copyOfRange(large, 0, index));
			byte[] remaining = Arrays.copyOfRange(large, index+subset.length, large.length);
			large = remaining;
			index = firstOccurrence(large,subset);
		}
		out.add(large);
		return out;
			
	}
	
	public static boolean equalsTo(byte[] first, byte[] second){
		if ( (first == null) || (first.length <= 0 ))
			return false;
		if ( (second == null) || (second.length <= 0 ))
			return false;
		if ( second.length != first.length)
			return false;
		
		for (int i = 0; i < first.length; i++) {
			if (first[i] != second[i]){
				return false;
			}
				
		}
		return true;
			
	}
	
	public static int firstOccurrence(byte[] large, byte[] subset){
		if ( (large == null) || (large.length <= 0 ))
			return -1;
		if ( (subset == null) || (subset.length <= 0 ))
			return -1;
		if ( subset.length > large.length)
			return -1;
		
		for (int i = 0; i < large.length; i++) {
			if (large[i] == subset[0]){
				int index = i;
				boolean found = true;
				for (int k = 0; k < subset.length; k++){
					if ( (large.length<=i+k) || (large[i+k] != subset[k]) ){
						found = false;
						break;
					}
				}
				if (found)
					return index;
			}
				
		}
		return -1;
			
	}

	public static byte[] trim(byte[] file) {
		final byte[] carriageReturn = "\r\n".getBytes();
		byte[] remaining = Arrays.copyOfRange(file,0,file.length);
		int index = firstOccurrence(remaining, carriageReturn);
		while(index==0){
			remaining= Arrays.copyOfRange(remaining,2,remaining.length);
			index = firstOccurrence(remaining, carriageReturn);
		}
		
		while(endsWith(remaining, carriageReturn)){
			remaining= Arrays.copyOfRange(remaining,0,remaining.length-2);
		}
		return remaining;
			
	}
	public static boolean endsWith(byte[] file, byte[] end) {
		if ( (file == null) || (file.length <= 0 ))
			return  false;
		if ( (end == null) || (end.length <= 0 ))
			return false;
		if ( end.length > file.length)
			return false;
		
		for (int i = 1; i <= end.length; i++) {
			if (end[end.length-i] != file[file.length-i])
				return false;
		}
		return true;
	}
	
}
