/**
 * This file is part of FatturazioneElettronica - Signature and timestamp facilities
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Signature and timestamp facilities is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Signature and timestamp facilities is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Signature and timestamp facilities.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.trento.comune.j4sign.verification.servlet;


import it.trento.comune.j4sign.verification.CertificationAuthorities;
import it.trento.comune.j4sign.verification.RootsVerifier;
import it.trento.comune.j4sign.verification.VerifyResult;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.Security;
import java.security.cert.CertStore;
import java.security.cert.CertStoreException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import javax.security.auth.x500.X500Principal;

import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.CMSSignedDataStreamGenerator;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;
import org.bouncycastle.util.encoders.Base64;

public class Verifier {
	private Logger log = Logger.getLogger(this.getClass().getName());

	private String filepath;
	private int current;

	private CertificationAuthorities CAroot;

	private boolean done;

	private boolean canceled;

	private boolean passed;

	private int differentSigners;

	private ArrayList<X500Principal> signersList;

	private Iterator currentSigner;

	private CMSSignedData cms;

	private Hashtable<X500Principal, VerifyResult> risultati;

	private Hashtable<X500Principal, SignerInformationStore> counterSignatures;

	private String CRLerror = "";

	private Vector<String> errors = new Vector<String>();

	private boolean isDownloadCRLForced;

	private RootsVerifier rootsVerifier = null;

	public Verifier(String f, boolean isDownloadCRLForced, RootsVerifier rv)
			throws Exception {

		filepath = f;
		signersList = new ArrayList();
		this.isDownloadCRLForced = isDownloadCRLForced;
		this.rootsVerifier = rv;

		readFile();

		initCARoots();

		if (CAroot != null) {
			risultati = new Hashtable();
		}

	}

	public Verifier(byte[] file, boolean isDownloadCRLForced, RootsVerifier rv)
			throws Exception {

		signersList = new ArrayList();
		this.isDownloadCRLForced = isDownloadCRLForced;
		this.rootsVerifier = rv;


		// codifica file Base64 o DER?
		byte[] signedBytes;
		try {
			// se Base64, decodifica (italian law!)
			signedBytes = Base64.decode(file);
			// Decodifica base64 completata
			// setMessage("Il file firmato è in formato Base64");
		} catch (Exception e) {
			// il file non e' in formato base64
			// quindi è in DER (against italian law!)
			// setMessage("Il file firmato è in formato DER");
			signedBytes = file;

		}
		
		readFile(file);

		initCARoots();

		if (CAroot != null) {
			risultati = new Hashtable();
		}

	}

	public void initCARoots() throws Exception {
		try {
			CAroot = this.rootsVerifier.getRoots();
		} catch (Exception ex) {
			log.severe("Errore nell'inizializzazione delle CA: " + ex);
			errors.add("Errore nell'inizializzazione delle CA: " + ex);
			throw (new Exception(ex));
		}
		if (CAroot == null)
			throw (new Exception("Errore nell'inizializzazione delle CA"));
	}

	public Vector<String> getErrors() {
		return errors;
	}

	public static byte[] getSignedContentBytes(String filepath)
			throws IOException {

		byte[] buffer = new byte[1024];

		FileInputStream is = null;

		is = new FileInputStream(filepath);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		while (is.read(buffer) > 0) {
			baos.write(buffer);
		}

		byte[] risultato = baos.toByteArray();

		// codifica file Base64 o DER?
		byte[] signedBytes;
		try {
			// se Base64, decodifica (italian law!)
			signedBytes = Base64.decode(risultato);
			// Decodifica base64 completata
			// setMessage("Il file firmato è in formato Base64");
		} catch (Exception e) {
			// il file non e' in formato base64
			// quindi è in DER (against italian law!)
			// setMessage("Il file firmato è in formato DER");
			signedBytes = risultato;

		}

		return signedBytes;
	}
	public void readFile(byte[] file) {
		// Estrazione del certificato dal file (ora codificato DER)
		try {
			this.cms = new CMSSignedData(file);
		} catch (CMSException ex1) {
			log.severe("Errore nell'estrazione del certificato dal file");
			errors.add("Errore nell'estrazione del certificato dal file");
			ex1.printStackTrace();
		} catch (IllegalArgumentException ex1) {
			log.severe("Errore nell'estrazione del certificato dal file");
			errors.add("Errore nell'estrazione del certificato dal file");
			ex1.printStackTrace();
		} 
		if (errors.isEmpty()) {
			org.bouncycastle.jce.provider.BouncyCastleProvider p = new org.bouncycastle.jce.provider.BouncyCastleProvider();
			if (Security.getProvider(p.getName()) == null)
				Security.addProvider(p);

			// recupero dal CMS la lista dei certificati

			try {
				CertStore certs = cms
						.getCertificatesAndCRLs("Collection", "BC");
			} catch (CMSException ex2) {
				log.severe("Errore nel CMS");
				errors.add("Errore nel CMS");
			} catch (NoSuchProviderException ex2) {
				log.severe("Non esiste il provider del servizio");
				errors.add("Non esiste il provider del servizio");
			} catch (NoSuchAlgorithmException ex2) {
				log.severe("Errore nell'algoritmo");
				errors.add("Errore nell'algoritmo");
			}

			// Recupero i firmatari.
			SignerInformationStore signers = cms.getSignerInfos();

			Collection c = signers.getSigners();
			differentSigners = cms.getSignerInfos().size();

			// non avrebbe senso che fossero uguali
			// quindi fa il ciclo tra i firmatari
			// PERO' PUO' CAPITARE CHE CI SIA UN FIRMATARIO CHE FIRMA DUE VOLTE
			// E IN QUESTO CASO DOVREBBE FARE IL GIRO SUI CERTIFICATI!!!
			currentSigner = c.iterator();
			if (!currentSigner.hasNext()) {
				done = true;
			}
		} else {
			canceled = true;
		}
	}
	public void readFile() throws IOException{
		readFile(getSignedContentBytes(filepath));
	}

	public void verify() {
		org.bouncycastle.jce.provider.BouncyCastleProvider p = new org.bouncycastle.jce.provider.BouncyCastleProvider();
		if (Security.getProvider(p.getName()) == null)
			Security.addProvider(p);

		X509Certificate cert = null;
		CertStore certs = null;

		passed = false;

		try {

			certs = this.cms.getCertificatesAndCRLs("Collection", "BC");

		} catch (CMSException ex2) {
			log.severe("Errore nel CMS");
			errors.add("Errore nel CMS");
		} catch (NoSuchProviderException ex2) {
			log.severe("Non esiste il provider del servizio");
			errors.add("Non esiste il provider del servizio");
		} catch (NoSuchAlgorithmException ex2) {
			log.severe("Errore nell'algoritmo");
			errors.add("Errore nell'algoritmo");
		}

		if (certs != null) {
			SignerInformation signer = (SignerInformation) currentSigner.next();

			Collection certCollection = null;
			try {
				certCollection = certs.getCertificates(signer.getSID());
			} catch (CertStoreException ex1) {
				log.severe("Errore nel CertStore");
				errors.add("Errore nel CertStore");
			}

			if (certCollection.size() == 1) {
				// Iterator certIt = certCollection.iterator();
				// X509Certificate cert = (X509Certificate)
				// certIt.next();

				cert = (X509Certificate) certCollection.toArray()[0];
				// CertValidity cv=new CertValidity(cert, CAroot);
				// log.info(i + ") Verifiying signature from:\n"
				// + cert.getSubjectDN());

				// inserisce in una lista i DN dei firmatari
				signersList.add(cert.getSubjectX500Principal());

				log.info("Signer CN: " + getCommonName(cert));

				VerifyResult vr = new VerifyResult(certs, cert, cms, CAroot,
						signer, isDownloadCRLForced);

				passed = vr.getPassed();
				CRLerror = vr.getCRLerror();

				risultati.put(cert.getSubjectX500Principal(), vr);

			} else {
				log
						.severe("There is not exactly one certificate for this signer!");
				errors
						.add("There is not exactly one certificate for this signer!");
			}
			if (!currentSigner.hasNext()) {
				done = true;
			}

		}

	}

	public boolean isDone() {
		return done;
	}

	/**
	 * Returns the Common name of given certificate<br>
	 * <br>
	 * Restituisce il CN del subjct certificato in oggetto
	 * 
	 * @param userCert
	 *            certificato da cui estrarre il Common Name
	 * @return la stringa contenente il CN
	 */
	public static String getCommonName(X509Certificate userCert) {
		String DN = userCert.getSubjectDN().toString();
		int offset = DN.indexOf("CN=");
		int end = DN.indexOf(",", offset);
		String CN;
		if (end != -1) {
			CN = DN.substring(offset + 3, end);
		} else {
			CN = DN.substring(offset + 3, DN.length());
		}
		return CN;
	}

	byte[] getFile() {
		return (byte[]) cms.getSignedContent().getContent();
	}

	public Hashtable getRisultati() {
		return risultati;
	}

}
