/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.common;

import it.fe.core.ejb.generic.NestedCriterion;
import it.fe.core.ejb.generic.NestedOrder;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.hibernate.criterion.Conjunction;
import org.hibernate.criterion.CriteriaSpecification;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

public class ProfileUtils {


	/**
	 * PROBLEMA: È NECESSARIO GESTIRE SITUAZIONI IN CUI ESISTANO PIÙ CRITERI DI FILTRO SULLO STESSO CAMPO
	 * Usando due logiche distinte per i criteri web e quelli di profilo.
	 * 
	 * Profilo: se esistono due profili sullo stesso campo, ma con valore distinto: devo usare una logica OR:
	 * 		es. invoice.supplier.supplierName , HERA
	 * 			invoice.supplier.supplierName , ENEL
	 * 			--> DEVO PRENDERE TUTTE LE FATTURE con fornitore Enel OPPURE Hera
	 * 
	 * Web: se esiste un criterio di filtro compatibile con quello dei profili, devo ragionare con una logica restrittiva:
	 * 		es. (PROFILO) 
	 * 				invoice.supplier.supplierName , HERA
	 * 				invoice.supplier.supplierName , ENEL
	 * 			(WEB)
	 * 				invoice.supplier.supplierName , TELECOM
	 * 
	 * 		--> (i.s.sn == HERA OR i.s.sn == ENEL) AND (i.s.sn == TELECOM)
	 * 			in pratica, restringo sull'insieme delle fatture già filtrate
	 */
	public static DetachedCriteria computeDetachedCriteriaWithFilters(Order orderCriteria, String order, Class clazz,Map<String, List<Criterion>> criteriWeb,  Map<String, List<Criterion>> criteriProfilo) {
		
		boolean orderAdded = (orderCriteria == null); //se non ho ordinamento, devo ignorarlo nelle successive fasi.
		DetachedCriteria out = DetachedCriteria.forClass(clazz);
		
		//Join dei criteri di filtro secondo le regole stabilite nel commento ad inizio funzione
		if ( (criteriProfilo != null) && (criteriProfilo.keySet() != null) )
			for (String property : criteriProfilo.keySet()){
				List<Criterion> criteri = criteriProfilo.get(property);
				if  ( (criteri == null) || (criteri.size() == 0) ) 
					continue;
				
				else if (criteri.get(0) instanceof NestedCriterion){
					
					String outerClass = ((NestedCriterion)criteri.get(0)).getOuterPropertyName();
					DetachedCriteria propertyCriteria = out.createCriteria(outerClass);
					
					/**
					 * Devo verificare la presenza di criteri di ordinamento su una proprietà innestata e aggiungerli al criterio opportuno in fase di creazione.
					 * Hibernate criteria, infatti, non consente di creare due criteri distinti per la stessa proprietà (es. uno per le restrictions e uno per l'order)
					 * E' quindi necessario creare gli ordinamenti innestati in maniera contestuale alla creazione della medesima proprietà innestata
					 * 
					 */
					
					//Se sto aggiungendo un sottocriterio sul quale è stato anche richiesto l'ordinamento...
					if ( (!orderAdded) &&  (order != null) && (orderCriteria != null) && (orderCriteria instanceof NestedOrder) && (((NestedOrder)orderCriteria).getOuterPropertyName().equals(outerClass))){
						String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
						boolean asc = ((NestedOrder)orderCriteria).isAscending();
						Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
						propertyCriteria.addOrder(newOrder);
						orderAdded=true;
					}
					
					if (criteri.size() == 1){
						//TODO: verificare se ci sono criteri di filtro su web e metterli in AND
						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
							propertyCriteria.add((((NestedCriterion)criteri.get(0)).getInnerCriterion()));
						else {
							Conjunction conjuntion =Restrictions.conjunction();
							conjuntion.add((((NestedCriterion)criteri.get(0)).getInnerCriterion()));
							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
								conjuntion.add( ((NestedCriterion) webCriterion).getInnerCriterion()); //TODO: attenzione: suppongo che sia un NestedCriterion anche quello della parte web perché o è quello delle property
							}
							propertyCriteria.add(conjuntion);
						}
					}
					else {
						Disjunction disjunction = Restrictions.disjunction();
						for (Criterion criterion : criteri) {
							disjunction.add(((NestedCriterion)criterion).getInnerCriterion());
						}
						
						//TODO: verificare se ci sono criteri di filtro su web e metterli in AND
						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
							propertyCriteria.add(disjunction);
						else {
							Conjunction conjuntion =Restrictions.conjunction();
							conjuntion.add(disjunction);
							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
								conjuntion.add(((NestedCriterion)webCriterion).getInnerCriterion());
							}
							propertyCriteria.add(conjuntion);
						}
						
					}
				}
					
				else{
					
					if (criteri.size() == 1)
						//verifico se ci sono criteri di filtro su web e li metto in AND con i critreri di profilo
						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
							out.add(criteri.get(0));
						else {
							Conjunction conjuntion =Restrictions.conjunction();
							conjuntion.add(criteri.get(0));
							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
								conjuntion.add(webCriterion);
							}
							out.add(conjuntion);
						}
					else {
						Disjunction disjunction = Restrictions.disjunction();
						for (Criterion criterion : criteri) {
							disjunction.add(criterion);
						}
						//verifico se ci sono criteri di filtro su web e li metto in AND con i critreri di profilo
						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
							out.add(disjunction);
						else {
							Conjunction conjuntion =Restrictions.conjunction();
							conjuntion.add(disjunction);
							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
								conjuntion.add(webCriterion);
							}
							out.add(conjuntion);
						}
						
					}
						
				}
			}
		
		//Ciclo sui criteri web rimanenti (e cioè non andati in join con i criteri dei profili)
		if ( (criteriWeb != null) && (criteriWeb.keySet() != null) )
			for (String property : criteriWeb.keySet()){
				if ( (criteriProfilo.get(property) != null) && (criteriProfilo.get(property).size()>0) )
					continue;
				else{
					List<Criterion> criteri = criteriWeb.get(property);
					if  ( (criteri == null) || (criteri.size() == 0) ) 
						continue;
					
					else if (criteri.get(0) instanceof NestedCriterion){
						String outerClass = ((NestedCriterion)criteri.get(0)).getOuterPropertyName();
						DetachedCriteria propertyCriteria = out.createCriteria(((NestedCriterion)criteri.get(0)).getOuterPropertyName());
						/**
						 * Devo verificare la presenza di criteri di ordinamento su una proprietà innestata e aggiungerli al criterio opportuno in fase di creazione.
						 * Hibernate criteria, infatti, non consente di creare due criteri distinti per la stessa proprietà (es. uno per le restrictions e uno per l'order)
						 * E' quindi necessario creare gli ordinamenti innestati in maniera contestuale alla creazione della medesima proprietà innestata
						 * 
						 * il codice sotto è analogo a quello già scritto nella sezione precedente per la gestione dei filtri di profilo:
						 * abbiamo garanzia di non aggiungere due volte lo stesso ordinamento perché questo secondo ciclo itera solo sulle proprietà web rimaste fuori dalla precedente join con i criteri di profilo
						 */
						
						//Se sto aggiungendo un sottocriterio sul quale è stato anche richiesto l'ordinamento...
						if ( (!orderAdded) &&  (order != null) && (orderCriteria != null) && (orderCriteria instanceof NestedOrder) && (((NestedOrder)orderCriteria).getOuterPropertyName().equals(outerClass))){
							String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
							boolean asc = ((NestedOrder)orderCriteria).isAscending();
							Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
							propertyCriteria.addOrder(newOrder);
							orderAdded=true;
						}
						
						if (criteri.size() == 1)
							propertyCriteria.add(((NestedCriterion)criteri.get(0)).getInnerCriterion());
						else {
							Conjunction c = Restrictions.conjunction();
							for (Criterion criterion : criteri) {
								c.add(((NestedCriterion)criterion).getInnerCriterion());
							}
							propertyCriteria.add(c);
						}
					}
					else {
						if (criteri.size() == 1)
							out.add(criteri.get(0));
						else {
							Conjunction conjunction = Restrictions.conjunction();
							for (Criterion criterion : criteri) {
								conjunction.add(criterion);
							}
							out.add(conjunction);
						}
					}
					
				}
					
			}
		
		//Infine aggiungo l'ordinamento (solo se questo non è di tipo nested, perché in tal caso è stato gestito a monte: vedere sopra)
		if ( (!orderAdded) &&  (order != null) && (orderCriteria != null)  ){
			if (! (orderCriteria instanceof NestedOrder)){
					out.addOrder(orderCriteria);
			}
			else{
				String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
				String outerProperty = ((NestedOrder)orderCriteria).getOuterPropertyName();
				boolean asc = ((NestedOrder)orderCriteria).isAscending();
				Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
				out.createCriteria(outerProperty ).addOrder(newOrder);
			}
		}
		return out;
	}
	
	/**
	 * Questo metodo traduce ordinamento e criteri web in un detachedCriteria Hibernate. <br />
	 * Effettua i join fino a 3 livelli di profondità dall'entità root (es. invoice.invoiceLine.meter.sector), 
	 * ricavandoli dai NestedCriterion.
	 * 
	 * 
	 * @param orderCriteria
	 * @param order
	 * @param clazz
	 * @param criteriWeb
	 * @return
	 */
	public static DetachedCriteria computeDetachedCriteria(Order orderCriteria, String order, Class clazz,Map<String, List<Criterion>> criteriWeb) {

		boolean orderAdded = (orderCriteria == null); //se non ho ordinamento, devo ignorarlo nelle successive fasi.
		
		//Root
		DetachedCriteria out = DetachedCriteria.forClass(clazz);

		// Mappa proprietà valori i criteri del profilo

		// Ciclo sui criteri web e creo il criterio finale
		if ((criteriWeb != null) && (criteriWeb.keySet() != null))
			for (String property : criteriWeb.keySet()) {
				//Criterion per la clausula WHERE
				List<Criterion> criteri = criteriWeb.get(property);
				
				if ((criteri == null) || (criteri.size() == 0))
					continue;

				else {
					List<Criterion> research = new ArrayList<Criterion>();
					
					for(int i=0; i<criteri.size(); i++){
					
						//Primo join
						if (criteri.get(i) instanceof NestedCriterion) {
							DetachedCriteria propertyCriteria = out
							.createCriteria(((NestedCriterion) criteri.get(i)).getOuterPropertyName());
							
							//Criterion per la clausula WHERE
							//research.add(((NestedCriterion) criteri.get(i)).getInnerCriterion());
							//Criterion research = ((NestedCriterion) criteri.get(i)).getInnerCriterion();
							

							//Secondo join
							Criterion nestedCriterionSecondLevel = ((NestedCriterion) criteri.get(i)).getInnerCriterion();							
							if (nestedCriterionSecondLevel instanceof NestedCriterion){
								propertyCriteria = propertyCriteria
								.createCriteria(((NestedCriterion) nestedCriterionSecondLevel).getOuterPropertyName());
								
								//research.add(((NestedCriterion) nestedCriterionSecondLevel).getInnerCriterion());
								//research = ((NestedCriterion) nestedCriterionSecondLevel).getInnerCriterion();
								
								
								//Terzo join
								Criterion nestedCriterionThirdLevel = ((NestedCriterion) nestedCriterionSecondLevel).getInnerCriterion();
								if (nestedCriterionThirdLevel instanceof NestedCriterion){
									propertyCriteria = propertyCriteria
									.createCriteria(((NestedCriterion) nestedCriterionThirdLevel).getOuterPropertyName());
									
									research.add(((NestedCriterion) nestedCriterionThirdLevel).getInnerCriterion());
									//research = ((NestedCriterion) nestedCriterionThirdLevel).getInnerCriterion();
								}
								else{
									research.add(((NestedCriterion) nestedCriterionSecondLevel).getInnerCriterion());
								}
							}
							else {								
								research.add(((NestedCriterion) criteri.get(i)).getInnerCriterion());
								
							}

		
							String outerClass = ((NestedCriterion) criteri.get(i))
									.getOuterPropertyName();
							/**
							 * Devo verificare la presenza di criteri di ordinamento su
							 * una proprietà innestata e aggiungerli al criterio
							 * opportuno in fase di creazione. Hibernate criteria,
							 * infatti, non consente di creare due criteri distinti per
							 * la stessa proprietà (es. uno per le restrictions e uno
							 * per l'order) E' quindi necessario creare gli ordinamenti
							 * innestati in maniera contestuale alla creazione della
							 * medesima proprietà innestata
							 * 
							 * il codice sotto è analogo a quello già scritto nella
							 * sezione precedente per la gestione dei filtri di profilo:
							 * abbiamo garanzia di non aggiungere due volte lo stesso
							 * ordinamento perché questo secondo ciclo itera solo sulle
							 * proprietà web rimaste fuori dalla precedente join con i
							 * criteri di profilo
							 * 
							 * TODO ordinamento per proprietà innestate di secondo e terzo livello
							 * 
							 */
		
							// Se sto aggiungendo un sottocriterio sul quale è stato
							// anche richiesto l'ordinamento...
							if ((!orderAdded) && (order != null)
									&& (orderCriteria != null)
									&& (orderCriteria instanceof NestedOrder)
									&& (((NestedOrder) orderCriteria)
											.getOuterPropertyName().equals(outerClass))) {
								String innerProperty = ((NestedOrder) orderCriteria)
										.getInnerPropertyName();
								boolean asc = ((NestedOrder) orderCriteria)
										.isAscending();
								Order newOrder = (asc) ? Order.asc(innerProperty)
										: Order.desc(innerProperty);
								propertyCriteria.addOrder(newOrder);
								orderAdded=true;
							}
							if(research.size() == 1){
								propertyCriteria.add(research.get(0));
							}
							else if(research != null && research.size() > 1) {
								Conjunction c = Restrictions.conjunction();
								for (Criterion criterion : research) {
									c.add(criterion);
								}
								propertyCriteria.add(c);
							}
							
//							if (criteri.size() == 1)
//								propertyCriteria.add(research);
//							else {
//								Conjunction c = Restrictions.conjunction();
//								for (Criterion criterion : criteri) {
//									c.add(((NestedCriterion) criterion)
//											.getInnerCriterion());
//								}
//								propertyCriteria.add(c);
//							}
						}
						else {
							if (criteri.size() == 1)
								out.add(criteri.get(i));
							else {
								Conjunction conjunction = Restrictions.conjunction();
								for (Criterion criterion : criteri) {
									conjunction.add(criterion);
								}
								out.add(conjunction);
							}
						}
					}
				}
			}
		//Infine aggiungo l'ordinamento (solo se questo non è di tipo nested, perché in tal caso è stato gestito a monte: vedere sopra)
		if ( (!orderAdded) &&  (order != null) && (orderCriteria != null)  ){
			if (! (orderCriteria instanceof NestedOrder)){
					out.addOrder(orderCriteria);
			}
			else{
				String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
				String outerProperty = ((NestedOrder)orderCriteria).getOuterPropertyName();
				boolean asc = ((NestedOrder)orderCriteria).isAscending();
				Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
				out.createCriteria(outerProperty ).addOrder(newOrder);
			}
		}
		

		out.setResultTransformer(CriteriaSpecification.DISTINCT_ROOT_ENTITY);
		return out;
	}
	
	public static List<Criterion> extractRestrictions(String propertyName, Map<String, List<Criterion>> criteriWeb){
		return criteriWeb.get(propertyName);
	}
	
}
