/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.common;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class RenderingUtils {

	public static Boolean verifyArchivingCompleted(Invoice invoice){
		return JNDIUtils.getTentativoVersamentoManager().isArchived(invoice);
	}
	
	public static String encodeXML(Invoice invoice){
		invoice = JNDIUtils.getInvoiceManager().fetchXMLFile(invoice);
		String xml = new String(invoice.getXmlFile().getFileBlob());
		if (xml != null)
			return xml;
//			return xml.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
		else 
			return xml;
	}
	
//	public static String encodeXML(String xml){
//		if (xml != null)
//			return xml.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
//		else 
//			return xml;
//	}
	
	public static Boolean verifyRefused(Invoice invoice) {
		if ((invoice == null) || (invoice.getInvoiceState() == null))
			return false;
		return stateEquals(invoice, InvoiceState.TECHNICALLY_REJECTED)
				|| stateEquals(invoice, InvoiceState.ACCOUNTABLY_REJECTED);
	}

	public static Boolean verifyAccounted(Invoice invoice) {
		if ((invoice == null) || (invoice.getInvoiceState() == null))
			return false;
		return stateEquals(invoice, InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED);
	}

	public static Boolean verifyOnError(Invoice invoice) {
		if ((invoice == null))
			return false;
		return invoice.isError();
	}

	public static Boolean verifyOnWorking(Invoice invoice) {
		if ((invoice == null) || (invoice.getInvoiceState() == null))
			return false;
		if ((!verifyRefused(invoice)) && (!verifyOnError(invoice))
				&& (!verifyAccounted(invoice)))
			return true;
		else
			return false;
	}

	public static Boolean stateEquals(Invoice invoice, String invoiceState) {
		boolean result = false;
		if (invoice == null)
			return result;
		if (invoiceState == null || invoiceState.equals(""))
			return result;

		result = invoice.getInvoiceState().getDescription()
				.equals(invoiceState);

		return result;

	}

	public static Long daysToExpiration(Date expirationDate) {
		if (expirationDate == null)
			return new Long(0);
		else {
			long msRemaining = expirationDate.getTime() - new Date().getTime();
			if (msRemaining % (1000 * 60 * 60 * 24) == 0)
				return msRemaining / (1000 * 60 * 60 * 24);
			else
				return (msRemaining / (1000 * 60 * 60 * 24)) + 1;
		}
	}	
	
	public static Long dateDiffAsDays(Date a, Date b) {
		if ((a == null) || (b == null))
			return new Long(0);
		else {
			long msRemaining = a.getTime() - b.getTime();
			if (msRemaining % (1000 * 60 * 60 * 24) == 0)
				return msRemaining / (1000 * 60 * 60 * 24);
			else
				return (msRemaining / (1000 * 60 * 60 * 24)) + 1;
		}
	}

	@SuppressWarnings("unchecked")
	public static List persistentBagToList(List persistentBag) {
		List returnList = new ArrayList();
		if (persistentBag != null){
			for (Object o : persistentBag) {
				returnList.add(o);
			}
		}
		return returnList;
	}

	public static Boolean dummy() {
		return false;
	}

	public static String cutMessage(String message, Integer chars) {
		if (message == null)
			return "";
		else if (message.length() <= chars)
			return message;
		else
			return message.substring(0, chars) + "...";
	}

	
	/**
	 * Crea una mappa "Contatore, Riassunto contatore".<br />
	 * Si assume che in una fattura ogni contatore abbia un solo riepilogo associato.
	 * 
	 * @param invoiceId
	 * @param contractId
	 * @return Map&lt;Meter, MeterSummary&gt;
	 */
	public static Map<Meter, MeterSummary> meterByContract(String invoiceId, String contractId) {
		Map<Meter,MeterSummary> out = new HashMap<Meter, MeterSummary>();		
		
		
		String query = "SELECT DISTINCT m, mS" +
						" FROM InvoiceLine iL" +
						" LEFT JOIN iL.meter m " +
						" LEFT JOIN m.supplyInfoList sI" +
						" LEFT JOIN iL.invoice.meterSummary mS" +
						" LEFT JOIN mS.meter m2" +
						" WHERE (iL.invoice.invoiceId = " + invoiceId +" )" +
						" AND (sI.previousDocument.previousDocumentId = " + contractId + ")" ;
				
		List<Object[]> metersInfo = JNDIUtils.getCrudManager().findByQuery(query);
		
		for (Object[] object : metersInfo) {
			Meter m = new Meter();
			m = (Meter)object[0];

			MeterSummary mS = new MeterSummary();
			mS = (MeterSummary)object[1];
			
			out.put(m, mS);
		}
		
		return out;
	}
	
	/**
	 * Ritorna l'indirizzo formattato col seguente stile:<br />
	 * <i>Indirizzo, civico - CAP Citta - Dipartimento - Casella postale (Nazione)</i>
	 * 
	 * @param pa (PostalAddress)
	 * @return indirizzo formattato
	 */
	public static String renderPostalAddress(PostalAddress pa){
		String stringPostalAddress = "";
		
		if(pa != null){			
			stringPostalAddress = pa.getStreetName();
			
			if((pa.getAdditionalStreetName() != null) && (!pa.getAdditionalStreetName().equals(""))){
				stringPostalAddress = stringPostalAddress + " " + pa.getAdditionalStreetName();
			}
			
			if((pa.getBuildingNumber() != null) && (!pa.getBuildingNumber().equals(""))){
				stringPostalAddress = stringPostalAddress + ", " + pa.getBuildingNumber();
			}
			
			if((pa.getPostalZone() != null) && (pa.getPostalZone().equals(""))){
				stringPostalAddress = stringPostalAddress + " - " + pa.getPostalZone();
			}
			
			if((pa.getCityName() != null) && (!pa.getCityName().equals(""))){
				stringPostalAddress = stringPostalAddress + " " + pa.getCityName();
			}
			
			if((pa.getDepartment() != null) && (!pa.getDepartment().equals(""))){
				stringPostalAddress = stringPostalAddress + " - " + pa.getDepartment();
			}
			
			if((pa.getPostBox() != null) && (!pa.getPostBox().equals(""))){
				stringPostalAddress = stringPostalAddress + " - " + pa.getPostBox();
			}
			
			if((pa.getCountry() != null) && (!pa.getCountry().equals(""))){
				stringPostalAddress = stringPostalAddress + " (" + pa.getCountry() + ")";
			}
		}
		
		return stringPostalAddress;
	}
	
	public static List<Reading> readingForMeter(List<Reading> invoiceReading, Meter meter){
		List<Reading> meterReading = new ArrayList<Reading>();
		
		for(int i=0; i<invoiceReading.size(); i++) {
			if(invoiceReading.get(i).getMeter().getMeterId().equals(meter.getMeterId())){
				meterReading.add(invoiceReading.get(i));
			}
		}
		
		return meterReading;
	}
	
	public static List<TaxDetail> clearTaxDetailHera(Invoice invoice){
		List<TaxDetail> clearTaxDetails = new ArrayList<TaxDetail>();
		
		//controllo fornitore HERA
		if(("04245520376".equalsIgnoreCase(invoice.getSupplier().getVatId())) || ("02221101203".equalsIgnoreCase(invoice.getSupplier().getVatId()))){
			for(TaxDetail taxDetail : invoice.getTaxDetails()) {
				//elimina i taxDetail con descrizione G
				//G = controllo totali IVA effettuato da Hera
				if(!"G".equalsIgnoreCase(taxDetail.getDescription())){
					clearTaxDetails.add(taxDetail);
				}
			}
		}
		else {
			clearTaxDetails = invoice.getTaxDetails();
		}
		
		return clearTaxDetails;		
	}

}
