/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 *  @author fulvio di marco
 *  
*
 * 
 * 
 */
package it.fe.web.common;

import it.fe.core.par.user.ProfileDetail;

import java.util.Set;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;

import org.apache.log4j.Logger;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

public class SessionUtils {
	
	private static final String PROFILE_DETAIL = "FE_SHARED_USERPROFILEDETAIL";
	private static final Logger logger = Logger.getLogger(SessionUtils.class);
	
	/**
	 * Extracts current user's set of ProfileDetails from HttpSession
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Set<ProfileDetail> getCurrentUserProfileDetail() {
		return (Set<ProfileDetail>) getPortletSession().getAttribute(PROFILE_DETAIL, PortletSession.APPLICATION_SCOPE);
	}
	
	/**
	 * Gets a PortletSession's instance from FacesContext
	 * 
	 * @return
	 */
	public static PortletSession getPortletSession() {
		return (PortletSession) FacesContext
				.getCurrentInstance()
					.getExternalContext()
						.getSession(false);
	}
	
	/**
	 * Gets a PortletRequestInstance from FacesContext
	 * 
	 * @return
	 */
	public static PortletRequest getPortletRequest() {
		return (PortletRequest) FacesContext
				.getCurrentInstance()
					.getExternalContext()
						.getRequest();
	}
	
	public static String getCurrentUserScreenname() {
		User liferayUser = null;
		FacesContext fc = FacesContext.getCurrentInstance();
		ExternalContext externalContext = fc.getExternalContext();
		if (externalContext.getUserPrincipal() == null){
			logger.info("current principal is null"); 
		}
		else{ 
			Long id = Long.parseLong(externalContext.getUserPrincipal().getName());
		try {
			liferayUser = UserLocalServiceUtil.getUserById(id); 
		}
		catch (PortalException ex) {
			logger.error(ex.getMessage());
		}
		catch (SystemException ex) {
			logger.error(ex.getMessage());
		} 
		} 
		return liferayUser.getScreenName();
	}

}
