/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.common;

import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.Invoice;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.xml.utils.PrefixResolver;
import org.apache.xml.utils.PrefixResolverDefault;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class UBLUtils {

	
	public static Double toDouble (double d){
		return new Double(d);
	}
	public static String renderDocumentType (String input){
		if( (input == null) || (input.equals("")) )
			return "--";
		else if (input.equalsIgnoreCase(Invoice.DocumentType.DebitNote)){
			return "Nota debito";
		}
		else if (input.equalsIgnoreCase(Invoice.DocumentType.CreditNote)){
			return "Nota credito";
		}
		else if (input.equalsIgnoreCase(Invoice.DocumentType.Invoice)){
			return "Fattura";
		}
		else 
			return "";
	}
	public static String getInvoiceNumberFromUBL(FEFile file) {
		if (file == null)
			return "";
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		domFactory.setNamespaceAware(true); // never forget this!
		DocumentBuilder builder;
		try {
			builder = domFactory.newDocumentBuilder();
			Document doc = builder.parse(new ByteArrayInputStream(file
					.getFileBlob()));

			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			final PrefixResolver resolver = new PrefixResolverDefault(doc
					.getDocumentElement());
			NamespaceContext ctx = new NamespaceContext() {
				public String getNamespaceURI(String prefix) {
					return resolver.getNamespaceForPrefix(prefix);
				}

				public String getPrefix(String arg0) {
					return null;
				}

				public Iterator getPrefixes(String arg0) {
					return null;
				}
			};

			xpath.setNamespaceContext(ctx);
			Element e = doc.getDocumentElement();

			String nodeName = "ID";
			String prefix = "cbc";
			// controlla la presenza del namespace
			if (!(xpath.evaluate(nodeName, e).equals(""))) {
				return xpath.evaluate(nodeName, e);
			} else if (!xpath.evaluate(prefix + ":" + nodeName, e).equals("")) {
				return xpath.evaluate(prefix + ":" + nodeName, e);
			}

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return "";

	}

	public static String getInvoiceTotalFromUBL(FEFile file) {
		if (file == null)
			return "";
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		domFactory.setNamespaceAware(true); // never forget this!
		DocumentBuilder builder;
		try {
			builder = domFactory.newDocumentBuilder();
			Document doc = builder.parse(new ByteArrayInputStream(file
					.getFileBlob()));

			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			xpath.setNamespaceContext(new NamespaceContext() {
				public String getNamespaceURI(String prefix) {
					if (prefix.equals("cbc")) {
						return "urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2";
					}
					else if (prefix.equals("cac")) {
						return "urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2";
					} 

					return XMLConstants.NULL_NS_URI;
				}

				public String getPrefix(String namespaceURI) {
					if (namespaceURI.equals("urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2")) {
						return "cbc";
					}
					else if (namespaceURI.equals("urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents")) {
						return "cac";
					}
					return null;
				}

				public Iterator getPrefixes(String namespaceURI) {
					ArrayList list = new ArrayList();

					if (namespaceURI.equals("urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2")) {
						list.add("cbc");
					}
					else if (namespaceURI.equals("urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2")) {
						list.add("cac");
					}
					return list.iterator();
				}
			});

			XPathExpression names = xpath.compile("//cac:LegalMonetaryTotal/cbc:PayableAmount/text()");
		
			NodeList nodesResult =(NodeList) names.evaluate(doc, XPathConstants.NODESET);
			if ( (nodesResult != null ) && (nodesResult.getLength() >0) )
				return nodesResult.item(0).getNodeValue();
			
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return "";

	}

}
