/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.administration;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.authorization.Permission;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.user.Context;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AdministrationUtils {

	public static String getActionsAsString (String stateDescription,Invoice invoice) {
		String NoResult="-";
		String result = "";
		String query = "SELECT username, endTime " +
		" FROM TaskTimings WHERE invoiceId = " + invoice.getInvoiceId() +
		" AND username IS NOT NULL AND endTime IS NOT NULL "+
		" ORDER BY endTime DESC";
		
		List<Object[]> out = JNDIUtils.getCrudManager().findByNativeQuery(query);
		
		if ( (out ==  null) || (out.size() == 0) )
			return "-";
		String username = ((String)out.get(0)[0]);
		Date end = ((Date)out.get(0)[1]);
		
		if ( (username == null) || (end == null) || (username.equals("")) ) 
			return NoResult;
		else{
			SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy, HH:mm");
			result+= username+ ", "+sdf.format(end);
		}
		return result;
	}
	
	public static String getResponsibilityAsString (String stateDescription,Invoice invoice) {
		String NoResult="-";
		InvoiceState state = JNDIUtils.getInvoiceStateManager().getInvoiceStateByDescription(stateDescription);
		if (state == null)
			return NoResult;
		Map<String,List<String>> responsibilities = determineResponsibility(new ArrayList<InvoiceState>(Arrays.asList(new InvoiceState[]{state})), invoice);
		
		if ( (responsibilities == null) || (responsibilities.keySet() == null) || (!responsibilities.keySet().contains(stateDescription)) || (responsibilities.get(stateDescription)== null))
			return NoResult;
		
		else{
			String out ="";
			List<String> responsibles = responsibilities.get(stateDescription);
			for (String responsible : responsibles) {
				out+=responsible+", ";
			}
			if (out.length() > 0)
				return out.substring(0,out.length()-2);
			else return out;
		}
	}
	
	public static Date getDateActions (Invoice invoice) {
		Date result = new Date();
		String query = "SELECT username, endTime" +
		" FROM TaskTimings WHERE invoiceId = " + invoice.getInvoiceId() +
		" AND endTime IS NOT NULL " +
		" ORDER BY endTime DESC";
		
		List<Object[]> out = JNDIUtils.getCrudManager().findByNativeQuery(query);
		
		if ( (out ==  null) || (out.size() == 0) )
			return null;
		result = ((Date)out.get(0)[1]);
		

		return result;
	}
	
	public static String getDateActionsAsString (String stateDescription,Invoice invoice) {
		String NoResult="-";
		String result = "";
		String query = "SELECT username, endTime" +
		" FROM TaskTimings WHERE invoiceId = " + invoice.getInvoiceId() +
		" AND endTime IS NOT NULL " +
		" ORDER BY endTime DESC";
		
		List<Object[]> out = JNDIUtils.getCrudManager().findByNativeQuery(query);
		
		if ( (out ==  null) || (out.size() == 0) )
			return "-";
		Date end = ((Date)out.get(0)[1]);
		
		if ( (end == null) ) 
			return NoResult;
		else{
			SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
			result = sdf.format(end);
		}
		return result;
	}
	
	public static String getInfoActionsAsString (Invoice invoice) {
		String NoResult="-";
		String result = "";
		String query = "SELECT username, endTime" +
		" FROM TaskTimings WHERE invoiceId = " + invoice.getInvoiceId() +
		" AND endTime IS NOT NULL " +
		" ORDER BY endTime DESC";
		
		List<Object[]> out = JNDIUtils.getCrudManager().findByNativeQuery(query);
		
		if ( (out ==  null) || (out.size() == 0) )
			return "-";
		
		String username = ((String)out.get(0)[0]);
		Date end = ((Date)out.get(0)[1]);
		
		if ( (end == null) ) 
			return NoResult;
		else{
			SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
			result = " il " + sdf.format(end) + " da " + username;
		}
		return result;
	}
	
	/**
	 * Ritorna una mappa Stato, Lista di utenti abilitati in scrittura
	 * 
	 * @param stateDescription
	 * @param invoice
	 * @return
	 */
	public static Map<String,List<String>> determineResponsibility (List<InvoiceState> states,Invoice invoice) {
		Permission permission = JNDIUtils.getPermissionManager().getPermissionByCode("W");
		Map<String,List<String>> out = new HashMap<String, List<String>>();

		if ( (states == null) || (permission == null) )
			return null;
		
		//foreach state
		for (InvoiceState invoiceState : states) {
			out.put(invoiceState.getDescription(), new ArrayList<String>());
		
			//query
			//3 parametri + context ALL
			//1 - invoiceStateId
			//2 - permissionId
			//3 - tableName
			String query = "SELECT u.username, pd.roleId, pd.startDate, pd.endDate, c.description, c.columnName, pd.contextValue, c.tableName " +
					"FROM ProfileDetails pd, Contexts c, Users u where pd.roleId in " +
					"( select rd.roleId from RoleDetails rd where rd.invoiceStateId="+ invoiceState.getInvoiceStateId()+
					" AND rd.permissionId="+  permission.getPermissionId() + ")" +
					" AND c.contextId=pd.contextId " +
					" AND u.userId=pd.userId" +
					" AND (c.tableName='Invoice' OR c.description='ALL')";
			
			/**
			 * es. di output
			 * 		userId 	roleId 	startDate 	endDate 	description 	columnName 	contextValue 	tableName
						1 		5 	2010-03-26 	2011-03-26 	FORNITORE 	supplier.supplierName 	HERA S.p.a. 	Invoice
						2 		5 	2010-03-26 	2011-03-26 	FORNITORE 	supplier.supplierName 	HERA Comm S.R.L. 	Invoice
			 */

			List<Object[]> profiles = JNDIUtils.getCrudManager().findByNativeQuery(query);
			if (profiles == null)
				break;
			//per ciascuna entry ottenuta al passo precedente...
			for (Object[] object : profiles) {
				String username= (String)object[0];
				Long roleId = ((Number)object[1]).longValue();
				Date startDate = (Date)object[2];
				Date endDate = (Date)object[3];
				String contextDescription = (String)object[4];
				String columnName= (String)object[5];
				String contextValue= (String)object[6];
				String tableName= (String)object[7];
			
				//TODO: gestione DATE di scadenza dei profili
				
				//... se il contesto è ALL --> questo utente può vedere qualunque fattura
				if ( (contextDescription != null) && (contextDescription.equals(Context.ALL)) )
					out.get(invoiceState.getDescription()).add(username);
				
				// ... altrimenti faccio una query su DB con columnName e contextValue e prendo gli id --> se l'ID della fattura rientra tra quegli ID, allora vuol dire che è visibile dall'utente --> aggiungo l'utente allo stato
				Map<String,Object> params = new HashMap<String, Object>();
				params.put("value", contextValue);
				List ids = JNDIUtils.getCrudManager().findByQuery("select invoiceId from Invoice where "+columnName+"=:value",  params);
				
				if (ids == null)
					break;
				
				if (ids.contains(invoice.getInvoiceId()))
					out.get(invoiceState.getDescription()).add(username);
			}		
		
		}
		
		return out;
	}
}
