/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.administration;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.common.utils.PasswordUtils;
import it.fe.core.ejb.mail.MailService;
import it.fe.core.ejb.user.RoleManager;
import it.fe.core.par.user.Context;
import it.fe.core.par.user.ProfileDetail;
import it.fe.core.par.user.Role;
import it.fe.core.par.user.User;
import it.fe.core.par.user.UserPassword;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;

import org.apache.log4j.Logger;

public class UserAdminHandler implements Serializable {

	private static final long serialVersionUID = 1L;
	
	private final static Logger logger = Logger.getLogger(UserAdminHandler.class);
	
	private String name;
	private String surname;
	private String email;
	private String username;
	private String role;
	private List<User> availableUsers;
	private List<SelectItem> availableRoles;
	private User userToDelete;
	
	public void addUser() {
		
		try {
			
			if (name == "" || surname == "" || email == "" || username == "") {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"E' necessario compilare tutti i campi", "E' necessario compilare tutti i campi"));
				return;
			}
			
			// TODO: controllo sulla validità dell'indirizzo mail
			 
			// controllo che lo username non esista già sul db
			User user = JNDIUtils.getUserManager().getUserByUsername(username);
			
			if (user != null) {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Utente "+username+" già esistente", "Utente "+username+" già esistente"));
				return;
			}
			
			user = new User();
			user.setName(name);
			user.setSurname(surname);
			user.setEmail(email);
			user.setUsername(username);
			user.setUoDenomination(FEPropertiesUtil.getUoDenomination());
			
			String generatedPassword = PasswordUtils.getRandomPassword(9);
			String generatedPasswordHashed = PasswordUtils.encrypt(PasswordUtils.TYPE_SHA, generatedPassword);
			
			UserPassword userPassword = new UserPassword();
			userPassword.setStartDate(new Date());
			userPassword.setPassword(generatedPasswordHashed);
			user.addUserPassword(userPassword);
			
			// ProfileDetail (Role/Context)
			ProfileDetail profileDetail = new ProfileDetail();
			// Role
			Role dbRole = JNDIUtils.getRoleManager().getRoleById(Long.valueOf(role).longValue());
			if (dbRole == null) {
				throw new Exception("Impossibile trovare il ruolo selezionato");
			}
			profileDetail.setRole(dbRole);
			
			// TODO: per ora setto arbitrariamente Context: FORNITORI
			Context dbContext = JNDIUtils.getContextManager().getContextByDescription("ALL");
			if (dbContext == null) {
				throw new Exception("Impossibile trovare il contesto");
			}
			profileDetail.setContext(dbContext);
			
			//TODO: per ora setto arbitrariamente contextValue = ALL
			profileDetail.setContextValue(Context.ALL);
			
			Date now = new Date();
			profileDetail.setStartDate(now);
			profileDetail.setEndDate(null);
			
			Set<ProfileDetail> profiles = new HashSet<ProfileDetail>();
			profiles.add(profileDetail);
			user.setProfile(profiles);
			
			if (JNDIUtils.getUserManager().insertUser(user) < 0 ) {
				logger.error("An error occurred while persisting new user to db");
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Si è verificato un errore nell'inserimento dell'utente", "Si è verificato un errore nell'inserimento dell'utente"));
				return;
			}
						
			// invio mail
			Map<String,Object> parameters = new HashMap<String,Object>();
			parameters.put(MailService.FROM, FEPropertiesUtil.getMailFrom());
			parameters.put(MailService.SUBJECT, "FE: Attivazione account");
			parameters.put(MailService.RECIPIENTS, email);
			String body = 	"Salve " +name+ " "+surname+", \n\n" +
							"è stato attivato il suo account su FE con\n\n" +
							"username: "+username+"\n" +
							"password: "+generatedPassword+"\n\n";
			parameters.put(MailService.BODY, body);
			int result = JNDIUtils.getMailService().sendEmail(parameters);
			
			if (result < 0) {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Si è verificato un errore nell'invio mail", "Si è verificato un errore nell'invio mail"));
			}
			
			else {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, "Account creato correttamente. E' stata inviata una mail a "+email, "Account creato correttamente. E' stata inviata una mail a "+email));
			}
				
		} catch (Exception e) {
			logger.error(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getMessage(), e.getMessage()));
			return;
		}
		
		
	}
	
	public void deleteUser(){
		if (userToDelete != null) {
			int result = JNDIUtils.getUserManager().removeUserById(userToDelete.getUserId());
			if ( result < 0) {
				logger.error("An error occurred while removing user");
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Errore: non è stato possibile eliminare l'utente", "Errore: non è stato possibile eliminare l'utente"));
			}
			else {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, "L'utente "+userToDelete.getUsername()+" è stato cancellato da FE", "L'utente "+userToDelete.getUsername()+" è stato cancellato da FE"));
			}
			// forcing model reload
			availableUsers = null;
		}
		
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the surname
	 */
	public String getSurname() {
		return surname;
	}

	/**
	 * @param surname the surname to set
	 */
	public void setSurname(String surname) {
		this.surname = surname;
	}

	/**
	 * @return the email
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * @param email the email to set
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @return the username
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * @param username the username to set
	 */
	public void setUsername(String username) {
		this.username = username;
	}

	public void setAvailableUsers(List<User> availableUsers) {
		this.availableUsers = availableUsers;
	}

	public List<User> getAvailableUsers() {
		if (availableUsers == null) {
			availableUsers = JNDIUtils.getUserManager().getUsers();
		}
		return availableUsers;
	}

	public void setUserToDelete(User userToDelete) {
		this.userToDelete = userToDelete;
	}

	public User getUserToDelete() {
		return userToDelete;
	}

	public void setAvailableRoles(List<SelectItem> availableRoles) {
		this.availableRoles = availableRoles;
	}

	public List<SelectItem> getAvailableRoles() {
		if ( availableRoles == null ) {
			availableRoles = new ArrayList<SelectItem>();
			availableRoles.add(new SelectItem("","Seleziona il ruolo"));
			RoleManager roleManager = JNDIUtils.getRoleManager();
			/*List<Role> dbRetrievedRoles = JNDIUtils.getRoleManager().getRoles();
			for (Role role : dbRetrievedRoles) {
				availableRoles.add(new SelectItem(role.getRoleId(),formatString(role.getDescription())));
			}*/
			Role administrator = roleManager.getRoleByDescription(Role.AMMINISTRATORE);
			availableRoles.add(new SelectItem(administrator.getRoleId(),formatString(administrator.getDescription())));
			Role technicalValidator = roleManager.getRoleByDescription(Role.FUNZIONARIO_TECNICO);
			availableRoles.add(new SelectItem(technicalValidator.getRoleId(),formatString(technicalValidator.getDescription())));
			Role consulter = roleManager.getRoleByDescription(Role.CONSULTATORE);
			availableRoles.add(new SelectItem(consulter.getRoleId(),formatString(consulter.getDescription())));
			
		}
		return availableRoles;
	}
	
	private String formatString(String stringToBeFormatted) {
		stringToBeFormatted = stringToBeFormatted.replace("_", " ");
		stringToBeFormatted = stringToBeFormatted.toLowerCase();
		stringToBeFormatted = stringToBeFormatted.substring(0,1).toUpperCase() + stringToBeFormatted.substring(1);
		return stringToBeFormatted;
	}

	public void setRole(String role) {
		this.role = role;
	}

	public String getRole() {
		return role;
	}


}
