/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
*
 *
 */
package it.fe.web.controller.invoice;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.comunemodena.ContoContrattuale;
import it.fe.core.par.comunemodena.InvoiceAdditionalData;
import it.fe.core.par.comunemodena.InvoiceLineAdditionalData;
import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.unimore.cia.Anagrafico;
import it.fe.core.par.unimore.cia.Causale;
import it.fe.core.par.unimore.cia.CentroCosto;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.process.exception.JbpmProcessException;
import it.fe.core.ws.comunemodena.sib.LabelValueBean;
import it.fe.core.ws.comunemodena.sib.SibService;
import it.fe.core.ws.comunemodena.sib.SibServiceService;
import it.fe.web.common.JSFUtils;
import it.fe.web.common.SessionUtils;
import it.fe.web.controller.profile.ComuneModenaProfilesHandler;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;

import org.apache.commons.validator.GenericValidator;
import org.richfaces.event.SimpleToggleEvent;

/**
 * Handler di fatture specifico per UniMoRe per la gestione dei dati aggiuntivi;
 * estende la classe FilterableInvoicesHandler
 * 
 * 
 * @author Simone Riccucci, Stefano Monti, Fulvio Di Marco
 * 
 */
public class ComuneModenaFilterableInvoicesHandler extends FilterableInvoicesHandler {

	private static final long serialVersionUID = 1L;
	
	private ComuneModenaProfilesHandler comuneModenaProfileHandler;

	public ComuneModenaFilterableInvoicesHandler() {
		comuneModenaProfileHandler = (ComuneModenaProfilesHandler) JSFUtils
				.getManagedBean("comuneModenaProfileHandler");
		SibServiceService SIBproxy=null;
		try {
			SIBproxy = new SibServiceService(new URL(FEPropertiesUtil.getPA_Backend_WSDL_URL()));
			SIBservice = SIBproxy.getSibServicePort();
		} catch (MalformedURLException e) {
			e.printStackTrace();
			//Log.error("Errore durante inizializzazione Web Service SIB",e);
		}
	}
	
	@Override
	public String openDetail() {


		checked = new HashMap<Long, Boolean>();
		currentAllChecked = false;
		
		return super.openDetail();
	}

	protected SibService SIBservice;
	
	/*
	 * 
	 * Gestione dati aggiuntivi
	 */

	protected String tipoFattura;
	protected List<SelectItem> tipiFattura;
	
	protected String codiceSettore = "0";
	protected List<SelectItem> settori = null;
	protected String codiceDivisione = "0";
	protected List<SelectItem> divisioni = null;
	protected String codiceUfficio = "0";
	protected List<SelectItem> uffici = null;
	
	protected Integer annoEsercizio;
	protected Date inizioCompetenzaEconomica;
	protected Date fineCompetenzaEconomica;
	protected Invoice documentoRef;
	protected ContoContrattuale contoContrattuale;
	
	protected String causaleDaCopiare = "Seleziona la causale";
	protected List<SelectItem> causaliDisponibiliHeader = new LinkedList<SelectItem>();
	protected Map<Long, List<SelectItem>> causaliDisponibili;
	protected Map<Long, String> causaleRiga;
	
	protected String subCausaleDaCopiare = "Seleziona la sub causale";
	protected List<SelectItem> subCausaliDisponibiliHeader = new LinkedList<SelectItem>();
	protected Map<Long, List<SelectItem>> subCausaliDisponibili;
	protected Map<Long, String> subCausaleRiga;
	
	protected Integer areaCommercialeDaCopiare;
	protected List<SelectItem> areeCommercialiDisponibiliHeader;
	protected Map<Long, List<SelectItem>> areeCommercialiDisponibili;
	protected Map<Long, Integer> areaCommerciale;
	
	protected String flagTipoDaCopiare;
	protected Map<Long, String> flagTipo;
	
	protected Map<Long, Boolean> checked = new HashMap<Long, Boolean>();
	protected boolean currentAllChecked = false;
	
	// modal panel attributes
	protected Long invoiceLineSelected=0L;
	protected boolean approvalModalPanelRendered = false;
	/*protected boolean missingAdditionalData;
	protected boolean supplierCodeSaved;
	protected boolean financialYearSaved;
	protected boolean budgetSaved;
	protected boolean uoSaved;
	protected boolean paymentMethodSaved;
	protected boolean bankAccountSaved;
	protected boolean invoiceLinesHasAdditionalDataSaved;*/

	public Long getInvoiceLineSelected() {
		return invoiceLineSelected;
	}

	public void setInvoiceLineSelected(Long invoiceLineSelected) {
		this.invoiceLineSelected = invoiceLineSelected;
	}

	public String technicallyApproveInvoice() {		
		try {
			JNDIUtils.getProcessManager().approveTechnically(selectedInvoice, SessionUtils.getCurrentUserScreenname());
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		}
		//selectedInvoice.setInvoiceState(JNDIUtils.getInvoiceStateManager().getInvoiceStateByDescription(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED));
		// forcing data model reload
		this.dataModel = null;
		this.updateEditable();
		this.updateInvoiceStateAndError();
		this.updateInvoiceState();
		// se la fattura contiene errori di inserimento in SIB lo segnalo
		if (!selectedInvoice.getInvoiceState().getDescription().equals(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED)) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore nell'inserimento nel sistema contabile: "+selectedInvoice.getErrorMessage(), selectedInvoice.getErrorMessage()));
		}
		else if (selectedInvoice.getInvoiceState().getDescription().equals(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED))
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"La fattura è stata inserita correttamente nel sistema contabile", "La fattura è stata inserita correttamente nel sistema contabile"));  
		return null;
	}
	
	private void updateInvoiceState() {
		Invoice invoice = JNDIUtils.getInvoiceManager().getInvoiceById(this.selectedInvoice.getInvoiceId().longValue());
		this.selectedInvoice.setInvoiceState(invoice.getInvoiceState());
	}

	public String technicallyRejectInvoice() {
		return super.technicallyRejectInvoice();
	}
	
	public String editAdditionalData() {
		return super.editAdditionalData();
	}
	
	public String discardAdditionalData() {
		return super.discardAdditionalData();
	}
	
	public String saveAdditionalData() {
		
		InvoiceAdditionalData invoiceAdditionalData = null;
		
		List<AdditionalData> additionalData = this.selectedInvoice.getAdditionalData();
		if(additionalData.size() == 0) {
			invoiceAdditionalData = new InvoiceAdditionalData();
			additionalData.add(invoiceAdditionalData);
		}
		else {
			for(AdditionalData data: additionalData) {
				if(data.getClass().getName().equals(InvoiceAdditionalData.class.getName())) {
					invoiceAdditionalData = (InvoiceAdditionalData)data;
				}
			}
		}
		
		if(invoiceAdditionalData == null)
			invoiceAdditionalData = new InvoiceAdditionalData();
		
		invoiceAdditionalData.setAnnoEsercizio(this.annoEsercizio);
		invoiceAdditionalData.setCodiceSettore(Integer.parseInt(codiceSettore));
		int codiceDivisione = Integer.parseInt(this.codiceDivisione.split("_")[0]);
		invoiceAdditionalData.setCodiceDivisione(codiceDivisione);
		int codiceUfficio = Integer.parseInt(this.codiceUfficio.split("_")[0]);
		invoiceAdditionalData.setCodiceUfficio(codiceUfficio);
		//invoiceAdditionalData.setContoContrattuale(contoContrattuale);
		invoiceAdditionalData.setDocumentoRef(documentoRef);
		invoiceAdditionalData.setInizioCompetenzaEconomica(inizioCompetenzaEconomica);
		invoiceAdditionalData.setFineCompetenzaEconomica(fineCompetenzaEconomica);
		invoiceAdditionalData.setTipoFattura(tipoFattura);
		
		for(InvoiceLine invoiceLine: this.selectedInvoice.getInvoiceLines()) {
			InvoiceLineAdditionalData invoiceLineAdditionalData = null;
			
			List<AdditionalData> additionalDataLine = invoiceLine.getAdditionalData();
			if(additionalDataLine.size() == 0) {
				invoiceLineAdditionalData = new InvoiceLineAdditionalData();
				additionalDataLine.add(invoiceLineAdditionalData);
			}
			else {
				for(AdditionalData data: additionalDataLine) {
					if(data.getClass().getName().equals(InvoiceLineAdditionalData.class.getName())) {
						invoiceLineAdditionalData = (InvoiceLineAdditionalData)data;
					}
				}
			}
			
			if(invoiceLineAdditionalData == null)
				invoiceLineAdditionalData = new InvoiceLineAdditionalData();
			
			invoiceLineAdditionalData.setCausaleRiga(this.causaleRiga.get(invoiceLine.getInvoiceLineId()));
			invoiceLineAdditionalData.setSubCausaleRiga(this.subCausaleRiga.get(invoiceLine.getInvoiceLineId()));
			
			invoiceLineAdditionalData.setAreaCommerciale(this.areaCommerciale.get(invoiceLine.getInvoiceLineId()));
			//invoiceLineAdditionalData.setAreeCommerciali(this.areaCommerciali.get(invoiceLine.getInvoiceLineId()));
			//invoiceLineAdditionalData.setCapitoliArticoli(this.capitoliArticoli.get(invoiceLine.getInvoiceLineId()));
			//invoiceLineAdditionalData.setCentriDiCosto(this.centriDiCosto.get(invoiceLine.getInvoiceLineId()));
			invoiceLineAdditionalData.setFlagTipo(this.flagTipo.get(invoiceLine.getInvoiceLineId()));
		}
		
		return super.saveAdditionalData();
	}
	
	/**
	 * Utility method that helps with indication of postback
	 * 
	 * @return
	 */
	public boolean isPostback() {
		FacesContext context = FacesContext.getCurrentInstance();
        return context.getRenderKit().getResponseStateManager().isPostback(context);
	}
	
	public void showApprovalModalPanel() {
		this.setApprovalModalPanelRendered(true);
	}

	/**
	 * Listener for rich:tabPanel tab changes. May be used to ajaxilly loading
	 * tab contents on tab switching.
	 * 
	 * @param event
	 */
	public void tabChange(ValueChangeEvent event) {
		String newTab = (String) event.getNewValue();
		if (newTab.equals("invoiceAdditionalInfo")) {
			
			// Invoice.AdditionalData: lazy fetching and caching for further requests
			List<AdditionalData> invoiceAdditionalData = selectedInvoice.getAdditionalData();
			if ( !(invoiceAdditionalData instanceof ArrayList)) {
				List<AdditionalData> lazilyFetchedAdditionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(selectedInvoice);
				this.selectedInvoice.setAdditionalData(lazilyFetchedAdditionalData);
			}
			
			// Invoice.InvoiceLines & InvoiceLine.AdditionalData: lazy fetching and caching for further requests
			List<InvoiceLine> invoiceLines = selectedInvoice.getInvoiceLines();
			if ( !(invoiceLines instanceof ArrayList) ) {
				List<InvoiceLine> lazilyFetchedInvoiceLines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
				
				for (InvoiceLine il : lazilyFetchedInvoiceLines) {
					List<AdditionalData> lazilyFetchedInvoiceLineAdditionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(il);
					il.setAdditionalData(lazilyFetchedInvoiceLineAdditionalData);
				}
				this.selectedInvoice.setInvoiceLines(lazilyFetchedInvoiceLines);
			}
			
			if(this.selectedInvoice.getAdditionalData()!=null && this.selectedInvoice.getAdditionalData().size()==1) {
				InvoiceAdditionalData data=(InvoiceAdditionalData) this.selectedInvoice.getAdditionalData().get(0);
				this.tipoFattura = data.getTipoFattura();
				this.annoEsercizio = data.getAnnoEsercizio();
				this.inizioCompetenzaEconomica = data.getInizioCompetenzaEconomica();
				this.fineCompetenzaEconomica = data.getFineCompetenzaEconomica();
				if(data.getCodiceSettore() != null)
					this.codiceSettore = data.getCodiceSettore()+"";
				else
					this.codiceSettore = "";
				if(data.getCodiceDivisione() != null)
					this.codiceDivisione = data.getCodiceDivisione()+"_"+this.codiceSettore;
				else
					this.codiceDivisione = "";
				if(data.getCodiceUfficio() != null)
					this.codiceUfficio = data.getCodiceUfficio()+"_"+this.codiceDivisione;
				else
					this.codiceUfficio = "";
			} else {
				this.tipoFattura = "";
				this.annoEsercizio = null;
				this.inizioCompetenzaEconomica = null;
				this.fineCompetenzaEconomica = null;
				this.codiceSettore = "";
				this.codiceDivisione = "";
				this.codiceUfficio = "";
			}
			List<InvoiceLine> lines=this.selectedInvoice.getInvoiceLines();
			this.causaleRiga = new HashMap<Long,String>();
			this.subCausaleRiga = new HashMap<Long,String>();
			this.flagTipo = new HashMap<Long, String>();
			this.areaCommerciale = new HashMap<Long, Integer>();
			this.causaliDisponibili = new HashMap<Long, List<SelectItem>>();
			this.subCausaliDisponibili = new HashMap<Long, List<SelectItem>>();
			this.areeCommercialiDisponibili = new HashMap<Long, List<SelectItem>>();
			
			List<SelectItem> causaliRigaSIB = this.toSelectItem(this.SIBservice.getCausaleRiga());
			List<SelectItem> areeCommercialiSIB = this.toSelectItem(this.SIBservice.getAreeCommerciali());
			
			for(InvoiceLine line: lines) {
				List<AdditionalData> invoiceLineAdditionalData = line.getAdditionalData();
				if ( !(invoiceLineAdditionalData instanceof ArrayList)) {
					List<AdditionalData> lazilyFetchedAdditionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(line);
					line.setAdditionalData(lazilyFetchedAdditionalData);
					invoiceLineAdditionalData = lazilyFetchedAdditionalData;
				}
				InvoiceLineAdditionalData lineData=null;
				int size=invoiceLineAdditionalData.size();
				if(invoiceLineAdditionalData==null || size == 0) {
					lineData = new InvoiceLineAdditionalData();
					lineData.setCausaleRiga("");
					lineData.setSubCausaleRiga("");
				}
				else {
					for(AdditionalData data: invoiceLineAdditionalData) {
						if(data.getClass().getName().equals(InvoiceLineAdditionalData.class.getName()))
							lineData = (InvoiceLineAdditionalData) data;
					}
					if(lineData == null) {
						lineData = new InvoiceLineAdditionalData();
						lineData.setCausaleRiga("");
						lineData.setSubCausaleRiga("");
					}
				}
				
				this.causaleRiga.put(line.getInvoiceLineId(), lineData.getCausaleRiga());
				this.subCausaleRiga.put(line.getInvoiceLineId(), lineData.getSubCausaleRiga());
				this.areaCommerciale.put(line.getInvoiceLineId(), lineData.getAreaCommerciale());
				this.flagTipo.put(line.getInvoiceLineId(), lineData.getFlagTipo());
				
				this.causaliDisponibili.put(line.getInvoiceLineId(), causaliRigaSIB);
				String causaleRiga = lineData.getCausaleRiga();
				if(GenericValidator.isBlankOrNull(causaleRiga))
					causaleRiga = this.causaliDisponibili.get(line.getInvoiceLineId()).get(0).getValue().toString();
				this.subCausaliDisponibili.put(line.getInvoiceLineId(), this.toSelectItem(this.SIBservice.getSubCausaleRiga(causaleRiga)));
				
				this.areeCommercialiDisponibili.put(line.getInvoiceLineId(), areeCommercialiSIB);				
				
			}
			
			this.causaliDisponibiliHeader = causaliRigaSIB;
			this.subCausaliDisponibiliHeader = this.toSelectItem(this.SIBservice.getSubCausaleRiga(this.causaliDisponibiliHeader.get(0).getValue().toString()));
			this.areeCommercialiDisponibiliHeader = areeCommercialiSIB;
			
			if(this.settori == null || this.settori.size() == 0)
				this.settori = this.toSelectItem(this.SIBservice.getSettori());
			
			if(this.divisioni == null) {
				if(GenericValidator.isBlankOrNull(this.codiceSettore))
					this.codiceSettore = this.settori.get(0).getValue().toString();
				this.divisioni=this.toSelectItem(this.SIBservice.getDivisioni(this.codiceSettore));
			}
			
			if(this.uffici == null) {
				if(GenericValidator.isBlankOrNull(this.codiceDivisione))
					this.codiceDivisione = this.divisioni.get(0).getValue().toString();
				String divisione=this.codiceDivisione.split("_")[0];
				this.uffici=this.toSelectItem(this.SIBservice.getUffici(this.codiceSettore,divisione));
			}
			
			this.tipiFattura = this.toSelectItem(this.SIBservice.getTipiDocumento());

		}
		if (newTab.equals("invoiceSpecificInfo")) {
			List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(selectedInvoice);
			List<MeterSummary> summaries = JNDIUtils.getInvoiceManager().fetchMeterSummaries(selectedInvoice);
			this.selectedInvoice.setPreviousDocuments(documents);
			this.selectedInvoice.setMeterSummary(summaries);
		}
	}
	
	public void onChangeCausale() {
		if(this.invoiceLineSelected == 0) {
			this.subCausaliDisponibiliHeader=
				this.toSelectItem(this.SIBservice.getSubCausaleRiga(this.causaleDaCopiare));
		} else {
			this.subCausaliDisponibili.put(this.invoiceLineSelected, this.toSelectItem(this.SIBservice.getSubCausaleRiga(this.causaleRiga.get(this.invoiceLineSelected))));
		}
	}
	
	public void onChangeSettore() {
		this.divisioni=this.toSelectItem(this.SIBservice.getDivisioni(this.codiceSettore));
		if(this.divisioni.size()>0) {
			this.codiceDivisione=this.divisioni.get(0).getValue().toString();
			this.onChangeDivisione();
		} else {
			this.uffici = new LinkedList<SelectItem>();
		}
	}
	
	public void onChangeDivisione() {
		String divisione=this.codiceDivisione.split("_")[0];
		this.uffici=this.toSelectItem(this.SIBservice.getUffici(this.codiceSettore,divisione));
	}
	
	public SibService getSIBservice() {
		return SIBservice;
	}

	public void setSIBservice(SibService sIBservice) {
		SIBservice = sIBservice;
	}

	public Invoice getDocumentoRef() {
		return documentoRef;
	}

	public void setDocumentoRef(Invoice documentoRef) {
		this.documentoRef = documentoRef;
	}

	public ContoContrattuale getContoContrattuale() {
		return contoContrattuale;
	}

	public void setContoContrattuale(ContoContrattuale contoContrattuale) {
		this.contoContrattuale = contoContrattuale;
	}

	public Integer getAreaCommercialeDaCopiare() {
		return areaCommercialeDaCopiare;
	}

	public void setAreaCommercialeDaCopiare(Integer areaCommercialeDaCopiare) {
		this.areaCommercialeDaCopiare = areaCommercialeDaCopiare;
	}

	public List<SelectItem> getAreeCommercialiDisponibiliHeader() {
		return areeCommercialiDisponibiliHeader;
	}

	public void setAreeCommercialiDisponibiliHeader(
			List<SelectItem> areeCommercialiDisponibiliHeader) {
		this.areeCommercialiDisponibiliHeader = areeCommercialiDisponibiliHeader;
	}

	public Map<Long, List<SelectItem>> getAreeCommercialiDisponibili() {
		return areeCommercialiDisponibili;
	}

	public void setAreeCommercialiDisponibili(
			Map<Long, List<SelectItem>> areeCommercialiDisponibili) {
		this.areeCommercialiDisponibili = areeCommercialiDisponibili;
	}

	public Map<Long, Integer> getAreaCommerciale() {
		return areaCommerciale;
	}

	public void setAreaCommerciale(Map<Long, Integer> areaCommerciale) {
		this.areaCommerciale = areaCommerciale;
	}

	public String getFlagTipoDaCopiare() {
		return flagTipoDaCopiare;
	}

	public void setFlagTipoDaCopiare(String flagTipoDaCopiare) {
		this.flagTipoDaCopiare = flagTipoDaCopiare;
	}

	public Map<Long, String> getFlagTipo() {
		return flagTipo;
	}

	public void setFlagTipo(Map<Long, String> flagTipo) {
		this.flagTipo = flagTipo;
	}

	public void invoiceLinesPanelToggled(ActionEvent actionEvent) {
		SimpleToggleEvent toggleEvent = (SimpleToggleEvent) actionEvent;
	}



	public void reasonChangedListener(ValueChangeEvent event) {

	}

	public void cdrChangedListener(ValueChangeEvent event) {

	}

	public void selectAll() {
		boolean newValue;
		if(isCurrentAllChecked())
			newValue = false;
		else
			newValue = true;
		Iterator<Map.Entry<Long, Boolean>> i = checked.entrySet().iterator();
		while (i.hasNext()) {
			Map.Entry<Long, Boolean> pair = i.next();
			pair.setValue(newValue);
		}
		setCurrentAllChecked(newValue);
	}

	public void copiaCausali() {
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (checked.get(line.getInvoiceLineId())) {
				// setto il value della selectItem della riga corrente
				causaleRiga.put(new Long(line.getInvoiceLineId()), this.causaleDaCopiare);
				subCausaleRiga.put(new Long(line.getInvoiceLineId()), this.subCausaleDaCopiare);
				this.causaliDisponibili.put(new Long(line.getInvoiceLineId()), this.causaliDisponibiliHeader);
				this.subCausaliDisponibili.put(new Long(line.getInvoiceLineId()), this.subCausaliDisponibiliHeader);
			}
		}
	}
	
	public void copiaAreaCommerciale() {
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (checked.get(line.getInvoiceLineId())) {
				// setto il value della selectItem della riga corrente
				areaCommerciale.put(new Long(line.getInvoiceLineId()), this.areaCommercialeDaCopiare);
			}
		}
	}
	
	public void copiaFlagTipo() {
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (checked.get(line.getInvoiceLineId())) {
				// setto il value della selectItem della riga corrente
				flagTipo.put(new Long(line.getInvoiceLineId()), this.flagTipoDaCopiare);
			}
		}
	}

	public String getTipoFattura() {
		return tipoFattura;
	}

	public void setTipoFattura(String tipoFattura) {
		this.tipoFattura = tipoFattura;
	}

	public List<SelectItem> getTipiFattura() {
		/*List<SelectItem> tipi = new LinkedList<SelectItem>();
		tipi.add(new SelectItem("IVA","IVA"));
		return tipi;*/
		return this.tipiFattura;
	}

	public void setTipiFattura(List<SelectItem> tipiFattura) {
		this.tipiFattura = tipiFattura;
	}

	public String getCodiceSettore() {
		return codiceSettore;
	}

	public void setCodiceSettore(String codiceSettore) {
		this.codiceSettore = codiceSettore;
	}

	public List<SelectItem> getSettori() {
		/*List<LabelValueBean> beans;
		try {
			beans = this.SIBservice.getSettori();
		} catch (Exception e) {
			e.printStackTrace();
			//Log.error("Errore invocazione back-end", e);
			beans = new LinkedList<LabelValueBean>();
		}*/
		return this.settori;
	}

	private List<SelectItem> toSelectItem(List<LabelValueBean> beans) {
		List<SelectItem> l = new LinkedList<SelectItem>();
		l.add(new SelectItem("", "SELEZIONARE"));
		if(beans != null) {
			for(LabelValueBean bean: beans) {
				l.add(new SelectItem(bean.getValue(), bean.getLabel()));
			}
		}
		return l;
	}
	
	public void setSettori(List<SelectItem> settori) {
		this.settori = settori;
	}

	public String getCodiceDivisione() {
		return codiceDivisione;
	}

	public void setCodiceDivisione(String codiceDivisione) {
		this.codiceDivisione = codiceDivisione;
	}

	public List<SelectItem> getDivisioni() {
		/*List<LabelValueBean> beans;
		try {
			beans = this.SIBservice.getDivisioni(this.codiceSettore.toString());
		} catch (Exception e) {
			//Log.error("Errore invocazione back-end", e);
			e.printStackTrace();
			beans = new LinkedList<LabelValueBean>();
		}*/
		return this.divisioni;
	}

	public void setDivisioni(List<SelectItem> divisioni) {
		this.divisioni = divisioni;
	}

	public String getCodiceUfficio() {
		return codiceUfficio;
	}

	public void setCodiceUfficio(String codiceUfficio) {
		this.codiceUfficio = codiceUfficio;
	}

	public List<SelectItem> getUffici() {
		/*List<LabelValueBean> beans;
		try {
			beans = this.SIBservice.getUffici(this.codiceSettore.toString(), this.codiceDivisione.toString());
		} catch (Exception e) {
			//Log.error("Errore invocazione back-end", e);
			e.printStackTrace();
			beans = new LinkedList<LabelValueBean>();
		}*/
		return this.uffici;
	}

	public void setUffici(List<SelectItem> uffici) {
		this.uffici = uffici;
	}

	public Integer getAnnoEsercizio() {
		return annoEsercizio;
	}

	public void setAnnoEsercizio(Integer annoEsercizio) {
		this.annoEsercizio = annoEsercizio;
	}

	public Date getInizioCompetenzaEconomica() {
		return inizioCompetenzaEconomica;
	}

	public void setInizioCompetenzaEconomica(Date inizioCompetenzaEconomica) {
		this.inizioCompetenzaEconomica = inizioCompetenzaEconomica;
	}

	public Date getFineCompetenzaEconomica() {
		return fineCompetenzaEconomica;
	}

	public void setFineCompetenzaEconomica(Date fineCompetenzaEconomica) {
		this.fineCompetenzaEconomica = fineCompetenzaEconomica;
	}

	public String getCausaleDaCopiare() {
		return causaleDaCopiare;
	}

	public void setCausaleDaCopiare(String causaleDaCopiare) {
		this.causaleDaCopiare = causaleDaCopiare;
	}

	public Map<Long, String> getCausaleRiga() {
		return causaleRiga;
	}

	public void setCausaleRiga(Map<Long, String> causaleRiga) {
		this.causaleRiga = causaleRiga;
	}

	public String getSubCausaleDaCopiare() {
		return subCausaleDaCopiare;
	}

	public void setSubCausaleDaCopiare(String subCausaleDaCopiare) {
		this.subCausaleDaCopiare = subCausaleDaCopiare;
	}


	public Map<Long, String> getSubCausaleRiga() {
		return subCausaleRiga;
	}

	public void setSubCausaleRiga(Map<Long, String> subCausaleRiga) {
		this.subCausaleRiga = subCausaleRiga;
	}
	
	

	/*
	public String getSupplierCode() {
		if (FacesContext.getCurrentInstance().getRenderResponse()) {
			// verificare se la fattura possiede dei dati aggiuntivi.
			if (this.invoiceHasSavedAdditionalData(selectedInvoice, Anagrafico.class)) {
				List<Anagrafico> storedAdditionalData = this.getInvoiceStoredAdditionalData(selectedInvoice, Anagrafico.class);
				if ( (storedAdditionalData != null) && (!storedAdditionalData.isEmpty()) ) {
					// trattandosi dell'anagrafico del fornitore mi aspetto che sia uno solo
					supplierCode = storedAdditionalData.get(0).getCodice();
				}
			}
		}
		return supplierCode;
	}

	public void setSupplierCode(String supplierCode) {
		this.supplierCode = supplierCode;
	}
*/
	/**
	 * @return the supplierCodes
	 */
	/*public List<SelectItem> getSupplierCodes() {
		if (supplierCodes == null)
			updateSupplierCodes();
		return supplierCodes;
	}*/

	/**
	 * @param supplierCodes
	 *            the supplierCodes to set
	 */
	/*public void setSupplierCodes(List<SelectItem> supplierCodes) {
		this.supplierCodes = supplierCodes;
	}*/

	public List<SelectItem> getCausaliDisponibiliHeader() {
		return causaliDisponibiliHeader;
	}

	public void setCausaliDisponibiliHeader(
			List<SelectItem> causaliDisponibiliHeader) {
		this.causaliDisponibiliHeader = causaliDisponibiliHeader;
	}

	public List<SelectItem> getSubCausaliDisponibiliHeader() {
		return subCausaliDisponibiliHeader;
	}

	public void setSubCausaliDisponibiliHeader(
			List<SelectItem> subCausaliDisponibiliHeader) {
		this.subCausaliDisponibiliHeader = subCausaliDisponibiliHeader;
	}

	public void setCausaliDisponibili(Map<Long, List<SelectItem>> causaliDisponibili) {
		this.causaliDisponibili = causaliDisponibili;
	}

	public Map<Long, List<SelectItem>> getCausaliDisponibili() {
		return causaliDisponibili;
	}

	public Map<Long, List<SelectItem>> getSubCausaliDisponibili() {
		return subCausaliDisponibili;
	}

	public void setSubCausaliDisponibili(
			Map<Long, List<SelectItem>> subCausaliDisponibili) {
		this.subCausaliDisponibili = subCausaliDisponibili;
	}

	/**
	 * @return the checked
	 */
	public Map<Long, Boolean> getChecked() {
		return checked;
	}

	/**
	 * @param checked
	 *            the checked to set
	 */
	public void setChecked(Map<Long, Boolean> checked) {
		this.checked = checked;
	}

	/**
	 * @return the currentAllChecked
	 */
	public boolean isCurrentAllChecked() {
		return currentAllChecked;
	}

	/**
	 * @param currentAllChecked the currentAllChecked to set
	 */
	public void setCurrentAllChecked(boolean currentAllChecked) {
		this.currentAllChecked = currentAllChecked;
	}

	/**
	 * @return the supplierCodeSaved
	 */
	public boolean isSupplierCodeSaved() {
		return this.invoiceHasSavedAdditionalData(selectedInvoice, Anagrafico.class);
	}



	/**
	 * @return the invoiceLinesHasAdditionalDataSaved
	 */
	public boolean isInvoiceLinesHasAdditionalDataSaved() {
		
		List<InvoiceLine> invoiceLines = selectedInvoice.getInvoiceLines();
		
		// lazy fetching and caching for furter requests
		if (! (invoiceLines instanceof ArrayList) ) {
			List<InvoiceLine> lazilyFetchedInvoiceLines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
			for (InvoiceLine il : lazilyFetchedInvoiceLines) {
				List<AdditionalData> lazilyFetchedInvoiceLineAdditionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(il);
				il.setAdditionalData(lazilyFetchedInvoiceLineAdditionalData);
			}
			this.selectedInvoice.setInvoiceLines(lazilyFetchedInvoiceLines);
		}
		
		// itero su tutte le righe e verifico se hanno sia Causale che CentroCosto salvati. Alla prima falsa esco per ottimizzare.
		boolean out = false;
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (this.invoiceLineHasSavedAdditionalData(line, Causale.class) && this.invoiceLineHasSavedAdditionalData(line, CentroCosto.class)) {
				out = true;
			}
			else {
				out = false;
				break;
			}
		}
		return out;
	}

	public boolean isMissingAdditionalData() {
		//TODO: Controllare i dati aggiuntivi
		return false;
	}
	
	/**
	 * @return the budgetSaved
	 */
	public boolean isBudgetSaved() {
		//TODO:
		return true;
	}
	
	public boolean isUoSaved() {
		//TODO:
		return true;
	}
	
	/**
	 * @return the supplierCodeSaved
	 */
	
	/**
	 * @return the approvalModalPanelRendered
	 */
	public boolean isApprovalModalPanelRendered() {
		return approvalModalPanelRendered;
	}

	/**
	 * @param approvalModalPanelRendered the approvalModalPanelRendered to set
	 */
	public void setApprovalModalPanelRendered(boolean approvalModalPanelRendered) {
		this.approvalModalPanelRendered = approvalModalPanelRendered;
	}
	
	/**
	 * Method used to retrieve and return additional data by class type
	 * 
	 * @param invoice
	 * @param clazz
	 * @return
	 */
	private boolean invoiceHasSavedAdditionalData(Invoice invoice, Class clazz) {
		boolean result = false;
		
		List<AdditionalData> additionalData = invoice.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(invoice);
			selectedInvoice.setAdditionalData(additionalData);
		} 
		if ( (additionalData!=null) && (!additionalData.isEmpty())) {
			for (AdditionalData ad : additionalData) {
				if (ad.getClass().equals(clazz)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}
	
	/**
	 * @param invoiceLine
	 * @param clazz
	 * @return
	 */
	private boolean invoiceLineHasSavedAdditionalData(InvoiceLine invoiceLine, Class clazz) {
		boolean result = false;
		List<AdditionalData> additionalData = invoiceLine.getAdditionalData();
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(invoiceLine);
			invoiceLine.setAdditionalData(additionalData);
		} 
		if ( (additionalData!=null) && (!additionalData.isEmpty()) ) {
			for (AdditionalData ad : additionalData) {
				if (ad.getClass().equals(clazz)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}
	
	/**
	 * Method used to retrieve and return additional data by class type
	 * 
	 * @param <T>
	 * @param invoiceLine
	 * @param clazz
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private <T> List<T> getInvoiceLineStoredAdditionalData(InvoiceLine invoiceLine, Class<T> clazz) {
		
		List<T> result = new ArrayList<T>();

		List<AdditionalData> additionalData = invoiceLine.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(invoiceLine);
			invoiceLine.setAdditionalData(additionalData);
		} 
		
		if (additionalData != null && !additionalData.isEmpty()) {
			for (AdditionalData ad : additionalData) {
				if (ad.getClass().equals(clazz)) {
					result.add((T) ad);
				}
			}
		}

		return result;
		
	}

	@SuppressWarnings("unchecked")
	private <T> List<T> getInvoiceStoredAdditionalData(Invoice invoice,
			Class<T> clazz) {
		List<T> result = new ArrayList<T>();

		List<AdditionalData> additionalData = invoice.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(invoice);
			// saving them in order to improve performance for further requests
			selectedInvoice.setAdditionalData(additionalData);
		} 

		for (AdditionalData ad : additionalData) {

			if (ad.getClass().equals(clazz)) {
				result.add((T) ad);
			}

		}
		return result;
	}
	
	/**
	 * Helper method to save or replace AdditionalData inside Invoice's AdditionalData list
	 * 
	 * @param invoice
	 * @param newAdditionalData
	 * @param clazz
	 */
	public Invoice saveOrReplaceAdditionalData(Invoice invoice, AdditionalData newAdditionalData, Class clazz) {
		
		List<AdditionalData> additionalDataList = invoice.getAdditionalData();
		
		// this is to solve java.util.ConcurrentModificationException
		List<AdditionalData> out = new ArrayList<AdditionalData>(additionalDataList);
		
		if ((additionalDataList != null)) {
			
			// il dato non è nullo e non è presente fra i dati aggiuntivi -> lo aggiungo
			if( (newAdditionalData != null) && (!additionalDataList.contains(newAdditionalData))) {
				// lo aggiungo alla nuova lista
				out.add(newAdditionalData);
			}
			
			// il dato può essere nullo o già presente fra i dati aggiuntivi
			else {
				for (AdditionalData ad : additionalDataList) {
					if(ad.getClass().equals(clazz)) {
						if (newAdditionalData == null) {
							out.remove(additionalDataList.indexOf(ad));
						}
						else {
							// lo dovrei sostituire nella lista originaria, ergo lo sostituisco nella nuova lista
							out.set(additionalDataList.indexOf(ad), newAdditionalData);
							//out.add(newAdditionalData);
						} 
					}
				}
			}
			
		}
		invoice.setAdditionalData(out);
		return invoice;
	}
	
	/**
	 * Helper method to save or replace AdditionalData inside InvoiceLine's AdditionalData list
	 * 
	 * @param invoiceLine
	 * @param newAdditionalData
	 * @param clazz
	 */
	public InvoiceLine saveOrReplaceAdditionalData(InvoiceLine invoiceLine, AdditionalData newAdditionalData, Class clazz) {
		
		List<AdditionalData> additionalDataList = invoiceLine.getAdditionalData();
		
		// this is to solve java.util.ConcurrentModificationException
		List<AdditionalData> out = new ArrayList<AdditionalData>(additionalDataList);
		
		if ((additionalDataList != null)) {
			
			// il dato non è nullo e non è presente fra i dati aggiuntivi -> lo aggiungo
			if( (newAdditionalData != null) && (!additionalDataList.contains(newAdditionalData))) {
				// lo aggiungo alla nuova lista
				out.add(newAdditionalData);
			}
			
			// il dato può essere nullo o già presente fra i dati aggiuntivi
			else {
				
				for (AdditionalData ad : additionalDataList) {
					if(ad.getClass().equals(clazz)) {
						if (newAdditionalData == null) {
							out.remove(additionalDataList.indexOf(ad));
						}
							
						else {
							// lo dovrei sostituire nella lista originaria, ergo lo sostituisco nella nuova lista
							out.set(additionalDataList.indexOf(ad), newAdditionalData);
							//out.add(newAdditionalData);
						}
							
					}
				}
				
			}
			
		}
		invoiceLine.setAdditionalData(out);
		return invoiceLine;
	}	
}
