/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.invoice;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.generic.NestedCriterion;
import it.fe.core.par.authorization.RoleDetail;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.user.Context;
import it.fe.core.par.user.ProfileDetail;
import it.fe.core.par.user.Role;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;
import it.fe.core.process.exception.JbpmProcessException;
import it.fe.web.common.DateUtils;
import it.fe.web.common.JSFUtils;
import it.fe.web.common.ProfileUtils;
import it.fe.web.common.SessionUtils;
import it.fe.web.controller.generic.ProfileFilterableHandler;
import it.fe.web.controller.profile.UnimoreCiaProfilesHandler;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.apache.log4j.Logger;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.security.auth.PrincipalThreadLocal;
import com.liferay.portal.util.PortalUtil;

/**
 * Handler di fatture; estende la classe ProfileFilterableHandler e ne ridefinisce i metodi astratti per gestire opportunamente i filtri lato web e lato profilo utente
 * 
 * 
 * @author Stefano Monti, Fulvio Di Marco
 *
 */

public class FilterableInvoicesHandler extends ProfileFilterableHandler {

	private static final long serialVersionUID = 1L;
	private static final String INVOICE_WAITING = "waiting";
	protected Logger logger = Logger.getLogger(FilterableInvoicesHandler.class.getName());

	protected Invoice selectedInvoice;
	public Invoice shownInvoice;
	protected List<Meter> shownInvoiceUtilityInfo;
	protected String invoiceNumber;
	protected String supplierName;
	protected Date dateBegin;
	protected Date dateEnd;
	protected Date receiptDateBegin;
	protected Date receiptDateEnd;
	protected boolean editable;
	protected boolean formDisabled;
	protected String validationResult = INVOICE_WAITING;
	protected UnimoreCiaProfilesHandler ciaProfilesHandler;
	protected List<AccountingCostAggregation> accountingCostAggregation ;
	protected String documentType;
	
	public FilterableInvoicesHandler() {}
	
	/**
	 * RequiredState e RequiredPermission vengono usate per generare i criteri di filtro sulla base dei profili utente
	 * Devono essere inizializzate come parametri del ManagedBean
	 */
	protected String requiredState;
	protected String requiredPermission;

	
	public Date getDateBegin() {
		return dateBegin;
	}

	public void setDateBegin(Date dateBegin) {
		this.dateBegin = dateBegin;
	}

	public Date getDateEnd() {
		return dateEnd;
	}

	public void setDateEnd(Date dateEnd) {
		this.dateEnd = dateEnd;
	}
	
	public String getRequiredState() {
		return requiredState;
	}

	public void setRequiredState(String requiredState) {
		this.requiredState = requiredState;
	}

	public String getRequiredPermission() {
		return requiredPermission;
	}

	public void setRequiredPermission(String requiredPermission) {
		this.requiredPermission = requiredPermission;
	}

	public String getSupplierName() {
		return supplierName;
	}

	public void setSupplierName(String supplierName) {
		this.supplierName = supplierName;
	}

	public String getInvoiceNumber() {
		return invoiceNumber;
	}

	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}

	public Invoice getSelectedInvoice() {
		return selectedInvoice;
	}

	public void setSelectedInvoice(Invoice selectedInvoice) {
		this.selectedInvoice = selectedInvoice;
	}
	
	public Invoice getShownInvoice() {		
		return shownInvoice;
	}

	public void setShownInvoice(Invoice shownInvoice) {
		this.shownInvoice = shownInvoice;
	}

	public List<Meter> getShownInvoiceUtilityInfo() {
		return shownInvoiceUtilityInfo;
	}

	public void setShownInvoiceUtilityInfo(List<Meter> shownInvoiceUtilityInfo) {
		this.shownInvoiceUtilityInfo = shownInvoiceUtilityInfo;
	}

	public Date getReceiptDateBegin() {
		return receiptDateBegin;
	}

	public void setReceiptDateBegin(Date receiptDateBegin) {
		this.receiptDateBegin = receiptDateBegin;
	}

	public Date getReceiptDateEnd() {
		return receiptDateEnd;
	}

	public void setReceiptDateEnd(Date receiptDateEnd) {
		this.receiptDateEnd = receiptDateEnd;
	}

	public String openDetail() {
		this.selectedInvoice = (Invoice) dataModel.getRowData();
		this.fetchLazyCollections((Invoice) dataModel.getRowData());
		this.fetchAccountingCosts((Invoice) dataModel.getRowData());
		this.updateEditable();
		InvoiceInfoHandler invoiceInfoHandler = (InvoiceInfoHandler) JSFUtils.getManagedBean("invoiceInfoHandler");
		invoiceInfoHandler.setMeterSummariesList(selectedInvoice.getMeterSummary());
		invoiceInfoHandler.setPreviousDocumentList(selectedInvoice.getPreviousDocuments());
		
		invoiceInfoHandler.setCurrentDocumentNumber(null);
		invoiceInfoHandler.setCurrentDocument(null);
		invoiceInfoHandler.setCurrentInvoiceId(null);
		invoiceInfoHandler.setMeterInfo(null);
		
		return "openDetail";
	}
	
	protected void fetchAccountingCosts(Invoice rowData) {
		accountingCostAggregation = new ArrayList<AccountingCostAggregation>();
		List<Object[]> accountingCostsMap = JNDIUtils.getInvoiceManager().fetchInvoiceLineAccountingCosts(selectedInvoice);
		if (accountingCostsMap != null){
			for (Object[] current: accountingCostsMap){
				accountingCostAggregation.add(new AccountingCostAggregation((String)current[0],(Double)current[2],(String)current[1]));
			}
		}
		
	}

	public void utilityInvoiceInfo(){
		this.shownInvoice = (Invoice) dataModel.getRowData();

		List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(shownInvoice);
		this.shownInvoice.setPreviousDocuments(documents);

	}
	

	protected void fetchLazyCollections(Invoice selectedInvoice) {
		List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(selectedInvoice);
		List<MeterSummary> summaries = JNDIUtils.getInvoiceManager().fetchMeterSummaries(selectedInvoice);
		List<InvoiceLine> lines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
		List<TaxDetail> taxDetails = JNDIUtils.getInvoiceManager().fetchTaxDetails(selectedInvoice);
		List<Reading> readingList = JNDIUtils.getInvoiceManager().fetchReadings(selectedInvoice);
		
		
		
		this.selectedInvoice.setInvoiceLines(lines);
		this.selectedInvoice.setPreviousDocuments(documents);
		this.selectedInvoice.setMeterSummary(summaries);
		this.selectedInvoice.setTaxDetails(taxDetails);
		this.selectedInvoice.setReadingList(readingList);
	}
	
	public void loadInvoiceLines() {
		List<InvoiceLine> lines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
		this.selectedInvoice.setInvoiceLines(lines);
	}
	
	public void updateInvoiceStateAndError() {
		Invoice invoice = JNDIUtils.getInvoiceManager().getInvoiceById(this.selectedInvoice.getInvoiceId().longValue());
		this.selectedInvoice.setInvoiceState(invoice.getInvoiceState());
		this.selectedInvoice.setError(invoice.isError());
		this.selectedInvoice.setErrorMessage(invoice.getErrorMessage());
	}

	public String technicallyApproveInvoice() {
		try {
			JNDIUtils.getProcessManager().approveTechnically(selectedInvoice, SessionUtils.getCurrentUserScreenname());
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		}
		selectedInvoice.setInvoiceState(JNDIUtils.getInvoiceStateManager().getInvoiceStateByDescription(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED));
		// forcing data model reload
		this.dataModel = null;
		this.updateEditable();
		this.updateInvoiceStateAndError();
		// se la fattura contiene errori di inserimento in CIA lo segnalo
		if (!selectedInvoice.getInvoiceState().getDescription().equals(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED))
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore nell'inserimento nel sistema contabile", "Si è verificato un errore nell'inserimento nel sistema contabile"));  
		else if (selectedInvoice.getInvoiceState().getDescription().equals(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED))
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"La fattura è stata inserita correttamente nel sistema contabile", "La fattura è stata inserita correttamente nel sistema contabile"));  
		return null;
	}
	
	public String technicallyRejectInvoice() {
		try {
			JNDIUtils.getProcessManager().rejectTechnically(selectedInvoice, SessionUtils.getCurrentUserScreenname());
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		} 
		selectedInvoice.setInvoiceState(JNDIUtils.getInvoiceStateManager().getInvoiceStateByDescription(InvoiceState.TECHNICALLY_REJECTED));
		// forcing data model reload
		this.dataModel = null;
		this.updateEditable();
		this.updateInvoiceStateAndError();
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"La fattura è stata rifiutata", "La fattura è stata rifiutata."));  
		return null;
	}
	
	public String editAdditionalData() {
		String result = "";
		try {
			result = JNDIUtils.getProcessManager().editAdditionalData(selectedInvoice, SessionUtils.getCurrentUserScreenname());
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		} 
		this.updateEditable();
		//this.setAdditionalDataFormEnabled(true);
		return result;
	}
	
	public String saveAdditionalData() {
		String result = "";
		try {
			JNDIUtils.getInvoiceManager().updateInvoice(selectedInvoice);
			String screenName = SessionUtils.getCurrentUserScreenname();
			result = JNDIUtils.getProcessManager().saveAdditionalData(selectedInvoice, screenName);
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		} 
		this.updateEditable();
		//this.setAdditionalDataFormEnabled(false);
		return result;
	}
	
	public String discardAdditionalData() {
		String result = "";
		try {
			JNDIUtils.getProcessManager().discardAdditionalData(selectedInvoice, SessionUtils.getCurrentUserScreenname());
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		} 
		this.updateEditable();
		//this.setAdditionalDataFormEnabled(false);
		return result;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	protected Class determineClass() {
		return Invoice.class;
	}

	
/**
 * PROBLEMA: È NECESSARIO GESTIRE SITUAZIONI IN CUI ESISTANO PIÙ CRITERI DI FILTRO SULLO STESSO CAMPO
 * Usando due logiche distinte per i criteri web e quelli di profilo.
 * 
 * Profilo: se esistono due profili sullo stesso campo, ma con valore distinto: devo usare una logica OR:
 * 		es. invoice.supplier.supplierName , HERA
 * 			invoice.supplier.supplierName , ENEL
 * 			--> DEVO PRENDERE TUTTE LE FATTURE con fornitore Enel OPPURE Hera
 * 
 * Web: se esiste un criterio di filtro compatibile con quello dei profili, devo ragionare con una logica restrittiva:
 * 		es. (PROFILO) 
 * 				invoice.supplier.supplierName , HERA
 * 				invoice.supplier.supplierName , ENEL
 * 			(WEB)
 * 				invoice.supplier.supplierName , TELECOM
 * 
 * 		--> (i.s.sn == HERA OR i.s.sn == ENEL) AND (i.s.sn == TELECOM)
 * 			in pratica, restringo sull'insieme delle fatture già filtrate
 */
	@Override
	protected DetachedCriteria determineDetachedCriteria() {
		return ProfileUtils.computeDetachedCriteriaWithFilters(determineOrder(), super.order, determineClass(), determineWebRestrictions(),this.criteriProfilo);
		
//		Order orderCriteria = determineOrder();
//		boolean orderAdded = (orderCriteria == null); //se non ho ordinamento, devo ignorarlo nelle successive fasi.
//		DetachedCriteria out = DetachedCriteria.forClass(Invoice.class);
	
//		//Mappa proprietà valori i criteri del profilo
//		Map<String, List<Criterion>> criteriWeb= determineWebRestrictions();
//		
//		//Join dei criteri di filtro secondo le regole stabilite nel commento ad inizio funzione
//		if ( (criteriProfilo != null) && (criteriProfilo.keySet() != null) )
//			for (String property : criteriProfilo.keySet()){
//				List<Criterion> criteri = criteriProfilo.get(property);
//				if  ( (criteri == null) || (criteri.size() == 0) ) 
//					continue;
//				
//				else if (criteri.get(0) instanceof NestedCriterion){
//					
//					String outerClass = ((NestedCriterion)criteri.get(0)).getOuterPropertyName();
//					DetachedCriteria propertyCriteria = out.createCriteria(outerClass);
//					
//					/**
//					 * Devo verificare la presenza di criteri di ordinamento su una proprietà innestata e aggiungerli al criterio opportuno in fase di creazione.
//					 * Hibernate criteria, infatti, non consente di creare due criteri distinti per la stessa proprietà (es. uno per le restrictions e uno per l'order)
//					 * E' quindi necessario creare gli ordinamenti innestati in maniera contestuale alla creazione della medesima proprietà innestata
//					 * 
//					 */
//					
//					//Se sto aggiungendo un sottocriterio sul quale è stato anche richiesto l'ordinamento...
//					if ( (!orderAdded) &&  (super.order != null) && (orderCriteria != null) && (orderCriteria instanceof NestedOrder) && (((NestedOrder)orderCriteria).getOuterPropertyName().equals(outerClass))){
//						String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
//						boolean asc = ((NestedOrder)orderCriteria).isAscending();
//						Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
//						propertyCriteria.addOrder(newOrder);
//						orderAdded=true;
//					}
//					
//					if (criteri.size() == 1){
//						//TODO: verificare se ci sono criteri di filtro su web e metterli in AND
//						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
//							propertyCriteria.add((((NestedCriterion)criteri.get(0)).getInnerCriterion()));
//						else {
//							Conjunction conjuntion =Restrictions.conjunction();
//							conjuntion.add((((NestedCriterion)criteri.get(0)).getInnerCriterion()));
//							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
//								conjuntion.add( ((NestedCriterion) webCriterion).getInnerCriterion()); //TODO: attenzione: suppongo che sia un NestedCriterion anche quello della parte web perché o è quello delle property
//							}
//							propertyCriteria.add(conjuntion);
//						}
//					}
//					else {
//						Disjunction disjunction = Restrictions.disjunction();
//						for (Criterion criterion : criteri) {
//							disjunction.add(((NestedCriterion)criterion).getInnerCriterion());
//						}
//						
//						//TODO: verificare se ci sono criteri di filtro su web e metterli in AND
//						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
//							propertyCriteria.add(disjunction);
//						else {
//							Conjunction conjuntion =Restrictions.conjunction();
//							conjuntion.add(disjunction);
//							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
//								conjuntion.add(((NestedCriterion)webCriterion).getInnerCriterion());
//							}
//							propertyCriteria.add(conjuntion);
//						}
//						
//					}
//				}
//					
//				else{
//					
//					if (criteri.size() == 1)
//						//verifico se ci sono criteri di filtro su web e li metto in AND con i critreri di profilo
//						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
//							out.add(criteri.get(0));
//						else {
//							Conjunction conjuntion =Restrictions.conjunction();
//							conjuntion.add(criteri.get(0));
//							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
//								conjuntion.add(webCriterion);
//							}
//							out.add(conjuntion);
//						}
//					else {
//						Disjunction disjunction = Restrictions.disjunction();
//						for (Criterion criterion : criteri) {
//							disjunction.add(criterion);
//						}
//						//verifico se ci sono criteri di filtro su web e li metto in AND con i critreri di profilo
//						if ( ((extractRestrictions(property,criteriWeb) == null) || ((extractRestrictions(property,criteriWeb).size() == 0))) )
//							out.add(disjunction);
//						else {
//							Conjunction conjuntion =Restrictions.conjunction();
//							conjuntion.add(disjunction);
//							for (Criterion webCriterion : extractRestrictions(property,criteriWeb) ){
//								conjuntion.add(webCriterion);
//							}
//							out.add(conjuntion);
//						}
//						
//					}
//						
//				}
//			}
//		
//		//Ciclo sui criteri web rimanenti (e cioè non andati in join con i criteri dei profili)
//		if ( (criteriWeb != null) && (criteriWeb.keySet() != null) )
//			for (String property : criteriWeb.keySet()){
//				if ( (criteriProfilo.get(property) != null) && (criteriProfilo.get(property).size()>0) )
//					continue;
//				else{
//					List<Criterion> criteri = criteriWeb.get(property);
//					if  ( (criteri == null) || (criteri.size() == 0) ) 
//						continue;
//					
//					else if (criteri.get(0) instanceof NestedCriterion){
//						String outerClass = ((NestedCriterion)criteri.get(0)).getOuterPropertyName();
//						DetachedCriteria propertyCriteria = out.createCriteria(((NestedCriterion)criteri.get(0)).getOuterPropertyName());
//						/**
//						 * Devo verificare la presenza di criteri di ordinamento su una proprietà innestata e aggiungerli al criterio opportuno in fase di creazione.
//						 * Hibernate criteria, infatti, non consente di creare due criteri distinti per la stessa proprietà (es. uno per le restrictions e uno per l'order)
//						 * E' quindi necessario creare gli ordinamenti innestati in maniera contestuale alla creazione della medesima proprietà innestata
//						 * 
//						 * il codice sotto è analogo a quello già scritto nella sezione precedente per la gestione dei filtri di profilo:
//						 * abbiamo garanzia di non aggiungere due volte lo stesso ordinamento perché questo secondo ciclo itera solo sulle proprietà web rimaste fuori dalla precedente join con i criteri di profilo
//						 */
//						
//						//Se sto aggiungendo un sottocriterio sul quale è stato anche richiesto l'ordinamento...
//						if ( (!orderAdded) &&  (super.order != null) && (orderCriteria != null) && (orderCriteria instanceof NestedOrder) && (((NestedOrder)orderCriteria).getOuterPropertyName().equals(outerClass))){
//							String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
//							boolean asc = ((NestedOrder)orderCriteria).isAscending();
//							Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
//							propertyCriteria.addOrder(newOrder);
//							orderAdded=true;
//						}
//						
//						if (criteri.size() == 1)
//							propertyCriteria.add(((NestedCriterion)criteri.get(0)).getInnerCriterion());
//						else {
//							Conjunction c = Restrictions.conjunction();
//							for (Criterion criterion : criteri) {
//								c.add(((NestedCriterion)criterion).getInnerCriterion());
//							}
//							propertyCriteria.add(c);
//						}
//					}
//					else {
//						if (criteri.size() == 1)
//							out.add(criteri.get(0));
//						else {
//							Conjunction conjunction = Restrictions.conjunction();
//							for (Criterion criterion : criteri) {
//								conjunction.add(criterion);
//							}
//							out.add(conjunction);
//						}
//					}
//					
//				}
//					
//			}
//		
//		//Infine aggiungo l'ordinamento (solo se questo non è di tipo nested, perché in tal caso è stato gestito a monte: vedere sopra)
//		if ( (!orderAdded) &&  (super.order != null) && (orderCriteria != null)  ){
//			if (! (orderCriteria instanceof NestedOrder)){
//					out.addOrder(orderCriteria);
//			}
//			else{
//				String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
//				String outerProperty = ((NestedOrder)orderCriteria).getOuterPropertyName();
//				boolean asc = ((NestedOrder)orderCriteria).isAscending();
//				Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
//				out.createCriteria(outerProperty ).addOrder(newOrder);
//			}
//		}
//		return out;
	}
	
	
	@SuppressWarnings("unchecked")
	@Override
	protected Map<String,List<Criterion>> determineWebRestrictions() {
		Map<String,List<Criterion>> out = new HashMap<String, List<Criterion>>();
	
		if ((this.invoiceNumber != null)&&(!this.invoiceNumber.equals("")))
			out.put("invoiceNumber",new ArrayList(Arrays.asList(new Criterion[]{Restrictions.ilike("invoiceNumber", this.invoiceNumber, MatchMode.ANYWHERE)})));
		
		//TODO: attenzione: è necessario garantire consistenza tra il nome del filtro lato web e lato profilo!!!
		//Altrimenti la successiva fase di join dei cirteri non funziona
		if ((this.supplierName!= null)&&(!this.supplierName.equals("")))
			out.put("supplier.supplierName",new ArrayList(Arrays.asList(new Criterion[]{new NestedCriterion("supplier", Restrictions.ilike("supplierName", supplierName, MatchMode.ANYWHERE))})));

		if ((this.dateBegin!= null) && (this.dateEnd!= null))
			out.put("paymentDetail.paymentDueDate",new ArrayList(Arrays.asList(new Criterion[]{new NestedCriterion("paymentDetail", Restrictions.between("paymentDueDate", DateUtils.beginOfDay(dateBegin),DateUtils.endOfDay(dateEnd)))})));
		else if (this.dateBegin!= null)
			out.put("paymentDetail.paymentDueDate",new ArrayList(Arrays.asList(new Criterion[]{new NestedCriterion("paymentDetail", Restrictions.ge("paymentDueDate",DateUtils.beginOfDay(dateBegin)))})));
		else if (this.dateEnd!= null)	
			out.put("paymentDetail.paymentDueDate",new ArrayList(Arrays.asList(new Criterion[]{new NestedCriterion("paymentDetail", Restrictions.le("paymentDueDate", DateUtils.endOfDay(dateEnd)))})));
		
		if ((this.receiptDateBegin!= null) && (this.receiptDateBegin!= null))
			out.put("receiptDate",new ArrayList(Arrays.asList(new Criterion[]{Restrictions.between("receiptDate", DateUtils.beginOfDay(receiptDateBegin),DateUtils.endOfDay(receiptDateEnd))})));
		else if (this.receiptDateBegin!= null)
			out.put("receiptDate",new ArrayList(Arrays.asList(new Criterion[]{Restrictions.ge("receiptDate",DateUtils.beginOfDay(receiptDateBegin))})));
		else if (this.receiptDateEnd!= null)	
			out.put("receiptDate",new ArrayList(Arrays.asList(new Criterion[]{Restrictions.le("receiptDate",DateUtils.endOfDay(receiptDateEnd))})));
		
		
		out.put("invoiceState.description", new ArrayList(Arrays.asList(new Criterion[]{new NestedCriterion("invoiceState", Restrictions.ilike("description", getRequiredState() , MatchMode.START))})));
		
		return out;
	}

	@Override
	public void resetSearchParameters() {
		this.invoiceNumber = null;
		this.supplierName = null;
		this.dateBegin = null;
		this.dateEnd = null;
		this.receiptDateBegin=null;
		this.receiptDateEnd=null;
		
	}

	/**
	 * Verifica e prepara i criteri di accesso alle fatture sulla base del profilo dell'utente
	 * Metodo con side effect: riempie anche la struttura dati associata alle Criteria di restrizione delle fatture, 
	 * in maniera da poterle riutilizzare nelle query successive
	 */	
	public boolean determineUserPermissions() {
		//variabile che verifica se sono stati trovati profili che consentano la visualizzazione di qualche informazione 
		boolean checkedPermission = false;
		
		InvoiceState portletState = JNDIUtils.getInvoiceStateManager().getInvoiceStateByDescription(getRequiredState());
		
//		1. prendo tutti i profileDetail associati all'utente 
		Set<ProfileDetail> profileDetails = SessionUtils.getCurrentUserProfileDetail();
		
//		2. prendo i ruoli dell'utente da profiledetails
		Set<Role> userRoles = new HashSet<Role>();
		for (ProfileDetail pd : profileDetails) 
			userRoles.add(pd.getRole());
		
//		3. carico da dettaglio_ruoli gli stati associati ai ruoli degli utenti
//		3.1 fetch del dettaglio ruoli
		Set<Role> userRoles_detailed = new HashSet<Role>();
		for (Role role : userRoles) 
			userRoles_detailed.add(JNDIUtils.getRoleManager().loadRoleDetail(role));

//		3.2 reperimento ruoli abilitati
		Set<Role> userRoles_filtered = new HashSet<Role>();
		Set<String> userRoleNames_filtered = new HashSet<String>();
		for (Role role : userRoles_detailed ) 
			for (RoleDetail roleDetail : role.getRoleDetail())
				//4. tengo i soli ruoli associati allo stato della portlet (getPortletState())
				//5. tengo i soli ruoli con permesso coerente con quello della portlet (getPortletPermission())
				if ( roleDetail.getInvoiceState().getDescription().equals(getRequiredState()) && (roleDetail.getPermission().getCode().equals(getRequiredPermission())) ){
					logger.trace(" Role: "+role.getDescription()+", state: "+roleDetail.getInvoiceState().getDescription()+", permission :"+ roleDetail.getPermission().getCode());
					userRoles_filtered.add(role);
					userRoleNames_filtered.add(role.getDescription());
				}

		Set<ProfileDetail> valid_profiles = new HashSet<ProfileDetail>();
		for (ProfileDetail profile : profileDetails ) 
			if (userRoleNames_filtered.contains( profile.getRole().getDescription()) )
				valid_profiles.add(profile);

		//Se non ci sono profili validi, l'utente non può vedere nulla
		if (valid_profiles.size() <= 0)
			return false;

		//i profili rimasti a questo punto sono tutti e soli quelli compatibili con il ruolo ed il permesso associato alla portlet
		super.validUserProfiles=valid_profiles;
		
		//TODO: gestione date profili!!
		
		//inizio a riempire la struttura dei criteri di filtro user-related
		
		//Gestione di ALL; è necessario gestire due tipi di wildcard ALL
		// - gestire un nuovo Contesto ALL --> tutte le fatture, indipendentemente dal contesto
		// - gestire contextValue messo a ALL --> cioè tutte le fatture di un dato contesto (es.:tutte le fatture con QUALUNQUE fornitore) ) 
		//inizio a riempire la struttura dei criteri di filtro user-related
		//TODO: verificare 
		
		for (ProfileDetail profileDetail : super.validUserProfiles) {
			
			//Il Contesto ALL automaticamente azzera tutti gli altri filtri sullo stesso stato/permesso
			if (profileDetail.getContext().getDescription().equals(Context.ALL)){
				super.criteriProfilo = new HashMap<String, List<Criterion>>();
				return true;
			}
			else if (profileDetail.getContextValue().equals(Context.ALL)){
				//se il Contesto corrente ha valore ALL significa che non devo aggiungere un filtro sul contesto
				//non esco dal ciclo di controllo perché ci possono essere altri contesti (collegati a questo utente/ruolo) 
				//che invece generano dei filtri
				//aggiorno la condizione di permessi verificati, perché questo profilo mi consente di visualizzare tutte le fatture 
				//di un dato contesto (pur senza generare filtri perché il contextValue è ad ALL)
				checkedPermission = true;
				continue;
			}
			
			String columnName = profileDetail.getContext().getColumnName();
			String tableName = profileDetail.getContext().getTableName();
			String value = profileDetail.getContextValue();
			//TODO: attenzione alla convenzione tableName == className
			//TODO: attenzione alla convenzione columnName == propertyName
			
			Criterion criteriaToBeAdded = null;  
			if  ( (tableName != null) && (tableName.equals(Invoice.class.getSimpleName())) && (columnName != null) && (!columnName.equals("")) && (value!= null) && (!value.equals(""))){
				if (columnName.contains(".")){
					String outerProperty = columnName.substring(0,columnName.indexOf("."));
					String innerProperty = columnName.substring(columnName.indexOf(".")+1);
					criteriaToBeAdded = new NestedCriterion(outerProperty, Restrictions.eq(innerProperty, value));
				}
				else
					criteriaToBeAdded = Restrictions.eq(columnName, value);
			}
			if (criteriaToBeAdded != null)
				if (super.criteriProfilo.keySet().contains (columnName) )
					super.criteriProfilo.get(columnName).add(criteriaToBeAdded);
				else 
					super.criteriProfilo.put(columnName, new ArrayList<Criterion>(Arrays.asList(new Criterion[]{criteriaToBeAdded})));
			
		}
	
		//l'utente ha profili coerenti con quelli della portlet ma i profili 
		// - non fanno match con la classe da filtrare (Fattura), oppure
		// - hanno nomi di colonne o valori non significativi (cioè condizioni di errore)
		//ATTENZIONE: per convenzione, se un utente non ha profili significativi per la classe fattura, non vede alcuna fattura!! 
		//(si sarebbe potuta scegliere la convenzione opposta...)
		
		//se i criteri di filtro sono vuoti possono essersi verificate due condizioni:
		// - non ho profili che consentano la visualizzazione --> devo restituire false
		// - ho profili che consentano la visualizzazione, ma con contextValue a ALL e che quindi non hanno generato filtri --> devo restituire true
		// questa doppia condizione è stata gestita mediante variabile checkedPermission
		if ( (criteriProfilo == null) || (super.criteriProfilo.keySet() == null) || (super.criteriProfilo.keySet().size() == 0) )  
			return checkedPermission;
		else		//se i criteri di filtro sono pieni, significa che esiste almeno un profilo che può consentire la visualizzazione (poi saranno i filtri sui contesti a mostrare o meno elementi(fatture)) 
			return true;

	}

	/*
	 * 
	 * ==============
	 * Helper methods
	 * ==============
	 * 
	 */


	
	protected void updateEditable() {
		try {
			this.editable = JNDIUtils.getProcessManager().isEditable(selectedInvoice);
		} catch (JbpmProcessException e) {
			logger.error(e.getMessage());
			this.editable = false;
		}
	}

	public void setEditable(boolean editable) {
		this.editable = editable;
	}

	public boolean isEditable() {
		return this.editable && "Ricevuta".equals(selectedInvoice.getInvoiceState().getDescription());
	}

	public void setValidationResult(String validationResult) {
		this.validationResult = validationResult;
	}

	public String getValidationResult() {
		return validationResult;
	}

	/**
	 * !(fe:stateEquals(fe:getManagedBean(MBean).selectedInvoice,'Ricevuta') and !fe:getManagedBean(MBean).editable)
	 * 
	 * @return the formDisabled
	 */
	public boolean isFormDisabled() {
		return ! ("Ricevuta".equals(selectedInvoice.getInvoiceState().getDescription()) && !(isEditable()) );
	}

	/**
	 * @param formDisabled the formDisabled to set
	 */
	public void setFormDisabled(boolean formDisabled) {
		this.formDisabled = formDisabled;
	}

	public List<AccountingCostAggregation> getAccountingCostAggregation() {
		return accountingCostAggregation;
	}

	public void setAccountingCostAggregation(
			List<AccountingCostAggregation> accountingCostAggregation) {
		this.accountingCostAggregation = accountingCostAggregation;
	}

	public String getDocumentType() {
		return documentType;
	}

	public void setDocumentType(String documentType) {
		this.documentType = documentType;
	}

}
