/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.PeppolValidation;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.user.Organization;
import it.fe.core.par.user.OrganizationProperty;
import it.fe.core.par.user.User;
import it.fe.web.common.SessionUtils;
import it.fe.web.controller.peppol.limeclient.RuntimeMessageDownloader;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.transform.TransformerException;

import org.busdox.transport.lime.api.interfaces.MessageException;

/**
 * Handler per la gestione del download di documenti da AP dell'infrastruttura
 * Peppol, mediante protocollo LIME.
 * 
 * @author Stefano Monti
 * 
 */
public class LIMEInboxHandler extends PeppolHandler {

	protected boolean leaveMessages = false;
	public boolean pollingEnabled = false;
	protected RuntimeMessageDownloader messageDownloader;
	private int downloadedMessages;
	protected User user;
	protected DownloadProgressCallback callback;
	

	public LIMEInboxHandler() {
		super();
		user = JNDIUtils.getUserManager().getUserByUsername(SessionUtils.getCurrentUserScreenname());
		//Override della proprietà receiverID: questo controller si aspetta il participant ID della azienda come UOID dell'utente che si è loggato.
		super.receiverID = user.getUoId();
		callback = new DownloadProgressCallback();
		
		String user = SessionUtils.getCurrentUserScreenname();
		User currentUser = JNDIUtils.getUserManager().getUserByUsername(user);
		if (( currentUser != null) && (currentUser.getUoId() != null) && (!currentUser.getUoId().equals("")) ){
			
			Organization organization = JNDIUtils.getOrganizationManager().getOrganizationByUoID(currentUser.getUoId());
			if (organization != null){
				OrganizationProperty peppolServer = JNDIUtils.getOrganizationPropertyManager().getOrganizationPropertyByOrganizationAndLabel(organization, "peppolServer");
				OrganizationProperty peppolUsername = JNDIUtils.getOrganizationPropertyManager().getOrganizationPropertyByOrganizationAndLabel(organization, "peppolUsername");
				OrganizationProperty peppolPassword= JNDIUtils.getOrganizationPropertyManager().getOrganizationPropertyByOrganizationAndLabel(organization, "peppolPassword");
			
				if ( (peppolServer != null) && (peppolServer.getValue() != null) &&  (!peppolServer.getValue().equals("")) ){
					super.serverAddress = peppolServer.getValue();
				}
				if ( (peppolUsername != null) && (peppolUsername.getValue() != null) &&  (!peppolUsername.getValue().equals("")) ){
					super.username = peppolUsername.getValue();
				}
				
				if ( (peppolPassword != null) && (peppolPassword.getValue() != null) &&  (!peppolPassword.getValue().equals("")) ){
					super.password = peppolPassword.getValue();
				}
			}
			
		}
		
	}
	
	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}
	
	public String refresh() {
		updatePeppolDocuments();
		return "";
	}

	public void updatePeppolDocuments() {
		peppolDocuments = new ArrayList<PeppolDocument>();

		Map<String, Object> parameters = new HashMap<String , Object>(); 
		parameters.put("recipientID", user.getUoId());
		
		
		List<OriginalDocument> originalDocuments = null;
		if (! isAdmin){
			originalDocuments = JNDIUtils
				.getCrudManager()
				.findByQuery(
						"from OriginalDocument r where r.transmissionType.description = 'PEPPOL_INCOMING' AND r.recipient LIKE :recipientID order by r.originalDocumentId DESC", parameters);
		}
		else{
			originalDocuments = JNDIUtils
					.getCrudManager()
					.findByQuery(
							"from OriginalDocument r where r.transmissionType.description = 'PEPPOL_INCOMING' order by r.originalDocumentId DESC", parameters);
			
		}
		if (originalDocuments != null) {
			for (OriginalDocument originalDocument : originalDocuments) {
				List<FEFile> receiptDocuments = JNDIUtils
						.getOriginalDocumentManager().fetchOriginalDocument(
								originalDocument.getOriginalDocumentId())
						.getReceiptDocuments();

				List<PeppolValidation> validations = JNDIUtils.getCrudManager()
						.findByQuery(
								"from PeppolValidation v where v.originalDocumentId = "
										+ originalDocument
												.getOriginalDocumentId());

				if (validations == null)
					validations = new ArrayList<PeppolValidation>();
				
				PeppolDocument pd = new PeppolDocument(originalDocument,
						receiptDocuments.get(0), validations);
				peppolDocuments.add(pd);
			}
		}

	}

	/**
	 * Questi metodi sono pensati per realizzare una gestione periodica e automatizzata della fase di fetch
	 * Al momento è lasciata manuale e on demand: l'utente clicca sul pulsante download e viene chiamato il metodo startPoll che scarica una sola volta le fatture attualemnte presenti sul server LIME
	 * 
	 */
	public void start() {
		try {
			pollingEnabled = true;
			startPoll();

		} catch (Exception e) {
			pollingEnabled = false;
			e.printStackTrace();
		}
	}

	public void stop() {
		pollingEnabled = false;
		messageDownloader = null;
	}

	
	
	public void startPoll() {
		messageDownloader = new RuntimeMessageDownloader(serverAddress,username, password, receiverID, dropFolder);
		this.downloadedMessages = 0;
		try {
			this.setCallback(new DownloadProgressCallback());
			getCallback().setStarted(true);
			getCallback().setValue(0);
			List<Long> messageIds = messageDownloader.downloadMessages(getCallback());
//			List<Long> messageIds = messageDownloader.fakeDownloadMessages();
			this.downloadedMessages = messageIds.size();
			for (Long messageId : messageIds) {
				applyValidations(messageId,
						determineDomesticValidationRequired(messageId));
			}
			
			this.setCallback(new DownloadProgressCallback());
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MessageException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (downloadedMessages > 0)
			updatePeppolDocuments();
	}

	public int getDownloadedMessages() {
		return downloadedMessages;
	}

	public boolean isPollingEnabled() {
		return pollingEnabled;
	}

	public void setPollingEnabled(boolean pollingEnabled) {
		this.pollingEnabled = pollingEnabled;
	}

	public DownloadProgressCallback getCallback() {
		return callback;
	}

	public void setCallback(DownloadProgressCallback callback) {
		this.callback = callback;
	}
}
