/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol;

import it.fe.core.par.PeppolValidation;
import it.fe.web.common.JSFUtils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.probatron.Session;
import org.probatron.ValidationReport;

public class ValidationFactory {

	public static Logger logger = Logger.getLogger(ValidationFactory.class
			.getName());

	/**
	 * 
	 * @param schematronFileName
	 *            : the final XSLT for schematron validation
	 * @param UBLFile
	 * @return the validation output file
	 */
	public static byte[] validateSchematronXSLT(String schematronFileName,
			byte[] UBLFile) {
		SchemaFactory factory = SchemaFactory
				.newInstance("http://www.w3.org/2001/XMLSchema");
		try {
			ByteArrayOutputStream os = new ByteArrayOutputStream();

			TransformerFactory tFactory = TransformerFactory.newInstance();

			Transformer transformer = tFactory
					.newTransformer(new javax.xml.transform.stream.StreamSource(
							schematronFileName));

			transformer.transform(new javax.xml.transform.stream.StreamSource(
					new ByteArrayInputStream(UBLFile)),
					new javax.xml.transform.stream.StreamResult(os));

			return os.toByteArray();

		} catch (Exception e) {

			logger.log(Level.WARNING, e.toString());

		}
		return new byte[] {};
	}

	public static byte[] validateSchematronSCH(String schematronFileName, 
			byte[] UBLFile) {
		String fileName = String.valueOf(new Date().getTime());
		File f = null;
		try {
			f = File.createTempFile(fileName,".tmp");
			FileOutputStream fos = new FileOutputStream(f);
			fos.write(UBLFile);
			fos.close();
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	
		Session theSession = new Session();
	
		if (schematronFileName.startsWith("file://"))
			theSession.setSchemaDoc(schematronFileName);
		else 
			theSession.setSchemaDoc("file://"+schematronFileName);

		try {
			// ValidationReport vr = theSession.doValidation(
			// "file:///home/stefano/WorkspaceVari/NuovoWorkspace/Validation/ValidazionePeppol/IstanzeDaPhloc/BII04+minimal+invoice+example+02.xml"
			// );
			
			ValidationReport vr = theSession
					.doValidation("file://"+f.getAbsolutePath());
			
			ByteArrayOutputStream os = new ByteArrayOutputStream();
			vr.streamOut(os);

			
			return os.toByteArray();
		} catch (Exception e) {
			logger.log(Level.WARNING, e.toString());
			return new byte[] {};
		}
		
	}

	public static String determineValidationStatus(byte[] byteArray) {
		String file = new String(byteArray);
		if (file.toLowerCase().contains("flag=\"fatal\""))
			return PeppolValidation.Status.ERROR.toString();
		else if (file.toLowerCase().contains("flag=\"warning\""))
			return PeppolValidation.Status.WARNING.toString();
		else 
			return PeppolValidation.Status.VALID.toString();
	}

	public static byte[] applyXSLTransformation(String styleSheetFilename, byte[] fileBytes){
		TransformerFactory tFactory = TransformerFactory.newInstance();
		try {

			Transformer transformer = tFactory
					.newTransformer(new javax.xml.transform.stream.StreamSource(
							new File(styleSheetFilename)));

			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			ByteArrayInputStream bais = new ByteArrayInputStream(fileBytes);
			transformer.transform(new javax.xml.transform.stream.StreamSource(
					bais),
					new javax.xml.transform.stream.StreamResult(baos));
			return baos.toByteArray();
		} catch (Exception e) {
			logger.log(Level.WARNING, e.toString());
			return new byte[]{};
		} 
	}
	
	public static String validateXml(String schemaFileName, byte[] UBLFile) {
		SchemaFactory factory = SchemaFactory
				.newInstance("http://www.w3.org/2001/XMLSchema");

		// 5. Check the document
		try {
			Schema schema = factory.newSchema(new File(schemaFileName));

			Validator validator = schema.newValidator();

			Source source = new StreamSource(new ByteArrayInputStream(UBLFile));

			validator.validate(source);

			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"invoiceValidationPassed");

			logger.log(Level.FINE, msg);

			return "valid";

		} catch (Exception ex) {
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"invoiceValidationFailed");

			logger.log(Level.WARNING, msg + ex.toString());
		}
		return "error";
	}


}
