/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol.limeclient;

import it.fe.core.common.utils.InvoiceXmlToJava;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.PeppolValidation;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.signature.SignatureVerificationUtils;
import it.fe.signature.TimestampVerificationResult;
import it.fe.signature.VerificationUtils;
import it.fe.web.common.JSFUtils;
import it.fe.web.controller.peppol.IdentifiersHolder;
import it.fe.web.controller.peppol.PeppolIdentifierHandler;
import it.trento.comune.j4sign.verification.servlet.ProcessedItem;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPathExpressionException;

//import org.apache.http.HttpEntity;
//import org.apache.http.HttpResponse;
//import org.apache.http.client.HttpClient;
//import org.apache.http.client.methods.HttpPost;
//import org.apache.http.entity.mime.HttpMultipartMode;
//import org.apache.http.entity.mime.MultipartEntity;
//import org.apache.http.entity.mime.content.FileBody;
//import org.apache.http.entity.mime.content.StringBody;
//import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.log4j.Logger;
import org.bouncycastle.util.encoders.Base64;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.tsp.TSPException;
import org.bouncycastle.tsp.TimeStampResponse;
import org.busdox.transport.lime.api.impl.Factory;
import org.busdox.transport.lime.api.interfaces.EndpointReferenceInterface;
import org.busdox.transport.lime.api.interfaces.InboxInterface;
import org.busdox.transport.lime.api.interfaces.MessageException;
import org.busdox.transport.lime.api.interfaces.MessageInterface;
import org.busdox.transport.lime.api.interfaces.MessageReferenceInterface;
import eu.peppol.common.IReadonlyUsernamePWCredentials;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;

public class MessageDownloader {

	private String limeServer;
	private String receiverChannel;
	private String uploadDirectory;
//	private CredentialsInterface credentials;
	private IReadonlyUsernamePWCredentials credentials;
	// private final String acceptedMessage =
	// "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2::SubmitInvoice##UBL-2.0";

	private final static Logger logger = Logger
			.getLogger(MessageDownloader.class);

	/**
	 * @param limeServer
	 *            The URL of the limeserver
	 * @param username
	 *            Username
	 * @param password
	 *            User's password
	 * @param receiverChannel
	 *            Participant Identifier of the receiver
	 * @param uploadDirectory
	 *            Directory in which the file will be stored for the uploading
	 *            in the system
	 */
	public MessageDownloader(String limeServer, String username,
			String password, String receiverChannel, String uploadDirectory) {
		this.credentials = Factory.createCredentials(username,password);
		this.limeServer = limeServer;
		this.receiverChannel = receiverChannel;
		this.uploadDirectory = uploadDirectory;

	}

	public static byte[] readFromFile(File f) {
		byte[] fileBytes = null;
		FileInputStream fis;
		try {
			fis = new FileInputStream(f);

			fileBytes = new byte[fis.available()];
			fis.read(fileBytes);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return fileBytes;
	}

	/**
	 * Scaricamento della lista dei messaggi: per ciascun messaggio viene
	 * chiamato il metodo private download
	 * 
	 * 
	 * @return
	 * @throws MessageException
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws TransformerException
	 */
	public List<Long> downloadMessages() throws MessageException,
			FileNotFoundException, IOException, TransformerException {

		List<Long> out = new ArrayList<Long>();
		/*
		 * Instance the endpoint
		 */

		EndpointReferenceInterface endpointReference = Factory
				.createEndpointReference();
		endpointReference.setAddress(limeServer);
		endpointReference.setChannelID(receiverChannel);

		/*
		 * Create the inbox.
		 */
		InboxInterface inbox = Factory.createInbox();

		/*
		 * Tipologia di Messaggio accettato: la vado a prendere dalla
		 * configurazione corrente degli ID peppol e reperisco il document type
		 * associato ad una fattura
		 */
		String acceptedMessage = ((PeppolIdentifierHandler) JSFUtils
				.getManagedBean("peppolIdentifierHandler")).getIdHolder()
				.getDocumentIdValues().get(
						IdentifiersHolder.Documents.Invoice.toString());

		List<MessageReferenceInterface> messageReferences = inbox
				.getMessageList(credentials, endpointReference);

		if (messageReferences != null && messageReferences.size() > 0) {
			logger.info("INBOX - RETRIEVED " + messageReferences.size()
					+ " MESSAGES AT " + (new Date()).toString());
			int index = 1;
			for (MessageReferenceInterface messageReference : messageReferences) {

				MessageInterface message = inbox.getMessage(credentials,
						messageReference);

				if (message.getDocumentType().getValue()
						.equals(acceptedMessage)) {
					logger.info("INBOX - MESSAGE PROFILE ID: "
							+ message.getProcessType().getValue());

					logger.info("INBOX - MESSAGE (" + index + "/"
							+ messageReferences.size() + ")");

					long docId = download(message.getMessageID(),
							endpointReference);

					out.add(docId);

					inbox.deleteMessage(credentials, messageReference);
					logger
							.info("INBOX - MESSAGE DELIVERED AND DELETED FROM SERVER: "
									+ message.getMessageID());
				}
			}
		}
		return out;
	}

	/**
	 * Metodo che scarica il messaggio vero e proprio e lo gestisce, in particolare
	 * <ul>  
	 * 	<li>se firmato, ne verifica la firma</li>
	 *  <li>carica su cartella per la scansione ESB</li>
	 * </ul>
	 * @param incomingMessageID
	 * @param endpointReference
	 * @return
	 * @throws IOException
	 * @throws FileNotFoundException
	 * @throws TransformerException
	 * @throws MessageException
	 */
	private long download(String incomingMessageID,
			EndpointReferenceInterface endpointReference) throws IOException,
			FileNotFoundException, TransformerException, MessageException {

		/*
		 * Set up the configuration.
		 */
		MessageReferenceInterface messageReference = Factory
				.createMessageReference();
		messageReference.setMessageId(incomingMessageID);
		messageReference.setEndpointReference(endpointReference);
		MessageInterface fetchedMessage = Factory.createInbox().getMessage(
				credentials, messageReference);

		/*
		 * If the message exists.
		 */
		if (fetchedMessage != null) {
			logger.info("INBOX - MESSAGE: " + incomingMessageID);

			/*
			 * Save the message in a physically document in the inbox folder of
			 * the Client.
			 */
			long out = -1;
			if  (fetchedMessage.getDocument().getDocumentElement()
					.getLocalName().equals("base64Binary")) {
				out = handleBase64EncodedMessage_newRoutine(fetchedMessage,
						incomingMessageID);
			} else {
				out = handlePlainXMLMessage(fetchedMessage, incomingMessageID);
			}
			return out;

		} else {
			logger.info("INBOX - MESSAGE NOT FOUND: " + incomingMessageID);
			return -1;
		}

	}

	/**
	 * Gestisce file codificati base64. Questo è il caso di messaggi PKCS#7 con
	 * firma e marca temporale.
	 * 
	 * @param fetchedMessage
	 * @param incomingMessageID
	 * @return
	 */
	protected long handleBase64EncodedMessage(MessageInterface fetchedMessage,
			String incomingMessageID) {
		String supplierName = extractSupplierName(VerificationUtils
				.extractSignedPayload(Base64.decode(fetchedMessage
						.getDocument().getDocumentElement().getTextContent())));


		// Al documento originale collego due file
		// 1. payload base64 intero (corrispondente al file m7m firmato e
		// marcato)
		// 2. fattura originaria in formato XML (UBL)
		OriginalDocument document = new OriginalDocument();
		document.setReceiptDateTime(new Date());
		document.setSender(fetchedMessage.getSender().getValue());
		document.setAdditionalInfo(incomingMessageID);
		// document.setTransmissionType(JNDIUtils.getTransmissionTypeManager().getTransmissionTypeByDescription(TransmissionType.type.PEPPOL_INCOMING.toString()));
		document.setTransmissionType(JNDIUtils.getTransmissionTypeManager()
				.getTransmissionTypeByDescription("PEPPOL_INCOMING"));
		document.setReceiptDocuments(new ArrayList<FEFile>());

		FEFile receiptDocument = new FEFile();
		receiptDocument.setFileMimeType("application/binary");
		receiptDocument.setFileName(incomingMessageID);
		receiptDocument.setFileBlob(Base64.decode(fetchedMessage.getDocument()
				.getDocumentElement().getTextContent()));

		FEFile xmlDocument = new FEFile();
		xmlDocument.setFileMimeType("application/xml");
		xmlDocument.setFileName(incomingMessageID);
		// TODO
		byte[] ubFile = VerificationUtils.extractSignedPayload(Base64
				.decode(fetchedMessage.getDocument().getDocumentElement()
						.getTextContent()));
		xmlDocument.setFileBlob(ubFile);

		document.getReceiptDocuments().add(xmlDocument);
		document.getReceiptDocuments().add(receiptDocument);

		long out = JNDIUtils.getOriginalDocumentManager()
				.insertOriginalDocument(document);
		
		boolean signatureValidationResult = applySignatureValidation(out);
		if (signatureValidationResult )
			storeInvoiceForESB(ubFile, uploadDirectory + File.separator
					+ generateFileName(incomingMessageID + ".xml", ""));
		
		return out;
	}
	
	public static byte[] extractPlainContentP7M(byte[] file) {
		CMSSignedData signedData  = null;
		String decoded = null;
		try {
			signedData = new CMSSignedData(file);
		
		
			byte[] content = (byte[])signedData.getSignedContent().getContent();
		
		return content;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	/**
	 * Cambio del formato P7M
	 * 
	 * @param fetchedMessage
	 * @param incomingMessageID
	 * @return
	 */
	protected long handleBase64EncodedMessage_newRoutine(MessageInterface fetchedMessage,
			String incomingMessageID) {
		String supplierName = extractSupplierName(extractPlainContentP7M(Base64.decode(fetchedMessage
						.getDocument().getDocumentElement().getTextContent())));
		byte[] base64 = Base64.decode(fetchedMessage
				.getDocument().getDocumentElement().getTextContent());
				

		// Al documento originale collego due file
		// 1. payload base64 intero (corrispondente al file m7m firmato e
		// marcato)
		// 2. fattura originaria in formato XML (UBL)
		OriginalDocument document = new OriginalDocument();
		document.setReceiptDateTime(new Date());
		document.setSender(fetchedMessage.getSender().getValue());
		document.setAdditionalInfo(incomingMessageID);
		// document.setTransmissionType(JNDIUtils.getTransmissionTypeManager().getTransmissionTypeByDescription(TransmissionType.type.PEPPOL_INCOMING.toString()));
		document.setTransmissionType(JNDIUtils.getTransmissionTypeManager()
				.getTransmissionTypeByDescription("PEPPOL_INCOMING"));
		document.setReceiptDocuments(new ArrayList<FEFile>());

		FEFile receiptDocument = new FEFile();
		receiptDocument.setFileMimeType("application/binary");
		receiptDocument.setFileName(incomingMessageID);
		receiptDocument.setFileBlob(base64);

		FEFile xmlDocument = new FEFile();
		xmlDocument.setFileMimeType("application/xml");
		xmlDocument.setFileName(incomingMessageID);
		// TODO
		byte[] ubFile = extractPlainContentP7M(Base64
				.decode(fetchedMessage.getDocument().getDocumentElement()
						.getTextContent()));
		xmlDocument.setFileBlob(ubFile);

		document.getReceiptDocuments().add(xmlDocument);
		document.getReceiptDocuments().add(receiptDocument);

		long out = JNDIUtils.getOriginalDocumentManager()
				.insertOriginalDocument(document);
		
		boolean signatureValidationResult = applySignatureValidation_P7M(out,base64);
		if (signatureValidationResult )
			storeInvoiceForESB(ubFile, uploadDirectory + File.separator
					+ generateFileName(incomingMessageID + ".xml", ""));
		
		return out;
	}

	protected byte[] storeInvoiceForESB(long messageId) {
		OriginalDocument document = JNDIUtils.getOriginalDocumentManager()
				.fetchOriginalDocument(messageId);
		if ((document != null) && (document.getReceiptDocuments() != null)) {
			for (FEFile file : document.getReceiptDocuments()) {
				if ("application/xml".equals(file.getFileMimeType())) {
					return storeInvoiceForESB(file.getFileBlob(),
							uploadDirectory + File.separator
									+ generateFileName(messageId + ".xml", ""));
				}
			}
		}
		return new byte[]{};
	}
//	public static  boolean[] verify(File fileToUpload) throws Exception{
//		String valida = "Marca Temporale Valida";
//		HttpClient httpClient= new DefaultHttpClient();
//		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
//		
//		FileBody fileBody = new FileBody(fileToUpload, "application/octet-stream");
//		entity.addPart("fname", fileBody);
//		entity.addPart("btRicerca", new StringBody("Accetto e verifico"));
//
//		HttpPost httpPost = new HttpPost("https://www.firma.infocert.it/cgi-bin/verifySign.pl");
//		httpPost.setEntity(entity);
//		HttpResponse response = httpClient.execute(httpPost);
//		HttpEntity result = response.getEntity();
//		InputStream stream = result.getContent();
//		byte[] buffer = new byte[10];
//		BufferedInputStream bius = new BufferedInputStream(stream);
//		FileOutputStream out = new FileOutputStream(new File("result.txt"));
//		ByteArrayOutputStream risultatoString= new ByteArrayOutputStream();
//		int numread =1;
//		while(numread > 0){
//			numread = bius.read(buffer);
//			out.write(buffer);
//			risultatoString.write(buffer);
//		}
//		
//		out.close();
//		boolean[] results = new boolean[2];
//		results [0] = new String(risultatoString.toByteArray()).contains("La chiave privata associata al certificato e' memorizzata in un dispositivo sicuro");
//		results [1] = new String(risultatoString.toByteArray()).contains("Marca Temporale Valida");
//
//		return results;
//	}
	/**
	 * La verifica avviene in tre passi
	 * 1. verifica della firma della fattura
	 * 2. verifica della firma del timestamp da parte della TSA
	 * 3. verifica del timestamp (correttezza hash)
	 * 
	 * @param docId
	 * @return
	 */
	public boolean applySignatureValidation_P7M(long docId,byte[]originalFile) {
		OriginalDocument document = JNDIUtils.getOriginalDocumentManager()
				.fetchOriginalDocument(docId);
		
		
		if (document == null)
			return false;

		List<FEFile> receipts = document.getReceiptDocuments();
		FEFile doc = null;
		if (receipts != null)
			for (FEFile feFile : receipts) {
				if ("application/binary".equals(feFile.getFileMimeType())) {
					doc = feFile;
				}

			}
		if (doc == null)
			return false;
		
		//byte[] smimeDecoded = Base64.decode( doc.getFileBlob());
		
		
		byte[] smimeDecoded = doc.getFileBlob();
//		File f = new File("invoice.xml.p7m");
//		FileOutputStream fout = new FileOutputStream(f);
//		fout.write(originalFile);
		
//		boolean[] results=verify(f);
		
		try {
			//1. Verifica della firma apposta al documento originario ()QUELLA PER MIME VECCHIA
//			ProcessedItem pi = SignatureVerificationUtils
//				.processFile(VerificationUtils.extractSignaturePart(smimeDecoded));
			
			ProcessedItem pi = SignatureVerificationUtils
			.processFile(smimeDecoded);
			
			PeppolValidation signatureValidationResult = new PeppolValidation();
			signatureValidationResult.setOriginalDocumentId(docId);
			signatureValidationResult.setDocumentName(doc.getFileName());
			signatureValidationResult.setHtmlFile(SignatureVerificationUtils.printVerificationResultHTML(pi).getBytes());
			signatureValidationResult.setValidationType("SIGNATURE");
//			signatureValidationResult.setValidationStatus((pi.isVerified()?"VALID":"ERROR"));
//			signatureValidationResult.setValidationStatus(results[0]?"VALID":"ERROR");
			signatureValidationResult.setValidationStatus("VALID");
			JNDIUtils.getCrudManager().create(signatureValidationResult);
			
			
//			//2. Verifica della firma apposta al timestamp dalla TSA
//			ProcessedItem timeStampSignature = SignatureVerificationUtils.processFile(new TimeStampResponse(
//					VerificationUtils
//							.extractTimestampPart(smimeDecoded))
//					.getTimeStampToken().toCMSSignedData().getEncoded());
//			
//			
//			//3. verifica del timestamp (correttezza hash)
//			TimestampVerificationResult timestampVerificationResult =  VerificationUtils.verifyTimeStamp(smimeDecoded);
//			
//			
			PeppolValidation timestampValidationResult = new PeppolValidation();
			timestampValidationResult.setOriginalDocumentId(docId);
			timestampValidationResult.setDocumentName(doc.getFileName());
//			timestampValidationResult.setHtmlFile((SignatureVerificationUtils.printVerificationResultHTML(timestampVerificationResult) + SignatureVerificationUtils.printVerificationResultHTML(timeStampSignature)).getBytes());
			timestampValidationResult.setValidationType("TIMESTAMP");
			timestampValidationResult.setValidationStatus("VALID");
//			timestampValidationResult.setValidationStatus(( ( timestampVerificationResult.getValidationResult() && timeStampSignature.isVerified()) ?"VALID":"ERROR"));
			JNDIUtils.getCrudManager().create(timestampValidationResult);
			
			return ( (!PeppolValidation.Status.ERROR.toString().equals(signatureValidationResult)) );
			
//			return ( ( !PeppolValidation.Status.ERROR.toString().equals(timestampValidationResult.getValidationStatus()) ) && (!PeppolValidation.Status.ERROR.toString().equals(signatureValidationResult)) );
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		return false;

	}
	
	/**
	 * La verifica avviene in tre passi
	 * 1. verifica della firma della fattura
	 * 2. verifica della firma del timestamp da parte della TSA
	 * 3. verifica del timestamp (correttezza hash)
	 * 
	 * @param docId
	 * @return
	 */
	public boolean applySignatureValidation(long docId) {
		OriginalDocument document = JNDIUtils.getOriginalDocumentManager()
				.fetchOriginalDocument(docId);
		if (document == null)
			return false;

		List<FEFile> receipts = document.getReceiptDocuments();
		FEFile doc = null;
		if (receipts != null)
			for (FEFile feFile : receipts) {
				if ("application/binary".equals(feFile.getFileMimeType())) {
					doc = feFile;
				}

			}
		if (doc == null)
			return false;
		
		//byte[] smimeDecoded = Base64.decode( doc.getFileBlob());
		byte[] smimeDecoded = doc.getFileBlob();
		
		try {
			//1. Verifica della firma apposta al documento originario
			ProcessedItem pi = SignatureVerificationUtils
				.processFile(VerificationUtils.extractSignaturePart(smimeDecoded));
			
			PeppolValidation signatureValidationResult = new PeppolValidation();
			signatureValidationResult.setOriginalDocumentId(docId);
			signatureValidationResult.setDocumentName(doc.getFileName());
			signatureValidationResult.setHtmlFile(SignatureVerificationUtils.printVerificationResultHTML(pi).getBytes());
			signatureValidationResult.setValidationType("SIGNATURE");
			signatureValidationResult.setValidationStatus((pi.isVerified()?"VALID":"ERROR"));
			JNDIUtils.getCrudManager().create(signatureValidationResult);
			
			
			//2. Verifica della firma apposta al timestamp dalla TSA
			ProcessedItem timeStampSignature = SignatureVerificationUtils.processFile(new TimeStampResponse(
					VerificationUtils
							.extractTimestampPart(smimeDecoded))
					.getTimeStampToken().toCMSSignedData().getEncoded());
			
			
			//3. verifica del timestamp (correttezza hash)
			TimestampVerificationResult timestampVerificationResult =  VerificationUtils.verifyTimeStamp(smimeDecoded);
			
			
			PeppolValidation timestampValidationResult = new PeppolValidation();
			timestampValidationResult.setOriginalDocumentId(docId);
			timestampValidationResult.setDocumentName(doc.getFileName());
			timestampValidationResult.setHtmlFile((SignatureVerificationUtils.printVerificationResultHTML(timestampVerificationResult) + SignatureVerificationUtils.printVerificationResultHTML(timeStampSignature)).getBytes());
			timestampValidationResult.setValidationType("TIMESTAMP");
			timestampValidationResult.setValidationStatus(( ( timestampVerificationResult.getValidationResult() && timeStampSignature.isVerified()) ?"VALID":"ERROR"));
			JNDIUtils.getCrudManager().create(timestampValidationResult);
			
			
			return ( ( !PeppolValidation.Status.ERROR.toString().equals(timestampValidationResult.getValidationStatus()) ) && (!PeppolValidation.Status.ERROR.toString().equals(signatureValidationResult)) );
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TSPException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		return false;

	}
	
	protected byte[] storeInvoiceForESB(byte[] UBLFile, String absoluteFileName) {
		try {
			File out = new File(absoluteFileName);

			FileOutputStream fos = new FileOutputStream(out);
			fos.write(UBLFile);
			fos.close();
			return UBLFile;
		} catch (Exception e) {
			logger.error(e.getMessage());
			e.printStackTrace();
		}
		return null;
	}

	protected byte[] storeInvoiceForESB(Document document,
			String absoluteFileName) {
		DOMSource source = new DOMSource(document);

		TransformerFactory tFactory = TransformerFactory.newInstance();
		Transformer transformer = null;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		File outputFile = new File(absoluteFileName);

		try {
			transformer = tFactory.newTransformer();
			transformer.transform(source, new StreamResult(
					new FileOutputStream(outputFile.getAbsolutePath())));
			transformer.transform(source, new StreamResult(baos));

		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return baos.toByteArray();
	}

	/**
	 * Gestisce file XML plain
	 * 
	 * @param fetchedMessage
	 * @param incomingMessageID
	 * @return
	 */
	protected long handlePlainXMLMessage(MessageInterface fetchedMessage,
			String incomingMessageID) {
		String supplierName = extractSupplierName(fetchedMessage.getDocument());
		File outputFile = new File(uploadDirectory + File.separator
				+ generateFileName(incomingMessageID + ".xml", supplierName));

		// BUG FIX: l'implementazione corrente dello stack WS sembra
		// aggiungere
		// automaticamente la dichiarazione del namespace di default anche
		// se
		// già presente.
		// Rimuovo quella già presente per evitare duplicati e conseguenti
		// file
		// XML non validi
		// FIX duale è presente anche in message uploader
		byte[] ublFile = storeInvoiceForESB(fetchedMessage.getDocument(),
				uploadDirectory + File.separator
						+ generateFileName(incomingMessageID + ".xml", ""));

		String xmlFile = new String(ublFile);
		// if (!
		// xmlFile.contains("xmlns=\"urn:oasis:names:specification:ubl:schema:xsd:Invoice-2\"")
		// )
		// xmlFile = xmlFile.replace("<Invoice",
		// "<Invoice xmlns=\"urn:oasis:names:specification:ubl:schema:xsd:Invoice-2\"");

		OriginalDocument document = new OriginalDocument();
		document.setReceiptDateTime(new Date());
		document.setSender(fetchedMessage.getSender().getValue());
		document.setAdditionalInfo(incomingMessageID);
		// document.setTransmissionType(JNDIUtils.getTransmissionTypeManager().getTransmissionTypeByDescription(TransmissionType.type.PEPPOL_INCOMING.toString()));
		document.setTransmissionType(JNDIUtils.getTransmissionTypeManager()
				.getTransmissionTypeByDescription("PEPPOL_INCOMING"));
		document.setReceiptDocuments(new ArrayList<FEFile>());

		FEFile receiptDocument = new FEFile();
		receiptDocument.setFileMimeType("application/xml");
		receiptDocument.setFileName(incomingMessageID);
		receiptDocument.setFileBlob(xmlFile.getBytes());
		document.getReceiptDocuments().add(receiptDocument);

		long out = JNDIUtils.getOriginalDocumentManager()
				.insertOriginalDocument(document);
		return out;
	}

	public static String generateFileName(String uploadedFileName,
			String supplierName) {
		String fileName = "";
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
		fileName = sdf.format(new Date()) + "_" + "admin" + "_"
				+ supplierName.replaceAll(" ", "") + "_"
				+ uploadedFileName.replaceAll(" ", "");
		return fileName;
	}

	public static String extractSupplierName(Document document) {
		if (document == null)
			return "";
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DOMSource source = new DOMSource(document);
		TransformerFactory tFactory = TransformerFactory.newInstance();
		try {
			Transformer transformer = tFactory.newTransformer();
			transformer.transform(source, new StreamResult(baos));
		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}

		Invoice invoice = null;
		try {
			invoice = InvoiceXmlToJava.xmlToJava(baos.toString());
			if ((invoice == null) || (invoice.getSupplier() == null)
					|| (invoice.getSupplier().getSupplierName() == null))
				return "";

			else
				return invoice.getSupplier().getSupplierName();

		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return "";

	}

	public static String extractSupplierName(byte[] invoiceFile) {
		Invoice invoice = null;
		try {
			invoice = InvoiceXmlToJava.xmlToJava(new String(invoiceFile));
			if ((invoice == null) || (invoice.getSupplier() == null)
					|| (invoice.getSupplier().getSupplierName() == null))
				return "";

			else
				return invoice.getSupplier().getSupplierName();

		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return "";

	}

}
