/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol.limeclient;

import it.fe.core.common.exception.InvalidInvoiceException;
import it.fe.core.common.utils.InvoiceXmlToJava;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.PeppolValidation;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.TransmissionType;
import it.fe.signature.SignatureVerificationUtils;
import it.fe.signature.TimestampVerificationResult;
import it.fe.signature.VerificationUtils;
import it.fe.web.common.JSFUtils;
import it.fe.web.controller.peppol.DownloadProgressCallback;
import it.fe.web.controller.peppol.IdentifiersHolder;
import it.fe.web.controller.peppol.PeppolIdentifierHandler;
import it.trento.comune.j4sign.verification.servlet.ProcessedItem;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

//import org.apache.http.HttpEntity;
//import org.apache.http.HttpResponse;
//import org.apache.http.client.HttpClient;
//import org.apache.http.client.methods.HttpPost;
//import org.apache.http.entity.mime.HttpMultipartMode;
//import org.apache.http.entity.mime.MultipartEntity;
//import org.apache.http.entity.mime.content.FileBody;
//import org.apache.http.entity.mime.content.StringBody;
//import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.commons.exec.CommandLine;
import org.apache.commons.exec.DefaultExecutor;
import org.apache.log4j.Logger;
import org.bouncycastle.util.encoders.Base64;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.tsp.TSPException;
import org.bouncycastle.tsp.TimeStampResponse;
import org.busdox.transport.lime.api.impl.Credentials;
import org.busdox.transport.lime.api.impl.Factory;
import org.busdox.transport.lime.api.interfaces.EndpointReferenceInterface;
import org.busdox.transport.lime.api.interfaces.InboxInterface;
import org.busdox.transport.lime.api.interfaces.MessageException;
import org.busdox.transport.lime.api.interfaces.MessageInterface;
import org.busdox.transport.lime.api.interfaces.MessageReferenceInterface;
import eu.peppol.common.IReadonlyUsernamePWCredentials;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Classe client di interazione con il protocollo LIME. Utilizza una libreria
 * esterna LIME-CLIENT a riga di comando al fine di gestire in maniera separata
 * e disaccoppiata le librerie WS del container Liferay (JBossWS) da quelle
 * necessarie per LIME (stack Metro v.2.2.1), già contenute nella libreria Java
 * a riga di comando.
 * 
 * @author Stefano Monti
 * 
 */
public class RuntimeMessageDownloader {

	private String limeServer;
	private String receiverChannel;
	private String uploadDirectory;
	private IReadonlyUsernamePWCredentials credentials;
	private String messageInvoice;

	private final static Logger logger = Logger
			.getLogger(RuntimeMessageDownloader.class);
	private static final String jbpmProcessDefinitionKey = "Generic";

	/**
	 * @param limeServer
	 *            The URL of the limeserver
	 * @param username
	 *            Username
	 * @param password
	 *            User's password
	 * @param receiverChannel
	 *            Participant Identifier of the receiver
	 * @param uploadDirectory
	 *            Directory in which the file will be stored for the uploading
	 *            in the system
	 */
	public RuntimeMessageDownloader(String limeServer, String username, String password, String receiverChannel, String uploadDirectory) {
		this.credentials = Factory.createCredentials(username, password);
		this.limeServer = limeServer;
		this.receiverChannel = receiverChannel;
		this.uploadDirectory = uploadDirectory;

	}

	public static byte[] readFromFile(File f) {
		byte[] fileBytes = null;
		FileInputStream fis;
		try {
			fis = new FileInputStream(f);

			fileBytes = new byte[fis.available()];
			fis.read(fileBytes);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return fileBytes;
	}

	/**
	 * Scaricamento della lista dei messaggi: per ciascun messaggio viene
	 * chiamato il metodo private download
	 * 
	 * 
	 * @return
	 * @throws MessageException
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws TransformerException
	 */
	public List<Long> downloadMessages(DownloadProgressCallback callback) throws MessageException,
			FileNotFoundException, IOException, TransformerException {

		
		File folder = new File(uploadDirectory + "/"
				+ receiverChannel.replaceAll(":", "_"));
		
		if (!folder.exists()){
			logger.warn("Folder does not exist: "+uploadDirectory + "/"
		+ receiverChannel.replaceAll(":", "_"));
			boolean result = folder.mkdirs();
			logger.warn("Folder "+uploadDirectory + "/"
					+ receiverChannel.replaceAll(":", "_") + "created "+result);
		}

		
		callback.setValue(10);
		callback.setProgressMessage("Reperimento documenti");
		List<Long> out = new ArrayList<Long>();
		String downloadFilesCommand = "java -jar /tmp/fe/transport-lime-client-console-1.0-jar-with-dependencies.jar "
				+ " -d '"
				+ uploadDirectory
				+ "/"
				+ receiverChannel.replaceAll(":", "_")
				+ "'"
				+ " -f filename"
				+ " -h '"
				+ limeServer
				+ "'"
				+ " -o RECEIVE "
				+ " -p '"
				+ credentials.getPassword()
				+ "' -s "
				+ receiverChannel
				+ " -r "
				+ receiverChannel
				+ " -t INVOICE"
				+ " -u "
				+ credentials.getUsername();

		logger.info("Calling PEPPOL LIME Client: " + downloadFilesCommand);

		CommandLine cmdLine = CommandLine.parse(downloadFilesCommand);
		DefaultExecutor executor = new DefaultExecutor();
		int exitValue = executor.execute(cmdLine);
		
		File[] listOfFiles = folder.listFiles();
		callback.setValue(30);
		
		if (listOfFiles != null){	
			
			callback.setProgressMessage("Processamento documenti");
			callback.setTotalMessages(listOfFiles.length);
			
			for (int i = 0; i < listOfFiles.length; i++) {
				
				
				callback.setCurrentMessage(i+1);
				
				if (listOfFiles[i].isFile()) {
					DocumentBuilderFactory dbFactory = DocumentBuilderFactory
							.newInstance();
					DocumentBuilder dBuilder = null;
					try {
						dBuilder = dbFactory.newDocumentBuilder();
						Document doc = dBuilder.parse(listOfFiles[i]);
						callback.setProgressMessage("Processamento documento "+listOfFiles[i].getName());
						
						handlePlainXMLMessage(doc, listOfFiles[i].getName()
								.replaceAll("uuid:", "").replaceAll(".xml", ""));
						
						
					} catch (ParserConfigurationException e) {
						logger.error("", e);
					} catch (SAXException e) {
						logger.error("", e);
					}
	
					logger.info("File downloaded:"
							+ listOfFiles[i].getName());
				}
				callback.setValue(callback.getValue()+new Double(70/listOfFiles.length).intValue());
			}
			
			
			//Ciclo cancellazione messaggi scaricati
			for (int i = 0; i < listOfFiles.length; i++) {
				if (listOfFiles[i].isFile()) {
					
					List<OriginalDocument> documents = JNDIUtils.getOriginalDocumentManager().findOriginalDocumentByTypeAndAddditionalData(TransmissionType.type.PEPPOL_INCOMING.toString(), listOfFiles[i].getName()
							.replaceAll("uuid:", "").replaceAll(".xml", ""));
					if ((documents != null) && (documents.size() > 0) ){
						String filename = listOfFiles[i].getName();
						listOfFiles[i].delete();
						
						String deleteFilesCommand = "java -jar /tmp/fe/transport-lime-client-console-1.0-jar-with-dependencies.jar "
								+ " -i '"
								+ filename.replaceAll(".xml", "")
								+ "'"
								+ " -d '"
								+ uploadDirectory
								+ "/"
								+ receiverChannel.replaceAll(":", "_")
								+ "'"
								+ " -f filename"
								+ " -h '"
								+ limeServer
								+ "'"
								+ " -o DELETE "
								+ " -p '"
								+ credentials.getPassword()
								+ "' -s "
								+ receiverChannel
								+ " -r "
								+ receiverChannel
								+ " -t INVOICE"
								+ " -u "
								+ credentials.getUsername();
						try{
							CommandLine deleteCmdLine = CommandLine.parse(deleteFilesCommand);
							DefaultExecutor deleteExecutor = new DefaultExecutor();
							int deleteExitValue = deleteExecutor.execute(deleteCmdLine);
						}
						catch (Exception e) {
							logger.error("", e);
						}
					}
					else{
						logger.error("Errore nel processamento dei file: mantenuta copia del file " + listOfFiles[i].getName());
					}
					
				}
			}
			callback.setValue(100);
			callback.setStarted(false);
		}
		
		return out;
	}

	protected byte[] storeInvoiceForESB(long messageId) {
		OriginalDocument document = JNDIUtils.getOriginalDocumentManager()
				.fetchOriginalDocument(messageId);
		if ((document != null) && (document.getReceiptDocuments() != null)) {
			for (FEFile file : document.getReceiptDocuments()) {
				if ("application/xml".equals(file.getFileMimeType())) {
					return storeInvoiceForESB(file.getFileBlob(),
							uploadDirectory + File.separator
									+ generateFileName(messageId + ".xml", ""));
				}
			}
		}
		return new byte[] {};
	}

	/**
	 * La verifica avviene in tre passi 1. verifica della firma della fattura 2.
	 * verifica della firma del timestamp da parte della TSA 3. verifica del
	 * timestamp (correttezza hash)
	 * 
	 * @param docId
	 * @return
	 */
	public boolean applySignatureValidation(long docId) {
		OriginalDocument document = JNDIUtils.getOriginalDocumentManager()
				.fetchOriginalDocument(docId);
		if (document == null)
			return false;

		List<FEFile> receipts = document.getReceiptDocuments();
		FEFile doc = null;
		if (receipts != null)
			for (FEFile feFile : receipts) {
				if ("application/binary".equals(feFile.getFileMimeType())) {
					doc = feFile;
				}

			}
		if (doc == null)
			return false;

		// byte[] smimeDecoded = Base64.decode( doc.getFileBlob());
		byte[] smimeDecoded = doc.getFileBlob();

		try {
			// 1. Verifica della firma apposta al documento originario
			ProcessedItem pi = SignatureVerificationUtils
					.processFile(VerificationUtils
							.extractSignaturePart(smimeDecoded));

			PeppolValidation signatureValidationResult = new PeppolValidation();
			signatureValidationResult.setOriginalDocumentId(docId);
			signatureValidationResult.setDocumentName(doc.getFileName());
			signatureValidationResult.setHtmlFile(SignatureVerificationUtils
					.printVerificationResultHTML(pi).getBytes());
			signatureValidationResult.setValidationType("SIGNATURE");
			signatureValidationResult
					.setValidationStatus((pi.isVerified() ? "VALID" : "ERROR"));
			JNDIUtils.getCrudManager().create(signatureValidationResult);

			// 2. Verifica della firma apposta al timestamp dalla TSA
			ProcessedItem timeStampSignature = SignatureVerificationUtils
					.processFile(new TimeStampResponse(VerificationUtils
							.extractTimestampPart(smimeDecoded))
							.getTimeStampToken().toCMSSignedData().getEncoded());

			// 3. verifica del timestamp (correttezza hash)
			TimestampVerificationResult timestampVerificationResult = VerificationUtils
					.verifyTimeStamp(smimeDecoded);

			PeppolValidation timestampValidationResult = new PeppolValidation();
			timestampValidationResult.setOriginalDocumentId(docId);
			timestampValidationResult.setDocumentName(doc.getFileName());
			timestampValidationResult
					.setHtmlFile((SignatureVerificationUtils
							.printVerificationResultHTML(timestampVerificationResult) + SignatureVerificationUtils
							.printVerificationResultHTML(timeStampSignature))
							.getBytes());
			timestampValidationResult.setValidationType("TIMESTAMP");
			timestampValidationResult
					.setValidationStatus(((timestampVerificationResult
							.getValidationResult() && timeStampSignature
							.isVerified()) ? "VALID" : "ERROR"));
			JNDIUtils.getCrudManager().create(timestampValidationResult);

			return ((!PeppolValidation.Status.ERROR.toString().equals(
					timestampValidationResult.getValidationStatus())) && (!PeppolValidation.Status.ERROR
					.toString().equals(signatureValidationResult)));

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TSPException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return false;

	}

	protected byte[] storeInvoiceForESB(byte[] UBLFile, String absoluteFileName) {
		try {
			File out = new File(absoluteFileName);

			FileOutputStream fos = new FileOutputStream(out);
			fos.write(UBLFile);
			fos.close();
			return UBLFile;
		} catch (Exception e) {
			logger.error(e.getMessage());
			e.printStackTrace();
		}
		return null;
	}

	protected byte[] storeInvoiceForESB(Document document,
			String absoluteFileName) {
		DOMSource source = new DOMSource(document);

		TransformerFactory tFactory = TransformerFactory.newInstance();
		Transformer transformer = null;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		File outputFile = new File(absoluteFileName);

		try {
			transformer = tFactory.newTransformer();
			transformer.transform(source, new StreamResult(
					new FileOutputStream(outputFile.getAbsolutePath())));
			transformer.transform(source, new StreamResult(baos));

		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return baos.toByteArray();
	}

	/**
	 * Gestisce file XML plain
	 * 
	 * @param fetchedMessage
	 * @param incomingMessageID
	 * @return
	 */
	protected long handlePlainXMLMessage(Document invoiceDocument,
			String incomingMessageID) {
		// String supplierName = extractSupplierNameXPath(invoiceDocument);
		String sender = extractSenderIDXPath(invoiceDocument);

		byte[] ublFile = storeInvoiceForESB(
				invoiceDocument,
				uploadDirectory + File.separator
						+ generateFileName(incomingMessageID + ".xml", ""));
		File toBeDeleted = new File(uploadDirectory + File.separator
		+ generateFileName(incomingMessageID + ".xml", ""));
		toBeDeleted.delete();

		String xmlFile = new String(ublFile);

		// Salvataggio documento originario
		OriginalDocument document = new OriginalDocument();
		document.setReceiptDateTime(new Date());
		document.setSender(sender);
		document.setAdditionalInfo(incomingMessageID);
		document.setTransmissionType(JNDIUtils.getTransmissionTypeManager()
				.getTransmissionTypeByDescription("PEPPOL_INCOMING"));
		document.setReceiptDocuments(new ArrayList<FEFile>());
		document.setRecipient(receiverChannel	);

		FEFile receiptDocument = new FEFile();
		receiptDocument.setFileMimeType("application/xml");
		receiptDocument.setFileName(incomingMessageID);
		receiptDocument.setFileBlob(xmlFile.getBytes());
		document.getReceiptDocuments().add(receiptDocument);

		long out = JNDIUtils.getOriginalDocumentManager()
				.insertOriginalDocument(document);
		document.setOriginalDocumentId(out);
		

		// Processamento della fattura e salvataggio su tabella Invoice
		
		try{
			readInvoice(document, invoiceDocument);
		}
		catch (Exception e) {
			document.setReceiveStatus(OriginalDocument.receiveStatusType.ERRORE.toString());
			document.setExceptionMessage(e.getLocalizedMessage());
		}

		return out;
	}

	private String extractSenderIDXPath(Document invoiceDocument) {
		String supplierExpression = "/Invoice/AccountingSupplierParty/Party/PartyName/Name";
		XPath xPath = XPathFactory.newInstance().newXPath();
		try {
			String supplierName = xPath.compile(supplierExpression).evaluate(
					invoiceDocument);
			return supplierName;
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return "";

	}

	public static String generateFileName(String uploadedFileName,
			String supplierName) {
		String fileName = "";
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
		fileName = sdf.format(new Date()) + "_" + "admin" + "_"
				+ supplierName.replaceAll(" ", "") + "_"
				+ uploadedFileName.replaceAll(" ", "");
		return fileName;
	}

	public void readInvoice(OriginalDocument holder, Document doc) throws Exception{
		logger.info("Processing invoice document from" + holder.getSender());
		String exceptionJbpm = "";
		String exceptionInvoice = "";
		String messageInvoice = "";
		int countExceptionInvoice = 0;
		
		boolean workflowStarted = false;

		Invoice invoice = new Invoice();

		invoice = InvoiceXmlToJava.xmlToJava(doc);
		logger.info("Invoice successfully parsed");			
		
		// setta nella fattura la data di ricezione del documento originale
		invoice.setReceiptDate(holder.getReceiptDateTime());

		// setta il documento originale
		// per ora gestisce solo 1 documento originale (caso Hera)
		List<OriginalDocument> originalDocuments = new ArrayList<OriginalDocument>();
		originalDocuments.add(JNDIUtils.getOriginalDocumentManager().fetchOriginalDocument(holder.getOriginalDocumentId()));
		invoice.setOriginalDocuments(originalDocuments);

		// setta il file xml
		// nome file: p.iva + invoiceNumber
		invoice.setXmlFile(new FEFile(generateInvoiceXML(doc), invoice
				.getSupplier().getVatId()
				+ "-"
				+ invoice.getInvoiceNumber()
				+ ".xml"));
		logger.info("Populating invoice structure");
		int checkInvoice = JNDIUtils.getInvoiceManager().checkInvoice(
				invoice);
		if (checkInvoice < 0) {
			JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
					holder.getOriginalDocumentId(),
					OriginalDocument.receiveStatusType.ERRORE.toString(),
					"Verifica di consistenza dei dati fallita");
			logger.error("Verifica di consistenza dei dati fallita");
			
			return;
		} else {

			// conrolla che la fattura non sia già  stata inserita in FE
			if (JNDIUtils.getInvoiceManager().findInvoiceByNumberAndVatId(
					invoice.getInvoiceNumber(),
					invoice.getSupplier().getVatId()) == null) {

				Long invoiceId = JNDIUtils.getInvoiceManager().insertInvoice(
						invoice);
				logger.info("Salvata fattura num. " + invoiceId + " nel db");

//					// Chiamata wf
//					workflowStarted= startWorkflow(JNDIUtils.getInvoiceManager().getInvoiceById(
//							invoiceId));

			} else {
				// controlla che la fattura già inserita abbia un wf
				// esistente associato
				invoice = JNDIUtils.getInvoiceManager()
						.findInvoiceByNumberAndVatId(
								invoice.getInvoiceNumber(),
								invoice.getSupplier().getVatId());
//					if ((invoice.getProcessInstanceId() == null)
//							|| (invoice.getProcessInstanceId().equals(""))) {
//						workflowStarted= startWorkflow(invoice);
//					} else {
					countExceptionInvoice++;
					messageInvoice = "FATTURA NUM. "
							+ invoice.getInvoiceNumber() + " di "
							+ invoice.getSupplier().getSupplierName();
					logger.error(messageInvoice + " GIA' INSERITA IN FE DB");

					exceptionInvoice = exceptionInvoice + "||"
							+ messageInvoice;
//					}
			}

			if ((countExceptionInvoice > 0) || (!exceptionJbpm.equals(""))) {
				String exceptionMessage = "";
				if (!exceptionInvoice.equals("")) {
					exceptionMessage = "Trovate " + countExceptionInvoice
							+ " fatture già  prensenti nel db FE:"
							+ exceptionInvoice;
				}

				if (!exceptionJbpm.equals("")) {
					exceptionMessage = exceptionMessage + exceptionJbpm;
				}

				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						holder.getOriginalDocumentId(),
						OriginalDocument.receiveStatusType.ERRORE
								.toString(), exceptionMessage);
			} else {
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						holder.getOriginalDocumentId(),
						OriginalDocument.receiveStatusType.OK.toString(),
						null);
			}

		}

	}

	private byte[] generateInvoiceXML(Document doc) {
		try {
			DOMSource domSource = new DOMSource(doc);
			StringWriter writer = new StringWriter();
			StreamResult result = new StreamResult(writer);
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer transformer = tf.newTransformer();
			transformer.transform(domSource, result);
			return writer.toString().getBytes("UTF-8");
		} catch (TransformerException ex) {
			ex.printStackTrace();
			
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public boolean startWorkflow(Invoice invoice) {
		String processInstanceKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		if(JNDIUtils.getProcessServiceFacade().findProcessInstanceById(jbpmProcessDefinitionKey, processInstanceKey) == null){
			
			Map<String,Object> variables = new HashMap<String,Object>();
			variables.put("invoiceId", invoice.getInvoiceId());
			variables.put("invoiceNumber", invoice.getInvoiceNumber());
			variables.put("supplierName", invoice.getSupplier().getSupplierName());

			String processInstanceId = JNDIUtils.getProcessServiceFacade().start(jbpmProcessDefinitionKey, variables, processInstanceKey);
			if(processInstanceId != null){
				logger.info("Start workflow with Process Instance Key " + processInstanceId);
				
				//Aggiorna il Process Instance Id nella fattura
				invoice = JNDIUtils.getInvoiceManager().updateProcessInstanceId(invoice.getInvoiceId(), processInstanceId);
				return true;
			}
			else {
				logger.error("Unable to start workflow");
				return false;
			}
		}
		else{
			logger.error("Process Instance " + processInstanceKey + " alredy exist");
			return false;
		}
	}

}
