/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.profile;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.common.utils.PasswordUtils;
import it.fe.core.par.user.User;
import it.fe.core.par.user.UserPassword;
import it.fe.web.common.SessionUtils;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.apache.log4j.Logger;

import com.liferay.portal.PortalException;
import com.liferay.portal.PwdEncryptorException;
import com.liferay.portal.SystemException;
import com.liferay.portal.util.PortalUtil;

public class ProfileHandler implements Serializable {

	private static final long serialVersionUID = 1L;
	
	private final static Logger logger = Logger.getLogger(ProfileHandler.class);
	
	private String currentPassword;
	private String newPassword;
	private String confirmNewPassword;
	
	public void changePassword() {
				
		
		try {
		
			if (currentPassword == "" || newPassword == "" || confirmNewPassword == "") {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"E' necessario compilare tutti i campi", "E' necessario compilare tutti i campi"));
				return;
			}
			
			if (!newPassword.equals(confirmNewPassword)) {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Le nuove password non coincidono", "Le nuove password non coincidono"));
				return;
			}
			
			String username = PortalUtil.getUser(SessionUtils.getPortletRequest()).getScreenName();
			
			if (username == null || username == "") {
				return;
			}
			
			// ottengo le password attualmente su db per l'utente
			User user = JNDIUtils.getUserManager().getUserByUsername(username);
			
			if (user != null) {
				
				List<UserPassword> retrievedPasswords = user.getPasswords();
				UserPassword matchingPassword = null;
				String formCurrentPasswordEncrypted = PasswordUtils.encrypt(PasswordUtils.TYPE_SHA, currentPassword, null);
				// ciclo su tutte le password valide presenti sul db e le confronto con l'hash di quella inserita mediante la form
				boolean isActualPasswordValid = false;
				for (UserPassword userPassword : retrievedPasswords) {
					if ( userPassword.getEndDate() == null || userPassword.getEndDate().compareTo(new Date()) < 0 ) {
						if (userPassword.getPassword().equals(formCurrentPasswordEncrypted)) {
							isActualPasswordValid = true;
							matchingPassword = userPassword;
							break;
						}
					}
				}
				
				if (isActualPasswordValid) {
					// se coincidono salvo su DB l'hash della nuova password
					String newPasswordEncrypted;
					
					newPasswordEncrypted = PasswordUtils.encrypt(PasswordUtils.TYPE_SHA, newPassword, null);
					
					if (newPasswordEncrypted != "") {
						UserPassword newUserPassword = new UserPassword();
						newUserPassword.setPassword(newPasswordEncrypted);
						newUserPassword.setStartDate(new Date());
						user.addUserPassword(newUserPassword);
						user.removeUserPassword(matchingPassword);
						user = JNDIUtils.getUserManager().updateUser(user);
						if (user != null) {
							FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, "Password cambiata con successo", "Password cambiata con successo"));
						}
				
					}
				}
				else {
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Password attuale non valida", "Password attuale non valida"));
					return;
				}
			}
			else {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Utente non trovato", "Utente non trovato"));
				return;
			}
			
		} catch (PwdEncryptorException e) {
			logger.error(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getMessage(), e.getMessage()));
			return;
		} catch (PortalException e) {
			logger.error(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getMessage(), e.getMessage()));
			return;
		} catch (SystemException e) {
			logger.error(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getMessage(), e.getMessage()));
			return;
		} 
		
	}

	/**
	 * @return the currentPassword
	 */
	public String getCurrentPassword() {
		return currentPassword;
	}

	/**
	 * @param currentPassword the currentPassword to set
	 */
	public void setCurrentPassword(String currentPassword) {
		this.currentPassword = currentPassword;
	}

	/**
	 * @return the newPassword
	 */
	public String getNewPassword() {
		return newPassword;
	}

	/**
	 * @param newPassword the newPassword to set
	 */
	public void setNewPassword(String newPassword) {
		this.newPassword = newPassword;
	}

	/**
	 * @return the confirmNewPassword
	 */
	public String getConfirmNewPassword() {
		return confirmNewPassword;
	}

	/**
	 * @param confirmNewPassword the confirmNewPassword to set
	 */
	public void setConfirmNewPassword(String confirmNewPassword) {
		this.confirmNewPassword = confirmNewPassword;
	}

}
