/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.reception;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.generic.NestedCriterion;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.web.common.DateUtils;
import it.fe.web.common.ProfileUtils;
import it.fe.web.controller.generic.FilterableHandler;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;

public class ReceptionHandler extends FilterableHandler {

	@Override
	protected Class determineClass() {
		return OriginalDocument.class;
	}

	/*
	 * Proprietà di filtraggio/ordinamento
	 */
	private String transmissionType;
	private String sender;
	private Date dateBegin;
	private Date dateEnd;
	private String receiveStatus;
	private OriginalDocument selectedReception;

	public String getTransmissionType() {
		return transmissionType;
	}

	public void setTransmissionType(String transmissionType) {
		this.transmissionType = transmissionType;
	}

	public String getSender() {
		return sender;
	}

	public void setSender(String sender) {
		this.sender = sender;
	}

	public Date getDateBegin() {
		return dateBegin;
	}

	public void setDateBegin(Date dateBegin) {
		this.dateBegin = dateBegin;
	}

	public Date getDateEnd() {
		return dateEnd;
	}

	public void setDateEnd(Date dateEnd) {
		this.dateEnd = dateEnd;
	}

	public String getReceiveStatus() {
		return receiveStatus;
	}

	public void setReceiveStatus(String receiveStatus) {
		this.receiveStatus = receiveStatus;
	}

	public OriginalDocument getSelectedReception() {
		return selectedReception;
	}

	public void setSelectedReception(OriginalDocument selectedReception) {
		this.selectedReception = selectedReception;
	}

	@Override
	protected DetachedCriteria determineDetachedCriteria() {
			return ProfileUtils.computeDetachedCriteria(determineOrder(), super.order, determineClass(), determineWebRestrictions());
	}

	@Override
	protected Map<String, List<Criterion>> determineWebRestrictions() {
		Map<String, List<Criterion>> out = new HashMap<String, List<Criterion>>();

		if ((this.transmissionType != null)
				&& (!this.transmissionType.equals("")))
			out.put("transmissionType.description", new ArrayList(Arrays
					.asList(new Criterion[] { new NestedCriterion(
							"transmissionType", Restrictions.ilike(
									"description", transmissionType,
									MatchMode.START)) })));

		if ((this.sender != null) && (!this.sender.equals("")))
			out.put("sender", new ArrayList(Arrays
					.asList(new Criterion[] { Restrictions.ilike("sender",
							sender, MatchMode.START) })));

		if ((this.receiveStatus != null) && (!this.receiveStatus.equals("")))
			out.put("receiveStatus",
					new ArrayList(Arrays.asList(new Criterion[] { Restrictions
							.ilike("receiveStatus", receiveStatus,
									MatchMode.START) })));

		if ((this.dateBegin != null) && (this.dateEnd != null))
			out.put("receiptDateTime", Arrays
					.asList(new Criterion[] { Restrictions.and(Restrictions.ge(
							"receiptDateTime", DateUtils.beginOfDay(this.dateBegin)), Restrictions
							.le("receiptDateTime",DateUtils.endOfDay( this.dateEnd))) }));

		else if (this.dateBegin != null)
			out.put("receiptDateTime", Arrays
					.asList(new Criterion[] { Restrictions.ge(
							"receiptDateTime", DateUtils.beginOfDay(this.dateBegin)) }));

		else if (this.dateEnd != null)
			out.put("receiptDateTime", Arrays
					.asList(new Criterion[] { Restrictions.le(
							"receiptDateTime", DateUtils.endOfDay(this.dateEnd)) }));

		return out;
	}

	@Override
	public void resetSearchParameters() {
		this.transmissionType = null;
		this.sender = null;
		this.dateBegin = null;
		this.dateEnd = null;
		this.receiveStatus = null;

	}

	public String openDetail() {
		this.selectedReception = JNDIUtils.getOriginalDocumentManager()
				.fetchOriginalDocument(
						((OriginalDocument) dataModel.getRowData())
								.getOriginalDocumentId());
		return "openDetail";
	}

}
