/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.search;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.generic.NestedCriterion;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Sector;
import it.fe.web.common.DateUtils;
import it.fe.web.common.JSFUtils;
import it.fe.web.common.ProfileUtils;
import it.fe.web.controller.generic.FilterableHandler;
import it.fe.web.controller.invoice.InvoiceInfoHandler;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;
import javax.portlet.ActionResponse;
import javax.portlet.PortletResponse;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;

import org.apache.log4j.Logger;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;

public class SearchController extends FilterableHandler implements Serializable {

	private static final long serialVersionUID = 1L;

	private static final Logger logger = Logger
			.getLogger(SearchController.class);
	
	protected Invoice selectedInvoice;
	public Invoice shownInvoice;
	protected List<Meter> shownInvoiceUtilityInfo;	
	private String invoiceNumber;
	private String contractNumber;
	private String supplier;
	private String vatId;
	private String sector;
	private String cause;
	private String invoiceState;
	private Date startDate;
	private Date endDate;
	private SelectItem[] availableInvoiceStates = null;
	private SelectItem[] availableSector = null;

	public String search() {
		filter();
		PortletResponse portletResponse = JSFUtils.getPortletResponse();
		ActionResponse actionResponse = (ActionResponse) portletResponse;
		try {
			actionResponse.setWindowState(WindowState.MAXIMIZED);
		} catch (WindowStateException e) {
			logger.error(e.getMessage());
		}
		return "searchResults";
	}

	public String openSearchForm() {
		PortletResponse portletResponse = JSFUtils.getPortletResponse();
		ActionResponse actionResponse = (ActionResponse) portletResponse;
		try {
			actionResponse.setWindowState(WindowState.NORMAL);
		} catch (WindowStateException e) {
			logger.error(e.getMessage());
		}
		return "searchForm";
	}

	/**
	 * @return the invoiceNumber
	 */
	public String getInvoiceNumber() {
		return invoiceNumber;
	}

	/**
	 * @param invoiceNumber
	 *            the invoiceNumber to set
	 */
	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}

	/**
	 * @return the contractNumber
	 */
	public String getContractNumber() {
		return contractNumber;
	}

	/**
	 * @param contractNumber
	 *            the contractNumber to set
	 */
	public void setContractNumber(String contractNumber) {
		this.contractNumber = contractNumber;
	}

	/**
	 * @return the supplier
	 */
	public String getSupplier() {
		return supplier;
	}

	/**
	 * @param supplier
	 *            the supplier to set
	 */
	public void setSupplier(String supplier) {
		this.supplier = supplier;
	}

	public void setVatId(String vatId) {
		this.vatId = vatId;
	}

	public String getVatId() {
		return vatId;
	}

	/**
	 * @return the sector
	 */
	public String getSector() {
		return sector;
	}

	/**
	 * @param sector the sector to set
	 */
	public void setSector(String sector) {
		this.sector = sector;
	}

	/**
	 * @return the cause
	 */
	public String getCause() {
		return cause;
	}

	/**
	 * @param cause
	 *            the cause to set
	 */
	public void setCause(String cause) {
		this.cause = cause;
	}

	/**
	 * @return the invoiceState
	 */
	public String getInvoiceState() {
		return invoiceState;
	}

	/**
	 * @param invoiceState
	 *            the invoiceState to set
	 */
	public void setInvoiceState(String invoiceState) {
		this.invoiceState = invoiceState;
	}

	/**
	 * @return the startDate
	 */
	public Date getStartDate() {
		return startDate;
	}

	/**
	 * @param startDate
	 *            the startDate to set
	 */
	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	/**
	 * @return the endDate
	 */
	public Date getEndDate() {
		return endDate;
	}

	/**
	 * @param endDate
	 *            the endDate to set
	 */
	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}
	
	public Invoice getSelectedInvoice() {
		return selectedInvoice;
	}

	public void setSelectedInvoice(Invoice selectedInvoice) {
		this.selectedInvoice = selectedInvoice;
	}
	
	public Invoice getShownInvoice() {		
		return shownInvoice;
	}

	public void setShownInvoice(Invoice shownInvoice) {
		this.shownInvoice = shownInvoice;
	}

	public List<Meter> getShownInvoiceUtilityInfo() {
		return shownInvoiceUtilityInfo;
	}

	public void setShownInvoiceUtilityInfo(List<Meter> shownInvoiceUtilityInfo) {
		this.shownInvoiceUtilityInfo = shownInvoiceUtilityInfo;
	}

	@Override
	protected Class determineClass() {
		return Invoice.class;
	}

	@Override
	protected Map<String, List<Criterion>> determineWebRestrictions() {
		Map<String, List<Criterion>> out = new HashMap<String, List<Criterion>>();

		if ((this.invoiceNumber != null) && (!this.invoiceNumber.equals("")))
			out.put("invoiceNumber", Arrays
					.asList(new Criterion[] { Restrictions.ilike(
							"invoiceNumber", this.invoiceNumber,
							MatchMode.ANYWHERE) }));
		if ((this.contractNumber != null) && (!this.contractNumber.equals("")))
			out.put("previousDocuments.documentCode", Arrays
					.asList(new Criterion[] { new NestedCriterion(
							"previousDocuments", Restrictions.and(Restrictions
									.ilike("documentCode", this.contractNumber,
											MatchMode.ANYWHERE), Restrictions
									.ilike("documentType", "CONTRATTO",
											MatchMode.EXACT))) }));

		// TODO: attenzione: è necessario garantire consistenza tra il nome del
		// filtro lato web e lato profilo!!!
		// Altrimenti la successiva fase di join dei cirteri non funziona
		if ((this.supplier != null) && (!this.supplier.equals("")))
			out.put("supplier.supplierName", Arrays
					.asList(new Criterion[] { new NestedCriterion("supplier",
							Restrictions.ilike("supplierName", supplier,
									MatchMode.ANYWHERE)) }));
		
		if ((this.vatId != null) && (!this.vatId.equals("")))
			out.put("supplier.vatId", Arrays
					.asList(new Criterion[] { new NestedCriterion("supplier",
							Restrictions.ilike("vatId", supplier,
									MatchMode.START)) }));

		if ((this.invoiceState != null) && (!this.invoiceState.equals("")))
			out.put("invoiceState.description", Arrays
					.asList(new Criterion[] { new NestedCriterion(
							"invoiceState", Restrictions.eq("description",
									invoiceState)) }));

		if ((startDate != null) || (endDate != null))
			out.put("paymentDetail.paymentDueDate", Arrays
					.asList(new Criterion[] { new NestedCriterion(
							"paymentDetail", Restrictions.between(
									"paymentDueDate", DateUtils
											.beginOfDay(startDate), DateUtils
											.endOfDay(endDate))) }));


		if ((this.sector != null) && (!this.sector.equals("")))
			out.put("invoiceLines.meter.sector.sectorName", Arrays
					.asList(new Criterion[] { new NestedCriterion("invoiceLines", 
												new NestedCriterion("meter",
												new NestedCriterion("sector", Restrictions.eq("sectorName", sector)))) }));
		
		
		
		return out;
	}

	@Override
	public void resetSearchParameters() {
		this.invoiceNumber = null;
		this.contractNumber = null;
		this.supplier = null;
		this.vatId = null;
		this.cause = null;
		this.invoiceState = null;
		this.startDate = null;
		this.endDate = null;
		this.sector = null;
	}

	/**
	 * Il metodo salva in un oggetto associato all'handler invece di effettuare
	 * una nuova query tutte le volte perché si suppone che gli stati
	 * disponibili delle fatture non siano variabili. Per far vedere eventuali
	 * modifiche, sarà necessario effettuare logout poi login (ricaricare
	 * l'handler in sessione).
	 * 
	 * @return
	 */
	public SelectItem[] getAvailableInvoiceStates() {
		if (availableInvoiceStates == null) {
			List<SelectItem> items = new ArrayList<SelectItem>();
			items.add(new SelectItem("", "-------"));
			List<InvoiceState> states = JNDIUtils.getCrudManager()
					.findByCriteria(InvoiceState.class, 0, 100, null, null);
			if (states != null)
				for (InvoiceState invoiceState : states)
					items.add(new SelectItem(invoiceState.getDescription(),
							invoiceState.getDescription()));

			availableInvoiceStates = items.toArray(new SelectItem[] {});

		}
		return availableInvoiceStates;
	}
	

	public SelectItem[] getAvailableSector() {
		if (availableSector == null) {
			List<SelectItem> items = new ArrayList<SelectItem>();
			items.add(new SelectItem("", "-------"));
			List<Sector> sectors = JNDIUtils.getCrudManager()
					.findByCriteria(Sector.class, 0, 100, null, null);
			if (sectors != null)
				for (Sector s : sectors)
					items.add(new SelectItem(s.getSectorName(),
							s.getSectorName()));

			availableSector = items.toArray(new SelectItem[] {});

		}
		return availableSector;
	}

	/**
	 * @param availableSector the availableSector to set
	 */
	public void setAvailableSector(SelectItem[] availableSector) {
		this.availableSector = availableSector;
	}

	/**
	 * Questo metodo determina gli effettivi criteri di filtraggio a partire dai
	 * criteri di filtro impostati dall'utente lato Web
	 */
	@Override
	protected DetachedCriteria determineDetachedCriteria() {
		return ProfileUtils.computeDetachedCriteria(determineOrder(),
				super.order, determineClass(), determineWebRestrictions());
	}
	
	public String openDetail() {
		this.selectedInvoice = (Invoice) dataModel.getRowData();
		this.fetchLazyCollections((Invoice) dataModel.getRowData());
		InvoiceInfoHandler invoiceInfoHandler = (InvoiceInfoHandler) JSFUtils.getManagedBean("invoiceInfoHandler");
		invoiceInfoHandler.setMeterSummariesList(selectedInvoice.getMeterSummary());
		invoiceInfoHandler.setPreviousDocumentList(selectedInvoice.getPreviousDocuments());
		
		invoiceInfoHandler.setCurrentDocument(null);
		invoiceInfoHandler.setMeterInfo(null);
		
		return "openDetail";
	}
	
	public void utilityInvoiceInfo(){
		this.shownInvoice = (Invoice) dataModel.getRowData();

		List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(shownInvoice);
		this.shownInvoice.setPreviousDocuments(documents);

	}
	
	private void fetchLazyCollections(Invoice selectedInvoice) {
		List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(selectedInvoice);
		List<InvoiceLine> lines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
		List<TaxDetail> taxDetails = JNDIUtils.getInvoiceManager().fetchTaxDetails(selectedInvoice);
		List<MeterSummary> summaries = JNDIUtils.getInvoiceManager().fetchMeterSummaries(selectedInvoice);
		this.selectedInvoice.setPreviousDocuments(documents);
		this.selectedInvoice.setInvoiceLines(lines);
		this.selectedInvoice.setTaxDetails(taxDetails);
		this.selectedInvoice.setMeterSummary(summaries);
	}

}
