/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.session;

import it.fe.core.par.user.ProfileDetail;
import it.fe.web.common.SessionUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.util.PortalUtil;

/**
 * @author fulvio di marco
 * 
*
 *
 */
public class ProfileDetailSessionHandler implements Serializable {

	private static final long serialVersionUID = 1L;
	
	private static final Logger logger = Logger.getLogger(ProfileDetailSessionHandler.class);
	
	private List<ProfileDetail> profiles;
	private String printableProfiles;
	private String firstName;
	private String lastName;
	private String screenName;
	private String email;
	
	public ProfileDetailSessionHandler() {
		if (profiles == null) {
			Set<ProfileDetail> profilesSet = SessionUtils.getCurrentUserProfileDetail();
			profiles = new ArrayList<ProfileDetail>();
			for(ProfileDetail pd : profilesSet) {
				profiles.add(pd);
			}
		}
	}
	

	public void setProfiles(List<ProfileDetail> profiles) {
		this.profiles = profiles;
	}
	
	public List<ProfileDetail> getProfiles() {
		if (profiles == null) {
			Set<ProfileDetail> profilesSet = SessionUtils.getCurrentUserProfileDetail();
			profiles = new ArrayList<ProfileDetail>();
			for(ProfileDetail pd : profilesSet) {
				profiles.add(pd);
			}
		}
		return profiles;
	}

	public void setPrintableProfiles(String printableProfiles) {
		this.printableProfiles = printableProfiles;
	}

	public String getPrintableProfiles() {
		return this.profilesToString();
	}
	
	private String profilesToString() {
		String output = "";
		for(ProfileDetail pd : this.getProfiles()) {
			if (pd != null) {
				if (pd.getContext() !=null && pd.getRole() !=null) {
					output += pd.getContext().getDescription() + " - " + pd.getRole().getDescription();
				}
			}
		}
		return output;
	}

	public void setScreenName(String screenName) {
		this.screenName = screenName;
	}

	public String getScreenName() {
		if (screenName == null) {
			try {
				screenName = PortalUtil.getUser(SessionUtils.getPortletRequest()).getScreenName();
			} catch (PortalException e) {
				logger.error(e);
			} catch (SystemException e) {
				logger.error(e);
			}
		}
		return screenName;	
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getEmail() {
		if (email == null) {
			try {
				email = PortalUtil.getUser(SessionUtils.getPortletRequest()).getEmailAddress();
			} catch (PortalException e) {
				logger.error(e);
			} catch (SystemException e) {
				logger.error(e);
			}
		}
		return email;	
	}


	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}


	public String getFirstName() {
		if (firstName == null) {
			try {
				firstName = PortalUtil.getUser(SessionUtils.getPortletRequest()).getFirstName();
			} catch (PortalException e) {
				logger.error(e);
			} catch (SystemException e) {
				logger.error(e);
			}
		}
		return firstName;	
	}


	public void setLastName(String lastName) {
		this.lastName = lastName;
	}


	public String getLastName() {
		if (lastName == null) {
			try {
				lastName = PortalUtil.getUser(SessionUtils.getPortletRequest()).getLastName();
			} catch (PortalException e) {
				logger.error(e);
			} catch (SystemException e) {
				logger.error(e);
			}
		}
		return lastName;	
	}
	
	

}
