/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.upload;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.invoice.Supplier;
import it.fe.web.common.SessionUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;

import org.apache.log4j.Logger;
import org.primefaces.event.FileUploadEvent;
import org.primefaces.model.UploadedFile;
import org.richfaces.event.UploadEvent;
import org.richfaces.model.UploadItem;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.util.PortalUtil;

/**
 * @author fulvio di marco
 * 
*
 *
 *
 */
public class InvoiceUploadController implements Serializable {

	private static final long serialVersionUID = 1L;
	
	private static final Logger logger = Logger.getLogger(InvoiceUploadController.class);
	
	private int uploadsAvailable = 1;
	private ArrayList<File> files = new ArrayList<File>();
	private List<Supplier> suppliers;
	private String selectedSupplier = null;
	//private Map<String, String> supplierItems;
	private ArrayList<SelectItem> supplierItems;
	private String user;
	private Date date;
 	
	private static final char extensionSeparator = '.';
	private static final File outputDir = new File(FEPropertiesUtil.getInvoiceManualUploadDir());
	private static final String DATE_FORMAT = "yyyyMMdd";
	
	public int getSize() {
        if (getFiles().size()>0){
            return getFiles().size();
        } else {
        	return 0;
        }
    }
	
	/**
	 * Action listener method that will be notified after file uploaded.
	 * 
	 * Note: RichFaces only.
	 * 
	 * @param event
	 * @throws Exception
	 */
	public void listener(UploadEvent event) throws Exception {
		List<UploadItem> items = event.getUploadItems();
		for(UploadItem i : items) {
			logger.info("Uploading file "+i.getFileName()+"; "+i.getContentType()+"; "+i.getFileSize());
			File uploadedFile = null;
			// uploaded file is stored in a temporary folder
			if (i.isTempFile()) {
				uploadedFile = i.getFile();
				// moving file to expected output dir
				String generatedFileName = generateFileName(i.getFileName());
				if (generatedFileName == null) {
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"Selezionare il fornitore", "Selezionare il fornitore"));
				}
				else {
					if (!this.checkDeployDir()) {
						FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore nella creazione della cartella di caricamento fatture", "Si è verificato un errore nella creazione della cartella di caricamento fatture"));
						return;
					}
					uploadedFile.renameTo(new File(outputDir, generatedFileName));
					uploadedFile = new File(outputDir, generatedFileName);
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"Caricamento file effettuato con successo.", null));
					files.add(uploadedFile);
				    uploadsAvailable--;
				}
				
			}
			// uploaded file is stored in the RAM
			else {
				int dot = i.getFileName().lastIndexOf(extensionSeparator);
				//logger.info("index of extension separator: "+dot);
				//logger.info(i.getFileName().substring(0, dot) + " - " + i.getFileName().substring(dot + 1));
				//File file = File.createTempFile(i.getFileName().substring(0, dot), "."+i.getFileName().substring(dot + 1), outputDir);
				String generatedFileName = generateFileName(i.getFileName());
				if (generatedFileName == null) {
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"Selezionare il fornitore", "Selezionare il fornitore"));
				}
				else {
					if (!this.checkDeployDir()) {
						FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore nella creazione della cartella di caricamento fatture", "Si è verificato un errore nella creazione della cartella di caricamento fatture"));
						return;
					}
					uploadedFile = new File(outputDir+"/"+generatedFileName);
					FileOutputStream fos = new FileOutputStream(uploadedFile);
					fos.write(i.getData());
			        fos.close();
			        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"Caricamento file effettuato con successo.", null));
			        files.add(uploadedFile);
			        uploadsAvailable--;
				}
				
			}
			
	       
		}
		
		//FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"Caricamento file effettuato con successo.", null));
		
	}
	
	
	/**
	 * Utility method used to check if manual deploy folder exists. In case it does not exist, this method creates it.
	 */
	private boolean checkDeployDir() {
		File uploadDirectory = new File(FEPropertiesUtil.getInvoiceManualUploadDir());
		if (uploadDirectory.isDirectory() && uploadDirectory.exists()) {
			return true;
		}
		else {
			if (uploadDirectory.mkdir()) {
				return true;
			}
			else {
				return false;
			}
		}
	}

	/**
	 * Action listener method that will be notified after file uploaded.
	 * 
	 * Note: Primefaces only.
	 * 
	 * @param event
	 */
	public void handleFileUpload(FileUploadEvent event) {
	     UploadedFile file = event.getFile();
	     //application code
	}
	
	public String clearUploadedData() {
        files.clear();
        setUploadsAvailable(10);
        this.selectedSupplier=null;
        return null;
    }
	
	private String generateFileName(String uploadedFileName) {
		if (getSelectedSupplier() == null) {
			return null;
		}
		String fileName = "";
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
		fileName=sdf.format(getDate())+"_"+user+"_"+getSelectedSupplier().replaceAll(" ","")+"_"+uploadedFileName.replaceAll(" ","");
		return fileName;
		
	}
	
	public void supplierChanged(ValueChangeEvent event) {
		setSelectedSupplier((String) event.getNewValue());
	}


	/**
	 * @return the uploadsAvailable
	 */
	public int getUploadsAvailable() {
		return uploadsAvailable;
	}


	/**
	 * @param uploadsAvailable the uploadsAvailable to set
	 */
	public void setUploadsAvailable(int uploadsAvailable) {
		this.uploadsAvailable = uploadsAvailable;
	}


	/**
	 * @return the files
	 */
	public ArrayList<File> getFiles() {
		return files;
	}


	/**
	 * @param files the files to set
	 */
	public void setFiles(ArrayList<File> files) {
		this.files = files;
	}

	public void setSuppliers(List<Supplier> suppliers) {
		this.suppliers = suppliers;
	}

	public List<Supplier> getSuppliers() {
		if (suppliers == null) {
			suppliers = JNDIUtils.getSupplierManager().getSuppliers();
		}
		return suppliers;
	}

//	public void setSupplierItems(Map<String, String> supplierItems) {
//		this.supplierItems = supplierItems;
//	}

//	public Map<String, String> getSupplierItems() {
//		if (supplierItems == null) {
//			supplierItems = new LinkedHashMap<String, String>();
//			supplierItems.put(null, "Seleziona il fornitore");
//			for (Supplier supplier : getSuppliers()) {
//				supplierItems.put(supplier.getSupplierName(), supplier.getSupplierName());
//			}
//		}
//		return supplierItems;
//	}

	public void setUser(String user) {
		this.user = user;
	}

	public String getUser() {
		if (user == null) {
			try {
				user = PortalUtil.getUser(SessionUtils.getPortletRequest()).getScreenName();
			} catch (PortalException e) {
				logger.error(e);
			} catch (SystemException e) {
				logger.error(e);
			}
		}
		return user;	
	}

	public void setDate(Date date) {
		this.date = date;
	}

	public Date getDate() {
		return new Date();
	}

	public void setSelectedSupplier(String selectedSupplier) {
		this.selectedSupplier = selectedSupplier;
	}

	public String getSelectedSupplier() {
//		if (selectedSupplier == null) {
//			selectedSupplier = "default"; // default value;
//	    }
	    return selectedSupplier;

	}

	public void setSupplierItems(ArrayList<SelectItem> supplierItems) {
		this.supplierItems = supplierItems;
	}

	/**
	 * @return the supplierItems
	 */
	public ArrayList<SelectItem> getSupplierItems() {
		if (supplierItems == null) {
			supplierItems = new ArrayList<SelectItem>();
			supplierItems.add(new SelectItem(null, "Seleziona un fornitore"));
			for (Supplier supplier : getSuppliers()) {
				supplierItems.add(new SelectItem(supplier.getVatId(), supplier.getSupplierName()));
			}
		}
		return supplierItems;
	}



}
